package jabba;

import java.awt.LayoutManager;
import java.awt.Container;
import java.awt.Component;
import java.awt.Insets;
import java.awt.Dimension;


/**
 * This layout manager orders components in a top-down manner.
 */
class ColumnLayout implements LayoutManager {

  int xInset = 5;
  int yInset = 5;
  int yGap = 2;

  /**
   * Empty.
   */
  public void addLayoutComponent(String s, Component c) {}

  /**
   * Orders the components.
   * @param cont container for the components to be placed in.
   */
  public void layoutContainer(Container cont) {
      Insets insets = cont.getInsets();
      int height = yInset + insets.top;

      Component[] children = cont.getComponents();
      Dimension compSize = null;
      for (int i = 0; i < children.length; i++) {
        compSize = children[i].getPreferredSize();
        children[i].setSize(compSize.width, compSize.height);
        children[i].setLocation( xInset + insets.left, height);
        height += compSize.height + yGap;
      }

  }

  /**
   * Returns minimum size required by ordered components.
   * @param cont container for which the size computation 
   * is going to be performed
   */
  public Dimension minimumLayoutSize(Container cont) {
      Insets insets = cont.getInsets();
      int height = yInset + insets.top;
      int width = 0 + insets.left + insets.right;

      Component[] children = cont.getComponents();
      Dimension compSize = null;
      for (int i = 0; i < children.length; i++) {
        compSize = children[i].getMinimumSize();
        height += compSize.height + yGap;
        width = Math.max(width, compSize.width + insets.left + insets.right + xInset*2);
      }
      height += insets.bottom;
      return new Dimension( width, height);
  }

  /**
   * Returns combined preferred components' size.
   * @param cont container, the computation is going to be performed for 
   */
  public Dimension preferredLayoutSize(Container cont) {
      Insets insets = cont.getInsets();
      int height = yInset + insets.top;
      int width = 0 + insets.left + insets.right;

      Component[] children = cont.getComponents();
      Dimension compSize = null;
      for (int i = 0; i < children.length; i++) {
        compSize = children[i].getPreferredSize();
        height += compSize.height + yGap;
        width = Math.max(width, compSize.width + insets.left + insets.right + xInset*2);
      }
      height += insets.bottom;
      return new Dimension( width, height);
  }

  /**
   * Empty.
   */
  public void removeLayoutComponent(Component c) {}
}
