package jabba;

import com.sun.java.swing.table.AbstractTableModel;
import com.sun.java.swing.event.TableModelEvent;

/** 
 * Receives messages from Receiver object and directly accesses it.
 */
class ViewerEquipmentTableDataModel 
    extends AbstractJabbaTableDataModel 
    implements JabbaConstants {
  private final static String[] columnNames = {
    "Name", "Type", "Last seen", "Quality", "Seen at"
  };
  
  // przykladowe dlugie wartosci
  private final static Object[] longValues = {
    "qqqqqqqqqqqq", "SUN_multimedia_workstation", 
    "00-00-00", new Integer(100), "ICS corridor" };

  // ktora kolumna gdzie (indeksy w tablicy columnNames)
  final static int COLUMN_NAME = 0;
  final static int COLUMN_LOCATION_NAME = 4;
  final static int COLUMN_QUALITY = 3;
  final static int COLUMN_TIME = 2;
  final static int COLUMN_TYPE = 1;

  final static int ATTRIBUTE_INDEX = -2;

  // glowny obiekt programu
  Jabba jabba;

  // obiekt przechowujacy dane
  Receiver receiver;

  ViewerEquipmentTableDataModel( Jabba jabba ) {
    super();
    this.jabba = jabba;
    receiver = jabba.receiver;
    jabba.console.addWriter( this, "1st equipment table model" );
  }

  // pomocnicza w okresleniu szerokosci kolumn
  public Object getLongValue( int clmn ) {
    return longValues[clmn];
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public int getRowCount() {
    return receiver.getObservedObjectsCount();
  }

  public Class getColumnClass(int col) {
    Object o = getValueAt(0,col);
    if( o!=null ) return o.getClass();
    else return java.lang.String.class;
  }

  public Object getValueAt( int row, int clmn ) {
    switch( clmn ) {
    case COLUMN_NAME: 
      return receiver.getEquipmentName(row);
    case COLUMN_LOCATION_NAME: 
      return receiver.getEquipmentLocationDescription(row);
    case COLUMN_QUALITY: 
      return receiver.getEquipmentSightingQuality(row);
    case COLUMN_TIME:
      return receiver.getEquipmentSightingTime(row);
    case COLUMN_TYPE:
      return receiver.getEquipmentType(row);

    case ATTRIBUTE_INDEX:
      return new Integer(row);
    case ATTRIBUTE_SELECTED:
	return receiver.getEquipmentRecordSelected(row);
    case ATTRIBUTE_REFERENCE:
      return receiver.getEquipmentRecordReference(row);

    default: break;
    }

    return new String("N.A.");
  }

  public String getColumnName( int clmn ) {
    return columnNames[clmn];
  }

  // implementacja JabbaDataModel
  public void equipment_nameChanged(int pos) {
    jabba.console.messageFrom( this, "nameChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_NAME );
  }

  public void equipment_locationChanged(int pos) {
    jabba.console.messageFrom( this, "locationChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_LOCATION_NAME );
  }

  public void equipment_typeChanged(int pos) {
    jabba.console.messageFrom( this, "typeChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_TYPE );
  }

  public void equipment_qualityChanged(int pos) { 
    jabba.console.messageFrom( this, "qualityChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_QUALITY );
  }
  
  public void equipment_timeChanged(int pos) {
    jabba.console.messageFrom(this, "timeChanged("+pos+") called." );
    fireTableCellUpdated( pos,COLUMN_TIME );
  }

  public void equipment_added(int pos) {
    fireTableRowsInserted(pos-1,pos);
  }

  public void equipment_removed(int pos) {
    fireTableRowsDeleted(pos,pos);
  }

}
