package jabba;

import com.sun.java.swing.table.AbstractTableModel;
import com.sun.java.swing.event.TableModelEvent;

/**
 * Displays location info obtained from the receiver.
 */
class ViewerLocationsTableDataModel 
    extends AbstractJabbaTableDataModel 
    implements JabbaConstants {
  private final static String[] columnNames = {
    "Number", "Description", "Phones", "People #" 
  };

  private final static Object[] longValues = {
    "666a", "ICS corridor", "0997,0998", new Integer(77)
  };

  // indeksy w tablicy columnNames
  final static int COLUMN_NUMBER=0;
  final static int COLUMN_DESCRIPTION=1;
  final static int COLUMN_PHONES=2;
  final static int COLUMN_PEOPLE=3;

  Jabba jabba;
  Receiver receiver;

  ViewerLocationsTableDataModel( Jabba jabba ) {
    super();
    this.jabba = jabba;
    receiver = jabba.receiver;
    jabba.console.addWriter( this, "1st loc. table model" );
  }

  public Object getLongValue( int clmn ) {
    return longValues[clmn];
  }

  public int getRowCount() {
    return receiver.getObservedObjectsCount();
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public Class getColumnClass( int clmn ) {
    Object o = getValueAt(0,clmn);
    if( o!=null ) return o.getClass();
    else return java.lang.String.class;
  }

  public String getColumnName( int clmn ) {
    return columnNames[clmn];
  }

  public Object getValueAt( int row, int column ) {
    switch( column ) {
    case COLUMN_NUMBER:
      return receiver.getLocationNumber(row);
    case COLUMN_DESCRIPTION:
      return receiver.getLocationDescription(row);
    case COLUMN_PHONES:
      return receiver.getLocationPhones(row);
    case COLUMN_PEOPLE:
      return receiver.getLocationPeopleNumber(row);

    case ATTRIBUTE_INDEX:
      return new Integer(row);
    case ATTRIBUTE_SELECTED:
      return receiver.getLocationRecordSelected(row);
    case ATTRIBUTE_REFERENCE:
	return receiver.getLocationRecordReference(row);

    default: break;
    }
    return new String("N.A.");
  }

  // implementacja JabbaDataModel
  
  public void location_peopleNumberChanged( int row ) {
    jabba.console.messageFrom( this, "peopleNumberChanged("+row+") called." );
    fireTableCellUpdated( row,COLUMN_PEOPLE );
  }

  public void location_added(int pos) {
    fireTableRowsInserted(pos-1,pos);
  }

  public void location_removed(int pos) {
    fireTableRowsDeleted(pos,pos);
  }

}
