package jabba;

import com.sun.java.swing.JTable;
import com.sun.java.swing.JPanel;
import com.sun.java.swing.JButton;
import com.sun.java.swing.JScrollPane;
import com.sun.java.swing.table.TableColumn;
import com.sun.java.swing.table.TableColumnModel;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

class ViewerMainTab extends JPanel implements ViewerTab {
  JTable table;
  ViewerMainTableDataModel dataModel;
  String name;

  Jabba jabba;
  Viewer viewer;

  ViewerMainTab( Jabba jabba, Viewer viewer ) {
    this( jabba,viewer,"People" );
  }

  ViewerMainTab( Jabba jabba, Viewer viewer, String name ) {
    super( false );
    this.jabba = jabba;
    this.viewer = viewer;
    this.name = name;
    jabba.console.addWriter( this,"Viewer main tab" );

    setLayout( new BorderLayout() );
  }

  public void regenerate() {
    jabba.console.messageFrom( this, "regenerate() called." );

    // usuwamy stara tabele
    removeAll();

    // tworzymy model danych
    dataModel = new ViewerMainTableDataModel(jabba);

    jabba.receiver.registerDataModel( dataModel,Receiver.USER_MESSAGES );

    // tworzymy tabele
    TableSorter ts = new TableSorter(dataModel);
    table = new JTable(ts,null,ts.getSelectionModel());
    table.getColumnModel().getColumn( dataModel.COLUMN_QUALITY ).setCellRenderer(new QualityRenderer());
    table.setFont( Viewer.defaultFont );
    ts.addMouseListenerToHeaderInTable(table);

    table.setPreferredScrollableViewportSize( new Dimension(500,100) );
    
    ViewerTabHelper.initColumnSizes( table,dataModel );
    table.addMouseListener( new MouseAdapter() {
      public void mouseClicked( MouseEvent ev ) {
	if( ev.getClickCount()==2 ) addNewPersonTab();
      }
    } );

    // tworzymy przyciski
    JPanel btnPanel = new JPanel( new FlowLayout(FlowLayout.RIGHT) );
    JButton btnSelectPresent = new JButton( "Select present users" );
    btnSelectPresent.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent ev ) {
	table.clearSelection();
	int val;
	for( int i=0; i<table.getRowCount(); i++ ) {
	  val = ((Integer)table.getValueAt(i,table.convertColumnIndexToView(dataModel.COLUMN_QUALITY))).intValue();
	  if( val>0 ) {
	    table.addRowSelectionInterval(i,i);
	  }
	}
      }
    } );

    btnPanel.add( btnSelectPresent );


    this.add( JTable.createScrollPaneForTable(table),BorderLayout.CENTER );
    this.add( btnPanel,BorderLayout.SOUTH );
  }    

  public void addNewPersonTab() {
    jabba.console.messageFrom( this, "addNewPersonTab() called." );

    String tabName = (String)(table.getValueAt( table.getSelectedRow(),
      table.convertColumnIndexToView(ViewerMainTableDataModel.COLUMN_NAME) ));

    int person_index = ((Integer)table.getValueAt( table.getSelectedRow(),
      ViewerMainTableDataModel.ATTRIBUTE_INDEX )).intValue();

    Object person_ref = table.getValueAt( table.getSelectedRow(),
      ViewerMainTableDataModel.ATTRIBUTE_REFERENCE );

    DismissableViewerTab vt=viewer.createPersonTab( tabName,person_index,person_ref );
    vt.setParentTab( this );
    viewer.addTab( vt );
    viewer.selectTab( vt );
  }

  public void update() {
    jabba.console.messageFrom( this, "update() called." );

    if( dataModel==null ) regenerate();
    else jabba.receiver.updateDataModel( dataModel,Receiver.USER_MESSAGES );
  }
    

  public JabbaDataModel getJabbaDataModel() {
    return dataModel;
  }

  public JPanel getPanel() {
    return this;
  }

  public String getName() {
    return name;
  }

}
