
#include <stream.h>
#include "abng_core_types.h"
#include "db_utils.hh"

ABng_Core_System::Sensor::Pattern 
 Sensor_Types::init_pattern() {

  ABng_Core_System::Sensor::Pattern pattern;
  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.id_pattern.is_any = 1;
  return pattern;
}

ostream& operator << (
  ostream &str, 
  ABng_Core_System::Sensor::Value_Description& value
) {
  str << value.item_id << "  " 
      << value.id.sensor_num << " " 
      << value.id.network_num << " " 
      << value.id.group_name;
  return str;
}

ABng_Core_System::ABng_Location_Description::Pattern 
 ABng_Location_Description_Types::init_pattern() {

  ABng_Core_System::ABng_Location_Description::Pattern pattern;

  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.loc_descr_pattern.is_any = 1;
  pattern.loc_descr_pattern.property_list_pattern.constraints= 
    Domain_Repository::UNRESTRICTED;
  pattern.sensor_list_pattern.is_any = 1;
  pattern.sensor_list_pattern.constraints =
    Domain_Repository::UNRESTRICTED;

  return pattern;
}

ostream& operator << (
  ostream& str, 
  ABng_Core_System::ABng_Location_Description::Value_Description& value
) {

  str << value.item_id << " ";
  str << "(" << value.loc_descr << ") " 
      << value.sensor_list.length() << " ";
  for(int i = 0; i < value.sensor_list.length(); i ++) {
    str << "(" << value.sensor_list[i] << ") ";
  }
  return str;
}


ABng_Core_System::Badge::Pattern 
 Badge_Types::init_pattern() {

  ABng_Core_System::Badge::Pattern pattern;
  pattern.is_any = 1;
  pattern.item_id_pattern.is_any = 1;
  pattern.id_pattern.is_any = 1;
  return pattern;
}

ostream& operator << (
  ostream& str, 
  ABng_Core_System::Badge::Value_Description& value
) {

  str << value.item_id << " "
#ifdef ORBIX
      << (_IDL_SEQUENCE_octet&) value.id << dec << " "
#else
      << value.id << dec << " "
#endif
      << (CORBA::is_nil( value.holder) ? 0 : value.holder->item_id());
  return str;
}

ABng_Core_System::Badge_Holder::Pattern 
 Badge_Holder_Types::init_pattern() {

  ABng_Core_System::Badge_Holder::Pattern pattern;
  Domain_Repository::User::Pattern user_pattern;

  pattern.is_any= 1;
  pattern.item_id_pattern.is_any= 1;
  pattern.my_badge_pattern.is_any= 1;
  pattern.current_badge_pattern.is_any= 1;
  pattern.home_sensor_pattern.is_any= 1;
  pattern.type_specific_object_pattern.is_any= 1;
  pattern.type_specific_object_pattern.object_pattern.user_obj_pattern(user_pattern);
  pattern.assigned_pattern.is_any= 1;
  pattern.visitor_pattern.is_any= 1;
  return pattern;
}

ostream& operator << (
  ostream& str, 
  ABng_Core_System::Badge_Holder::Value_Description& value
) {

  str << value.item_id << " "
      << "(" << value.my_badge << ") "
      << "(" << value.current_badge << ") ";

   if (value.home_sensor.item_id == 0) 
       str << "no home sensor" << " ";
   else
       str << "(" << value.home_sensor << ") ";

   if (value.type_specific_object._d() == ABng_Core::LOCATABLE_USER)
       str << "(" <<value.type_specific_object.user_description() << ") ";
   else
       str << "(" << value.type_specific_object.equipment_description() << ") ";

   str << (value.assigned ? "true" : "false") << ' ';
   str << (value.visitor ? "true" : "false") << ' ';

  return str;
}
