// $Id: basic_client.cc,v 1.2 1998/07/29 13:14:45 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/basic_client.cc,v $
//
// $Log: basic_client.cc,v $
// Revision 1.2  1998/07/29 13:14:45  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
// File:          rep_client.cc
// Author:        Malgorzata Steinder (gosia@ics.agh.edu.pl)
// Date Created:  16.03.98
//
// General Description:
//  Implementation of the Rep_Client class which is a skeleton for ABng 
//  repository utilities.

#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include "basic_client.h"
#include "compatibility.hh"


template<class Repository_Types>
  void Basic_Client<Repository_Types>::init_rep_ptr() {

  CORBA::Object_var obj_ptr;   
  try {
     obj_ptr = Bootstrap::resolve((*srv_rep)());
  } catch(...) {
    cout << "Repository cannot be resolved " << endl;
    ABng_Server_Manager::Name name= (*srv_rep)();
    for (int i=0; i<name.length(); i++)
      cout << name[i] << " ";
    cout << endl;
    exit(1);
  }
  if (CORBA::is_nil(obj_ptr)){     
     cerr << "Repository client error: ";
     cerr << "Repository object reference is nill!" 
          << endl;
     exit(-1);
  }

  try {
      my_rep = Repository::_narrow(obj_ptr);
      if (CORBA::is_nil( my_rep)) {
	cerr << "Repository client error: ";
	cerr << "exception when trying to _narrow! (got nil ref)" << endl;
	exit(-1);	
      }
  } catch (CORBA::SystemException& se){
      cerr << "Repository client error: ";
      cerr << "exception when trying to _narrow! " << &se << endl;
      exit(-1);
  }
     

}

template<class Repository_Types>
void Basic_Client<Repository_Types>::init(int argc, char **argv) {

  try {
    int argc = 0; char **argv = 0;
    orb_ptr = CORBA::ORB_init(argc, argv, ORB_IDENTIFIER); 
    boa_ptr = orb_ptr->BOA_init(argc, argv, BOA_IDENTIFIER);
  }
  catch(...) {
    cerr << "Cannot initialise ORB or BOA" << endl;
    exit(1);
  }
  Bootstrap::init( orb_ptr);

  init_rep_ptr();
}


template<class Repository_Types>
Basic_Client<Repository_Types>::Item_ptr 
  Basic_Client<Repository_Types>::init_item_ptr(char* str) {
  Item_ptr i;
  if(strncmp(str, "IOR", 3)) {
      Domain_Repository::Repository_Item_Id id = atoi(str);
      if (id == 0) 
	return Item::_duplicate( Item::_nil());
      Pattern pattern = init_pattern_id(id);
      Items *is;
      try {
         is = my_rep->find(pattern);
      } catch(Domain_Repository::Search_Not_Implemented) {
         cerr << "Exception: Domain_Repository::Search_Not_Implemented"<< endl;
         exit(1);
     } catch(Domain_Repository::Unknown_Object_Ref) {
         cerr << "Exception: Domain_Repository::Unknown_Object_Ref" << endl;
         exit(1);
     } catch(Domain_Repository::Destination_Unreachable &de) {
         cerr << "Exception: Domain_Repository::Destination_Unreachable "
	      << de.server_name
              << endl;
         exit(1);
     } catch(CORBA::SystemException ex) {
         cerr << "Exception: CORBA::SystemException " << ex << endl;
#ifdef OMNIORB
	 cerr << "ID " << ex.NP_RepositoryId() << endl;
#endif
         exit(1);
     } catch(...) {
         cerr << "Exception: Unknown" << endl;
         exit(1);
     }
     if(is->length() == 0) {
         cerr << "No items with this id found" << endl;
         exit(1);
     }
     if(is->length() > 1) {
         cerr << "Too many items with this id found" << endl;
         exit(1);
     }
     i = Item::_duplicate((*is)[0]);
     return i;
  }
  else {
    CORBA::Object_ptr obj;
    try {
       obj = orb_ptr->string_to_object(str);
       if(obj == NULL) {
	 cerr << "Wrong object reference" << endl;
         exit(1);
       }
       i = Item::_narrow(obj);
       if(i == NULL) {
         cerr << "The reference of the wrong object type" << endl;
         exit(1);
       }
    } catch(CORBA::SystemException ex) {
       cerr << "Exception: CORBA::SystemException " << ex << endl;
       exit(1);
    } catch(...) {
       cerr << "Exception: Unknown" << endl;
       exit(1);
    } 
  }
  return i;
}

template<class Repository_Types>
Basic_Client<Repository_Types>::Pattern
 Basic_Client<Repository_Types>::init_pattern() {
   Pattern pattern = Repository_Types::init_pattern();
   return pattern;
}

template<class Repository_Types>
Basic_Client<Repository_Types>::Pattern
Basic_Client<Repository_Types>::init_pattern_id(
  Domain_Repository::Repository_Item_Id id
  ) {

  Pattern pattern = init_pattern();

  pattern.is_any = 0;
  pattern.item_id_pattern.is_any = 0;
  pattern.item_id_pattern.value = id;

  return pattern;
}






