// -*- Mode: C++; -*-
// $Id: basic_rep_i.h,v 1.2 1998/07/29 13:14:47 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/basic_rep_i.h,v $
//
// $Log: basic_rep_i.h,v $
// Revision 1.2  1998/07/29 13:14:47  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:56:16  mencnar
// Initial revision
//
//
// File:          basic_rep_i.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  02.02.98
//
// General Description:
//  Header file for the template implementation base class for ABng 
//  repositories.

#ifndef __BASIC_REP_I_H__
#define __BASIC_REP_I_H__

#include "observed_i.h"
#include "db_utils.hh"
#include <rw/tphasht.h>
#include <rw/tpdlist.h>
#include "compatibility.hh"

template <class Data_Object, class Observer_List, class Observer_List_Iterator>
struct T_Notify_Params {
  Data_Object data_object_p;
  Observer_List obs_list;
  Domain_Repository::Change_Type type_p;
  Domain_Repository::Repository_Item_Id item_id;
  
  T_Notify_Params( Observer_List &_obs_list);
  ~T_Notify_Params();
};

template <class Item_Types, class Observer_Types>
class No_Update_Basic_Rep_I : public Observed_I<Observer_Types> {

public:

  typedef Item_Types::Item Item;
  typedef Item_Types::Item_ptr Item_ptr;
  typedef Item_Types::Items Items;
  typedef Item_Types::Item_I Item_I;
  typedef Item_Types::Initial_Data Initial_Data;
  typedef Item_Types::Data_Object Data_Object;
  typedef Item_Types::Item::Description Description;
  typedef Item_Types::Item::Descriptions Descriptions;
  typedef Item_Types::Item::Pattern Pattern;
  typedef Item_Types::Item::Entity_Set Entity_Set;
  typedef Item_Types::Item::Query_Results Query_Results;
  typedef Item_Types::Item::Value_Description Value_Description;
  typedef Item_Types::Item::Value_Descriptions Value_Descriptions;
 
  typedef Basic_Observed_I<Observer_Types>::Observer_List Observer_List;
  typedef Basic_Observed_I<Observer_Types>::Observer_ptr Observer_ptr;

  typedef T_Notify_Params<Data_Object,Observer_List,Observer_List_Iterator>
    Notify_Params;

  No_Update_Basic_Rep_I(unsigned (*hash_item_p)(const Item_I&), 
#ifdef ObjectStore
    os_database* db_p
#elif FilePersistency
    const char *repository_name_p
#endif
  );

  virtual ~No_Update_Basic_Rep_I();

  virtual void init(int cold_start_p = 0);

  virtual Items* item_list (
    ENV_DECL1(env));

  virtual Items* find(
    const Pattern& data_record_pattern
    ENV_DECL(env));
 
 virtual Value_Descriptions* find_values (
   const Pattern& data_record_pattern
   ENV_DECL(env));

 virtual Query_Results* complex_query (
   const Entity_Set& es,
   const Domain_Repository::Query_Options &qo,
   Observer_ptr obs
   ENV_DECL(env));
		     
  virtual Item_ptr add (
    const Initial_Data& data_record
    ENV_DECL(env));

  virtual void remove (
    Item_ptr object_to_remove
    ENV_DECL(env));

  virtual Items* private_find(
    const Pattern& data_record_pattern
    ENV_DECL(env)) = 0;
 
 virtual Value_Descriptions* private_find_values (
   const Pattern& data_record_pattern
   ENV_DECL(env));
		     
  virtual Item_ptr private_add (
    const Initial_Data& data_record
    ENV_DECL(env));

  virtual void private_remove (
    Item_ptr object_to_remove
    ENV_DECL(env));

private:
  ABng_S_Guarded notifier_guard;
  omni_condition notifier_cond;
  RWTPtrDlist< Notify_Params> notifier_queue;

  void insert_into_notifier( Notify_Params *ntf);
  static void static_notify_worker( void *args);
  void notify_worker();


protected:
  // complex query helper methods
  virtual Description get_null_description( Item_I *i) { 
    Description d; 
    if (i) d.item_id= i->item_id();
    return d; 
  }
  virtual Value_Description get_null_value_description( Item_I *i) { 
    Value_Description vd; 
    if(i) vd.item_id= i->item_id();
    return vd; 
  }

  // Add helper methods
  virtual int validate_add_data(const Initial_Data& data_record) {return 1;}
  virtual Item_I *create_item(const Initial_Data& data_record);
  virtual int add_permitted(Item_I* item_p);
  virtual void init_corba_object(Item_I *item_p) = 0;
  virtual void insert_item(Item_I* item_p);

  // Remove helper methods
  virtual int validate_remove_data(Item_ptr item_p);
  virtual int remove_permitted(Item_I* item_p);
  virtual void uninit_corba_object(Item_I* item_p);
  virtual void remove_item(Item_I* item_p);

  // Find helper methods
  virtual void collect_values(Item_I *item_p, Value_Description& vd) = 0;
  virtual Value_Descriptions *collect_value_descriptions( Items &matching_items);

  // Common helper methods
  virtual Item_I *deref_item(Item_ptr item_p);
  virtual void construct_data_object(Data_Object& data_object_p, 
    Item_ptr item_p)  = 0;
  virtual void notify_observers(
    Observer_List& obs_list,
    Item_I *item_p,
    Domain_Repository::Change_Type type_p);
  virtual void save();
  virtual void save(ostream &);
  virtual void restore (istream &);
  virtual Domain_Repository::Repository_Item_Id get_item_counter() {
    return item_counter;
  }
  virtual Domain_Repository::Repository_Item_Id assign_id() {
    return item_counter++;
  }  
  virtual DB_Header *create_db_header() = 0;
  static void gather_item(Item_I* current_item, void* apply_data);  

  static void write_ior(const CORBA::Object_ptr obj_p, 
    const char *file_name);

  virtual void create_corba_objects() = 0;
  virtual void destroy_corba_objects() = 0;

  // Attributes:

  // internal item id counter of the repository
  Domain_Repository::Repository_Item_Id item_counter;

  typedef RWTPtrHashTable<Item_I> Item_List;
  typedef RWTPtrHashTableIterator<Item_I> Item_List_Iterator;
  typedef RWTValHashDictionary<Domain_Repository::Repository_Item_Id, 
    Item_I*> Id_Item_Dictionary;
  typedef RWTValHashDictionaryIterator<Domain_Repository::Repository_Item_Id, 
    Item_I*> Id_Item_Dictionary_Iterator;

  // list of items
  Item_List my_item_list;
  Id_Item_Dictionary my_item_deref;

  unsigned (*hash_item)(const Item_I&);

#ifdef ObjectStore
  os_database *my_db;  
#elif defined FilePersistency
  char* my_repository_name;
#endif
};


template <class Item_Types, class Observer_Types>
class Basic_Rep_I : public No_Update_Basic_Rep_I<Item_Types, Observer_Types> {

public:
  Basic_Rep_I(unsigned (*hash_item_p)(const Item_I&), 
#ifdef ObjectStore
    os_database* db_p
#elif FilePersistency
    const char *repository_name_p
#endif
	      ) :
    No_Update_Basic_Rep_I<Item_Types, Observer_Types>(
    hash_item_p,
#ifdef ObjectStore
    db_p
#elif FilePersistency
    repository_name_p
#endif
							) {}

  virtual void update (
    Item_ptr object_to_update, 
    const Initial_Data& data_record_pattern
    ENV_DECL(env));

  virtual void private_update (
    Item_ptr object_to_update, 
    const Initial_Data& data_record_pattern
    ENV_DECL(env));

protected:
  // Update helper methods
  virtual int validate_update_data(Item_ptr item_p, 
				   const Initial_Data& data_record_pattern);

  virtual int update_permitted(Item_I* item_p,
			       const Initial_Data& data_record_pattern,
			       Item_I* objtu);

  virtual void update_item(Item_I* item_p, 
			   const Initial_Data& data_record_pattern);

};

#endif /* __BASIC_REP_I_H__ */

