// $Id: basic_server.cc,v 1.2 1998/07/29 13:14:48 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/basic_server.cc,v $
//
// $Log: basic_server.cc,v $
// Revision 1.2  1998/07/29 13:14:48  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
// File:          basic_server.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  02.02.98
//
// General Description:
//  Definition of the Basic_Server class implementing operations
//  commonly performed in CORBA servers.

#include <iostream.h>
#include <stdlib.h>
#include "basic_server.h"
#include "bootstrap_n.hh"
#include "compatibility.hh"

void Basic_Server::init_corba(int &argc, char ** &argv) {


  try {
    orb_ptr = CORBA::ORB_init(argc, argv, ORB_IDENTIFIER); 
    boa_ptr = orb_ptr->BOA_init(argc, argv, BOA_IDENTIFIER);
#ifdef OMNIORB
  // Turn off garbage collecting of unused connections
  //
  omniORB::idleConnectionScanPeriod( omniORB::idleIncoming, 0);
  omniORB::idleConnectionScanPeriod( omniORB::idleOutgoing, 0);
#endif
  }
  catch(...) {
    cerr << "Cannot initialise ORB or BOA" << endl;
    exit(1);
  }
  Bootstrap::init( orb_ptr);
};


void Basic_Server::run() {
  for (int i=0; i<thread_pool-1; i++) {
    omni_thread *thr;
    thr= new omni_thread( static_impl_is_ready, (void*) this);
    thr->start();
  }
  static_impl_is_ready( this);
}

void Basic_Server::static_impl_is_ready( void *args) {
 Basic_Server *ptr= (Basic_Server*) args;

#ifdef DEBUG
 cerr << "run ->impl_is_ready" << endl;
#endif
#ifdef ORBIX
 ptr->boa_ptr->processEvents( ptr->my_timeout);
#elif defined OMNIORB
 ptr->boa_ptr->impl_is_ready();
#endif
}

#ifdef ORBIX
void Basic_Server::set_orbix_conf(
  unsigned long max_retries_p,
  unsigned long connection_timeout_p,
  unsigned long default_tx_timeout_p,
  unsigned long object_table_size_p,
  int diagnostics_p,
  int propagate_tie_detele_p,
  int abort_slow_connection_p) {

  orb_ptr->setDiagnostics(diagnostics_p);
  boa_ptr->propagateTIEdelete(propagate_tie_detele_p);
  orb_ptr->maxConnectRetries(max_retries_p);
  orb_ptr->connectionTimeout(connection_timeout_p);
  orb_ptr->defaultTxTimeout(default_tx_timeout_p);
  orb_ptr->reSizeObjectTable(object_table_size_p);
  orb_ptr->abortSlowConnects(abort_slow_connection_p);
};
#endif  





