//
// $Id: bootstrap_n.hh,v 1.2 1998/07/29 13:14:51 mencnar Exp $
//
// $Log: bootstrap_n.hh,v $
// Revision 1.2  1998/07/29 13:14:51  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:56:04  mencnar
// Initial revision
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: Bootstrap class header file
//              Hides functionality of ABng_Server_Manager::Register interface
//

#ifndef __BOOTSTRAP_N_H__
#define __BOOTSTRAP_N_H__ 1

#include "bootstrap_base.hh"
#include "abng_server_manager.hh"
#include "compatibility.hh"
#include "abng_guarded.hh"
#include <rw/tpdlist.h>
#include <sys/time.h>

ostream & operator << ( ostream & ostr, 
			const ABng_Server_Manager::Name &name);

istream & operator >> ( istream &istr,
			ABng_Server_Manager::Name &name);

int operator == ( const ABng_Server_Manager::Name &name1,
		  const ABng_Server_Manager::Name &name2);

int operator == ( const ABng_Server_Manager::Name &name1,
		  const char *name2);

int operator >= ( const ABng_Server_Manager::Name &name1,
		  const ABng_Server_Manager::Name &name2);

ABng_Server_Manager::Name operator + ( const ABng_Server_Manager::Name &name,
				       const char *add_name);
ABng_Server_Manager::Name operator - ( const ABng_Server_Manager::Name &name1,
				       const ABng_Server_Manager::Name &name2);

class Bootstrap_listener;

class SM_CallbackImpl : public BOA_IMPL_BASE(ABng_Server_Manager,Callback) {
  Bootstrap_listener &listener;
public:
  SM_CallbackImpl( Bootstrap_listener &_listener) :
    listener( _listener) {
    CONNECT_IMPL( ABng_Server_Manager::Callback_var,this);
  }

  void server_startup( CORBA::Object_ptr server_reference,
		       const ABng_Server_Manager::Name &name,
		       ABng_Server_Manager::uid callback_identifier
		       ENV_DECL( env) );

  void server_shutdown( const ABng_Server_Manager::Name &name,
			ABng_Server_Manager::uid callback_identifier
			ENV_DECL( env) );
};

class Bootstrap_listener {
public: // for tests only, should be private

  SM_CallbackImpl callbackimpl;
public:
  Bootstrap_listener();

  ABng_Server_Manager::uid register_callback( 
				 const ABng_Server_Manager::Name &name);
  ABng_Server_Manager::uid register_context_callback( 
				 const ABng_Server_Manager::Name &name);

  void unregister_callback( ABng_Server_Manager::uid identifier);
  
  virtual void reference_resolved( const ABng_Server_Manager::Name &name,
                                   CORBA::Object_var server_ref,
				   ABng_Server_Manager::uid callback_id)= 0;
  virtual void server_shutdown( const ABng_Server_Manager::Name &name,
				ABng_Server_Manager::uid callback_id) {}
};


class Bootstrap {
public:
  static Bootstrap_base *base_impl;
private:
  static ABng_Server_Manager::Register_var register_object;
  static ABng_Server_Manager::Register_Manager_var register_manager_object;
  friend class SM_CallbackImpl;

public:
  static void init( CORBA::ORB_ptr orb_ptr,
		    int client_only= 0);
  static void register_server_in_NS( const ABng_Server_Manager::Name &name,
                                     CORBA::Object_ptr server_ref);

  static void unregister_server_from_NS( const ABng_Server_Manager::Name &name);

  /* time synchronisation stuff goes here */

  static ABng_Guarded time_guard;
  static unsigned long next_synch; // next synchronisation in "next_synch" seconds
  static omni_thread *synch_thread;
  static int synch_required;
  static int is_synch_required();

  static ABng_Server_Manager::Time gettimeofday();
  static void need_synch_time( unsigned long accuracy_microseconds);
  static void synch_time( unsigned long accuracy_microseconds);
  
  static void synch_time_worker( void *args);
  
  /* ----  */

  static ABng_Server_Manager::Name CTX_Root();
  static ABng_Server_Manager::Name CTX_ABng();
  static ABng_Server_Manager::Name CTX_Bootstrap_Client();
  static ABng_Server_Manager::Name CTX_Domain_Repository();
  static ABng_Server_Manager::Name CTX_Domain_Repository_Public();
  static ABng_Server_Manager::Name CTX_Domain_Repository_Management();
  static ABng_Server_Manager::Name CTX_ABng_Core();
  static ABng_Server_Manager::Name CTX_ABng_Core_Public();
  static ABng_Server_Manager::Name CTX_ABng_Core_Public_Views();
  static ABng_Server_Manager::Name CTX_ABng_Core_Management();
  static ABng_Server_Manager::Name CTX_ABng_Core_System();
  static ABng_Server_Manager::Name CTX_ABng_Core_System_Pollers();
  static ABng_Server_Manager::Name CTX_ABng_Core_System_Vsps();
  static ABng_Server_Manager::Name CTX_ABng_Core_Management_View_Managers();

  static ABng_Server_Manager::Name SRV_Bootstrap();
  static ABng_Server_Manager::Name SRV_Bootstrap_Client();
  static ABng_Server_Manager::Name SRV_Scheduler();
  static ABng_Server_Manager::Name SRV_Sensor_Repository();
  static ABng_Server_Manager::Name SRV_Badge_Repository();
  static ABng_Server_Manager::Name SRV_Badge_Holder_Repository();
  static ABng_Server_Manager::Name SRV_Location_Type_Repository();
  static ABng_Server_Manager::Name SRV_Location_Type_Repository_Manager();
  static ABng_Server_Manager::Name SRV_Location_Description_Repository();
  static ABng_Server_Manager::Name SRV_Location_Description_Repository_Manager();
  static ABng_Server_Manager::Name SRV_Audio_Devices_Repository();
  static ABng_Server_Manager::Name SRV_Audio_Devices_Repository_Manager();
  static ABng_Server_Manager::Name SRV_Property_Record_Repository();
  static ABng_Server_Manager::Name SRV_Property_Record_Repository_Manager();
  static ABng_Server_Manager::Name SRV_ABng_Location_Description_Repository();
  static ABng_Server_Manager::Name SRV_User_Repository();
  static ABng_Server_Manager::Name SRV_User_Repository_Manager();
  static ABng_Server_Manager::Name SRV_Equipment_Repository();
  static ABng_Server_Manager::Name SRV_Equipment_Repository_Manager();
  static ABng_Server_Manager::Name SRV_Meta_Equipment_Repository();
  static ABng_Server_Manager::Name SRV_Meta_Equipment_Repository_Manager();

  static ABng_Server_Manager::Name SRV_Away_Repository();
  static ABng_Server_Manager::Name SRV_Away_Repository_Manager();

  static ABng_Server_Manager::Name SRV_Local_Dependence();
  static ABng_Server_Manager::Name SRV_Main_Sighting_Processor();
                                 
  static CORBA::Object_var resolve( const ABng_Server_Manager::Name &name);

  friend class Bootstrap_listener;  
};


#endif __BOOTSTRAP_N_H__
