// $Id: db_utils.cc,v 1.3 1998/07/29 13:14:51 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/db_utils.cc,v $
//
// $Log: db_utils.cc,v $
// Revision 1.3  1998/07/29 13:14:51  mencnar
// time synchronisation,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.2  1998/06/16 12:55:00  mencnar
// *** empty log message ***
//
// Revision 1.1  1998/01/15 14:25:23  mencnar
// Initial revision
//
//

#include <rw/rwdate.h>
#include <rw/rwtime.h>
#include "domain_rep.hh"
#include "db_utils.hh"
#include <sys/types.h>
#include <sys/param.h>
#include <netdb.h>
#include <unistd.h>
#include <rw/tvdlist.h>
#include <strstream.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>


#if !defined (DECLARED_GETHOSTNAME)
extern "C" int          gethostname (char *, int);
#endif


struct in_addr get_host_address() {

  char name[MAXHOSTNAMELEN];
  gethostname( name, MAXHOSTNAMELEN);

  int buflen= 255;
  char *buf= (char *) malloc( buflen);
  int errno_r;
  struct hostent he;

  while ( gethostbyname_r( name, &he, buf, buflen, &errno_r) == NULL) {
    if (errno_r == ERANGE) {
      buflen*= 2;
      buf= (char *) realloc( buf, buflen);
    }
    else {
      struct in_addr ret;
      ret= ret;
      return ret;
    }
  }
  return *((struct in_addr *) he.h_addr_list[0]);
}

DB_Header::DB_Header( const char *_rep_name,
		      const char *_ver_name,
		      const char *_record_description,
		      int _dont_include_default_description) :
  rep_name( _rep_name),
  ver_name( _ver_name),
  record_description( _record_description),
  dont_include_default_description( _dont_include_default_description) {}  

ostream & operator << ( ostream &ostr, const DB_Header &db) {
  char buf[MAXHOSTNAMELEN];
  gethostname( buf, MAXHOSTNAMELEN);
  RWTime rwtime;

  ostr << "\
# Active Badge next generation (ABng)\n\
# Release: 1.0\n\
#\n\
# " << db.rep_name << " database\n\
# Last update: " << rwtime << "\n\
#\n\
# Generated by a process with pid " << (unsigned) getpid() << " on " << buf << "\n\
#\n\
# Format description (version " << db.ver_name << "):\n";

  if ( ! db.dont_include_default_description)
    ostr << "\
# 	<file>::=               <next_item_id>\n\
#  				<number_of_items>\n\
#				<data_records>\n\
#\n\
#       <data_records>::=       |\n\
#                               <data_record> <data_records>\n\
#\n";

  ostr << db.record_description;
  
  if ( db.record_description.contains("<sequence_of_octets>")) {
    ostr << "\
#       <sequence_of_octets>::= \"<\" <hex_numbers> \">\"\n\
#\n\
#       <hex_numbers>::=        |\n\
#                               <hex_number> <hex_numbers>\n"; 
  }
  ostr << endl;
  return ostr;
}

istream & operator >> ( istream &istr, DB_Header &) {
  char c;

  while ( 1) {
    istr >> ws;
    if ( istr.peek() == '#') {
      while ( 1) {
	c= istr.get();
	if (!istr) return istr;
	if (c == '\n') break;
      }
    }
    else break;
  }
  return istr;
}

ostream * FileTransaction::prepare() {
  size_t slash_index= file_name.last('/');
  RWCString dir_name= ".";
  if (slash_index != RW_NPOS) 
    dir_name= file_name(0, slash_index+1);

  tmp_file_name= tempnam( (const char *) dir_name, "abngtmp_");
#ifdef DEBUG
  //  cerr << "dir_name" << dir_name << ' ' << endl;
#endif
  if (tmp_file_name == NULL)
    tmp_file_name= strdup( (const char *) file_name);
#ifdef DEBUG
  //  cerr << "FileTransaction::prepare " << file_name << ' ' << tmp_file_name << endl;
#endif
  return (ofs= new ofstream( tmp_file_name));
}

void FileTransaction::commit() {
  if (tmp_file_name) {
#ifdef DEBUG
    //  cerr << "FileTransaction::commit " << file_name << ' ' << tmp_file_name << endl;
#endif
    if (make_backup) rename( file_name, file_name+".bak_file");
    rename( tmp_file_name, file_name);
  }
}

FileTransaction::~FileTransaction() {
  if (ofs) delete ofs;
  if (tmp_file_name) 
    if (file_name != tmp_file_name) unlink( tmp_file_name);
  if (tmp_file_name) free( tmp_file_name);
}

//-----------------------------------------------------------------TransactionLOG
//
TransactionLOG::TransactionLOG( const char *db_file_name,
                                size_t _min_log_size,
                                size_t _max_log_size,
				int _active) :
  log_file_name( NULL),
  min_log_size( _min_log_size),
  max_log_size( _max_log_size),
  active( _active)
{
  log_file_name= (char *) malloc( strlen(db_file_name)+strlen(".log")+1);
  strcpy( log_file_name, db_file_name);
  strcat( log_file_name, ".log");

}

TransactionLOG::~TransactionLOG() {
  if (log_file_name) free( log_file_name);
}

void TransactionLOG::start_transaction() {
  if (!active) return;
  check_and_reorganize();
  RWTime rwtime;
  ofs.close();
  ofs.open( log_file_name, ios::app);
  ofs << "<<" << rwtime << " | ";
}

void TransactionLOG::stop_transaction() {
  if (!active) return;
  ofs << " >>" << endl;
  ofs.close();
}

void TransactionLOG::fail_transaction() {
  if (!active) return;
  ofs << " ##>>failed" << endl;
  ofs.close();
}

void TransactionLOG::check_and_reorganize() {
  struct stat memstat;
  char buf[256];

  if (!active) return;
  if (stat( log_file_name, &memstat)) return;

  if (memstat.st_size <= max_log_size) return;
  ofs.close();
  FileTransaction ft( log_file_name,
                      0 /* do not make a backup copy */);
  ostream *ostr= ft.prepare();
  ifstream ifs( log_file_name);
  ifs.seekg( min_log_size);
  while (ifs) {
    ifs.read( buf, 256);
    ostr->write( buf, 256);
  }    
  ft.commit();
  ofs.open( log_file_name, ios::app);
}



ostream & operator << ( ostream &str, const Domain_Repository::Octets &seq) {
  str << "<";
  for (int i=0; i<seq.length(); i++) {
    if (i>0) str << " ";
    str << hex << (int) seq[i];
  }
  str << ">" << dec;
  return str;
}

istream & operator >> ( istream &istr, Domain_Repository::Octets &seq) {
  char c;
  istr >> ws;
  if (istr.peek() != '<') {
    istr.clear( iostream::failbit|istr.rdstate());
    return istr;
  }
  istr >> c;
  if (!istr) return istr;

  istr >> hex;
  RWTValDlist<CORBA::Octet> seq_list;
  do {
    int val;
    istr >> val;
    if (istr) seq_list.append( (CORBA::Octet) val);
  } while (istr);
  istr.clear( iostream::goodbit);
  istr >> ws;
  istr >> c;
  //  cout << "c== " << (int) c << "  " << c << endl;
  if (c != '>') {
    istr.clear( iostream::failbit|istr.rdstate());
    return istr;
  }
  int size= seq_list.entries();
  seq.length( size);

  for (int i=0; i<size; i++)
    seq[i]= seq_list[i];

  istr >> dec;
  return istr;
}

#ifdef ORBIX
ostream & operator << ( ostream &str, const CORBA::TypeCode_ptr tc) {
  CORBA::String_var buf( (char*) (*tc));

  str << '\"' << (const char *) buf << "\" ";
  return str;
}

istream & operator >> ( istream &str, CORBA::TypeCode_ptr tc) {
  RWCString buf;
  str >> buf;
  CORBA::TypeCode new_tc( (const char *) buf);
  (*tc)= new_tc;
  return str;
}

#endif

void save_string( ostream &ostr, const char *str) {
  ostr << '\"';
  if (str)
    for (;
	 *str;
	 str++) {
      if (*str == '\"') ostr << "\\\"";
      else if (*str == '\\') ostr << "\\\\";
      else ostr << *str;
    }   
  ostr << '\"';
}

void restore_string( istream &istr, char *&str) {
  char c;
  istr >> ws >> c;
  if (c != '\"') {
    cerr << "db_utils.cc: restore_string: expected \"" << endl;
    exit(1);
  }
  ostrstream strstr;
  while (istr) {
    istr >> c;
    if (c == '\\') istr >> c;
    else if (c == '\"') break;
    strstr << c;
  }
  if (!istr) {
    cerr << "db_utils.cc: restore_string: Unexpected end of stream" << endl;
    exit(1);
  }
  strstr << '\0';
  char *ret= strstr.str();
  str= CORBA::string_dup( ret);
  free(ret);
}
