#include <iostream.h>
#include "mtime.hh"
#include "bootstrap_n.hh"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fstream.h>

static ABng_Server_Manager::Time diff_time;
static long clock_drift=0; // microseconds per second
static ABng_Server_Manager::Time last_synchronized;
static int synch_initialized= 0;
static ABng_Guarded diffs_guard;

static void init_synch_diffs() {
  diff_time.seconds= diff_time.microseconds=
    last_synchronized.seconds= last_synchronized.microseconds= 0;
  synch_initialized= 1;
}

static inline void normalize_time( ABng_Server_Manager::Time &t) {
  while (t.microseconds>1000000) {
    t.microseconds-=1000000;
    t.seconds++;
  }
  while (t.microseconds<0) {
    t.microseconds+= 1000000;
    t.seconds--;
  }
  if (t.seconds<0) {
    if (t.microseconds>0) {
      t.seconds++;
      t.microseconds-= 1000000;
    }
  }
}

static int operator > ( const ABng_Server_Manager::Time &t1, 
		 const ABng_Server_Manager::Time &t2);

static ABng_Server_Manager::Time operator +( const ABng_Server_Manager::Time &t1, 
				      const ABng_Server_Manager::Time &t2);

static ABng_Server_Manager::Time operator -( const ABng_Server_Manager::Time &t1, 
				      const ABng_Server_Manager::Time &t2);

static ABng_Server_Manager::Time operator /( const ABng_Server_Manager::Time &t, 
					     int dv);

static int operator > ( const ABng_Server_Manager::Time &t1, 
			const ABng_Server_Manager::Time &t2) {

  ABng_Server_Manager::Time x1= t1;
  ABng_Server_Manager::Time x2= t2;
  normalize_time( x1);
  normalize_time( x2);
  if (x1.seconds > x2.seconds) return 1;
  if (x1.seconds < x2.seconds) return 0;
  if (x1.microseconds > x2.microseconds) return 1;
  return 0;
}

static ABng_Server_Manager::Time operator +( const ABng_Server_Manager::Time &t1, 
					     const ABng_Server_Manager::Time &t2) {
  ABng_Server_Manager::Time ret;
  ret.seconds= t1.seconds+t2.seconds;
  ret.microseconds= t1.microseconds+t2.microseconds;
  //  normalize_time(ret);
  return ret;
}

static ABng_Server_Manager::Time operator -( const ABng_Server_Manager::Time &t1, 
					     const ABng_Server_Manager::Time &t2) {
  ABng_Server_Manager::Time ret;
  ret.seconds= t1.seconds-t2.seconds;
  ret.microseconds= t1.microseconds-t2.microseconds;
  //  normalize_time(ret);
  return ret;
}

static ABng_Server_Manager::Time operator /( const ABng_Server_Manager::Time &t, 
					     int dv) {
  ABng_Server_Manager::Time ret= t;
  ret.microseconds+= t.seconds*1000000;
  ret.microseconds= t.microseconds/dv;
  ret.seconds= 0;
  return ret;
}

static inline ABng_Server_Manager::Time get_clock_diff() {
  struct timeval mtd;
  gettimeofday( &mtd, NULL);

  ABng_Server_Manager::Time new_diff= diff_time;
  new_diff.microseconds-= ((long) mtd.tv_sec + diff_time.seconds - last_synchronized.seconds)*clock_drift;
  //  cerr << "Diff correction " 
  //       << ((long) mtd.tv_sec + diff_time.seconds
  //                  - last_synchronized.seconds)*clock_drift;
  normalize_time( new_diff);
  return new_diff;
}

static ABng_Server_Manager::Time my_currtime() {
  if (!synch_initialized) init_synch_diffs();

  struct timeval mtd;
  gettimeofday( &mtd, NULL);

  ABng_Server_Manager::Time diff= get_clock_diff();
  ABng_Server_Manager::Time ret;
  ret.seconds= (long) mtd.tv_sec + diff.seconds;
  ret.microseconds= (long) mtd.tv_usec + diff.microseconds;
  normalize_time( ret);
  return ret;
}

ABng_Server_Manager::Time currtime() {
  auto_lock local_lock( diffs_guard);
  
  return my_currtime();
}


void my_synch_time( long eps_microseconds,
		    ABng_Server_Manager::TimeServer_var timesrv);

unsigned long synch_time( long eps_microseconds,
			  ABng_Server_Manager::TimeServer_var timesrv) {
  static ABng_Server_Manager::Time diff1;
  static unsigned int seed_r= 123127;

  if (!synch_initialized) init_synch_diffs();

  auto_lock local_lock( diffs_guard);
  unsigned long ret;

  if (last_synchronized.seconds + last_synchronized.microseconds == 0) {
    cerr << "Synchronizing for the first time" << endl;
    my_synch_time( eps_microseconds,
		   timesrv);
    last_synchronized= my_currtime();
    diff1= diff_time;
    ret= 60;
  }
  else {
    my_synch_time( eps_microseconds,
		   timesrv);
    ABng_Server_Manager::Time drift= (diff1-diff_time);
    ABng_Server_Manager::Time last_diff= diff_time;
    normalize_time( drift);
    drift= drift/((my_currtime()-last_synchronized).seconds);
    last_synchronized= my_currtime();
    
    cerr << "Drift " << drift.microseconds << "microseconds per second" << endl;
  
    clock_drift+= drift.microseconds;
    
    unsigned long ret;
    if (drift.microseconds != 0) {
      ret=  eps_microseconds / labs(drift.microseconds);
      ret= ( ret > 3600) ? ret : 3600;
    }
    else ret= 3600;
  }

  // Call this function again after returned number of seconds
  // to check for time drifts and synchronize again
  //

  return (ret*(100+(rand_r( &seed_r)*100)/RAND_MAX))/100 ;
}

void my_synch_time( long eps_microseconds,
		    ABng_Server_Manager::TimeServer_var timesrv) {
  char buf[2048];

  try {
    ABng_Server_Manager::Time org_timestamp;
    ABng_Server_Manager::Time recv_timestamp;
    ABng_Server_Manager::Time time_err;
    ABng_Server_Manager::Time rt;
    ABng_Server_Manager::Time sum_rt;
    ABng_Server_Manager::Time mean_rt;

    mean_rt.seconds= 1000;
    mean_rt.microseconds= 0;

    int first= 0;
    int norm_cnt= 0;
    int loop=0;
    int sum_counter= 0;
    sum_rt.seconds= sum_rt.microseconds= 0;

    do {
      loop++;
      org_timestamp= my_currtime();
      recv_timestamp= timesrv->get_current_time();
      normalize_time( rt= my_currtime()-org_timestamp);
      
      if (loop>4) {
	sum_rt= sum_rt+rt;
	if (loop>8)
	  mean_rt= sum_rt/(++sum_counter);
      }

      if ( rt > mean_rt) {
	//	cout << "Roundtrip " << rt.seconds << " " << rt.microseconds
	//	     << "  -- discarded" << endl;
	continue;
      }
	  
      if (first) time_err= (recv_timestamp-org_timestamp);
      time_err= (recv_timestamp-org_timestamp)+((org_timestamp-my_currtime())/2);
      normalize_time(time_err);
      //      cout << "Time corrected by " << time_err.seconds << " "
      //	   << time_err.microseconds << endl;
      //      cout << "Roundtrip " << rt.seconds << " " << rt.microseconds << endl;
      first= 0;
      diff_time= diff_time+time_err;
      normalize_time( diff_time);
      cout << "diff time " << diff_time.seconds 
	   << " " << diff_time.microseconds << endl;

      if (time_err.seconds==0 && time_err.microseconds<eps_microseconds)
	norm_cnt++;
      else norm_cnt= 0;
      struct timeval tv;
      tv.tv_sec= 0;
      tv.tv_usec= 100000;
      select(0,NULL,NULL,NULL,&tv);
    }
    while ( norm_cnt<5);
  }
  catch (CORBA::Exception &ex) {
    cerr << "synch_time caught " << ex << endl;
  } 
}



