// $Id: rep_client.cc,v 1.2 1998/07/29 13:14:54 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/common/rep_client.cc,v $
//
// $Log: rep_client.cc,v $
// Revision 1.2  1998/07/29 13:14:54  mencnar
// time synchronization,
// persistent observers,
// notify calls moved to another thread ( fixed deadlock)
// internal server names changed
//
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
// File:          rep_client.cc
// Author:        Malgorzata Steinder (gosia@ics.agh.edu.pl)
// Date Created:  16.03.98
//
// General Description:
//  Implementation of the Rep_Client class which is a skeleton for ABng 
//  repository utilities.

#include <stream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include "rep_client.h"


// for getopt()
extern char *optarg;
extern int optind;



template<class Repository_Types>
void Rep_Client<Repository_Types>::get_options(int argc, char **argv) {

  int c;
  while (( c = getopt (argc, argv, "irh")) != -1) {
    switch (c) {
      case 'h' : {
	print_info();
	exit(1);
      }
      case 'r': break;
      case 'i': ior_flag = 0; break;
      default: {
	print_info();
	exit(1);
      }
    }
  }
}

template<class Repository_Types>
void Rep_Client<Repository_Types>::init_fields(int argc, char **argv) {
  prog_name = strdup(argv[0]);

  if(argc < 2) {
      print_info();
      exit(1);
  }

  operation = strdup(argv[1]);

  if(strcmp(operation, "add") &&
     strcmp(operation, "remove") &&
     strcmp(operation, "find") &&
     strcmp(operation, "find_values") &&
     strcmp(operation, "update")
  ) {
    for(int i = 0; i < sizeof(operations); i ++) {
      if(operations[i].name == 0)  {
          print_info();
          exit(1);
      }
      if(!strcmp(operation, operations[i].name)) {
        int num = 0, j = 0;
        while(operations[i].parameters[j++]) num++;
	if(argc < num) {
          print_info();
          exit(1);
        }
        break;
      }
    }

    if(i >= sizeof(operations)) {
      print_info();
      exit(1);
    }
  }

  ior_flag = 1;

  my_argc = argc;
  my_argv = argv;
}

template<class Repository_Types>
void Rep_Client<Repository_Types>::init_rep_ptr() {

  Basic_Client<Repository_Types>::init_rep_ptr();

  CORBA::Object_var obj_ptr;   

  obj_ptr = Bootstrap::resolve((*srv_rep_mgr)());
  if (CORBA::is_nil(obj_ptr)){     
      cerr << "Repository_Manager client error: ";
      cerr << "Repository_Manager object reference is nill!" 
           << endl;
      exit(-1);
   }

   try {
      my_rep_mgr = Repository_Manager::_narrow(obj_ptr);
   } catch (CORBA::SystemException& se){
      cerr << "Repository_Manager client error: ";
      cerr << "exception when trying to _narrow! " << &se << endl;
      exit(-1);
   }
}

template<class Repository_Types>
void Rep_Client<Repository_Types>::init(int argc, char **argv) {

  init_fields(argc, argv);
  get_options(argc - 1, argv + 1);

  Basic_Client<Repository_Types>::init(argc, argv);
}




template<class Repository_Types>
void Rep_Client<Repository_Types>::run() {

   if(!strcmp(operation, "add")) {
        add(my_argv[my_argc - 1]);
        return;
   }
   if(!strcmp(operation, "remove")) {
        remove(my_argv[my_argc - 1]);
        return;
   }
   if(!strcmp(operation, "find")) {
        find();
        return;
   }
   if(!strcmp(operation, "find_values")) {
        find_values();
        return;
   }
   if(!strcmp(operation, "update")) {
        update(my_argv[my_argc - 2], my_argv[my_argc - 1]);
        return;
   }
   other_operations();
}



template<class Repository_Types>
void Rep_Client<Repository_Types>::add(char* data) {

   Initial_Data i_data = init_data(data);
   Item_ptr i;
   try {
       i = my_rep_mgr->add(i_data);
   } catch(Domain_Repository::Duplicate_Data_Record) {
     cerr << "Exception: Domain_Repository::Duplicate_Data_Record" << endl;
     exit(1);
   } catch(Domain_Repository::Unknown_Object_Ref) {
     cerr << "Exception: Domain_Repository::Unknown_Object_Ref" << endl;
     exit(1);
   } catch(Domain_Repository::Destination_Unreachable &de) {
     cerr << "Exception: Domain_Repository::Destination_Unreachable " 
	  << de.server_name
	  << endl;
     exit(1);
   } catch(Domain_Repository::Operation_Not_Allowed) {
     cerr << "Exception: Domain_Repository::Operation_Not_Allowed" << endl;
     exit(1);
   } catch(Domain_Repository::Invalid_Description) {
     cerr << "Exception: Domain_Repository::Invalid_Description" << endl;
     exit(1);
   } catch(CORBA::SystemException ex) {
     cerr << "Exception: CORBA::SystemException " << ex << endl;
     exit(1);
   } catch(...) {
     cerr << "Exception: Unknown" << endl;
     exit(1);
   } 
   if(!ior_flag) {
      try {
        cout  << "Operation successful: new item id "<< i->item_id() << endl;
      } catch(CORBA::SystemException ex) {
        cerr << "Exception: CORBA::SystemException " << ex << endl;
        exit(1);
      } catch(...) {
        cerr << "Exception: Unknown" << endl;
        exit(1);
      } 
   }
   else {
      try {
	CORBA::String_var str = orb_ptr->object_to_string(i);
        cout  << "Operation successful: new object ref "<< str << endl;
      } catch(CORBA::SystemException ex) {
        cerr << "Exception: CORBA::SystemException " << ex << endl;
        exit(1);
      } catch(...) {
        cerr << "Exception: Unknown" << endl;
        exit(1);
      } 
   }
}




template<class Repository_Types>
void Rep_Client<Repository_Types>::remove(char* str) {

   Item_ptr i = init_item_ptr(str);
   try {
     my_rep_mgr->remove(i);
   } catch(Domain_Repository::Unknown_Object_Ref) {
     cerr << "Exception: Domain_Repository::Unknown_Object_Ref" << endl;
     exit(1);
   } catch(Domain_Repository::Destination_Unreachable &de) {
     cerr << "Exception: Domain_Repository::Destination_Unreachable " 
	  << de.server_name
	  << endl;
     exit(1);
   } catch(Domain_Repository::Operation_Not_Allowed) {
     cerr << "Exception: Domain_Repository::Operation_Not_Allowed" << endl;
     exit(1);
   } catch(CORBA::SystemException ex) {
     cerr << "Exception: CORBA::SystemException " << ex << endl;
     exit(1);
   } catch(...) {
     cerr << "Exception: Unknown" << endl;
     exit(1);
   } 
   cout << "Operation successful" << endl;
}



template<class Repository_Types>
void Rep_Client<Repository_Types>::update(char* str1, char* str2) {

   Item_ptr i = init_item_ptr(str1);
   Initial_Data i_data = init_data(str2);

   try {
       my_rep_mgr->update(i, i_data);
   } catch(Domain_Repository::Duplicate_Data_Record) {
     cerr << "Exception: Domain_Repository::Duplicate_Data_Record" << endl;
     exit(1);
   } catch(Domain_Repository::Unknown_Object_Ref) {
     cerr << "Exception: Domain_Repository::Unknown_Object_Ref" << endl;
     exit(1);
   } catch(Domain_Repository::Destination_Unreachable &de) {
     cerr << "Exception: Domain_Repository::Destination_Unreachable " 
	  << de.server_name
	  << endl;
     exit(1);
   } catch(Domain_Repository::Operation_Not_Allowed) {
     cerr << "Exception: Domain_Repository::Operation_Not_Allowed" << endl;
     exit(1);
   } catch(Domain_Repository::Invalid_Description) {
     cerr << "Exception: Domain_Repository::Invalid_Description" << endl;
     exit(1);
   } catch(CORBA::SystemException ex) {
     cerr << "Exception: CORBA::SystemException " << ex << endl;
     exit(1);
   } catch(...) {
     cerr << "Exception: Unknown" << endl;
     exit(1);
   } 
   cout << "Operation successful" << endl;
}



template<class Repository_Types>
void Rep_Client<Repository_Types>::find() {
   Pattern pattern = init_pattern();

   Items *is;
   try {
     is = my_rep->find(pattern);
   } catch(Domain_Repository::Search_Not_Implemented) {
     cerr << "Exception: Domain_Repository::Search_Not_Implemented" << endl;
     exit(1);
   } catch(Domain_Repository::Unknown_Object_Ref) {
     cerr << "Exception: Domain_Repository::Unknown_Object_Ref" << endl;
     exit(1);
   } catch(Domain_Repository::Destination_Unreachable &de) {
     cerr << "Exception: Domain_Repository::Destination_Unreachable " 
	  << de.server_name
	  << endl;
     exit(1);
   } catch(CORBA::SystemException ex) {
     cerr << "Exception: CORBA::SystemException " << ex << endl;
     exit(1);
   } catch(...) {
     cerr << "Exception: Unknown" << endl;
     exit(1);
   } 
   cout << "Operation successful: ";
   if(is->length() == 1)
       cout << is->length() << " item has been found" << endl;
   else
       cout << is->length() << " items have been found" << endl;

   for(int i = 0; i < is->length(); i ++) {
      if(!ior_flag) {
         try {
           cout  << i << ". " << (*is)[i]->item_id() << endl;
         } catch(CORBA::SystemException ex) {
           cerr << "Exception: CORBA::SystemException " << ex << endl;
           exit(1);
         } catch(...) {
           cerr << "Exception: Unknown" << endl;
           exit(1);
         } 
      }
      else {
         try {
	   CORBA::String_var str = orb_ptr->object_to_string((*is)[i]);
           cout  << i << ". " << str << endl;
         } catch(CORBA::SystemException ex) {
           cerr << "Exception: CORBA::SystemException " << ex << endl;
           exit(1);
         } catch(...) {
           cerr << "Exception: Unknown" << endl;
           exit(1);
         } 
      }
   }
}



template<class Repository_Types>
void Rep_Client<Repository_Types>::find_values() {
   Pattern pattern = init_pattern();

   Value_Descriptions *vds;
   try {
     vds = my_rep->find_values(pattern);
   } catch(Domain_Repository::Search_Not_Implemented) {
     cerr << "Exception: Domain_Repository::Search_Not_Implemented" << endl;
     exit(1);
   } catch(Domain_Repository::Unknown_Object_Ref) {
     cerr << "Exception: Domain_Repository::Unknown_Object_Ref" << endl;
     exit(1);
   } catch(Domain_Repository::Destination_Unreachable &de) {
     cerr << "Exception: Domain_Repository::Destination_Unreachable " 
	  << de.server_name
	  << endl;
     exit(1);
   } catch(CORBA::SystemException ex) {
     cerr << "Exception: CORBA::SystemException " << ex << endl;
#ifdef OMNIORB
     cerr << "ID " << ex.NP_RepositoryId() << endl;
#endif
     exit(1);
   } catch(...) {
     cerr << "Exception: Unknown" << endl;
     exit(1);
   } 
   cout << "Operation successful: ";
   if(vds->length() == 1)
       cout << vds->length() << " item has been found" << endl;
   else
       cout << vds->length() << " items have been found" << endl;

   for(int i = 0; i < vds->length(); i ++)
     cout << i << ". " << (*vds)[i] << endl;
}




template<class Repository_Types>
void Rep_Client<Repository_Types>::print_info() {

  cout << "Usage:\t" << prog_name << " add [options] <" << description 
       << ">" << endl;
  cout << "\t" << prog_name << " remove [options] <item ID | item IOR>" 
       << endl;
  cout << "\t" << prog_name << " update [options] <item ID | item IOR> <" 
       << description << ">" << endl;
  cout << "\t" << prog_name << " find | find_values [options]" << endl;
  for(int i = 0; i < sizeof(operations); i ++) {
    if(operations[i].name == 0) break;
    cout << "\t" << prog_name << " " << operations[i].name << " [options]";
    for(int j = 0; operations[i].parameters[j]; j ++)
      cout << " <" << operations[i].parameters[j] << ">" << endl;
    cout << endl;
  }
  cout << endl;

  cout << "The following options may be used:" << endl;
  cout << "-i print RepositoryId instead of IOR" << endl;
  cout << "-r print IOR (default)"<< endl;
  cout << "-h print this help info" << endl;
}


//--------------------------------------------------------------------------
//                     HELPER METHODS
//--------------------------------------------------------------------------

template<class Repository_Types>
char* Rep_Client<Repository_Types>::read_id(char*& data) {

   long n = 0;
   while(*data == ' ') data ++;

   int bracket=  (*data == '<');
   if (bracket) data++;
     
   while(((!bracket && data[n] != ' ') || (bracket && data[n] != '>'))
	 && data[n] != '\0')
     n++;

   /*
   if (!data[n]) {
     cerr << "Wrong identifier. Stopped at: " << data << endl  << endl;
     print_info();
     exit(1);
   }
   */

   char* obj = new char[n + 1];
   strncpy(obj, data, n);
   obj[n] = '\0';

   data += n+bracket;
   return obj;
}

template<class Repository_Types>
CORBA::TypeCode_ptr Rep_Client<Repository_Types>::read_TC(char*& data) {
    char* str = read_id(data);

    if(!strcmp(str, "short")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_short);
    if(!strcmp(str, "long")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_long);
    if(!strcmp(str, "ushort")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_ushort);
    if(!strcmp(str, "ulong")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_ulong);
    if(!strcmp(str, "float")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_float);
    if(!strcmp(str, "double")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_double);
    if(!strcmp(str, "boolean")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_boolean);
    if(!strcmp(str, "char")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_char);
    if(!strcmp(str, "octet")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_octet);
    if(!strcmp(str, "string")) 
       return CORBA::TypeCode::_duplicate(CORBA::_tc_string);
    if(!strcmp(str, "seq")) {
       delete str;   
       str = read_id(data);
       CORBA::TypeCode_ptr stc= NULL;

       if(!strcmp(str, "short")) 
	 stc= Domain_Repository::_tc_Shorts;
       else if(!strcmp(str, "long")) 
         stc= Domain_Repository::_tc_Longs;
       else if(!strcmp(str, "ushort")) 
	 stc= Domain_Repository::_tc_Ushorts;
       else if(!strcmp(str, "ulong")) 
          stc= Domain_Repository::_tc_Ulongs;
       else if(!strcmp(str, "float")) 
          stc= Domain_Repository::_tc_Floats;
       else if(!strcmp(str, "double")) 
          stc= Domain_Repository::_tc_Doubles;
       else if(!strcmp(str, "boolean")) 
          stc= Domain_Repository::_tc_Booleans;
       else if(!strcmp(str, "char")) 
          stc= Domain_Repository::_tc_Chars;
       else if(!strcmp(str, "octet")) 
          stc= Domain_Repository::_tc_Octets;
       else if(!strcmp(str, "string")) 
          stc= Domain_Repository::_tc_Strings;



#ifdef OMNIORB
       if (stc) {
	 while (stc->kind() == CORBA::tk_alias)
	   stc= stc->content_type();
	 return CORBA::TypeCode::_duplicate( stc);
       }
#else
       return CORBA::TypeCode::_duplicate( stc);
#endif
       
    }
    cout << "The type specified is not supported. Use:" << endl
         << "    [ seq ] short" << endl
         << "    [ seq ] long" << endl
         << "    [ seq ] ushort" << endl
         << "    [ seq ] ulong" << endl
         << "    [ seq ] float" << endl
         << "    [ seq ] double" << endl
         << "    [ seq ] char" << endl
         << "    [ seq ] octet" << endl
         << "    [ seq ] boolean" << endl
         << "    [ seq ] string" << endl;
   cout << "Stopped at: " << data << endl << endl;
   print_info();
   exit(1);
}



template<class Repository_Types>
void Rep_Client<Repository_Types>::skip_digits(char*& data) {
   while(*data >= '0' && *data <= '9') data ++;
}

template<class Repository_Types>
short Rep_Client<Repository_Types>::read_short(char*& data) {
    read_spaces(data);
    short s;
    if(!(*data) || !sscanf(data, "%dh", &s)) {
      cerr << "Unable to read short" << endl;
      print_info();
      exit(1);
    }
    skip_digits(data);
    return s;
}

template<class Repository_Types>
long  Rep_Client<Repository_Types>::read_long(char*& data) {
    read_spaces(data);
    long l;
    if(!(*data) || !sscanf(data, "%dl", &l)) {
      cerr << "Unable to read long. Stopped at: " << data << endl;
      print_info();
      exit(1);
    }
    skip_digits(data);
    return l;
}

template<class Repository_Types>
unsigned short Rep_Client<Repository_Types>::read_ushort(char*& data) {
    read_spaces(data);
    unsigned short s;
    if(!(*data) || !sscanf(data, "%uh", &s)) {
      cerr << "Unable to read unsigned short. Stopped at: " << data << endl 
           << endl;
      print_info();
      exit(1);
    }
    skip_digits(data);
    return s;
}

template<class Repository_Types>
unsigned long  Rep_Client<Repository_Types>::read_ulong(char*& data) {
    read_spaces(data);
    unsigned long l;
    if(!sscanf(data, "%ul", &l)) {
      cerr << "Unable to read unsigned long. Stopped at: " << data << endl 
           << endl;
      exit(1);
    }
    skip_digits(data);
    return l;
}

template<class Repository_Types>
float  Rep_Client<Repository_Types>::read_float(char*& data) {
    read_spaces(data);
    float f;
    if(!(*data) || !sscanf(data, "%f", &f)) {
      cerr << "Unable to read float. Stopped at: " << data << endl  << endl;
      print_info();
      exit(1);
    }
    skip_digits(data);
    if(*data == '.') data ++;
    skip_digits(data);
    return f;
}

template<class Repository_Types>
double  Rep_Client<Repository_Types>::read_double(char*& data) {
    read_spaces(data);
    double d;
    if(!(*data) || !sscanf(data, "%fl", &d)) {
      cerr << "Unable to read double. Stopped at: " << data << endl << endl;
      print_info();
      exit(1);
    }
    skip_digits(data);
    if(*data == '.') data ++;
    skip_digits(data);
    return d;
}

template<class Repository_Types>
char  Rep_Client<Repository_Types>::read_char(char*& data) {
    read_spaces(data);
    char c = *data;
    data ++;
    return c;
}

template<class Repository_Types>
unsigned char  Rep_Client<Repository_Types>::read_octet(char*& data) {
    read_spaces(data);
    unsigned char c = *data;
    data ++;
    return c;
}

template<class Repository_Types>
char*  Rep_Client<Repository_Types>::read_string(char*& data) {
    read_spaces(data);
    char str[100];
    if(*data == '\'') {
      data ++;
      int i = 0;
      while(*data && (*data != '\'' || *(data - 1) == '\\')) {
        str[i++] = *data;
        data ++;
      }
      if(*data != '\'') {
        cerr << "Unable to read string. Stopped at end of string."
             << endl << endl;
        print_info();
        exit(1);
      }
      data++;
      str[i++]= '\0';
      return strdup(str);
    }
    if(!sscanf(data, "%s", str)) {
      cerr << "Unable to read string. Stopped at: " << data << endl << endl;
      print_info();
      exit(1);
    }
    data += strlen(str);
    return strdup(str);
}

template<class Repository_Types>
void  Rep_Client<Repository_Types>::read_spaces(char*& data) {
   while(*data == ' ') data ++;
}

template<class Repository_Types>
unsigned char  Rep_Client<Repository_Types>::read_boolean(char*& data) {
    read_spaces(data);
    if((data[0] == 'T' || data[0] == 't') &&  
       (data[1] == 'R' || data[1] == 'r') &&  
       (data[2] == 'U' || data[2] == 'u') &&  
       (data[3] == 'E' || data[3] == 'e')) {
       data += 4;
       return 1;
    }
    if((data[0] == 'F' || data[0] == 'f') &&  
       (data[1] == 'A' || data[1] == 'a') &&  
       (data[2] == 'L' || data[2] == 'l') &&  
       (data[3] == 'S' || data[3] == 's') &&  
       (data[4] == 'E' || data[4] == 'e')) {
       data += 5;
       return 0;
    }
    cerr << 
     "The correct values for boolean are \"true\" and \"false\". Stopped at: " 
         << data << endl << endl;
    print_info();
    exit(1);
}

template<class Repository_Types>
void* 
Rep_Client<Repository_Types>::read_void(CORBA::TypeCode_ptr tc, char*& data) {
  CORBA::Any *pany= new CORBA::Any();

  switch(tc->kind()) {
  case CORBA::tk_short: 
    (*pany) <<= (CORBA::Short) read_short(data);
    break;
  case CORBA::tk_long:
    (*pany) <<= (CORBA::Long) read_long(data);
    break;
  case CORBA::tk_ushort:
    (*pany) <<= (CORBA::UShort) read_ushort(data);
    break;
  case CORBA::tk_ulong:
    (*pany) <<= (CORBA::ULong) read_ulong(data);
    break;
  case CORBA::tk_float:
    (*pany) <<= (CORBA::Float) read_float(data);
    break;
  case CORBA::tk_double:
    (*pany) <<= (CORBA::Double) read_double(data);
    break;
  case CORBA::tk_char: {
    CORBA::Any::from_char fc( read_char(data));
    (*pany) <<= fc;
  }
  break;
  case CORBA::tk_octet: {
    CORBA::Any::from_octet fo( read_octet(data));
    (*pany) <<= fo;
  }
  break;
  case CORBA::tk_boolean: {
    CORBA::Any::from_boolean fb( read_boolean(data));
    (*pany) <<= fb;
  }
  break;
  case CORBA::tk_string: 
    (*pany) <<= read_string(data);
    break;
                             
  case CORBA::tk_sequence: {
    if(tc->equal(Domain_Repository::_tc_Shorts)) {
      Domain_Repository::Shorts value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] = read_short(data);
      }
      (*pany) <<= value;
      break;
    }
    if(tc->equal(Domain_Repository::_tc_Longs))  {
      Domain_Repository::Longs value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] = read_long(data);
      }
      (*pany) <<= value;
      break;      
    }
    if(tc->equal(Domain_Repository::_tc_Ushorts)) {
      Domain_Repository::Ushorts value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	i ++) {
	value[i] =read_ushort(data);
      }
      (*pany) <<= value;
      break;
    }
#ifdef OMNIORB
    if(tc->content_type()->kind() == CORBA::tk_ulong) {
#else
      if(tc->equal(Domain_Repository::_tc_Ulongs)) {
#endif 
	Domain_Repository::Ulongs value;
	value.length(read_ulong(data));
	for(int i = 0; 
	    i < value.length();
	    i ++) {
	  value[i] =read_ulong(data);
	}
	(*pany) <<= value;
	break;
#ifdef OMNIORB
      }
#else
    }
#endif 
    if(tc->equal(Domain_Repository::_tc_Floats)) {
      Domain_Repository::Floats value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] =read_float(data);
      }
      (*pany) <<= value;
      break;
    }
    if(tc->equal(Domain_Repository::_tc_Doubles)) {
      Domain_Repository::Doubles value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] =read_double(data);
      }
      (*pany) <<= value;
      break;
    }
    if(tc->equal(Domain_Repository::_tc_Chars)) {
      Domain_Repository::Chars value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] =read_char(data);
      }
      (*pany) <<= value;
      break;
    }
    if(tc->equal(Domain_Repository::_tc_Octets)) {
      Domain_Repository::Octets value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] =read_octet(data);
      }
      (*pany) <<= value;
      break;
    }
    if(tc->equal(Domain_Repository::_tc_Booleans)) {
      Domain_Repository::Booleans value;
      value.length(read_long(data));
      for(int i = 0; 
	  i < value.length();
	  i ++) {
	value[i] = read_boolean(data);
      }
      (*pany) <<= value;
      break;
    } 
#ifdef OMNIORB
    if(tc->content_type()->kind() == CORBA::tk_string) {
#else
      if(tc->equal(Domain_Repository::_tc_Strings)) {
#endif
	Domain_Repository::Strings value;
	value.length(read_long(data));
	for(int i = 0; 
	    i < value.length();
	    i ++) {
	  value[i] =read_string(data);
	}
	(*pany) <<= value;
	break;
#ifdef OMNIORB
      }
#else
    }
#endif
    cerr << "Not supported sequence type. Stopped at: " << data
	 << endl << endl;
    print_info();
    exit(1);
  }
  default: cerr << "Not supported type. Stopped at: " << data 
		<< endl << endl;
  print_info(); 
  exit(1);
  }
  return (void *) (pany->value());
}




