//
//  File:         abng_location_descr_i.cc
//  Author:       Malgorzata Steinder (gosia@ics.agh.edu.pl)
//  Date Created: 12.12.97
//
// General Description:
//   Implementation file for the ABng_Location_Description_I class
//

#include "abng_location_descr_i.h"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

//
// Implementation of constructor, copy constructor, destructor and operators
//                   _______________________________________________________

ostream& 
 operator << (ostream& str, 
              ABng_Core_System::ABng_Location_Description::Description& descr
) {
  str << "loc_descr = " << descr.loc_descr->item_id() << endl;
  str << "sensor_list = { " << endl;
  str << "                _maximum = " 
      << descr.sensor_list.maximum() << endl;
  str << "                _length = " 
      << descr.sensor_list.length() << endl;
  str << "                _buffer = {" << endl;
  for(int i = 0; i < descr.sensor_list.length(); i ++) {
       str << "                            ";
       str << descr.sensor_list[i]->item_id() << ", " << endl;
  }
  str << "                           }" << endl;
  str << "               } " << endl;
  return str;
}

 ABng_Loc_Descr_I::ABng_Loc_Descr_I (
    const ABng_Core_System::ABng_Location_Description::Description& descr_init
) {  
        set_descr(descr_init);

        //
	// init my_corba_object
        //
	my_corba_object = 
          (ABng_Core_System::ABng_Location_Description_ptr)NULL;
}

 ABng_Loc_Descr_I::ABng_Loc_Descr_I (
    const ABng_Loc_Descr_I& abng_location_description_p
) {
        set_descr(abng_location_description_p.my_descr);

	my_corba_object = abng_location_description_p.my_corba_object;
}                        

#ifdef FilePersistency
 ABng_Loc_Descr_I::ABng_Loc_Descr_I(istream& f_p) 
   {
	// read description from the file
	restore(f_p);

	// init my_corba_object
	my_corba_object = (ABng_Core_System::ABng_Location_Description_ptr)NULL;
}
#endif

 ABng_Loc_Descr_I::~ABng_Loc_Descr_I () {
}

 ABng_Loc_Descr_I::operator==(
    const ABng_Loc_Descr_I& abng_location_description_p
) {
	if(abng_location_description_p.get_loc_descr_id() !=
            get_loc_descr_id()
        )  return 0;

	return 1;
}

//
// Implementation of methods enabling access to the state
//

void
 ABng_Loc_Descr_I::set_descr(
    const ABng_Core_System::ABng_Location_Description::Description& descr_update
) {
        if(CORBA::is_nil(descr_update.loc_descr)) 
	  throw Domain_Repository::Invalid_Description();
        for(int j = 0; j < descr_update.sensor_list.length(); j ++)
          if(CORBA::is_nil(descr_update.sensor_list[j])) 
	     throw Domain_Repository::Invalid_Description();

        Domain_Repository::Repository_Item_Ids new_sensor_ids(
            descr_update.sensor_list.length()
        );
        new_sensor_ids.length(descr_update.sensor_list.length());
        for(j = 0; j < descr_update.sensor_list.length(); j ++) {
          try {
               new_sensor_ids[j] = descr_update.sensor_list[j]->item_id();
          } catch(...) {
	    cerr << "Cannot get sensor item_id" << endl;
               throw Domain_Repository::Invalid_Description();
          }
        }

        try {
           loc_descr_id = descr_update.loc_descr->item_id();
        } catch(CORBA::Exception &ex) {
           cerr << ex << endl;
	   cerr << "Cannot get id of location description" << endl;
           throw Domain_Repository::Invalid_Description();
        }

	Domain_Repository::Repository_Item_Id old_item_id = my_descr.item_id;
        my_descr = descr_update;
        my_descr.item_id = old_item_id;

        sensor_ids = new_sensor_ids;
}

const ABng_Core_System::ABng_Location_Description::Description&
 ABng_Loc_Descr_I::get_descr() const {
	return my_descr;  
}

void
 ABng_Loc_Descr_I::set_corba_object(
    const ABng_Core_System::ABng_Location_Description_ptr corba_object_p
) {
	my_corba_object = corba_object_p;
}

const ABng_Core_System::ABng_Location_Description_ptr
   ABng_Loc_Descr_I::get_corba_object (
) {
	return my_corba_object;
}

void 
 ABng_Loc_Descr_I::set_item_id(
    const Domain_Repository::Repository_Item_Id item_id_p
) {
	my_descr.item_id = item_id_p;
}

void 
 ABng_Loc_Descr_I::set_loc_descr(
    Domain_Repository::Location_Description_ptr ld
) {
   if(!CORBA::is_nil(ld))
     my_descr.loc_descr = 
         Domain_Repository::Location_Description::_duplicate(ld);
}

void
 ABng_Loc_Descr_I::remove_sensor(Domain_Repository::Repository_Item_Id id)  {

   for(int i = 0; i < sensor_ids.length(); i ++)
       if(sensor_ids[i] == id) break;
   //
   // Throw exception if found
   //
   if(i == sensor_ids.length()) 
       throw Domain_Repository::Unknown_Object_Ref();

   auto_lock local_lock(*this);
   //
   // Release (should be deleted) if found
   //
   //   CORBA::release(my_descr.sensor_list[i]);

   //
   // Remove the vacant place
   //
   for(; i < my_descr.sensor_list.length() - 1; i ++) {
      my_descr.sensor_list[i] = my_descr.sensor_list[i + 1];
      sensor_ids[i] = sensor_ids[i + 1];
   }
   my_descr.sensor_list.length(my_descr.sensor_list.length() - 1);
   sensor_ids.length(sensor_ids.length() - 1);

   ABng_Loc_Descr_Rep_I::instance()->public_notify_update(this);

}

int  
 ABng_Loc_Descr_I::contains_sensor(Domain_Repository::Repository_Item_Id id)  {
    for(int i = 0; i < sensor_ids.length(); i ++)
        if(sensor_ids[i] == id) {
          return 1;
        }
    return 0;
}

//
// Implementation of the System_Static_Data_Observed_E interface
//                   _____________________________

ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
 ABng_Loc_Descr_I::system_static_data_observer_list(
	ENV_DEF1(env)
)  {

     return ABng_Loc_Descr_Rep_I::instance()->system_static_data_observer_list(
               my_descr.item_id USE_ENV(env)
            );
}
                
void 
 ABng_Loc_Descr_I::attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DEF(env)
)  {

  	// initialize the sequence for this abng_location_description item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this ABng_location_description item id into the sequance
	observed_id_list_p.length(1);
        observed_id_list_p[0] = my_descr.item_id;
	
	// register observer into the repository list
	ABng_Loc_Descr_Rep_I::instance()->attach_system_static_data_observer(
            obs, observed_id_list_p USE_ENV(env)
        );
}

void 
 ABng_Loc_Descr_I::detach_system_static_data_observer(
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DEF(env)
)  {

	// initialize the sequence for this ABng_location_description item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this ABng_location_description item it into the sequance
        observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	ABng_Loc_Descr_Rep_I::instance()->detach_system_static_data_observer(
            obs, observed_id_list_p USE_ENV(env)
        );
}


//
// Implementation of IDL operations of the ABng_Location_Description interface
//                                         ______
 
ABng_Core_System::ABng_Location_Description::Description* 
 ABng_Loc_Descr_I::descr (
	ENV_DEF1(env)
) {

   if(CORBA::is_nil(my_descr.loc_descr))
     throw Domain_Repository::Destination_Unreachable();

   // duplicate my_descr
   ABng_Core_System::ABng_Location_Description::Description* 
     my_descr_duplicated =
       new ABng_Core_System::ABng_Location_Description::Description(my_descr);

   return my_descr_duplicated;
}

ABng_Core_System::ABng_Location_Description::Value_Description* 
 ABng_Loc_Descr_I::_private_value_descr (
	ENV_DEF1(env)
) {

   // duplicate my_descr
   ABng_Core_System::ABng_Location_Description::Value_Description* 
     my_value_descr =
	new ABng_Core_System::ABng_Location_Description::Value_Description();

   try {
     my_value_descr->item_id = get_descr().item_id;
     my_value_descr->loc_descr = *(get_descr().loc_descr->descr());
     my_value_descr->sensor_list.length(get_descr().sensor_list.length());
     for(int i = 0; i < get_descr().sensor_list.length(); i ++)
       my_value_descr->sensor_list[i] = *(get_descr().sensor_list[i]->descr());
   } catch(...) {
          throw Domain_Repository::Destination_Unreachable();
   }

   return my_value_descr;
} 

ABng_Core_System::ABng_Location_Description::Value_Description* 
 ABng_Loc_Descr_I::value_descr (
	ENV_DEF1(env)
) {

   auto_lock local_lock(*this);

   return _private_value_descr(USE_ENV1(env));
}
 
Domain_Repository::Repository_Item_Id 
 ABng_Loc_Descr_I::item_id (ENV_DEF1(env)) 
   {
	return my_descr.item_id;
}

Domain_Repository::Location_Description_ptr
 ABng_Loc_Descr_I::loc_descr(ENV_DEF1(env)) 
  {
    if(CORBA::is_nil(my_descr.loc_descr))
           throw Domain_Repository::Destination_Unreachable();

    return 
       Domain_Repository::Location_Description::_duplicate(my_descr.loc_descr);
}

ABng_Core_System::Sensors*
 ABng_Loc_Descr_I::sensor_list(ENV_DEF1(env)) 
  {
    for(int i = 0; i < my_descr.sensor_list.length(); i ++)
      if(CORBA::is_nil(my_descr.sensor_list[i]))
           throw Domain_Repository::Destination_Unreachable();

    ABng_Core_System::Sensors* sl = 
      new ABng_Core_System::Sensors(my_descr.sensor_list);
    return sl;
   }

void 
 ABng_Loc_Descr_I::add(ABng_Core_System::Sensor_ptr new_sensor
                       ENV_DEF(env)
) {

        if(new_sensor == NULL)
           throw Domain_Repository::Unknown_Object_Ref();

        //
        // Check if not a duplicate
        //
        Domain_Repository::Repository_Item_Id id = new_sensor->item_id();

        for(int i = 0; i < my_descr.sensor_list.length(); i ++)
           if(sensor_ids[i] == id)
               throw Domain_Repository::Duplicate_Data_Record(); 

	ABng_Core_System::ABng_Location_Description::Description 
           new_descr(my_descr);
        new_descr.sensor_list.length(my_descr.sensor_list.length() + 1);
        new_descr.sensor_list[new_descr.sensor_list.length() - 1] = 
	  ABng_Core_System::Sensor::_duplicate(new_sensor);
       

        if(ABng_Loc_Descr_Rep_I::instance()->validate_update_add_data(
                this, new_descr
           )
        ) {
            auto_lock local_lock(*this);
            set_descr(new_descr);
            ABng_Loc_Descr_Rep_I::instance()->public_notify_update(this);
        }
}

void
 ABng_Loc_Descr_I::remove(ABng_Core_System::Sensor_ptr sensor_to_remove
                          ENV_DEF(env)
)  {

        if(sensor_to_remove == NULL)
           throw Domain_Repository::Unknown_Object_Ref();

        //
        // Find this sensor id
        //
        Domain_Repository::Repository_Item_Id id;
        try {
            id = sensor_to_remove->item_id();
        } catch (...) {
        }

        remove_sensor(id);
}


#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________

void
 ABng_Loc_Descr_I::save(ostream &ostr) {

        ostr << my_descr.item_id << " " << loc_descr_id;
        
        ostr << " <";
        ostr << sensor_ids.maximum() << " ";
        ostr << sensor_ids.length() << " <";

        for(int i = 0; i < sensor_ids.length(); i++) {
	  ostr << sensor_ids[i] << " ";
        }
        ostr << ">>" << endl;

}


void
 ABng_Loc_Descr_I::restore(istream &istr) {

#ifdef DEBUG
        cerr << "ABng_Loc_Descr_I:: restore" << endl;
#endif

        istr >> my_descr.item_id;
        istr >> loc_descr_id;
        my_descr.loc_descr = NULL;
        
        int _tmpl;

        char c;
        do { istr >> c; } while (c != '<');

        istr >> _tmpl;
        sensor_ids = Domain_Repository::Repository_Item_Ids(_tmpl);
        my_descr.sensor_list = ABng_Core_System::Sensors(_tmpl);

        istr >> _tmpl;
        sensor_ids.length(_tmpl);
        my_descr.sensor_list.length(_tmpl);

        do { istr >> c; } while (c != '<');

        for(int i = 0; i < _tmpl; i ++) {
	  istr >> sensor_ids[i];
          my_descr.sensor_list[i] = NULL;
        }
        
        do { istr >> c; } while (c != '>');
        do { istr >> c; } while (c != '>');


#ifdef DEBUG
        cerr << "ABng_Loc_Descr_I  restored" << endl;
#endif
}
#endif /* FilePersistency */




