//
//  File:         abng_location_descr_rep_observer_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 16.12.97
//
// General Description:
//   Implementation file for the ABng_Location_Descr_Rep_Observer_I class
//

#ifdef ORBIX
#include <CORBA.h>
#elif defined OMNIORB
#include <omniORB2/CORBA.h>
#endif

#include "sensor_observer.h"

Sensor_Observer_I* Sensor_Observer_I::_instance = 0;

Sensor_Observer_I::Sensor_Observer_I () {
  // init my_corba_object
#ifdef ORBIX
  my_corba_object = (ABng_Core_System::System_Static_Data_Observer_ptr)
   new TIE_ABng_Core_System_System_Static_Data_Observer(Sensor_Observer_I)
        (this);
#elif defined OMNIORB
  my_corba_object = 
    new TIE_CLASS( ABng_Core_System,System_Static_Data_Observer)<Sensor_Observer_I,0>(this);
  CONNECT_IMPL( ABng_Core_System::System_Static_Data_Observer_var, my_corba_object);
#endif
  _instance = this;
}

Sensor_Observer_I::~Sensor_Observer_I ()
{
  try {
     sensor_rep->detach_add_system_static_data_observer(my_corba_object);
  } catch (...) {
  }
  CORBA::release(my_corba_object);
}

void 
 Sensor_Observer_I::notify (
    Domain_Repository::Repository_Item_Id id,
    const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
    Domain_Repository::Change_Type change
    ENV_DEF(env)
) {
    int id_p;
    ABng_Core_System::Sensor_ptr abng_location_descr_p;

    switch(change) {
       case Domain_Repository::OBJECT_ADDED:
#ifdef DEBUG
	  cerr << "Sensor_Observer: new ABng_location_descr added" << endl;
#endif	
	  abng_location_descr_p= sd.sensor_obj();
#ifdef DEBUG
	  // attempt to get the id of this abng_location_descr
	  try {
	      id_p = abng_location_descr_p->item_id();
	  } catch (CORBA::SystemException& se){
	      cerr << "abng_location_Descr_Rep monitor error: ";
	      cerr << "exception when trying to get the id "
                   << "of the new abng_location_descr! ";
	      cerr << endl << &se << endl;
	  }
	  cerr << "Sensor_Observer: new abng_location_descr id " 
               << id_p << endl;
#endif

	  // attempt to register observer in this abng_location_descr
	  try {
	       abng_location_descr_p->attach_system_static_data_observer(
                        my_corba_object
               );
	  } catch (CORBA::SystemException& se){
#ifdef DEBUG
	       cerr << "ABng_Location_Descr_Rep monitor error: ";
	       cerr << "exception when trying to register "
                    << "observer for the new abng_location_descr! ";
	       cerr << endl << &se << endl;
#endif
	  }
#ifdef DEBUG
	  cerr << "abng_location_Descr_Rep monitor: "
               << "observer for new abng_location_descr registered" << endl;
#endif
	  break;

      case Domain_Repository::OBJECT_REMOVED: {
#ifdef DEBUG
	  cerr << "Sensor_Observer: abng_location_descr removed" << endl;
#endif	
	  ABng_Loc_Descr_Rep_I::instance()->remove_sensor(id);

	  break;
       }
       case Domain_Repository::OBJECT_UPDATED:
#ifdef DEBUG
	  cerr << "Sensor_Observer: abng_location_descr updated" << endl;
#endif	
	  abng_location_descr_p= sd.sensor_obj();
#ifdef DEBUG
	  // attempt to get the id of this abng_location_descr
	  try {
	      id_p = abng_location_descr_p->item_id();
	  } catch (CORBA::SystemException& se){
	      cerr << "abng_location_Descr_Rep monitor error: ";
	      cerr << "exception when trying to get the id "
                   << "of the updated abng_location_descr! ";
	      cerr << endl << &se << endl;
	  }
	  cerr << "Sensor_Observer: updated abng_location_descr id " 
               << id_p << endl;
#endif
	  break;

       default:
#ifdef DEBUG
	  cerr << "Sensor_Observer error: unknow change type !!!" << endl;
#endif	
          break;
    }
}

void 
   Sensor_Observer_I::abort (
	ENV_DEF1(env)
   ) {
#ifdef DEBUG
	cerr << "Sensor_Observer: abort called!!!" << endl 
             << endl;	
	cerr << "Sensor_Observer: exiting!!" << endl;
#endif
	exit(0);	

}



void Sensor_Observer_I::reference_resolved(
                                      const ABng_Server_Manager::Name &name,
				      CORBA::Object_var server_ref) {
  if ( name == Bootstrap::SRV_Sensor_Repository()) {
         try {
             sensor_rep = 
                ABng_Core_Management::Sensor_Rep::_narrow( server_ref);
             sensor_rep->attach_add_system_static_data_observer(
                my_corba_object
              );
         }
         catch (...) {
             sensor_rep = ABng_Core_Management::Sensor_Rep::_nil();
         }
  }

  try {
     ABng_Core_System::Sensor::Pattern pattern;
     pattern.is_any = 1;
     ABng_Core_System::Sensors* list = sensor_rep->find(pattern);

     for(int i = 0; i < list->length(); i ++)
        (*list)[i]->attach_system_static_data_observer(my_corba_object);

     delete list;
   } catch(...) {
   }
}


