// $Id: away_rep_i.cc,v 1.1.1.1 1998/07/29 14:16:23 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/away_rep/away_rep_i.cc,v $
//
// $Log: away_rep_i.cc,v $
// Revision 1.1.1.1  1998/07/29 14:16:23  mencnar
// initial revision
//
//
// File:          away_rep_i.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.01.98
//
// General Description:
//  Implementation file for the Away_Rep_I class
//

#define EXCEPTIONS

#include <rw/rwtime.h>
#include <rw/rwdate.h>

#include "away_rep_i.hh"
#include "Naming.hh"
#include "db_utils.hh"



#include <unistd.h>


ostream & operator << ( ostream &ostr, 
			const Domain_Repository::Time &time) {

  static RWTime offset_time( RWDate(1,1,1970),0,0,0,RWZone::utc());
  RWTime t( offset_time.seconds()+time.seconds);
  return ostr << t;
}

//
// 		Constructor
//              ___________

Away_Rep_I::Away_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  ) : 
  Basic_Rep_I<Away_Types, ADO_Types>
(hash_away,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
 ),
heart_beat_cond( & heart_beat_sleeper.mutex()),
heart_beat_needed( 0)
{
  last_active_away_set= NULL;
  _instance= this;
  create_corba_objects();
  (new omni_thread( static_heart_beat, this))->start();
}



//
// 		Destructor
//		__________

Away_Rep_I::~Away_Rep_I() {
  destroy_corba_objects();
  delete ps_dev;
}

void Away_Rep_I::init( int init_v) {
  Basic_Rep_I<Away_Types, ADO_Types>::init( init_v);
  try {
    Bootstrap::register_server_in_NS( Bootstrap::SRV_Away_Repository(),
				      my_rep);
    Bootstrap::register_server_in_NS( Bootstrap::SRV_Away_Repository_Manager(),
				      my_rep_manager);

    register_callback( Bootstrap::SRV_Badge_Holder_Repository());
    register_callback( Bootstrap::SRV_Badge_Repository());
    register_context_callback( Bootstrap::CTX_ABng_Core_System_Pollers());
    ps_dev= new ProcessControlImpl( "away_repository", this);
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception while contacting server manager" << ex << endl;
    exit( 1);
  }
  wake_up_heart_beat();
}

Away_Rep_I* 
 Away_Rep_I::_instance = 0;

Away_Rep_I* 
 Away_Rep_I::instance() {
   return _instance;
}

void Away_Rep_I::reference_resolved( const ABng_Server_Manager::Name &name,
				     CORBA::Object_var server_ref,
				     ABng_Server_Manager::uid callback_id) {

  if ( name >= Bootstrap::CTX_ABng_Core_System_Pollers()) {
    try {
      ABng_Poller::AB_Poller_var pol = 
        ABng_Poller::AB_Poller::_narrow(server_ref);
      if (!CORBA::is_nil(pol)) {
	pol->detach_observer(my_poller_observer_obj);
	pol->attach_observer(my_poller_observer_obj);
	cerr << "Connected to a poller " << name << endl;
      }
    }
    catch (...) {
      cerr << "Connect to a poller failed" << endl;
      //      invalidate_reference(server_name, directory_name);
    }
    return;
  }
  else if ( name == Bootstrap::SRV_Badge_Holder_Repository()) {
#ifdef DEBUG
    cerr << "Badge Holder repository resolved" << endl;
#endif
    try {
      ABng_Core_Management::Badge_Holder_Rep_ptr tmp_ptr=
	ABng_Core_Management::Badge_Holder_Rep::_narrow( server_ref);

      if ( CORBA::is_nil( tmp_ptr)) return;
      {
	auto_lock local_lock( *this);

	r_badge_holder_rep= tmp_ptr;
      }
      resolve_badge_holders();
    }
    catch (...) {
#ifdef DEBUG
      cerr << "Caught exception while resolving badge holders" << endl;
#endif
    }
  }
  else if ( name == Bootstrap::SRV_Badge_Repository()) {
#ifdef DEBUG
    cerr << "Badge Holder repository resolved" << endl;
#endif
    try {
      ABng_Core_Management::Badge_Rep_ptr tmp_ptr=
	ABng_Core_Management::Badge_Rep::_narrow( server_ref);

      if ( CORBA::is_nil( tmp_ptr)) return;
      {
	auto_lock local_lock( *this);

	r_badge_rep= tmp_ptr;
      }
      attach_badges();
    }
    catch (...) {
#ifdef DEBUG
      cerr << "Caught exception while resolving badges" << endl;
#endif
    }    
  }
}

ABng_Core::Away::Description 
  Away_Rep_I::get_null_description( Item_I *item) {
  
  Description d;
  if (item) d.item_id= item->item_id();
  d.type_specific_object.user_obj( Domain_Repository::User::_nil());
  return d;
}

ABng_Core::Away::Value_Description 
  Away_Rep_I::get_null_value_description( Item_I *item) {

  Value_Description d;
  if (item) d.item_id= item->item_id();
  d.type_specific_object_value.user_description( 
				  Domain_Repository::User::Value_Description());
  return d;
}

void Away_Rep_I::attach_badges() {
  if (r_badge_holder_rep->_non_existent()) throw 0;

#ifdef DEBUG
  cerr << "attach_badges" << endl;
#endif
  auto_lock local_lock(*this); 
  // it is safe to set a lock here as there is no other servers dependent upon
  // the away server, so a deadlock is very unlikely to happen

  ABng_Core_System::Badge::Entity_Set es;
  Domain_Repository::Repository_Item_Ids ids( my_item_list.entries());
  ids.length( my_item_list.entries());
  RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
  for ( int i=0; 
	iterator();
	i++)
    ids[i]= iterator.key()->holder_item_id();
  
  es.s_ids( ids);
  Domain_Repository::Query_Options qo;
  qo.return_references= 0;
  qo.return_descriptions= 0;
  qo.return_value_descriptions= 0;
  qo.do_attach= 1;
  qo.do_detach= 0;
  try {
    ABng_Core_System::Badge::Query_Results_var qr=
      r_badge_rep->complex_query( es, qo, 
				  get_ssdo());
  }
  catch (...) {
#ifdef DEBUG
    cerr << "Exception" << endl;
#endif
  }  
#ifdef DEBUG
  cerr << "attach_badges...done" << endl;
#endif
}

void Away_Rep_I::resolve_badge_holders() {
  if (r_badge_holder_rep->_non_existent()) throw 0;

#ifdef DEBUG
  cerr << "resolve_badge_holders" << endl;
#endif
  auto_lock local_lock(*this); 
  // it is safe to set a lock here as there is no other servers dependent upon
  // the away server, so a deadlock is very unlikely to happen

  ABng_Core_System::Badge_Holder::Entity_Set es;
  Domain_Repository::Repository_Item_Ids ids( my_item_list.entries());
  ids.length( my_item_list.entries());
  RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
  for ( int i=0; 
	iterator();
	i++)
    ids[i]= iterator.key()->holder_item_id();
  
  es.s_ids( ids);
  Domain_Repository::Query_Options qo;
  qo.return_references= 1;
  qo.return_descriptions= 1;
  qo.return_value_descriptions= 1;
  qo.do_attach= 1;
  qo.do_detach= 0;
  try {
    ABng_Core_System::Badge_Holder::Query_Results_var qr=
      r_badge_holder_rep->complex_query( es, qo, 
					 get_ssdo());

    RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
    for (int i=0; 
	 iterator();
	 i++) {
      Observer_List *observer_list_p = 
	get_item_observer_list( iterator.key()->item_id());

      if (!qr->r_attach_results[i]) // badge holder removed from database
	iterator.key()->mark_for_delete();
    
      ABng_Core_System::Badge_Holder_var
	bh_var( ABng_Core_System::Badge_Holder::_duplicate( qr->r_references[i]));
					       
      iterator.key()->fast_change_holder( bh_var,
					  qr->r_descriptions[i],
					  qr->r_value_descriptions[i],
					  (ABng_Core::Away::Description &) 
					    iterator.key()->get_descr());
      select_active_message( iterator.key()->holder_item_id(),
			     1 /* don't remove cancelled/expired away messages,
				  they will be collected by a "heartbeat" */);
    }
  }
  catch (...) {
#ifdef DEBUG
    cerr << "Exception" << endl;
#endif
  }
  attach_badges();
#ifdef DEBUG
  cerr << "Resolve badge holders...done" << endl;
#endif
}



//
// Private functions
//


unsigned 
   Away_Rep_I::hash_away(
	const Away_I& away_i_p
   )
{
  return (unsigned) away_i_p.get_descr().start_time.seconds;
}


void
   Away_Rep_I::match_away(
	Away_I *current_away,
	void* apply_data
   )
{
  const ABng_Core::Away::Pattern& drp = 
    ((struct Away_Rep_I::MatchAwayApplyDataStruct*)apply_data)->_data_record_pattern;

  // check general patten matching rule
  if (drp.is_any){
    // any away matches this pattern
    ((struct MatchAwayApplyDataStruct*)apply_data)->_found_aways.insert( current_away);
    return;
  }

  // check matching of item_id
  if (!drp.item_id_pattern.is_any &&
      (drp.item_id_pattern.value != 
       current_away->get_descr().item_id))
    // this away item_id does not match pattern 
    return;

  // start_time
  if (!drp.start_time_pattern.is_any) {
    int is_before= current_away->get_descr().start_time<drp.start_time_pattern.value;
    
    if ((drp.start_time_pattern.before && !is_before) ||
	(!drp.start_time_pattern.before && is_before))
      return;
  }
  // stop_time
  if (!drp.stop_time_pattern.is_any) {
    int is_before= current_away->get_descr().stop_time<drp.stop_time_pattern.value;
    
    if ((drp.stop_time_pattern.before && !is_before) ||
	(!drp.stop_time_pattern.before && is_before))
      return;
  }
  // cancel_from_time
  if (!drp.cancel_from_time_pattern.is_any) {
    int is_before= current_away->get_descr().cancel_from_time <
      drp.cancel_from_time_pattern.value;
    
    if ((drp.cancel_from_time_pattern.before && !is_before) ||
	(!drp.cancel_from_time_pattern.before && is_before))
      return;
  }
  
  // message
  if (!drp.message_pattern.is_any) {
    if (strcmp( drp.message_pattern.value,
		current_away->get_descr().message))
      return;
  }
  // contact
  if (!drp.message_pattern.is_any) {
    if (strcmp( drp.contact_pattern.value,
		current_away->get_descr().contact))
      return;
  }

  // cancel_by_badge
  if (!drp.cancel_by_badge_pattern.is_any) {
    if ((drp.cancel_by_badge_pattern.value &&
	 !current_away->get_descr().cancel_by_badge) ||
	(!drp.cancel_by_badge_pattern.value &&
	 current_away->get_descr().cancel_by_badge))
      return;
  }
  
  // active
  if (!drp.active_pattern.is_any) {
    if ((drp.active_pattern.value &&
	 !current_away->get_descr().active) ||
	(!drp.active_pattern.value &&
	 current_away->get_descr().active))
      return;
  }

  // this away matches the pattern
  ((struct MatchAwayApplyDataStruct*)apply_data)->_found_aways.insert( current_away);
}

//
// Implementation of IDL operations of the System_Static_Data_Observer interface
//
void Away_Rep_I::notify (
        Domain_Repository::Repository_Item_Id id,
        const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
        Domain_Repository::Change_Type change
        ENV_DEF(env)
	) {

  auto_lock local_lock(*this);

  if (sd._d() == ABng_Core_System::System_Static_Data_Observer::BADGE_SD &&
      change == Domain_Repository::OBJECT_UPDATED) {
    RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
    while (iterator())
      if (iterator.key()->badge_item_id() == id) {
	iterator.key()->resolve_badge_holder( r_badge_holder_rep);
	select_active_message( iterator.key()->holder_item_id(),
			       1 /* don't remove cancelled/expired away messages,
				    they will be collected by a "heartbeat" */);
      }
  }
  else if (sd._d() == ABng_Core_System::System_Static_Data_Observer::BADGE_HOLDER_SD) {

    RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
    while (iterator())
      if (iterator.key()->holder_item_id() == id) {
	switch (change) {
	case Domain_Repository::OBJECT_REMOVED:
	  iterator.key()->mark_for_delete();
	  select_active_message( iterator.key()->holder_item_id(),
				 1 /* don't remove cancelled/expired away messages,
				      they will be collected by a "heartbeat" */);

	  notify_observers(*get_item_observer_list(iterator.key()->item_id()), 
			   iterator.key(), 
			   Domain_Repository::OBJECT_UPDATED);
	  break;
	case Domain_Repository::OBJECT_UPDATED:
	  iterator.key()->resolve_badge_holder( r_badge_holder_rep);
	  select_active_message( iterator.key()->holder_item_id(),
				 1 /* don't remove cancelled/expired away messages,
				      they will be collected by a "heartbeat" */);
	  notify_observers(*get_item_observer_list(iterator.key()->item_id()), 
			   iterator.key(), 
			   Domain_Repository::OBJECT_UPDATED);
	  break;      
	}
      }
  }
}


void Away_Rep_I::abort (
        ENV_DEF1(env)
	) {
  return;
}

ABng_Core::Away_ptr Away_Rep_I::add (
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& data_record
	ENV_DEF(env)
	) {

  auto_lock local_lock(*this);

  Item_I *new_item = create_item(data_record);
  add_permitted(new_item);
  init_corba_object(new_item);
  // assign a unique item id to it
  new_item->set_item_id(assign_id());
  insert_item(new_item);
  select_active_message( new_item->holder_item_id(),
			 1);
  notify_observers(my_add_observer_list, new_item, Domain_Repository::OBJECT_ADDED);
#ifdef FilePersistency
  save();
#endif
  return Item::_duplicate(new_item->get_corba_object());
}

void Away_Rep_I::remove (
	ABng_Core::Away_ptr object_to_remove
	ENV_DEF(env)
	) {
  auto_lock local_lock(*this);

  Item_I *item = deref_item(object_to_remove);
  if(!remove_permitted(item)) {
    return;
  }
  Observer_List *observer_list_p = 
    get_item_observer_list(item->item_id());
  item->mark_for_delete();
  select_active_message( item->holder_item_id(),
			 1);  
  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_REMOVED);
  remove_item(item);
  uninit_corba_object(item);
  delete item;
#ifdef FilePersistency
  // store updated state of the repository
  save();
#endif
}

void Away_Rep_I::update (
	ABng_Core::Away_ptr object_to_update, 
	const ABng_Core_Management::Away_Rep_Manager::Change_Description& 
              data_record_pattern
	ENV_DEF(env)
	) {
  auto_lock local_lock(*this);

  if(!validate_update_data(object_to_update, data_record_pattern)) {
    return;
  }
  Item_I *item = deref_item(object_to_update);
  if(!update_permitted(item, data_record_pattern,item)) {
    return;
  }
  my_item_list.remove(item);
  update_item(item, data_record_pattern);
  my_item_list.insert(item);
  Observer_List *observer_list_p = 
    get_item_observer_list(item->item_id());
  select_active_message( item->holder_item_id(),
			 1);  
  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_UPDATED);
#ifdef FilePersistency
  // store updated state of the repository
  save();
#endif  
}

void Away_Rep_I::sighting(
    const char *network_group_name,
    ABng_Poller::AB_Poller::Network_Id network, 
    ABng_Poller::AB_Poller::Station_Id station, 
    const ABng_Poller::AB_Poller::Badge_Id& badge,
    ABng_Poller::AB_Poller::Sighting_Code code,
    const ABng_Poller::AB_Poller::Field_Id& field,
    ABng_Poller::AB_Poller::Byte click_num,
    ABng_Poller::AB_Poller::Byte attributes,
    ABng_Poller::AB_Poller::Seq_Number seq,
    const ABng_Poller::AB_Poller::Time_Stamp& ts
    ENV_DEF(env) ) {
  
  ABng_Guarded::auto_lock local_lock(*this);
  
  ABng_Server_Manager::Time asm_time=
    Bootstrap::gettimeofday();
  Domain_Repository::Time dr_time;
  dr_time.seconds= asm_time.seconds;
  dr_time.microseconds= asm_time.microseconds;
  Domain_Repository::Repository_Item_Id holder_id= 0;

  RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
  while (iterator()) { 
    Away_I *item= iterator.key();
    
    if (!item->marked_for_delete()) 
      if (item->get_badge_id() == badge)
	if (item->cancelled_by_sighting( dr_time)) {
#ifdef DEBUG
	  cerr << "Away message cancelled by badge (id: " 
	       << (Domain_Repository::Octets&) badge << " )" << endl;
#endif
	  holder_id= item->holder_item_id();
	  item->mark_for_delete();
	}
  }

  if (holder_id) {
    select_active_message( holder_id, 0 /* delete cancelled away messages */);
    wake_up_heart_beat();
  }
}

static unsigned hash_item_id( Domain_Repository::Repository_Item_Id id) {
  return (unsigned) id;
}

void Away_Rep_I::static_heart_beat( void *args) {
  Away_Rep_I *rep= (Away_Rep_I*) args;
  
  while (1) {
    {
      ABng_S_Guarded::auto_lock local_lock( rep->heart_beat_sleeper);
      unsigned long s,ns;
      omni_thread::get_time( &s, &ns,
			     60, 0);
      if (! rep->heart_beat_needed) {
#ifdef OMNIORB
	rep->heart_beat_cond.timedwait( s,ns);
#else
	rep->heart_beat_cond.timed_wait( s, ns);
#endif
      }
      rep->heart_beat_needed= 0;
    }
    rep->heart_beat();
  }
}

void Away_Rep_I::wake_up_heart_beat() {
  ABng_S_Guarded::auto_lock local_lock( heart_beat_sleeper);
#ifdef DEBUG
  cerr << "wake_up_heart_beat" << endl;
#endif
  heart_beat_needed= 1;
  heart_beat_cond.signal();
}

void Away_Rep_I::heart_beat() {
  ABng_Guarded::auto_lock local_lock(*this);

#ifdef DEBUG
  cerr << "heart_beat" << endl;
#endif
  RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
  RWTValHashTable<Domain_Repository::Repository_Item_Id> holder_ids( hash_item_id);
  while (iterator()) { 
    Domain_Repository::Repository_Item_Id holder_id=
      iterator.key()->holder_item_id();
    if (!holder_ids.contains( holder_id))
      holder_ids.insert( holder_id);
  }

  RWTValHashTableIterator<Domain_Repository::Repository_Item_Id> id_iter(holder_ids);
  while (id_iter()) 
    select_active_message( id_iter.key(),
			   0 /* delete cancelled/expired away messages */);
}

void Away_Rep_I::select_active_message( Domain_Repository::Repository_Item_Id holder_id,
					int dont_remove) {
  last_active_away_set= NULL;
  Away_I *last_selected= NULL;

  Domain_Repository::Time last_start_time;
  last_start_time.seconds= last_start_time.microseconds= 0;

  ABng_Server_Manager::Time asm_time=
    Bootstrap::gettimeofday();
  Domain_Repository::Time dr_time;
  dr_time.seconds= asm_time.seconds;
  dr_time.microseconds= asm_time.microseconds;

  {
    RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
    while (iterator()) { 
      Away_I *item= iterator.key();
      
#ifdef DEBUG
      cerr << item->holder_item_id() << endl;
#endif

      if (item->holder_item_id() != holder_id ||
	  item->marked_for_delete()) continue;
      if (item->get_descr().stop_time < dr_time) {
	item->mark_for_delete();
	continue;
      }

	
      if (item->get_descr().start_time < dr_time &&
	  dr_time < item->get_descr().stop_time) {
      
	if ( last_start_time < item->get_descr().start_time) {
	  if (last_selected) {
#ifdef DEBUG
	    cerr << "Mark for delete" << endl;
#endif
	    last_selected->mark_for_delete();
	  }
	  last_selected= item;
	  last_start_time= item->get_descr().start_time;
	}
	else {
	  item->mark_for_delete();
#ifdef DEBUG
	  cerr << "Mark for delete" << endl;
#endif
	}
      }
    }
  }
  {
    RWTValDlist<Away_I*> pending_delete;
    RWTValDlist<Away_I*> pending_update;

    RWTPtrHashTableIterator<Away_I> iterator(my_item_list);
    while (iterator()) { 
      Away_I *item= iterator.key();
      if (item->holder_item_id() != holder_id) continue;
      if (item->marked_for_delete()) {
	item->reset_active();
	pending_delete.insert( item);
	continue;
      }
      if (item == last_selected &&
	  !item->get_descr().active) {
	item->set_active();
	pending_update.insert( item);
	continue;
      }
      if (item != last_selected &&
	  item->get_descr().active) {
	item->reset_active();
	pending_update.insert( item);
      }
    }

    int dirty_flag= 0;

    if (!dont_remove) {
      RWTValDlistIterator<Away_I*> del_iterator( pending_delete);
      while (del_iterator()) {
	dirty_flag= 1;
	Away_I *item=  del_iterator.key();
	last_active_away_set= last_selected;
	my_remove( item);
      }
    }
    RWTValDlistIterator<Away_I*> update_iterator( pending_update);
    while (update_iterator()) {
      dirty_flag= 1;
      last_active_away_set= last_selected;
      my_update( update_iterator.key());
    }
#ifdef FilePersistency
    if (dirty_flag) save();
#endif
  }
  last_active_away_set= last_selected;  
}

void Away_Rep_I::my_remove( Away_I* item) {
  Observer_List *observer_list_p = 
    get_item_observer_list(item->item_id());

  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_REMOVED);
  remove_item(item);
  uninit_corba_object(item);
  delete item;
}

void Away_Rep_I::my_update( Away_I* item) {

  Observer_List *observer_list_p = 
    get_item_observer_list(item->item_id());
#ifdef DEBUG
  cerr << "my_update (item_id) " << item->item_id() << endl;
#endif
  notify_observers(*observer_list_p, item, Domain_Repository::OBJECT_UPDATED);  
}

//
// Implementation of IDL operations of the Away_Data_Observed_R interface
//                                         ___________________________


ABng_Core::Aways* 
   Away_Rep_I::find (
	const ABng_Core::Away::Pattern& drp
	ENV_DEF(env)
   )
{
  auto_lock local_lock(*this);

  return private_find( drp USE_ENV(env));
}

ABng_Core::Aways* Away_Rep_I::private_find (
	const ABng_Core::Away::Pattern& drp
	ENV_DEF(env)) {

  // create a structure consisting of a pointer to the container
  //  and a reference of the pattern
  MatchAwayApplyDataStruct apply_data( drp);

  my_item_list.apply(match_away, (void*)&apply_data);

  // remote find
  if (apply_data._found_aways.entries() &&
      (!drp.is_any)) // not empty
    if (! drp.type_specific_object_pattern.is_any) { 
      // remote pattern really matters
      
      if (CORBA::is_nil( (ABng_Core_Management::Badge_Holder_Rep_ptr) 
			 r_badge_holder_rep))
        throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

      try {
        ABng_Core_System::Badge_Holder::Pattern bh_pattern;
        bh_pattern.is_any= 0;
        bh_pattern.item_id_pattern.is_any= 1;
        bh_pattern.my_badge_pattern.is_any= 1;
        bh_pattern.current_badge_pattern.is_any= 1;
        bh_pattern.home_sensor_pattern.is_any= 1;
        bh_pattern.type_specific_object_pattern= 
          drp.type_specific_object_pattern;
	bh_pattern.assigned_pattern.is_any= 1;
	bh_pattern.visitor_pattern.is_any= 1;

	ABng_Core_System::Badge_Holder::Value_Descriptions *
          ret_val( r_badge_holder_rep->find_values( bh_pattern));
        ABng_Core_System::Badge_Holder::Value_Descriptions_var
          auto_ret_val( ret_val);
	
        for ( RWTValDlistIterator<Away_I*> iter( apply_data._found_aways);
              iter(); ) {

          int i;
          Away_I * it= iter.key();
          for (i=0; i<ret_val->length(); i++)
            if ( it->holder_item_id() == 
                 (*ret_val)[i].item_id) break;

          if (i==ret_val->length()) // no corresponding item found
            iter.remove(); // remove current item
        }
      }
      catch (Domain_Repository::Destination_Unreachable &) {
        throw;
      }
      catch (CORBA::Exception &ex) {
#ifdef DEBUG
        cerr << "Away_Rep_I::private_find " << ex << endl;
#endif
        throw Domain_Repository::Destination_Unreachable("badge_holder_rep");
      }
    }
  //  ----
  ABng_Core::Aways *ret= 
    new ABng_Core::Aways( apply_data._found_aways.entries());
  ret->length( apply_data._found_aways.entries());

  int pos=0;
  for ( RWTValDlistIterator<Away_I*> iter( apply_data._found_aways);
        iter(); )
    (*ret)[pos++]= ABng_Core::Away::_duplicate( iter.key()->get_corba_object());

  return ret;
}


void Away_Rep_I::collect_values( 
    Away_I *lt, 
    ABng_Core::Away::Value_Description& ld) {

  ABng_Core::Away::Value_Description_var val_descr= lt->value_descr();
  ld= *(val_descr.operator->());
}


void Away_Rep_I::init_corba_object( Away_I *item_p) {

  // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
  ObjectStoreTIE_ABng_Core_Away( Away_I)* 
    new_away_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_Away(
      Away_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_Away(
        Away_I
	) (item_p);
#else
  TIE_ABng_Core_Away( Away_I) *
    new_away_tie = 
    new TIE_ABng_Core_Away( Away_I) (item_p);
#endif
  item_p->set_corba_object( new_away_tie);

#elif defined OMNIORB
  ABng_Core::Away_ptr new_away=
    new TIE_CLASS(ABng_Core,Away)<Away_I,0>(item_p);
  CONNECT_IMPL( ABng_Core::Away_var, new_away);
  item_p->set_corba_object( new_away);
#endif
}

DB_Header * Away_Rep_I::create_db_header() {

  return new DB_Header( "Away Repository", "1.0", "\
#  <data_record>::=  <item_id> <holder_item_id>\n\
#                    <start_time_seconds> <start_time_microseconds>\n\
#                    <stop_time_seconds> <stop_time_microseconds>\n\
#                    <cancel_from_time_seconds> <cancel_from_time_microseconds>\n\
#                    <message_string> <contact_string>\n\
#                    <cancel_by_badge>\n\
#\n");
    }

void Away_Rep_I::construct_data_object(
    ABng_Core::Away_Data_Observer::Away_Data_Object& data_object_p,
    ABng_Core::Away_ptr item_p) {

  data_object_p.away_obj= ABng_Core::Away::_duplicate( item_p);
  if (last_active_away_set)
    data_object_p.active_away_message= 
      ABng_Core::Away::_duplicate( last_active_away_set->get_corba_object());
}

void Away_Rep_I::create_corba_objects() {

#ifdef ORBIX 
#ifdef FilePersistency
  // create a TIE for the object
  my_rep = 
    new TIE_ABng_Core_Away_Rep(Away_Rep_I) (this);
  my_rep_manager = 
    new TIE_ABng_Core_Management_Away_Rep_Manager(Away_Rep_I) (this);
  my_ssdo=
    new TIE_ABng_Core_System_System_Static_Data_Observer(Away_Rep_I) (this);
  my_poller_observer_obj =
    new TIE_ABng_Poller_AB_Observer(Away_Rep_I) (this);
#endif /* FilePersistency */
#elif defined OMNIORB
  my_rep =
    new TIE_CLASS(ABng_Core,Away_Rep)<Away_Rep_I,0>(this);
  CONNECT_IMPL( ABng_Core::Away_Rep_var,my_rep);
  my_rep_manager =
    new TIE_CLASS(ABng_Core_Management,Away_Rep_Manager)<Away_Rep_I,0>(this);
  CONNECT_IMPL( ABng_Core_Management::Away_Rep_Manager_var,my_rep_manager);
  my_ssdo =
    new TIE_CLASS(ABng_Core_System,System_Static_Data_Observer)<Away_Rep_I,0>(this);
  CONNECT_IMPL( ABng_Core_System::System_Static_Data_Observer_var,my_ssdo);
  my_poller_observer_obj =
    new TIE_CLASS(ABng_Poller,AB_Observer)<Away_Rep_I,0>(this);
  CONNECT_IMPL( ABng_Poller::AB_Observer_var,my_poller_observer_obj);
#endif
}

void Away_Rep_I::destroy_corba_objects() {
 
  CORBA::release(my_rep);
  CORBA::release(my_rep_manager);
  CORBA::release(my_ssdo);
  CORBA::release(my_poller_observer_obj);
}









