//
//  File:         badge_holder_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 21.12.97
//
// General Description:
//   Header file for the implementation class of the Badge_Holder interface
//

#ifndef badge_holder_i_h
#define badge_holder_i_h

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "abng_core_management.hh"
#include "abng_guarded.hh"
#include "abng_string.h"
#include "compatibility.hh"

class Badge_Holder_I
{
 private:
  // state of the IDL Badge_Holder object
  ABng_Core_System::Badge_Holder::Description my_descr;

  // additional state
  Domain_Repository::Repository_Item_Id my_badge_id;
  Domain_Repository::Repository_Item_Id current_badge_id;
  Domain_Repository::Repository_Item_Id home_sensor_id;
  ABng_Core::Locatable_Type badge_holder_kind;
  Domain_Repository::Repository_Item_Id type_specific_object_id;

  // reference of CORBA object encapsulated this C++ object
  ABng_Core_System::Badge_Holder_ptr my_corba_object;

  // state lock
  ABng_Guarded &my_guard;

 public:
  Domain_Repository::Repository_Item_Id get_my_badge_id() const {
    return my_badge_id;
  }

  Domain_Repository::Repository_Item_Id get_current_badge_id() const {
    return current_badge_id;
  }

  Domain_Repository::Repository_Item_Id get_home_sensor_id() const {
    return home_sensor_id;
  }
  
  Domain_Repository::Repository_Item_Id get_type_specific_object_id() const {
    return type_specific_object_id;
  }
    
  // ctor
   Badge_Holder_I (
	const ABng_Core_System::Badge_Holder::Description& descr_init
   );

#ifdef FilePersistency
   Badge_Holder_I (
	istream &iostr
   );
#endif

  virtual 
   ~Badge_Holder_I ();

   operator==(	
	const Badge_Holder_I&
   );

  // methods enabling access to the state

  void
   set_descr (
	const ABng_Core_System::Badge_Holder::Description& descr_update
   );
 
  const ABng_Core_System::Badge_Holder::Description&
   get_descr () const;

  void
   set_corba_object (
	ABng_Core_System::Badge_Holder_ptr corba_object_p
   );

  const ABng_Core_System::Badge_Holder_ptr
   get_corba_object ();

  void 
   set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   );

  ABng_Core::Locatable_Type kind() { return badge_holder_kind; }


  void resolve_my_badge(ABng_Core_Management::Badge_Rep_var badge_rep);
  void resolve_current_badge(ABng_Core_Management::Badge_Rep_var badge_rep);
  void resolve_home_sensor(ABng_Core_Management::Sensor_Rep_var sensor_rep);
  void resolve_user(Domain_Repository::User_Rep_var user_rep);
  void resolve_equipment(Domain_Repository::Equipment_Rep_var equipment_rep);

  //
  // IDL operations of the System_Static_Data_Observed_E interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   system_static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual void 
   attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs  
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Unknown_Observer);

  //
  // IDL operations of the Badge_Holder interface
  //                       ____________
 
  virtual ABng_Core_System::Badge_Holder::Description* 
   descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);
 
  virtual ABng_Core_System::Badge_Holder::Value_Description* 
   value_descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

  virtual ABng_Core_System::Badge_Holder::Value_Description* 
   private_value_descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

  virtual Domain_Repository::Repository_Item_Id 
   item_id (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual CORBA::Boolean assigned( ENV_DECL1(env));
  virtual CORBA::Boolean visitor( ENV_DECL1(env));

  virtual ABng_Core_System::Badge_ptr 
   my_badge (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

  virtual ABng_Core_System::Badge_ptr 
   current_badge (
	ENV_DECL1(env)
  )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

  virtual ABng_Core_System::Sensor_ptr 
   home_sensor (
	ENV_DECL1(env)
  )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

  virtual ABng_Core::Locatable_Type_Specific_Object* 
   type_specific_object (
	ENV_DECL1(env)
  )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable);

#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream &iostr
   );

  virtual void
   restore (
	istream &iostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_System_Badge_Holder(Badge_Holder_I)
#else
DEF_TIE_ABng_Core_System_Badge_Holder(Badge_Holder_I)
#endif
#elif defined OMNIORB
// DEF_TIE__tie_Badge_Holder
#endif

// implementation of this class need method "instance" of class Badge_Holder_Rep_I
#include "badge_holder_rep_i.h"

#endif /* badge_holder_i_h */


