// -*- Mode: C++; -*-
//
//  File:         badge_holder_rep_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 21.12.97
//
// General Description:
//   Header file for Badge_Holder_Rep implementation class
//

#ifndef badge_holder_rep_i_hh
#define badge_holder_rep_i_hh

#include "badge_holder_i.h"
#include "basic_rep_i.h"
#include "bootstrap_n.hh"
#include "process_status.hh"

class H_SDO_Types {

public:
  	typedef ABng_Core_System::System_Static_Data_Observed_R Observed;
  	typedef ABng_Core_System::System_Static_Data_Observer Observer;
  	typedef ABng_Core_System::System_Static_Data_Observer_ptr Observer_ptr;
  	typedef ABng_Core_System::System_Static_Data_Observed_E::
    		System_Static_Data_Observers Observers;
};

class Holder_Types {

public:
	typedef ABng_Core_System::Badge_Holder Item;  
	typedef ABng_Core_System::Badge_Holder_ptr Item_ptr;  
  	typedef ABng_Core_System::Badge_Holders Items;  
  	typedef Badge_Holder_I Item_I;  
  	typedef ABng_Core_System::Badge_Holder::Description Initial_Data;
  	typedef ABng_Core_System::Badge_Holder::Description Description;
  	typedef ABng_Core_System::System_Static_Data_Observer::
		System_Static_Data_Object Data_Object;
};

class Badge_Holder_Rep_I : public Bootstrap_listener,
                     	   public Basic_Rep_I<Holder_Types, H_SDO_Types>,
			   public ServerStatusInterface
{
  static void register_my_servers( void *args);
  ABng_S_Guarded repos_guard;
  omni_condition repos_cond;
  unsigned long last_repos_resolved, last_repos_resolved_us;

 public:
  Domain_Repository::Static_Data_Observer_ptr observer_corba_object;

  // ctor:
  Badge_Holder_Rep_I (
#ifdef ObjectStore
	os_database* db
#elif defined FilePersistency
    	const char* repository_name_p
#endif
  );

  virtual void 
    init (
	int cold_start_p = 0
    );

  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static Badge_Holder_Rep_I* 
    instance ();

  // dctor:
  virtual 
     ~Badge_Holder_Rep_I ();

 private:
  // pointer to an unique istance of the class
  static Badge_Holder_Rep_I* _instance;

  // repository
  ABng_Core_Management::Badge_Holder_Rep* my_rep;

  ProcessControlImpl *ps_dev;

  // references of servers this one is depended on
  ABng_Core_Management::Badge_Rep_var r_badge_rep;
  ABng_Core_Management::Sensor_Rep_var r_sensor_rep;
  Domain_Repository::User_Rep_var r_user_rep;
  Domain_Repository::Equipment_Rep_var r_equipment_rep;

  virtual void 
   init_corba_object (
	Badge_Holder_I* item_p
   );

  virtual void 
   create_corba_objects (
   );

  virtual void 
   destroy_corba_objects (
   );

  void 
   reference_resolved (
	const ABng_Server_Manager::Name &name,
	CORBA::Object_var server_ref,
	ABng_Server_Manager::uid callback_id
   ); 

  void resolve_badges();
  void resolve_home_sensors();
  void resolve_users();
  void resolve_equipments();

  virtual DB_Header*
   create_db_header (
   );

  // calculate the hash value of the badge_holder
  static unsigned 
   hash_badge_holder(
	const Badge_Holder_I& badge_holder_i_p
   ); 

  // put the current badge_holder into the container
  static void
   gather_badge_holder(
	Badge_Holder_I* current_badge_holder,
	void* apply_data
   );

  // definition of structure containing parameters of match_badge_holder method
  struct MatchBadge_HolderApplyDataStruct {
    ABng_Core_System::Badge_Holders* _found_badge_holders;
    const ABng_Core_System::Badge_Holder::Pattern& _data_record_pattern;
    ABng_Core_System::Badge::Value_Descriptions* _my_badge_descrs;
    ABng_Core_System::Badge::Value_Descriptions* _current_badge_descrs;
    ABng_Core_System::Sensor::Value_Descriptions* _home_sensor_descrs;
    Domain_Repository::User::Value_Descriptions* _user_descrs;
    Domain_Repository::Equipment::Value_Descriptions* _equipment_descrs;

    ABng_Core_Management::Badge_Rep_var _r_badge_rep;
    ABng_Core_Management::Sensor_Rep_var _r_sensor_rep;
    Domain_Repository::User_Rep_var _r_user_rep;
    Domain_Repository::Equipment_Rep_var _r_equipment_rep;

    MatchBadge_HolderApplyDataStruct(
	ABng_Core_System::Badge_Holders* _found_badge_holders_p,
   	const ABng_Core_System::Badge_Holder::Pattern& _data_record_pattern_p,
        ABng_Core_Management::Badge_Rep_var _badge_rep,
        ABng_Core_Management::Sensor_Rep_var _sensor_rep,
        Domain_Repository::User_Rep_var _user_rep,
        Domain_Repository::Equipment_Rep_var _equipment_rep
    );

    ~MatchBadge_HolderApplyDataStruct();

    int 
     match_my_badge(
	const ABng_Core_System::Badge::Pattern&,
	Domain_Repository::Repository_Item_Id
     );

    int 
     match_current_badge(
	const ABng_Core_System::Badge::Pattern&,
	Domain_Repository::Repository_Item_Id
     );

    int 
     match_home_sensor(
	const ABng_Core_System::Sensor::Pattern&,
	Domain_Repository::Repository_Item_Id
     );

    int 
     match_user(
	const Domain_Repository::User::Pattern&,
	Domain_Repository::Repository_Item_Id
     );

    int 
     match_equipment(
	const Domain_Repository::Equipment::Pattern&,
	Domain_Repository::Repository_Item_Id
     );
  };

  // if the current badge_holder matches the pattern puts it into the container
  static void
   match_badge_holder(
	Badge_Holder_I* current_badge_holder,
	void* apply_data
   );

  virtual int 
   validate_add_data (
	const ABng_Core_System::Badge_Holder::Description &data_record
   );

  virtual int 
   Badge_Holder_Rep_I::validate_update_data (
	ABng_Core_System::Badge_Holder_ptr item_p,
	const ABng_Core_System::Badge_Holder::Description& data_record
   );

  virtual void 
   construct_data_object (
    	ABng_Core_System::System_Static_Data_Observer::
    	System_Static_Data_Object& data_object_p,
    	ABng_Core_System::Badge_Holder_ptr item_p
   );

  virtual void 
   collect_values (
	Badge_Holder_I *lt, 
    	ABng_Core_System::Badge_Holder::Value_Description& ld
   );

  virtual Description get_null_description( Item_I *item);
  virtual Value_Description get_null_value_description( Item_I *item);
  
public:
  //
  // IDL operations of the System_Static_Data_Observed_R interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_R::Observer_Entries*
   system_static_data_observer_list (
        ENV_DECL1(env)
   ) 
                throw (CORBA::SystemException);
             
  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   add_system_static_data_observer_list (
        ENV_DECL1(env)
   ) 
                throw (CORBA::SystemException);

  virtual void 
   attach_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs,  
        const Domain_Repository::Repository_Item_Ids& observed_id_list
        ENV_DECL(env)
   )
                throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids);
 
  virtual void 
   detach_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs, 
        const Domain_Repository::Repository_Item_Ids& observed_id_list
        ENV_DECL(env)
   ) 
                throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids, 
                       Domain_Repository::Unknown_Observer,
                       Domain_Repository::Unmatched_Observer_Item_Ids);
  
  virtual void 
   attach_add_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs  
        ENV_DECL(env)
   )
                throw (CORBA::SystemException);
 
  virtual void 
   detach_add_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs
        ENV_DECL(env)
   ) 
                throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer);
 
  // Suplementary operation
 
  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   system_static_data_observer_list (
        Domain_Repository::Repository_Item_Id item_id_p
        ENV_DECL(env)
   ) 
                throw (CORBA::SystemException);

  //
  // IDL operations of the Badge_Holder_Rep interface
  //                       ________________

  virtual ABng_Core_System::Badge_Holders* 
   badge_holder_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual ABng_Core_System::Badge_Holder_ptr 
   add (
	const ABng_Core_System::Badge_Holder::Description& data_record
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Duplicate_Data_Record);

  virtual void 
   remove (
	ABng_Core_System::Badge_Holder_ptr object_to_remove 
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Object_Ref);

  virtual ABng_Core_System::Badge_Holders* 
   find (
	const ABng_Core_System::Badge_Holder::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

   // no locking here 
  ABng_Core_System::Badge_Holders* 
   private_find (
	const ABng_Core_System::Badge_Holder::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);  

  virtual ABng_Core_System::Badge_Holder::Value_Descriptions* 
   find_values (
	const ABng_Core_System::Badge_Holder::Pattern& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable,
                       Domain_Repository::Search_Not_Implemented);

  virtual ABng_Core_System::Badge_Holder::Query_Results*
   complex_query(
		 const ABng_Core_System::Badge_Holder::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core_System::System_Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<Holder_Types, H_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual void 
   update (
	ABng_Core_System::Badge_Holder_ptr object_to_update, 
	const ABng_Core_System::Badge_Holder::Description& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
		       Domain_Repository::Unknown_Object_Ref,
		       Domain_Repository::Duplicate_Data_Record);

  //
  // IDL operations of the Static_Data_Observer interface
  //                       ___________________________
            
  virtual void 
   notify (
        Domain_Repository::Repository_Item_Id id,
	const Domain_Repository::Static_Data_Observer::Static_Data_Object& sd, 
	Domain_Repository::Change_Type change
        ENV_DECL(env)

   );

  virtual void 
   abort (
		ENV_DECL1(env)
   );

#ifdef ObjectStore
  //
  // for ObjectStore
  //

  static os_typespec* get_os_typespec();
#endif
};

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Management_Badge_Holder_Rep(Badge_Holder_Rep_I)
#else
DEF_TIE_ABng_Core_Management_Badge_Holder_Rep(Badge_Holder_Rep_I)
DEF_TIE_Domain_Repository_Static_Data_Observer(Badge_Holder_Rep_I)
#endif
#elif defined OMNIORB
// DEF_TIE__tie_Badge_Holder_Rep
#endif

#endif /* badge_holder_rep_i_hh */



