//
//  File:         abng_location_descr_rep_observer_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 16.12.97
//
// General Description:
//   Implementation file for the ABng_Location_Descr_Rep_Observer_I class
//

#include <CORBA.h>

#include "location_descr_observer.h"

extern CORBA::String_var str1;
CORBA::String_var str2;


Location_Descr_Observer_I* Location_Descr_Observer_I::_instance = 0;

Location_Descr_Observer_I::Location_Descr_Observer_I () {
  // init my_corba_object
  my_corba_object = (Domain_Repository::Static_Data_Observer_ptr) new 
   TIE_Domain_Repository_Static_Data_Observer(Location_Descr_Observer_I)
        (this);
  _instance = this;
}

Location_Descr_Observer_I::~Location_Descr_Observer_I ()
{
  try {
     location_descr_rep->detach_add_static_data_observer(my_corba_object);
  } catch (...) {
  }
  CORBA::release(my_corba_object);
}

void 
 Location_Descr_Observer_I::notify (
    Domain_Repository::Repository_Item_Id id,
    const Domain_Repository::Static_Data_Observer::Static_Data_Object& sd, 
    Domain_Repository::Change_Type change,
    CORBA::Environment &IT_env
) {
    int id_p;
    Domain_Repository::Location_Description_ptr abng_location_descr_p;

    switch(change) {
       case Domain_Repository::OBJECT_ADDED:
	  cerr << "Location_Descr_Observer: new ABng_location_descr added" 
               << endl;	
	  abng_location_descr_p= sd.location_descr_obj();
#ifdef DEBUG
	  // attempt to get the id of this abng_location_descr
	  try {
	      id_p = abng_location_descr_p->item_id();
	  } catch (CORBA::SystemException& se){
	      cerr << "Location_Descr_Rep monitor error: "
	           << "exception when trying to get the id " 
                   << "of the new abng_location_descr! "<< endl << &se << endl;
	  }
	  cerr << "Location_Descr_Observer: new abng_location_descr id " 
               << id_p << endl;
#endif
	  try {
	       abng_location_descr_p->attach_static_data_observer(
                        my_corba_object
               );
	  } catch (CORBA::SystemException& se){
#ifdef DEBUG
	       cerr << "Location_Descr_Rep monitor error: "
	            << "exception when trying to register observer "
                    << "for the new abng_location_descr! "
	            << endl << &se << endl;
#endif
	  }
#ifdef DEBUG
	  cerr << "abng_location_Descr_Rep monitor: "
               << "observer for new abng_location_descr registered" << endl;
#endif
	  break;

       case Domain_Repository::OBJECT_REMOVED: {
#ifdef DEBUG
	  cerr << "Location_Descr_Observer: abng_location_descr removed" 
               << endl;	
#endif
	  ABng_Core_System::ABng_Location_Description::Pattern pattern;
          pattern.is_any = 0;
          pattern.item_id_pattern.is_any = 1;
          pattern.loc_descr_pattern.is_any = 0;
          pattern.loc_descr_pattern.item_id_pattern.is_any = 0;
          pattern.loc_descr_pattern.item_id_pattern.value = id;
          pattern.loc_descr_pattern.type_name_pattern.is_any = 1;
          pattern.loc_descr_pattern.property_list_pattern.is_any = 1;

          ABng_Core_System::ABng_Location_Descriptions* loc_descrs;
          try {
	    loc_descrs = ABng_Loc_Descr_Rep_I::instance()->find(pattern);
          } catch(...) {
             return;
          }

          for(int i = 0; i < loc_descrs->length(); i ++) {
            try {
               ABng_Loc_Descr_Rep_I::instance()->remove((*loc_descrs)[i]);
            } catch(...) {
            }
          }

	  break;
    }
       case Domain_Repository::OBJECT_UPDATED:
#ifdef DEBUG
	  cerr << "Location_Descr_Observer: abng_location_descr updated" 
               << endl;	
#endif
	  abng_location_descr_p= sd.location_descr_obj();
#ifdef DEBUG
	  // attempt to get the id of this abng_location_descr
	  try {
	      id_p = abng_location_descr_p->item_id();
	  } catch (CORBA::SystemException& se){
	      cerr << "abng_location_Descr_Rep monitor error: ";
	      cerr << "exception when trying to get the id "
                   << "of the updated abng_location_descr! ";
	      cerr << endl << &se << endl;
	  }
	  cerr << "Location_Descr_Observer: updated abng_location_descr id " 
               << id_p << endl;
#endif
	  break;

       default:
#ifdef DEBUG
	  cerr << "Location_Descr_Observer error: unknow change type !!!" 
               << endl;	
#endif
          break;
    }
}

void 
   Location_Descr_Observer_I::abort (
	CORBA::Environment &IT_env
   ) {
#ifdef DEBUG
	cerr << "Location_Descr_Observer: abort called!!!" << endl 
             << endl;	
	cerr << "Location_Descr_Observer: exiting!!" << endl;
#endif
	exit(0);	

}


void Location_Descr_Observer_I::reference_resolved(
                                      const ABng_Server_Manager::Name &name,
				      CORBA::Object_var server_ref) {

  if ( name == Bootstrap::SRV_Location_Description_Repository()) {
         try {
             location_descr_rep = 
                Domain_Repository::Location_Description_Rep::_narrow(
                   server_ref
                );
             location_descr_rep->attach_add_static_data_observer(
                my_corba_object
              );
         }
         catch (...) {
             location_descr_rep = 
                   Domain_Repository::Location_Description_Rep::_nil();
         }
  }

  try {
       Domain_Repository::Location_Description::Pattern ld_pattern;
       ld_pattern.is_any = 1;
       ld_pattern.property_list_pattern.constraints = 
          Domain_Repository::UNRESTRICTED;
 
       Domain_Repository::Location_Description_Rep::Location_Descriptions
          * list = location_descr_rep->find(ld_pattern);

       for(int i = 0; i < list->length(); i ++)
           (*list)[i]->attach_static_data_observer(my_corba_object);

       delete list;
   } catch(...) {
   }
}


