// $Id: badge_i.cc,v 1.2 1998/07/29 13:31:06 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/badge_rep/badge_i.cc,v $
//
// $Log: badge_i.cc,v $
// Revision 1.2  1998/07/29 13:31:06  mencnar
// compiled as a library,
// complex_query implementation
//
// Revision 1.1.1.1  1998/06/17 14:43:58  mencnar
//
// Revision 1.2  1998/01/13 10:30:44  mencnar
// Various bug fixes
// Persistent storage implemented
// Name service integration
//
// Revision 1.1  1998/01/07 08:34:53  mencnar
// Initial revision
//
//
// File:          badge_i.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.01.98
//
// General Description:
//  Implementation file for the Badge_I class
//

#ifdef ORBIX
#define EXCEPTIONS
#endif

#include "badge_rep_i.hh"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

int operator == ( const Domain_Repository::Octets &first,
                  const Domain_Repository::Octets &second) {
  if (first.length() != second.length()) return 0;
  int size= first.length();
  for (int i=0; i<size; i++)
    if (first[i] != second[i]) return 0;
  return 1;
}


//
// Implementation of constructor, copy constructor, destructor and operators
//                   _______________________________________________________

  Badge_I::Badge_I (
	const ABng_Core_System::Badge::Description& descr_init
	) : 
    my_guard( *(Badge_Rep_I::instance())),
    badge_holder_id( 0)
{
  cerr << "Badge_I constructor (description) " << (int)this << endl;
  // init my_descr
  my_descr= descr_init;

   // init my_corba_object
  my_corba_object = (ABng_Core_System::Badge_ptr)NULL;

  if (! CORBA::is_nil( my_descr.holder)) {
    try {
      badge_holder_id= my_descr.holder->item_id();
    }
    catch (...) {
      throw Domain_Repository::Destination_Unreachable("badge_holder_repository");
    }
  }
  else badge_holder_id= 0;
}

  Badge_I::Badge_I (
	const Badge_I& badge_p
	) :
    my_guard( badge_p.my_guard)
{
  cerr << "Badge_I copy constructor" << endl;
  // copy my_descr
  my_descr.item_id = badge_p.my_descr.item_id;
  my_descr.id = badge_p.my_descr.id;

  badge_holder_id= badge_p.badge_holder_id;

  // copy m_corba_object
  my_corba_object = badge_p.my_corba_object;
}                        

#ifdef FilePersistency
   Badge_I::Badge_I (
	istream &iostr
	) :
     my_guard( *(Badge_Rep_I::instance()))
{
  cerr << "Badge_I stream constructor" << endl;
  // read description from the file
  restore( iostr);
  
  // init my_corba_object
  my_corba_object = (ABng_Core_System::Badge_ptr)NULL;

  my_descr.holder= NULL;
}
#endif

Badge_I::~Badge_I (
  )
{
  cerr << "Badge_I destructor " << (int) this << endl;
}

  Badge_I::operator== (
	const Badge_I& badge_p
  )
{
  if (!(my_descr.id == badge_p.my_descr.id))
    return 0;
  
  return 1;
}

//
// Implementation of methods enabling access to the state
//

void
  Badge_I::set_descr (
	const ABng_Core_System::Badge::Description& descr_update
  )
{
  // update my_descr
  /// NO !!!! my_descr.item_id = descr_update.item_id;
  my_descr.id = descr_update.id;
  my_descr.holder= descr_update.holder;

  if ( ! CORBA::is_nil( my_descr.holder)) {
    try {
      badge_holder_id= my_descr.holder->item_id();
    }
    catch (...) {}   
  }
  else badge_holder_id= 0;
}

const ABng_Core_System::Badge::Description&
   Badge_I::get_descr (
   ) const
{
	return my_descr;  
}

void
   Badge_I::set_corba_object (
	ABng_Core_System::Badge_ptr corba_object_p
   )
{
	my_corba_object = corba_object_p;
}

ABng_Core_System::Badge_ptr
   Badge_I::get_corba_object (
   ) 
{
	return my_corba_object;
}

void 
   Badge_I::set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
	my_descr.item_id = item_id_p;
}

void Badge_I::resolve_badge_holder( ABng_Core_Management::Badge_Holder_Rep_var holder_rep) {
  ABng_Core_System::Badge_Holder::Pattern pat;

  Domain_Repository::User::Pattern user_pattern;
  pat.is_any= 0;
  pat.item_id_pattern.is_any= 0;
  pat.item_id_pattern.value= badge_holder_id;
  pat.my_badge_pattern.is_any= 1;
  pat.current_badge_pattern.is_any= 1;
  pat.home_sensor_pattern.is_any= 1;
  pat.type_specific_object_pattern.is_any= 1;
  pat.type_specific_object_pattern.object_pattern.user_obj_pattern( user_pattern);
  pat.assigned_pattern.is_any= 1;
  pat.visitor_pattern.is_any= 1;

#ifdef DEBUG
  cerr << "resolve_badge_holder (item_id= " << badge_holder_id << ")" << endl;
#endif
  try {
    ABng_Core_System::Badge_Holders_var ret_val( 
		holder_rep->find( pat) );
    if (ret_val->length() == 1)
      my_descr.holder= ABng_Core_System::Badge_Holder::_duplicate( ret_val[0]);
    else
#ifdef DEBUG
      cerr << "not found" << endl;
#endif
  }
  catch (CORBA::COMM_FAILURE &ex) { 
    throw;
  }
  catch (...) {}
}

//
// Implementation of IDL operations of the Static_Data_Observed_E interface
//                                         _____________________________

ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   Badge_I::system_static_data_observer_list (
        ENV_DEF1(env)) {

     return Badge_Rep_I::instance()->system_static_data_observer_list(
				       my_descr.item_id USE_ENV(env));
}
                
void 
   Badge_I::attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DEF(env)) {

#ifdef DEBUG
  cerr << "Badge_I::attach_system_static_data_observer " << (int) obs << " "
#ifdef ORBIX
       << obs->_refCount()
#endif
       << endl;
#endif
	// ignore observed_id_list parameter
  	//  initialize the sequence for this badge item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this badge item it into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	Badge_Rep_I::instance()->attach_system_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

void 
   Badge_I::detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DEF(env)) {

     // ignore observed_id_list parameter
     //  initialize the sequence for this badge item id
     Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

     // put this  item it into the sequance
     observed_id_list_p.length(1);
     observed_id_list_p[0] = my_descr.item_id;

     // register observer into the repository list
     try {
       Badge_Rep_I::instance()->detach_system_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
     }
     catch ( Domain_Repository::Unknown_Observer &ex) {
       throw ex;
     }
     catch ( ...) {}
}

//
// Implementation of IDL operations of Scheduler_Request_Queue interface
//                                     _______________________

ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests*
   Badge_I::list_user_requests (
	const char * user_id
	ENV_DEF(env)) {

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();

  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch))
    return sch->list_user_requests( user_id);
}
   
ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests* 
   Badge_I::list_all_requests (
	ENV_DEF1(env)) {

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();

  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch))
    return sch->list_all_requests();
}

void
   Badge_I::remove_request (
	CORBA::Long request_id
	ENV_DEF(env)) {

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();

  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch))
	  sch->remove_request( request_id);
}

ABng_Core_System::Badge_Holder_ptr Badge_I::holder (
				     ENV_DEF1(env)) {

  if (badge_holder_id != 0 &&
      CORBA::is_nil( my_descr.holder)) 
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

  return ABng_Core_System::Badge_Holder::_duplicate( my_descr.holder);
}


//
// Implementation of IDL operations of the Badge_Commander interface
//                                         ________________

void Badge_I::prepare_params( const ABng_Core_System::Sensors& sensors,
			      ABng_Core_System::Badge_Ids &bids,
			      ABng_Core_System::Sensor_Ids &sids) {

    try {
      bids.length(1);
      bids[0]= my_descr.id;

      sids.length( sensors.length());
      for (int j=0; j<sensors.length(); j++) {
	sids[j]= *(sensors[j]->id());
      }    
    }
    catch (...) { 
      throw ABng_Core::Invalid_Argument();
    }
}

void Badge_I::map_exceptions( CORBA::Exception &ex) {
  try {
    throw ex;
  }
    catch ( ABng_Core_System::Unknown_Sensor_Id &ex) {
      throw ABng_Core::Invalid_Argument();
    }
    catch ( ABng_Core_System::Invalid_Sensor_Id &ex) {
      throw ABng_Core::Invalid_Argument();
    }
    catch ( ABng_Core_System::Unknown_Badge_Id &ex) {
      throw ABng_Core::Invalid_Argument();
    }
    catch ( ABng_Core_System::Invalid_Badge_Id &ex) {
      throw ABng_Core::Invalid_Argument();
    }
    catch ( CORBA::SystemException &ex) {
      throw Domain_Repository::Destination_Unreachable("scheduler");
    }  
}
		       

CORBA::Long Badge_I::issue_command ( 
		const ABng_Core_System::Sensors& sensors, 
		const ABng_Core_System::Octets& cmd, 
		CORBA::Boolean has_reply, 
		CORBA::Octet reply_code, 
		const ABng_Core::Command_Context& ctx, 
		ABng_Core_System::Scheduler_Caller_ptr reply_to
		ENV_DEF(env)) {

  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->issue_badge_command( sids, bids, cmd,
				       has_reply, reply_code,
				       ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");
}


CORBA::Long Badge_I::challenge (
	       const ABng_Core_System::Sensors& sensors,
	       const ABng_Core_System::Octets& random, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DEF(env)) {

  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->challenge( sids, bids, random,
			     ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");  
}

CORBA::Long Badge_I::sound (
	       const ABng_Core_System::Sensors& sensors,
	       const ABng_Core_System::Octets& sound, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DEF(env)) {
      
  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
	Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->sound( sids, bids, sound,
			 ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");  
}

CORBA::Long Badge_I::set_state (
              const ABng_Core_System::Sensors& sensors, 
	      CORBA::Octet status,
	      const ABng_Core_System::Octets &sound,
	      const ABng_Core::Command_Context& ctx, 
	      ABng_Core_System::Scheduler_Caller_ptr reply_to
	      ENV_DEF(env)) {
      
  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->set_state( sids, bids, status, sound,
			     ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");  
}

CORBA::Long Badge_I::get_state (
               const ABng_Core_System::Sensors& sensors, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DEF(env)) {
      
  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->get_state( sids, bids, 
			     ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");  
}

CORBA::Long Badge_I::set_home (
               const ABng_Core_System::Sensors& sensors, 
	       const ABng_Core_System::Octets& address, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DEF(env)) {
      
  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->set_home( sids, bids, address,
			    ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");  
}


CORBA::Long Badge_I::get_home (
               const ABng_Core_System::Sensors& sensors, 
	       const ABng_Core::Command_Context& ctx, 
	       ABng_Core_System::Scheduler_Caller_ptr reply_to
	       ENV_DEF(env)) {

  ABng_Core_System::Badge_Ids bids( 1);
  ABng_Core_System::Sensor_Ids sids( sensors.length());
  prepare_params( sensors, bids, sids);

  ABng_Core_System::Scheduler_var sch= 
    Badge_Rep_I::instance()->scheduler();
  if ( ! CORBA::is_nil( (ABng_Core_System::Scheduler_ptr) sch)) {
    try {
      return sch->get_home( sids, bids,
			    ctx, reply_to);
    }
    catch ( CORBA::Exception &ex) {
      map_exceptions( ex);
    }
  } else throw Domain_Repository::Destination_Unreachable("scheduler");  
}




//
// Implementation of IDL operations of the Badge interface
//                                         ______
 
ABng_Core_System::Badge::Description* 
  Badge_I::descr (
	ENV_DEF1(env)) {
  ABng_Guarded::auto_lock local_lock( my_guard);

  if (badge_holder_id != 0 &&
      CORBA::is_nil( my_descr.holder)) {
    cerr << "badge_holder_rep unreachable" << endl;
    cerr << "badge_holder_id = " << badge_holder_id << endl;
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");
  }

  // duplicate my_descr
  ABng_Core_System::Badge::Description* my_descr_duplicated =
    new ABng_Core_System::Badge::Description(my_descr);

  return my_descr_duplicated;
}

ABng_Core_System::Badge::Value_Description*
  Badge_I::value_descr (
      	ENV_DEF1(env)) {

  return descr();
}
 
Domain_Repository::Repository_Item_Id 
   Badge_I::item_id (
	ENV_DEF1(env)) {
  // no lock here: item_id is immutable

  return my_descr.item_id;
}

ABng_Core_System::Badge_Id* 
   Badge_I::id (
	ENV_DEF1(env)) {
  ABng_Guarded::auto_lock local_lock( my_guard);

  // duplicate id
  ABng_Core_System::Badge_Id* id_duplicated =
    new ABng_Core_System::Badge_Id(my_descr.id);

  return id_duplicated;
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________


void
   Badge_I::save (
	ostream &iostr
   )
{
#ifdef DEBUG
  // cerr << "Badge_I: saving Badge state to file" << endl; 
#endif

  iostr << dec << my_descr.item_id << ' ';

#ifdef DEBUG
  // cerr << "Badge_I: item id -> " << my_descr.item_id << endl; 
#endif

  iostr << my_descr.id << ' ' << dec << badge_holder_id << endl;
  // end of the record

#ifdef DEBUG
  // cerr << "Badge_I: state saved" << endl; 
#endif
}

void
   Badge_I::restore (
	istream &iostr
   )
{
  iostr >> dec >> my_descr.item_id >> my_descr.id >> badge_holder_id;
  if (!iostr) {
    cerr << "Badge_I error: reading item_id from file";
    exit(1);
  }

#ifdef DEBUG
  cerr << "Badge_I: read badge item_id = "; 
  cerr << my_descr.item_id << endl;
#endif


}
#endif /* FilePersistency */



