// $Id: badge_rep_i.cc,v 1.2 1998/07/29 13:31:07 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/badge_rep/badge_rep_i.cc,v $
//
// $Log: badge_rep_i.cc,v $
// Revision 1.2  1998/07/29 13:31:07  mencnar
// compiled as a library,
// complex_query implementation
//
// Revision 1.1.1.1  1998/06/17 14:43:58  mencnar
//
// Revision 1.1  1998/01/13 10:39:55  mencnar
// Initial revision
//
//
// File:          badge_rep_i.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.01.98
//
// General Description:
//  Implementation file for the Badge_Rep_I class
//

#define EXCEPTIONS

#include "badge_rep_i.hh"
#include "Naming.hh"
#include "db_utils.hh"

#include <rw/tvdlist.h>



//
// 		Constructor
//              ___________

Badge_Rep_I::Badge_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  ) : 
  Basic_Rep_I<Badge_Types, B_SDO_Types>(hash_badge,
#ifdef ObjectStore
    db_p
#elif defined FilePersistency
    repository_name_p
#endif
 ) {
  _instance= this;
  create_corba_objects();
}



//
// 		Destructor
//		__________

Badge_Rep_I::~Badge_Rep_I() {
  destroy_corba_objects();
  delete ps_dev;
}

void Badge_Rep_I::init( int init_v) {
  Basic_Rep_I<Badge_Types, B_SDO_Types>::init( init_v);
  try {
    Bootstrap::register_server_in_NS( Bootstrap::SRV_Badge_Repository(),
				      my_rep);

    register_callback( Bootstrap::SRV_Badge_Holder_Repository());
    register_callback( Bootstrap::SRV_Scheduler());
    ps_dev= new ProcessControlImpl( "badge_repository", this);
    load_observers();
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception while contacting server manager" << ex << endl;
    exit( 1);
  }
}

Badge_Rep_I* 
 Badge_Rep_I::_instance = 0;

Badge_Rep_I* 
 Badge_Rep_I::instance() {
   return _instance;
}

void Badge_Rep_I::reference_resolved( const ABng_Server_Manager::Name &name,
				      CORBA::Object_var server_ref,
				      ABng_Server_Manager::uid callback_id) {

  if ( name == Bootstrap::SRV_Scheduler()) {
    try {
      ABng_Core_System::Scheduler_ptr tmp_ptr=
	ABng_Core_System::Scheduler::_narrow( server_ref);

      if ( CORBA::is_nil( tmp_ptr)) return;

      auto_lock local_lock( *this);
      r_scheduler= tmp_ptr;
    }
    catch (...) {}
  }
  else if ( name == Bootstrap::SRV_Badge_Holder_Repository()) {
#ifdef DEBUG
    cerr << "Badge Holder repository resolved" << endl;
#endif
    try {
      ABng_Core_Management::Badge_Holder_Rep_ptr tmp_ptr=
	ABng_Core_Management::Badge_Holder_Rep::_narrow( server_ref);

      if ( CORBA::is_nil( tmp_ptr)) return;
      {
	auto_lock local_lock( *this);

	r_badge_holder_rep= tmp_ptr;
      }
      resolve_badge_holders();
    }
    catch (...) {
#ifdef DEBUG
      cerr << "Caught exception while resolving badge holders" << endl;
#endif
      auto_lock local_lock( *this);
      r_badge_holder_rep= ABng_Core_Management::Badge_Holder_Rep::_nil();
    }
  }
}

void Badge_Rep_I::resolve_badge_holders() {
  if (r_badge_holder_rep->_non_existent()) throw 0;

#ifdef DEBUG
  cerr << "resolve_badge_holders" << endl;
#endif

  RWTPtrHashTableIterator<Badge_I> iterator(my_item_list);
  while (iterator()) { 

    iterator.key()->resolve_badge_holder( r_badge_holder_rep);
    Observer_List *observer_list_p = 
      get_item_observer_list( iterator.key()->item_id());
    
    notify_observers(*observer_list_p, 
		     iterator.key(), 
		     Domain_Repository::OBJECT_UPDATED);
  }
}



//
// Private functions
//



int
Badge_Rep_I::remove_permitted(Badge_I * badge_i_p) {

  return 1;
}
   
int Badge_Rep_I::validate_add_data(
  const ABng_Core_System::Badge::Description &data_record) {

#ifdef ORBIX
  if ( CORBA::is_nil( data_record.holder)) 
    (( ABng_Core_System::Badge::Description &) data_record).holder= NULL;
  // a very nasty thing (to cast away a const), 
  // but masks Orbix bug (ver. 2.2, 2.3) 
#endif
  return 1;
}

int Badge_Rep_I::validate_update_data(Item_ptr item_p, 
  const ABng_Core_System::Badge::Description& data_record) {

#ifdef ORBIX
  if ( CORBA::is_nil( data_record.holder)) 
    (( ABng_Core_System::Badge::Description &) data_record).holder= NULL;
  // a very nasty thing (to cast away a const), 
  // but masks Orbix bug (ver. 2.2, 2.3) 
#endif
  return 1;
}

unsigned 
   Badge_Rep_I::hash_badge(
	const Badge_I& badge_i_p
   )
{
  //  cerr << "hash badge " << (int) &badge_i_p << endl;
  ABng_Core_System::Badge::Description badge_descr = 
    badge_i_p.get_descr();

  unsigned long ret= 0;

  for (int i=0; i<badge_descr.id.length(); i++)
    ret= ret*256+badge_descr.id[i];
  return ret;
}


void
   Badge_Rep_I::gather_badge(
	Badge_I *current_badge,
	void* apply_data
   )
{
  // project apply_data (void*) to Badges*
  ABng_Core_System::Badges *collected_badges =
    (ABng_Core_System::Badges*)apply_data;

  // increase the length of the container
  collected_badges->length(collected_badges->length() + 1);

  // put duplicated badges (CORBA object) into the container
  (*collected_badges)[collected_badges->length()-1] =
    ABng_Core_System::Badge::_duplicate(current_badge->get_corba_object());

#ifdef DEBUG
  //	cerr << "Gather badge -- refCount " << current_badge->get_corba_object()->_refCount() << endl;
#endif

}

void
   Badge_Rep_I::match_badge(
	Badge_I *current_badge,
	void* apply_data
   )
{
  const ABng_Core_System::Badge::Pattern& data_record_pattern = 
    ((struct Badge_Rep_I::MatchBadgeApplyDataStruct*)apply_data)->_data_record_pattern;

  // check general patten matching rule
  if (data_record_pattern.is_any){
    // any badge matches this pattern
    gather_badge(current_badge, 
		 ((struct MatchBadgeApplyDataStruct*)apply_data)->_found_badges);
    return;
  }

  // check matching of item_id
  if (!data_record_pattern.item_id_pattern.is_any &&
      (data_record_pattern.item_id_pattern.value != 
       current_badge->get_descr().item_id))
    // this badge item_id does not match pattern 
    return;

  if (!data_record_pattern.id_pattern.is_any &&
      (!(data_record_pattern.id_pattern.value ==
	 current_badge->get_descr().id)))
    return;

  // this badge matches the pattern
  gather_badge(current_badge, 
	       ((struct MatchBadgeApplyDataStruct*)apply_data)->_found_badges);
}



//
// Implementation of IDL operations of the System_Static_Data_Observed_R interface
//                                         ___________________________


ABng_Core_System::Badges* 
   Badge_Rep_I::find (
	const ABng_Core_System::Badge::Pattern& data_record_pattern
	ENV_DEF(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
  auto_lock local_lock(*this);

  return private_find( data_record_pattern USE_ENV(env));
}

ABng_Core_System::Badges* 
   Badge_Rep_I::private_find (
	const ABng_Core_System::Badge::Pattern& data_record_pattern
	ENV_DEF(env)) {

     // create a structure consisting of a pointer to the container
     //  and a reference of the pattern
     MatchBadgeApplyDataStruct apply_data(
	     (ABng_Core_System::Badges*)new 
	     ABng_Core_System::Badges(my_item_list.entries()),
	     data_record_pattern);

     // initialize the sequence for found badges
     //  pesimitic assumption - number of items matching pattern
     //  is equal to number of items

     // browse through hash table to collect badges
     my_item_list.apply(match_badge, (void*)&apply_data);

     // pointer to found_badges was stored inside apply_data thus return it
#ifdef DEBUG
     cerr << "match badge applied to " << my_item_list.entries() << " items" << endl;
#endif DEBUG
     return apply_data._found_badges;
}


void Badge_Rep_I::collect_values( 
    Badge_I *lt, 
    ABng_Core_System::Badge::Value_Description& ld) {

  ld= lt->get_descr();
}


void Badge_Rep_I::init_corba_object( Badge_I *item_p) {

  // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
  ObjectStoreTIE_ABng_Core_System_Badge( Badge_I)* 
    new_badge_tie =
    new(my_db, ObjectStoreTIE_ABng_Core_System_Badge(
      Badge_I)::get_os_typespec()) 
      ObjectStoreTIE_ABng_Core_System_Badge(
        Badge_I
	) (item_p);
#else
  TIE_ABng_Core_System_Badge( Badge_I) *
    new_badge_tie = 
    new TIE_ABng_Core_System_Badge( Badge_I) (item_p);
#endif
  item_p->set_corba_object( new_badge_tie);

#elif defined OMNIORB
  ABng_Core_System::Badge_ptr new_badge=
    new TIE_CLASS(ABng_Core_System,Badge)<Badge_I,0>(item_p);
  CONNECT_IMPL( ABng_Core_System::Badge_var, new_badge);
  item_p->set_corba_object( new_badge);
#endif
}

DB_Header * Badge_Rep_I::create_db_header() {

  return new DB_Header( "Badge Repository", "1.0", "\
#       <data_record>::=  <item_id> <badge_id> <badge_holder_id>\n\
#\n\
#       <badge_id>::=     <sequence_of_octets>\n\
#\n");
    }

void Badge_Rep_I::construct_data_object(
    ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& 
      data_object_p,
    ABng_Core_System::Badge_ptr item_p) {


  // construct union to store reference of added entity
  data_object_p.badge_obj(
    ABng_Core_System::Badge::_duplicate(item_p));
}

void Badge_Rep_I::create_corba_objects() {

#ifdef ORBIX 
#ifdef FilePersistency
  // create a TIE for the object
  my_rep = 
    new TIE_ABng_Core_Management_Badge_Rep(Badge_Rep_I) (this);
#endif /* FilePersistency */
#elif defined OMNIORB
  my_rep =
    new TIE_CLASS(ABng_Core_Management,Badge_Rep)<Badge_Rep_I,0>(this);
  CONNECT_IMPL( ABng_Core_Management::Badge_Rep_var,my_rep);
#endif
 
  write_ior(my_rep, 
    "./Badge_Rep_iiop.ref");
}

void Badge_Rep_I::destroy_corba_objects() {
 
  CORBA::release(my_rep);
}









