// $Id: badge_rep_i.hh,v 1.2 1998/07/29 13:31:08 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/badge_rep/badge_rep_i.hh,v $
//
// $Log: badge_rep_i.hh,v $
// Revision 1.2  1998/07/29 13:31:08  mencnar
// compiled as a library,
// complex_query implementation
//
// Revision 1.1.1.1  1998/06/17 14:43:58  mencnar
//
//
// File:          badge_rep_i.hh
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  07.01.98
//
// General Description:
//   Header file for Badge_Rep implementation class
//
 

#ifndef badge_rep_i_hh
#define badge_rep_i_hh

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <errno.h>
#include <string.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "rw/tphasht.h"
#include "rw/tvhasht.h"
#include "rw/tphdict.h"
#include "rw/tvhdict.h"

#include "abng_core_management.hh"
#include "badge_i.hh"
#include "bootstrap_n.hh"
#include "abng_guarded.hh"
#include "observed_i.h"
#include "basic_rep_i.h"
#include "process_status.hh"


class B_SDO_Types {

public:

  typedef ABng_Core_System::System_Static_Data_Observed_R Observed;
  typedef ABng_Core_System::System_Static_Data_Observer Observer;
  typedef ABng_Core_System::System_Static_Data_Observer_ptr Observer_ptr;
  typedef ABng_Core_System::System_Static_Data_Observed_E::
    System_Static_Data_Observers Observers;
};

class Badge_Types {

public:

  typedef ABng_Core_System::Badge Item;  
  typedef ABng_Core_System::Badge_ptr Item_ptr;  
  typedef ABng_Core_System::Badges Items;  
  typedef Badge_I Item_I;  
  typedef ABng_Core_System::Badge::Description Initial_Data;
  typedef ABng_Core_System::Badge::Description Description;
  typedef ABng_Core_System::System_Static_Data_Observer::
    System_Static_Data_Object Data_Object;
};


class Badge_Rep_I  : public Bootstrap_listener,
		     public Basic_Rep_I<Badge_Types, B_SDO_Types>,
                     public ServerStatusInterface { 


 public:
  Badge_Rep_I (
#ifdef ObjectStore
    os_database* db_p
#elif defined FilePersistency
    const char *repository_name_p
#endif
  );
 
  virtual ~Badge_Rep_I ();

  virtual void init( int init_v);
 
  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static Badge_Rep_I* 
    instance ();
 
 private:

  ABng_Core_System::Scheduler_var r_scheduler;
  ABng_Core_Management::Badge_Holder_Rep_var r_badge_holder_rep;
  

  // calculate the hash value of the badge
  static unsigned 
   hash_badge(
	const Badge_I& badge_i_p
   ); 

  // put the current badge into the container
  static void
   gather_badge(
	Badge_I* current_badge,
	void* apply_data
   );

  // definition of structure containing parameters of match_badge method
  struct MatchBadgeApplyDataStruct {
    ABng_Core_System::Badges* _found_badges;
    const ABng_Core_System::Badge::Pattern& _data_record_pattern;

    MatchBadgeApplyDataStruct(
	ABng_Core_System::Badges* _found_badges_p,
   	const ABng_Core_System::Badge::Pattern& _data_record_pattern_p):
	 	_found_badges(_found_badges_p),
		_data_record_pattern(_data_record_pattern_p) {}
  };

  // if the current badge matches the pattern puts it into the container
  static void
   match_badge(
	Badge_I* current_badge,
	void* apply_data
   );
 
  // put the current observer into the container
  static void
   gather_observers(
	Badge_I* current_badge,
	void* apply_data
   );

 protected:
  // ctor:
  Badge_Rep_I (
#ifdef ObjectStore
	os_database* db
#elif defined FilePersistency
#endif
  );

  void resolve_badge_holders();

//  static const char * ref_server_names[];

  virtual int remove_permitted( Badge_I *);

public:
  ABng_Core_System::Scheduler_var scheduler() {
    auto_lock local_lock( *this);
    return r_scheduler;
  }

  void reference_resolved( const ABng_Server_Manager::Name &name,
			   CORBA::Object_var server_ref,
			   ABng_Server_Manager::uid callback_id);

  //
  // IDL operations of the Static_Data_Observed_R interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_R::Observer_Entries*
   system_static_data_observer_list (
        ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException) {
        return observer_list(USE_ENV1(env));
  }
             
  virtual ABng_Core_System::System_Static_Data_Observed_E::
     System_Static_Data_Observers*
   add_system_static_data_observer_list (
	ENV_DECL1(env)
   ) 
	throw (CORBA::SystemException) {
        return add_observer_list( USE_ENV1(env));
   }
           
  virtual void 
  attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   )
	throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Repository_Item_Ids
        ) {
        attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void 
   detach_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs, 
        const Domain_Repository::Repository_Item_Ids& observed_id_list
        ENV_DECL(env)
   ) 
        throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Repository_Item_Ids, 
               Domain_Repository::Unknown_Observer,
               Domain_Repository::Unmatched_Observer_Item_Ids
        ) {
        detach_observer( obs, observed_id_list USE_ENV(env));
  }



  virtual void 
   attach_add_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs
	ENV_DECL(env)
   )
	throw (CORBA::SystemException) {
        attach_add_observer(obs USE_ENV(env));
  }
 
  virtual void 
   detach_add_system_static_data_observer (
        ABng_Core_System::System_Static_Data_Observer_ptr obs
        ENV_DECL(env)
   ) 
        throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Observer) {
        detach_add_observer(obs USE_ENV(env));
  }

  // Suplementary operation

  virtual ABng_Core_System::System_Static_Data_Observed_E::
     System_Static_Data_Observers*
   system_static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL(env)
   ) 
	throw (CORBA::SystemException) {
        return get_item_observers(item_id_p USE_ENV(env));
  }


  //
  // IDL operations of the Badge_Rep interface
  //                       __________

  
  virtual ABng_Core_System::Badges* 
   badge_list (
	ENV_DECL1(env)
   ) 
    throw (CORBA::SystemException) {
    return item_list( USE_ENV1(env));
  }

  virtual ABng_Core_System::Badges* 
   find (
	const ABng_Core_System::Badge::Pattern& data_record_pattern
	ENV_DECL(env)
   )
	throw (CORBA::SystemException,
               Domain_Repository::Search_Not_Implemented
        );

  virtual ABng_Core_System::Badges* 
   Badge_Rep_I::private_find (
	const ABng_Core_System::Badge::Pattern& data_record_pattern
	ENV_DECL(env)
   );

  virtual ABng_Core_System::Badge::Value_Descriptions* 
   find_values (
	const ABng_Core_System::Badge::Pattern& data_record_pattern
        ENV_DECL(env)) {

     return Basic_Rep_I<Badge_Types, B_SDO_Types>::find_values( 
		  data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core_System::Badge::Query_Results*
   complex_query(
		 const ABng_Core_System::Badge::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core_System::System_Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<Badge_Types, B_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }


  virtual ABng_Core_System::Badge_ptr 
   add (
	const ABng_Core_System::Badge::Description& data_record
	ENV_DECL(env)
   )
	 throw (CORBA::SystemException, 
                Domain_Repository::Duplicate_Data_Record
		) {
    return Basic_Rep_I<Badge_Types, B_SDO_Types>::add(data_record USE_ENV(env));
  }

  virtual int validate_add_data(
	 const ABng_Core_System::Badge::Description &data_record);

  virtual void 
   remove (
	ABng_Core_System::Badge_ptr object_to_remove
	ENV_DECL(env)
   )
	throw (CORBA::SystemException, 
               Domain_Repository::Unknown_Object_Ref
	       ) {
    
     Basic_Rep_I<Badge_Types, B_SDO_Types>::remove(object_to_remove USE_ENV(env));
  }

  virtual void 
   update (
	ABng_Core_System::Badge_ptr object_to_update, 
	const ABng_Core_System::Badge::Description& 
              data_record_pattern
	ENV_DECL(env)
   ) 
	throw (CORBA::SystemException, 
	       Domain_Repository::Unknown_Object_Ref,
	       Domain_Repository::Duplicate_Data_Record
	       ) {

    Basic_Rep_I<Badge_Types, B_SDO_Types>::update(object_to_update, 
      data_record_pattern USE_ENV(env));
  }

  virtual int validate_update_data(
      ABng_Core_System::Badge_ptr object_to_update,
      const ABng_Core_System::Badge::Description &data_record);
         
#ifdef ObjectStore
  //
  // for ObjectStore
  //

  static os_typespec* get_os_typespec();

#endif

  virtual void init_corba_object(Badge_I *item_p);
  virtual DB_Header *create_db_header();

  virtual void construct_data_object(
    ABng_Core_System::System_Static_Data_Observer::
    System_Static_Data_Object& data_object_p,
    ABng_Core_System::Badge_ptr item_p);


  virtual void collect_values( Badge_I *lt, 
    ABng_Core_System::Badge::Value_Description& ld);

  // pointer to an unique istance of the class
  static Badge_Rep_I* _instance;

  // repository
  ABng_Core_Management::Badge_Rep* my_rep;

  ProcessControlImpl *ps_dev;

  virtual void create_corba_objects();
  virtual void destroy_corba_objects();  

    
};

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Management_Badge_Rep(Badge_Rep_I)
#else
DEF_TIE_ABng_Core_Management_Badge_Rep(Badge_Rep_I)
#endif
#elif defined OMNIORB
  // DEF_TIE__tie_Badge_Rep
#endif
#endif /* badge_rep_i_hh */
