//
//  File:         meta_equipment_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 08.04.98
//
// General Description:
//   Implementation file for the Meta_Equipment_I class
//

#define EXCEPTIONS
#include "meta_equipment_i.h"
#include "db_utils.hh"

#include <rw/tvdlist.h>

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

//
// Implementation of constructor, destructor and operators
//                   _____________________________________

  Meta_Equipment_I::Meta_Equipment_I (
	const Domain_Repository::Meta_Equipment::Description& descr_init
  ) : 
	my_guard(*(Meta_Equipment_Rep_I::instance()))
{
      	// init my_descr
	my_descr.item_id = descr_init.item_id;

	my_descr.type = ABng_String::copy_string(descr_init.type);
	my_descr.vendor = ABng_String::copy_string(descr_init.vendor);
	my_descr.model = ABng_String::copy_string(descr_init.model);

	my_descr.my_dimensions = descr_init.my_dimensions;
	my_descr.my_control_info = descr_init.my_control_info;
       
        // check if valid
        long max_length = descr_init.specific_property_description_list.length();
        for(int i = 0; i < max_length - 1; i ++)
		for(int j = i + 1; j < max_length; j ++) {
			if(!strcmp(descr_init.specific_property_description_list[i].name,
			   descr_init.specific_property_description_list[j].name)) 
				throw Domain_Repository::Invalid_Description();
		}

	my_descr.specific_property_description_list = 
		descr_init.specific_property_description_list;

	my_descr.my_icon = descr_init.my_icon;

	// init my_corba_object
	my_corba_object = (Domain_Repository::Meta_Equipment_ptr)NULL;
}

#ifdef FilePersistency
   Meta_Equipment_I::Meta_Equipment_I (
	istream &iostr
   ) :
	my_guard(*(Meta_Equipment_Rep_I::instance()))
{
	// read description from the file
	restore(iostr);

	// init my_corba_object
	my_corba_object = (Domain_Repository::Meta_Equipment_ptr)NULL;
}
#endif


  Meta_Equipment_I::~Meta_Equipment_I (
  )
{
	ABng_String::free_string(my_descr.type);
	ABng_String::free_string(my_descr.vendor);
	ABng_String::free_string(my_descr.model);
}

  Meta_Equipment_I::operator== (
	const Meta_Equipment_I& meta_equipment_p
  )
{
	if (strcmp(my_descr.type, meta_equipment_p.my_descr.type))
		return 0;

	if (strcmp(my_descr.vendor, meta_equipment_p.my_descr.vendor))
		return 0;

	if (strcmp(my_descr.model, meta_equipment_p.my_descr.model))
		return 0;

	return 1;
}

//
// Implementation of methods enabling access to the state
//

void
  Meta_Equipment_I::set_descr (
	const Domain_Repository::Meta_Equipment::Description& descr_update
  )
{       
	// update my_descr
	/// NO !!!! my_descr.item_id = _descr_update.item_id;

	my_descr.type = ABng_String::copy_string(descr_update.type);

	my_descr.vendor = ABng_String::copy_string(descr_update.vendor);

	my_descr.model = ABng_String::copy_string(descr_update.model);

	my_descr.my_dimensions= descr_update.my_dimensions;

	// check if valid
        long max_length = descr_update.specific_property_description_list.length();
        for(int i = 0; i < max_length - 1; i ++)
		for(int j = i + 1; j < max_length; j ++) {
			if(!strcmp(descr_update.specific_property_description_list[i].name,
			   descr_update.specific_property_description_list[j].name)) 
				throw Domain_Repository::Invalid_Description();
		}

	my_descr.my_control_info= descr_update.my_control_info;

	my_descr.specific_property_description_list = 
		descr_update.specific_property_description_list;

	my_descr.my_icon = descr_update.my_icon;
}

const Domain_Repository::Meta_Equipment::Description&
   Meta_Equipment_I::get_descr (
   ) const
{
	return my_descr;  
}

void
   Meta_Equipment_I::set_corba_object (
	const Domain_Repository::Meta_Equipment_ptr corba_object_p
   )
{
	my_corba_object = corba_object_p;
}

const Domain_Repository::Meta_Equipment_ptr
   Meta_Equipment_I::get_corba_object (
   ) 
{
	return my_corba_object;
}

void 
   Meta_Equipment_I::set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
	my_descr.item_id = item_id_p;
}

//
// Implementation of IDL operations of the Static_Data_Observed_E interface
//                                         ______________________

Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   Meta_Equipment_I::static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return Meta_Equipment_Rep_I::instance()->static_data_observer_list(
		my_descr.item_id USE_ENV(env));
}
                
void 
   Meta_Equipment_I::attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
  	// initialize the sequence for this meta_equipment item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this meta_equipment item id into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	Meta_Equipment_Rep_I::instance()->attach_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

void 
   Meta_Equipment_I::detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer)
{
	// initialize the sequence for this meta_equipment item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this meta_equipment item it into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
	Meta_Equipment_Rep_I::instance()->detach_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

//
// Implementation of IDL operations of the Meta_Equipment interface
//                                         ______________
 
Domain_Repository::Meta_Equipment::Description* 
  Meta_Equipment_I::descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate my_descr
	Domain_Repository::Meta_Equipment::Description* my_descr_duplicated =
		new Domain_Repository::Meta_Equipment::Description(my_descr);

	return my_descr_duplicated;
}

Domain_Repository::Meta_Equipment::Value_Description* 
  Meta_Equipment_I::value_descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate my_descr
	Domain_Repository::Meta_Equipment::Value_Description* my_value_descr_duplicated =
		new Domain_Repository::Meta_Equipment::Description(my_descr);

	return my_value_descr_duplicated;
}
 
Domain_Repository::Repository_Item_Id 
   Meta_Equipment_I::item_id (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	// no lock here: item_id is immutable

	return my_descr.item_id;
}

char* 
   Meta_Equipment_I::type (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.type);
}

char* 
   Meta_Equipment_I::vendor (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.vendor);
}

char* 
   Meta_Equipment_I::model (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	ABng_Guarded::auto_lock local_lock(my_guard);
	
	return CORBA::string_dup(my_descr.model);
}

Domain_Repository::Meta_Equipment::Dimensions 
   Meta_Equipment_I::my_dimensions (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return my_descr.my_dimensions;
}

Domain_Repository::Meta_Equipment::Control_Info* 
   Meta_Equipment_I::my_control_info (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	Domain_Repository::Meta_Equipment::Control_Info* my_control_info = 
		new Domain_Repository::Meta_Equipment::Control_Info(my_descr.my_control_info);

	return my_control_info;
}

Domain_Repository::Property_Descriptions*
    Meta_Equipment_I::specific_property_description_list ( 
	ENV_DEF1(env)
    ) 
    throw (CORBA::SystemException)
{

	Domain_Repository::Property_Descriptions* specific_property_description_list = 
		new Domain_Repository::Property_Descriptions(
		my_descr.specific_property_description_list);

	return specific_property_description_list;
}

Domain_Repository::Meta_Equipment::Image* 
   Meta_Equipment_I::my_icon (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	Domain_Repository::Meta_Equipment::Image* my_icon = 
		new Domain_Repository::Meta_Equipment::Image(my_descr.my_icon);

	return my_icon;
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________

void
   Meta_Equipment_I::save (
	ostream &iostr
   )
{
	iostr << my_descr.item_id  << ' ';
	iostr << my_descr.type     << ' ';
	iostr << my_descr.vendor   << ' ';
	iostr << my_descr.model;

	// store dimension
	iostr << " <";
	iostr << my_descr.my_dimensions.height << " ";
	iostr << my_descr.my_dimensions.width  << " ";
	iostr << my_descr.my_dimensions.depth;
	iostr << ">";

	// store control info
	iostr << " <";
	if (my_descr.my_control_info._d() == Domain_Repository::Meta_Equipment::INFRA_RED) {
		iostr << "INFRA_RED";

		int len = my_descr.my_control_info.my_IR_control_data().commands.length();
		iostr << " < ";
		for(int i = 0; i < len; i++) 
			iostr << my_descr.my_control_info.my_IR_control_data().commands[i] << " ";
		iostr << ">";

		iostr << " "<< my_descr.my_control_info.my_IR_control_data().my_IR_data;
	}
	else
	if (my_descr.my_control_info._d() == Domain_Repository::Meta_Equipment::ANYTHING_ELSE) {
		iostr << "ANYTHING_ELSE";
		iostr << " " << my_descr.my_control_info.default_field();
	}
	iostr << ">";

	// store specific property description list
        iostr << " < ";
        iostr << my_descr.specific_property_description_list.maximum() << " ";
        iostr << my_descr.specific_property_description_list.length();

	for(int j = 0; j < my_descr.specific_property_description_list.length(); j++) {
	  iostr << " <" << my_descr.specific_property_description_list[j].name << " ";
#ifdef ORBIX
          iostr << (const char *) (my_descr.specific_property_description_list[j].type);
#elif defined OMNIORB
	  serialize_TypeCode( iostr,  my_descr.specific_property_description_list[j].type);
#endif
	  iostr << " >";
        }
        iostr << ">" << endl;

	// store icon
	iostr << " " << my_descr.my_icon;

	// end of this entity
	iostr << endl;
}

void
   Meta_Equipment_I::restore (
	istream &iostr
   )
{
	char c;

	iostr >> my_descr.item_id;
	my_descr.type 		= ABng_String::read_string(iostr);
	my_descr.vendor 	= ABng_String::read_string(iostr);
	my_descr.model 		= ABng_String::read_string(iostr);

	// restore dimension
	do { iostr >> c; } while (c != '<');
	if (!iostr) return;

	iostr >> my_descr.my_dimensions.height;
	iostr >> my_descr.my_dimensions.width;
	iostr >> my_descr.my_dimensions.depth;

	do { iostr >> c; } while (c != '>');
	if (!iostr) return;

	// restore control info
	do {iostr >> c; } while (c != '<');
	if (!iostr) return;

	char val[100];
	iostr >> val;
	if (!iostr) return;
	
	if (!strcmp(val, "INFRA_RED")) {
		Domain_Repository::IR_Control_Data *_ircd = new Domain_Repository::IR_Control_Data;
		RWTValDlist<CORBA::String_var> seq_list;

		do {iostr >> c; } while (c != '<');
		if (!iostr) return;
		do {
			iostr >> val;
			if (iostr && strcmp(val, ">")) seq_list.append(ABng_String::copy_string(val));
		} while (iostr && strcmp(val, ">"));
		if (!iostr) return;

		int size = seq_list.entries();
		_ircd->commands = Domain_Repository::Command_List(size);
		_ircd->commands.length(size);
		for (int i=0; i< size; i++)
			_ircd->commands[i]= seq_list[i];

		iostr >> _ircd->my_IR_data;
		if (!iostr) return;
		my_descr.my_control_info.my_IR_control_data(*_ircd);
	}
	else if (!strcmp(val, "ANYTHING_ELSE")) {
		int _df;
		iostr >> _df;
		if (!iostr) return;

		my_descr.my_control_info.default_field(_df);
	} 
	else {
		cerr << endl << "MetaEquipment restore error: unknown kind of control data" << endl;
		// simulate error
		iostr.clear(iostream::failbit|iostr.rdstate());
		return;
	}

	do { iostr >> c; } while (c != '>');
	if (!iostr) return;

	// restore specific property description list
	do { iostr >> c; } while (c != '<');
	if (!iostr) return;

	int pl_max, pl_len;
	iostr >> pl_max >> pl_len;
	if (!iostr || pl_max < pl_len) 
		return;

	my_descr.specific_property_description_list =  
		Domain_Repository::Property_Descriptions(pl_max);
	my_descr.specific_property_description_list.length(pl_len);

	for (int k=0; k < pl_len && iostr; k++) {
		do { iostr >> c; } while (c != '<');
		if (!iostr) return;

		iostr >> val;
		if (iostr && strcmp(val, ">"))
			my_descr.specific_property_description_list[k].name =
				ABng_String::copy_string(val);
		else {
			// simulate error
			iostr.clear(iostream::failbit|iostr.rdstate());
			return;
		}
#ifdef ORBIX
		iostr >> val;
		if (iostr && strcmp(val, ">")) {
			my_descr.specific_property_description_list[k].type =
				new CORBA::TypeCode(val);
			cerr << val << " "
			     << (int) my_descr.specific_property_description_list[k].type->kind() << endl;
		} else {
			// simulate error
			iostr.clear(iostream::failbit|iostr.rdstate());
			return;
		}
#elif defined OMNIORB
	        CORBA::TypeCode_ptr tmp_tc;
		deserialize_TypeCode( iostr, tmp_tc);
		my_descr.specific_property_description_list[k].type= tmp_tc;
#endif

		do { iostr >> c; } while (c != '>');
		if (!iostr) return;
	}

	if (!iostr || k != pl_len) {
		// simulate error
		iostr.clear(iostream::failbit|iostr.rdstate());
		return;
	}

	do { iostr >> c; } while (c != '>');
	if (!iostr) return;

	// restore icon
	iostr >> my_descr.my_icon;
}
#endif /* FilePersistency */

