//
//  File:         meta_equipment_rep_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 20.04.98
//
// General Description:
//   Implementation file for the Meta_Equipment_Rep_I class
//

#define EXCEPTIONS

#include "meta_equipment_rep_i.h"
#include "Naming.hh"
#include "db_utils.hh"

#include <rw/cstring.h>

//
// 		Constructor
//              ___________

   Meta_Equipment_Rep_I::Meta_Equipment_Rep_I (
#ifdef ObjectStore
    	os_database* db_p
#elif defined FilePersistency
    	const char *repository_name_p
#endif
    ) : Basic_Rep_I<ME_Types, ME_SDO_Types>(hash_meta_equipment,
#ifdef ObjectStore
    		db_p
#elif defined FilePersistency
    		repository_name_p
#endif
        ) 
{
  	_instance = this;
  	create_corba_objects();
}

void 
   Meta_Equipment_Rep_I::init (
	int cold_start_p
   ) 
{
	Basic_Rep_I<ME_Types, ME_SDO_Types>::init(cold_start_p);

	try {
		Bootstrap::register_server_in_NS(
			Bootstrap::SRV_Meta_Equipment_Repository_Manager(), 
			my_rep_manager);
		Bootstrap::register_server_in_NS(
			Bootstrap::SRV_Meta_Equipment_Repository(), my_rep);
    		register_callback(Bootstrap::SRV_Equipment_Repository());
		ps_dev= new ProcessControlImpl( "meta_equipment_repository", this);
	}
	catch (CORBA::Exception &ex) {
		cerr << "Caught exception while contacting server manager" << ex << endl;
		exit( 1);
	}
}

void 
   Meta_Equipment_Rep_I::init_corba_object ( 
	Meta_Equipment_I *item_p
   ) 
{
	 // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
#ifdef ObjectStore
	ObjectStoreTIE_Domain_Repository_Meta_Equipment(Meta_Equipment_I)* new_meta_equipment_tie =
		new(my_db, ObjectStoreTIE_Domain_Repository_Meta_Equipment(Meta_Equipment_I)::get_os_typespec())
		ObjectStoreTIE_Domain_Repository_Meta_Equipment(Meta_Equipment_I) (item_p);
#else
	TIE_Domain_Repository_Meta_Equipment( Meta_Equipment_I)* new_meta_equipment_tie = 
		new TIE_Domain_Repository_Meta_Equipment(Meta_Equipment_I) (item_p);
#endif
#elif defined OMNIORB
  Domain_Repository::Meta_Equipment_ptr
    new_meta_equipment_tie = 
    new TIE_CLASS( Domain_Repository,Meta_Equipment)<Meta_Equipment_I,0>(item_p);  
  CONNECT_IMPL( Domain_Repository::Meta_Equipment_var, new_meta_equipment_tie);
#endif
  item_p->set_corba_object(new_meta_equipment_tie);
}

void 
   Meta_Equipment_Rep_I::create_corba_objects (
   ) 
{
#ifdef ORBIX
#ifdef FilePersistency
  my_rep_manager = new TIE_Domain_Repository_Management_Meta_Equipment_Rep_Manager(Meta_Equipment_Rep_I) (this);
  // create a TIE for the object
  my_rep = new TIE_Domain_Repository_Meta_Equipment_Rep(Meta_Equipment_Rep_I) (this);
#endif /* FilePersistency */
#elif defined OMNIORB
  my_rep_manager =
    new TIE_CLASS(Domain_Repository_Management,Meta_Equipment_Rep_Manager)<Meta_Equipment_Rep_I,0>(this);
  CONNECT_IMPL( Domain_Repository_Management::Meta_Equipment_Rep_Manager_var,my_rep_manager);
    
  my_rep =
    new TIE_CLASS(Domain_Repository,Meta_Equipment_Rep)<Meta_Equipment_Rep_I,0>(this);
  CONNECT_IMPL( Domain_Repository::Meta_Equipment_Rep_var,my_rep);
    
#endif
  write_ior(my_rep_manager, "./Meta_Equipment_Rep_iiop.ref");
}

Meta_Equipment_Rep_I* Meta_Equipment_Rep_I::_instance = 0;

Meta_Equipment_Rep_I* 
   Meta_Equipment_Rep_I::instance(
   )
{
   	return _instance;
}

//
// 		Destructor
//		__________

   Meta_Equipment_Rep_I::~Meta_Equipment_Rep_I ()
{
  	destroy_corba_objects();
	delete ps_dev;
}

void 
   Meta_Equipment_Rep_I::destroy_corba_objects (
   ) 
{ 
	CORBA::release(my_rep_manager);
	CORBA::release(my_rep);
}

//
// Private functions
//

void 
   Meta_Equipment_Rep_I::reference_resolved ( 
	const ABng_Server_Manager::Name &name,
	CORBA::Object_var server_ref,
	ABng_Server_Manager::uid callback_id
   ) 
{
	auto_lock local_lock(*this);

	if ( name == Bootstrap::SRV_Equipment_Repository()) {
		try {
			r_equipment_rep = Domain_Repository::Equipment_Rep::_narrow(server_ref);
		}
		catch (...) {}
	}
}

int
   Meta_Equipment_Rep_I::remove_permitted (
	Meta_Equipment_I * meta_equipment_i_p
   ) 
{
	Domain_Repository::Equipments* equipments;
	Domain_Repository::Equipment::Pattern pattern;

	pattern.is_any = 0;

	pattern.item_id_pattern.is_any = 1;

	pattern.meta_data_pattern.is_any = 0;

	pattern.meta_data_pattern.item_id_pattern.is_any = 0;
	pattern.meta_data_pattern.item_id_pattern.value = meta_equipment_i_p->get_descr().item_id;

	pattern.meta_data_pattern.type_pattern.is_any = 1;
	pattern.meta_data_pattern.vendor_pattern.is_any = 1;
	pattern.meta_data_pattern.model_pattern.is_any = 1;
	pattern.meta_data_pattern.my_dimensions_pattern.is_any = 1;
	pattern.meta_data_pattern.my_control_info_pattern.is_any = 1;
	pattern.meta_data_pattern.specific_property_description_list_pattern.is_any = 1;
	pattern.meta_data_pattern.my_icon_pattern.is_any = 1;

	pattern.name_pattern.is_any = 1;

	pattern.last_command_pattern.is_any = 1;

	pattern.property_list_pattern.is_any = 1;

	if (CORBA::is_nil((Domain_Repository::Equipment_Rep_ptr)r_equipment_rep))
		return 1;

	try {
		equipments = r_equipment_rep->find(pattern);
	} catch (CORBA::COMM_FAILURE &ex) {
		r_equipment_rep = Domain_Repository::Equipment_Rep::_nil();
		return 1;
	} catch(...) {
		return 1;
	}

	int result = 1;
        
	if (equipments->length()) 
		result = 0;
	delete equipments;
     
	return result;
}

DB_Header* 
   Meta_Equipment_Rep_I::create_db_header (
   )
{
	return new DB_Header("Meta_Equipment Repository", "1.0", "\
#       <data_record>::=  	<item_id> <type> <vendor> <model>\n\
#				<dimensions> <control_info> <specific_properties>\n\
#				<icon>\n\
#\n\
#       <type>::=		<string>\n\
#	<vendor>::=		<string>\n\
#	<model>::=		<string>\n\
#	<dimensions>::=		\"<\" <height> <width> <depth> \">\"\n\
#	<control_info>::=	\"<\" <control_info_buf> \">\"\n\
#       <specific_properties>::=\"<\" <seq_maximum> <seq_length> <prop_descr_buffer> \">\"\n\
#	<icon>::=		<sequence_of_octets>\n\
#\n\
#	<height>:=		<integer>\n\
#	<width>:=		<integer>\n\
#	<depth>:=		<integer>\n\
#\n\
#	<control_info_buf>::=	|\n\
#				<IR_control_data> <default>\n\
#	<IR_control_data>::=	\"INFRA_RED\" <commands> <IR_control_data>\n\
#	<default>::=		\"ANYTHING_ELSE\" <int>\n\
#	<commands>:=		<sequence_of_strings>\n\
#	<IR_control_data>:=	<sequence_of_octets>\n\
#\n\
#       <prop_descr_buffer>::=  |\n\
#				<property_descr> <prop_descr_buffer>\n\
#       <property_descr>::= 	\"<\" <property_name> <type_code> \">\"\n\
#       <property_name>::=	<string>\n\
#       <type_code>::=		<string>\n\
#\n\
#       <sequence_of_strings>::=\"< \" <strings> \" >\"\n\
#       <strings>::=		|\n\
#				<string> <string>\n\
#\n\
#       <sequence_of_octets>::= \"<\" <hex_digits> \">\"\n\
#       <hex_digits>::=        	|\n\
#				<hex_digit> <hex_digits>\n\
#\n\
");
}

unsigned 
   Meta_Equipment_Rep_I::hash_meta_equipment (
	const Meta_Equipment_I& meta_equipment_i_p
   )
{
	Domain_Repository::Meta_Equipment::Description meta_equipment_descr = 
		meta_equipment_i_p.get_descr();
	RWCString _type(meta_equipment_descr.type);
	RWCString _vendor(meta_equipment_descr.vendor);
	RWCString _model(meta_equipment_descr.model);

	return 256 * _type.hash()  + 128 * _vendor.hash() + _model.hash();
}

void
   Meta_Equipment_Rep_I::gather_meta_equipment (
	Meta_Equipment_I *current_meta_equipment,
	void* apply_data
   )
{
	// project apply_data (void*) to Meta_Equipments*
	Domain_Repository::Meta_Equipments *collected_meta_equipments =
		(Domain_Repository::Meta_Equipments*)apply_data;

	// increase the length of the container
	collected_meta_equipments->length(collected_meta_equipments->length() + 1);

	// put duplicated meta_equipment (CORBA object) into the container
	(*collected_meta_equipments)[collected_meta_equipments->length() - 1] =
		Domain_Repository::Meta_Equipment::_duplicate(current_meta_equipment->get_corba_object());
}

int 
   operator == (
	const Octets &first, 
	const Octets &second
   ) 
{
	if (first.length() != second.length()) 
		return 0;

	int size = first.length();

	for (int i=0; i<size; i++)
		if (first[i] != second[i]) 
			return 0;

	return 1;
}

void
   Meta_Equipment_Rep_I::match_meta_equipment (
	Meta_Equipment_I *current_meta_equipment,
	void* apply_data
   )
{
  const Domain_Repository::Meta_Equipment::Pattern& data_record_pattern = 
    ((struct MatchMeta_EquipmentApplyDataStruct*)apply_data)->_data_record_pattern;

  Domain_Repository::Meta_Equipment::Description current_meta_equipment_descr;

  // check general patten matching rule
  if (data_record_pattern.is_any){
    // any meta_equipment matches this pattern
    gather_meta_equipment(current_meta_equipment, 
			  ((struct MatchMeta_EquipmentApplyDataStruct*)apply_data)->_found_meta_equipments);
    return;
  }
  
  current_meta_equipment_descr = current_meta_equipment->get_descr();

  // check matching of item_id
  if (!data_record_pattern.item_id_pattern.is_any && 
      (data_record_pattern.item_id_pattern.value != current_meta_equipment_descr.item_id))
    // this meta_equipment item_id does not match pattern 
    return;
  
  // check matching of type
  if (!data_record_pattern.type_pattern.is_any)
    if (!ABng_String::match_string(data_record_pattern.type_pattern.value, 
				   current_meta_equipment_descr.type))
      return;
  
  // check matching of vendor
  if (!data_record_pattern.vendor_pattern.is_any)
    if (!ABng_String::match_string(data_record_pattern.vendor_pattern.value, 
				   current_meta_equipment_descr.vendor))
      return;

  // check matching of model
  if (!data_record_pattern.model_pattern.is_any)
    if (!ABng_String::match_string(data_record_pattern.model_pattern.value, 
				   current_meta_equipment_descr.model))
      return;

  // check matching of my_dimensions
  if (!data_record_pattern.my_dimensions_pattern.is_any) {
    if (!data_record_pattern.my_dimensions_pattern.height_pattern.is_any)
      if (data_record_pattern.my_dimensions_pattern.height_pattern.value != 
	  current_meta_equipment_descr.my_dimensions.height)
	return;

    if (!data_record_pattern.my_dimensions_pattern.width_pattern.is_any)
      if (data_record_pattern.my_dimensions_pattern.width_pattern.value != 
	  current_meta_equipment_descr.my_dimensions.width)
	return;

    if (!data_record_pattern.my_dimensions_pattern.depth_pattern.is_any)
      if (data_record_pattern.my_dimensions_pattern.depth_pattern.value != 
	  current_meta_equipment_descr.my_dimensions.depth)
	return;
  }

  // check matching of my_control_info
  if (!data_record_pattern.my_control_info_pattern.is_any) {
    Domain_Repository::Meta_Equipment::Control_Info_Pattern::IRCI_Pattern _data_pat =
      data_record_pattern.my_control_info_pattern.my_control_data_pattern;
    
    // check matching of type 
    if (_data_pat._d() != current_meta_equipment_descr.my_control_info._d())
      return;
    
    if (_data_pat._d() == Domain_Repository::Meta_Equipment::INFRA_RED) 
      if (!_data_pat.my_IRCD_pattern().is_any) {
	if(!_data_pat.my_IRCD_pattern().commands_pattern.is_any) {
	  // match sequence of string
	  Domain_Repository::Sequence_Pattern_Constraints spc=
	    _data_pat.my_IRCD_pattern().commands_pattern.constraints;
	  
	}
	if(!_data_pat.my_IRCD_pattern().my_IR_data_pattern.is_any) {
	  // match sequence of octets
	  if (!(_data_pat.my_IRCD_pattern().my_IR_data_pattern.value == 
		current_meta_equipment_descr.my_control_info.my_IR_control_data().my_IR_data))
	    return;
	}	
      }
      else
	if (!_data_pat.default_field_pattern().is_any) 
	  if (_data_pat.default_field_pattern().value != 
	      current_meta_equipment_descr.my_control_info.default_field())
	    return;    
  }
  // check matching of specific_property_description_list
  if (!data_record_pattern.specific_property_description_list_pattern.is_any){
    
  }


  // check matching of my_icon
  if (!data_record_pattern.my_icon_pattern.is_any)
    if (!(data_record_pattern.my_icon_pattern.value == current_meta_equipment_descr.my_icon))
      return;
  
  // this meta_equipment matches the pattern
  gather_meta_equipment(current_meta_equipment, 
			((struct MatchMeta_EquipmentApplyDataStruct*)apply_data)->_found_meta_equipments);
}

void 
   Meta_Equipment_Rep_I::construct_data_object (
	Domain_Repository::Static_Data_Observer::Static_Data_Object& data_object_p,
	Domain_Repository::Meta_Equipment_ptr item_p
   ) 
{
	// construct union to store reference of added entity
	data_object_p.meta_equipment_obj(Domain_Repository::Meta_Equipment::_duplicate(item_p));
}

void 
   Meta_Equipment_Rep_I::collect_values ( 
	Meta_Equipment_I *lt, 
	Domain_Repository::Meta_Equipment::Value_Description& ld
   ) 
{
	ld = lt->get_descr();
}

//
// Implementation of IDL operations of the System_Static_Data_Observed_R interface
//                                         _____________________________

 Domain_Repository::Static_Data_Observed_R::Observer_Entries*
   Meta_Equipment_Rep_I::static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{	
	return observer_list(USE_ENV1(env));
}

 Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   Meta_Equipment_Rep_I::add_static_data_observer_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return add_observer_list(USE_ENV1(env));
}
                        
void 
   Meta_Equipment_Rep_I::attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                      Domain_Repository::Unknown_Repository_Item_Ids)
{
	attach_observer(obs, observed_id_list USE_ENV(env));
}

void 
   Meta_Equipment_Rep_I::detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs, 
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids, 
                       Domain_Repository::Unknown_Observer,
		       Domain_Repository::Unmatched_Observer_Item_Ids)
{
	detach_observer(obs, observed_id_list USE_ENV(env));
}

void 
   Meta_Equipment_Rep_I::attach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
	attach_add_observer(obs USE_ENV(env));
}

void 
   Meta_Equipment_Rep_I::detach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer)
{
	detach_add_observer(obs USE_ENV(env));
}

// Suplementary operation

 Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   Meta_Equipment_Rep_I::static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException)
{
	return get_item_observers(item_id_p USE_ENV(env));
}

//
// Implementation of IDL operations of the Meta_Equipment_Rep interface
//                                         __________________

Domain_Repository::Meta_Equipment_Rep*
   Meta_Equipment_Rep_I::rep(
	ENV_DEF1(env)
   ) 
  	throw (CORBA::SystemException) 
{
    return Domain_Repository::Meta_Equipment_Rep::_duplicate(my_rep);

}

Domain_Repository::Meta_Equipments* 
   Meta_Equipment_Rep_I::meta_equipment_list (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException)
{
	return item_list(USE_ENV1(env));
}
                
 Domain_Repository::Meta_Equipment_ptr 
   Meta_Equipment_Rep_I::add (
	const Domain_Repository::Meta_Equipment::Description& data_record
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Duplicate_Data_Record)
{
	return Basic_Rep_I<ME_Types, ME_SDO_Types>::add(data_record USE_ENV(env));
}

void 
   Meta_Equipment_Rep_I::remove (
	Domain_Repository::Meta_Equipment_ptr object_to_remove 
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Object_Ref)
{
	Basic_Rep_I<ME_Types, ME_SDO_Types>::remove(object_to_remove USE_ENV(env));
}

Domain_Repository::Meta_Equipments* 
   Meta_Equipment_Rep_I::find (
	const Domain_Repository::Meta_Equipment::Pattern& data_record_pattern
	ENV_DEF(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
  	auto_lock local_lock(*this);

  	return private_find(data_record_pattern USE_ENV(env));
}

Domain_Repository::Meta_Equipments* 
   Meta_Equipment_Rep_I::private_find (
	const Domain_Repository::Meta_Equipment::Pattern& data_record_pattern
	ENV_DEF(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
	// create a structure consisting of a pointer to the container
	//  and a reference of the pattern
	MatchMeta_EquipmentApplyDataStruct apply_data = MatchMeta_EquipmentApplyDataStruct(
		(Domain_Repository::Meta_Equipments*)new 
			Domain_Repository::Meta_Equipments(my_item_list.entries()),
			data_record_pattern);

	// browse through hash table to collect meta_equipments
	my_item_list.apply(match_meta_equipment, (void*)&apply_data);

	// pointer to found_meta_equipments was stored inside apply_data thus return it
	return apply_data._found_meta_equipments;
}

 Domain_Repository::Meta_Equipment::Value_Descriptions* 
   Meta_Equipment_Rep_I::find_values (
	const Domain_Repository::Meta_Equipment::Pattern& data_record_pattern
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented)
{
	return Basic_Rep_I<ME_Types, ME_SDO_Types>::find_values(data_record_pattern USE_ENV(env));
}

void 
   Meta_Equipment_Rep_I::update (
	Domain_Repository::Meta_Equipment_ptr object_to_update, 
	const Domain_Repository::Meta_Equipment::Description& data_record_pattern
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
		       Domain_Repository::Unknown_Object_Ref,
		       Domain_Repository::Duplicate_Data_Record)
{
	Basic_Rep_I<ME_Types, ME_SDO_Types>::update(object_to_update, data_record_pattern USE_ENV(env));
}
