// -*- Mode: C++; -*-
//
//  File:         meta_equipment_rep_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 20.04.98
//
// General Description:
//   Header file for Meta_Equipment_Rep implementation class
//

#ifndef meta_equipment_rep_i_h
#define meta_equipment_rep_i_h

#include "meta_equipment_i.h"
#include "basic_rep_i.h"
#include "bootstrap_n.hh"
#include "process_status.hh"

class ME_SDO_Types {

public:
  	typedef Domain_Repository::Static_Data_Observed_R Observed;
  	typedef Domain_Repository::Static_Data_Observer Observer;
  	typedef Domain_Repository::Static_Data_Observer_ptr Observer_ptr;
  	typedef Domain_Repository::Static_Data_Observed_E::Static_Data_Observers 
		Observers;
};

class ME_Types {

public:
	typedef Domain_Repository::Meta_Equipment Item;  
	typedef Domain_Repository::Meta_Equipment_ptr Item_ptr;  
  	typedef Domain_Repository::Meta_Equipments Items;  
  	typedef Meta_Equipment_I Item_I;  
  	typedef Domain_Repository::Meta_Equipment::Description Initial_Data;
  	typedef Domain_Repository::Meta_Equipment::Description Description;
  	typedef Domain_Repository::Static_Data_Observer::
		Static_Data_Object Data_Object;
};

class Meta_Equipment_Rep_I : 
                   public Bootstrap_listener,
                   public Basic_Rep_I<ME_Types, ME_SDO_Types>,
		   public ServerStatusInterface 
{
 public:
  // ctor:
  Meta_Equipment_Rep_I (
#ifdef ObjectStore
	os_database* db
#elif defined FilePersistency
    	const char *repository_name_p
#endif
  );

  virtual void 
   init (
	int cold_start_p = 0
   );

  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static Meta_Equipment_Rep_I* 
    instance ();

  // dctor:
  virtual 
     ~Meta_Equipment_Rep_I ();

 private:
  // pointer to an unique istance of the class
  static Meta_Equipment_Rep_I* _instance;

  // repository
  Domain_Repository_Management::Meta_Equipment_Rep_Manager_ptr my_rep_manager;
  Domain_Repository::Meta_Equipment_Rep_ptr my_rep;
  ProcessControlImpl *ps_dev;

  // references of servers this one is depended on
  Domain_Repository::Equipment_Rep_var r_equipment_rep;

  virtual void 
   init_corba_object (
	Meta_Equipment_I *item_p
   );

  virtual void 
   create_corba_objects (
   );

  virtual void 
   destroy_corba_objects (
   );

  void 
   reference_resolved (
	const ABng_Server_Manager::Name &name,
	CORBA::Object_var server_ref,
	ABng_Server_Manager::uid callback_id
   );

  int
   remove_permitted (
	Meta_Equipment_I * meta_equipment_i_p
   );

  virtual DB_Header*
   create_db_header (
   );

  // calculate the hash value of the meta_equipment
  static unsigned 
   hash_meta_equipment (
	const Meta_Equipment_I& meta_equipment_i_p
   ); 

  // put the current meta_equipment into the container
  static void
   gather_meta_equipment (
	Meta_Equipment_I* current_meta_equipment,
	void* apply_data
   );

  // definition of structure containing parameters of match_meta_equipment method
  struct MatchMeta_EquipmentApplyDataStruct {
    Domain_Repository::Meta_Equipments* _found_meta_equipments;
    const  Domain_Repository::Meta_Equipment::Pattern& _data_record_pattern;

    // its constructor
    MatchMeta_EquipmentApplyDataStruct (
	Domain_Repository::Meta_Equipments* _found_meta_equipments_p,
   	const Domain_Repository::Meta_Equipment::Pattern& _data_record_pattern_p
    ):
	 	_found_meta_equipments(_found_meta_equipments_p),
		_data_record_pattern(_data_record_pattern_p) {}
  };

  // if the current meta_equipment matches the pattern puts it into the container
  static void
   match_meta_equipment (
	Meta_Equipment_I* current_meta_equipment,
	void* apply_data
   );
 
  virtual void 
   construct_data_object (
    	Domain_Repository::Static_Data_Observer::Static_Data_Object& data_object_p,
    	Domain_Repository::Meta_Equipment_ptr item_p
   );

  virtual void 
   collect_values (
	Meta_Equipment_I *lt, 
    	Domain_Repository::Meta_Equipment::Value_Description& ld
   );

 public:
  //
  // IDL operations of the Static_Data_Observed_R interface
  //                       ______________________

  virtual Domain_Repository::Static_Data_Observed_R::Observer_Entries*
   static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
             
  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   add_static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
           
  virtual void 
   attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids);

  virtual void 
   detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs, 
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids, 
                       Domain_Repository::Unknown_Observer,
		       Domain_Repository::Unmatched_Observer_Item_Ids);
  
  virtual void 
   attach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs  
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_add_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer);

  // Suplementary operation

  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException);

  //
  // IDL operations of the Meta_Equipment_Rep interface
  //                       __________________

  virtual Domain_Repository::Meta_Equipment_Rep*
   rep (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Meta_Equipments* 
   meta_equipment_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual Domain_Repository::Meta_Equipment_ptr 
   add (
	const  Domain_Repository::Meta_Equipment::Description& data_record
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Duplicate_Data_Record);

  virtual void 
   remove (
	Domain_Repository::Meta_Equipment_ptr object_to_remove 
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Object_Ref);

  virtual Domain_Repository::Meta_Equipments* 
   find (
	const  Domain_Repository::Meta_Equipment::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

  // no locking here 
  Domain_Repository::Meta_Equipments* 
   private_find (
	const Domain_Repository::Meta_Equipment::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented); 
 
  virtual Domain_Repository::Meta_Equipment::Value_Descriptions* 
   find_values (
	const Domain_Repository::Meta_Equipment::Pattern& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

  virtual Domain_Repository::Meta_Equipment::Query_Results*
   complex_query(
		 const Domain_Repository::Meta_Equipment::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 Domain_Repository::Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<ME_Types, ME_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual void 
   update (
	Domain_Repository::Meta_Equipment_ptr object_to_update, 
	const Domain_Repository::Meta_Equipment::Description& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
		       Domain_Repository::Unknown_Object_Ref,
		       Domain_Repository::Duplicate_Data_Record);

#ifdef ObjectStore
  //
  // for ObjectStore
  //

  static os_typespec* get_os_typespec();
#endif
};

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_Meta_Equipment_Rep(Meta_Equipment_Rep_I)
DEF_ObjectStoreTIE_Domain_Repository_Management_Meta_Equipment_Rep_Manager(Meta_Equipment_Rep_I)
#else
DEF_TIE_Domain_Repository_Meta_Equipment_Rep(Meta_Equipment_Rep_I)
DEF_TIE_Domain_Repository_Management_Meta_Equipment_Rep_Manager(Meta_Equipment_Rep_I)
#endif
#endif

#endif /* meta_equipment_rep_i_h */
