// $Id: sched_barrier.hh,v 1.4 1998/07/29 13:51:12 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_barrier.hh,v $
//
// $Log: sched_barrier.hh,v $
// Revision 1.4  1998/07/29 13:51:12  mencnar
// Barrier uses not-reenrant locks
//
// Revision 1.3  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.2  1997/11/03 09:37:06  mencnar
// Bug fix:  Sched_Location_List now passed only by reference
//
// Revision 1.1  1997/10/23 09:18:39  mencnar
// Initial revision
//
//


#ifndef __sched_barrier_hh__
#define __shced_barrier_hh__


//------------------------------------------------------------Sched_Barrier
//
class Sched_Barrier : public ABng_S_Guarded {
  RWTPtrDlist<Sched_Request> sighting_queue;
  Sched_Barrier_Id last_sighting_pos;
  ABng_Core::Time last_sighting_time;
  Sched_Reply *my_reply;
  int cancel_reply;
  int request_in_progress;
  Seq_Number my_seq_number;
  
  void begin_request( Sched_Request &);

public:
  static ABng_Core::Time sighting_timeout;

  Sched_Barrier_Id barrier_id;
  omni_condition cond_reply_arrived;

  Sched_Location_List &location_list;

  Sched_Barrier(Sched_Location_List &loc_list,
		const Sched_Barrier_Id bar_id) :
	cond_reply_arrived( &mutex()),
	barrier_id( bar_id),
	my_reply( NULL), cancel_reply( 0),
	location_list( loc_list), request_in_progress(0),
        my_seq_number( 0) {
    unsigned long sec,nsec;
    omni_thread::get_time( 	&sec, &nsec);
    my_seq_number= (Seq_Number) nsec;
    last_sighting_time.seconds= last_sighting_time.microseconds= 0;
    last_sighting_pos= barrier_id;
  }
  
  void end_request();

  void insert_request( Sched_Request &req);
  void remove_request( Sched_Request &reg);

  void wait_till(
	ABng_Core::Time &till,
	Sched_Request &req);

  int wait_for_sighting(
	ABng_Core::Time &timeout,
	Sched_Request &req);

  int wait_for_reply(
	ABng_Core::Time &timeout,
	Sched_Request &req);

  void sighting_arrived( const Sched_Barrier_Id &new_location);
  void reply_arrived( Sched_Reply *reply);
  Seq_Number alloc_seq_number() { 
    auto_lock local_lock( *this);
    return ++my_seq_number; 
  };
  ~Sched_Barrier();
};


#endif  

// _sched_barrier_hh_
