// $Id: sched_main.hh,v 1.5 1998/07/29 13:49:40 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/scheduler/sched_main.hh,v $
//
// $Log: sched_main.hh,v $
// Revision 1.5  1998/07/29 13:49:40  mencnar
// "Server_Status" implementation
//
// Revision 1.4  1998/06/18 10:21:56  mencnar
// moved to CVS
//
// Revision 1.3  1997/10/31 15:19:43  mencnar
// Fixes in IDLs
// Memory leaks
//
// Revision 1.2  1997/10/24 08:03:37  mencnar
// User identification using Orbix principal
//
// Revision 1.1  1997/10/23 09:17:10  mencnar
// Initial revision
//
//

#ifndef __shed_main_hh__
#define __shed_main_hh__

#define EXCEPTIONS
#include <rw/tphdict.h>
#include <rw/tpdlist.h>
#include <rw/tvdlist.h>
#include <rw/cstring.h>
#include <errno.h>	// ETIMEDOUT
#include "abng_core_management.hh"
#include "abng_poller.hh"
#include "omnithread.h"
#include "bootstrap_n.hh"
#include "abng_guarded.hh"
#include "process_status.hh"

typedef ABng_Poller::AB_Poller::Badge_Id Badge_Id;
typedef ABng_Poller::AB_Poller::Network_Id Network_Id;
typedef ABng_Poller::AB_Poller::Station_Id Station_Id;
typedef ABng_Poller::AB_Poller::Time_Stamp Time_Stamp;
typedef ABng_Poller::AB_Poller::Tones Tones;
typedef ABng_Poller::AB_Poller::Seq_Number Seq_Number;
typedef ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests 
             Scheduler_Requests;
typedef ABng_Core_System::Scheduler_Request_Queue::Scheduler_Request 
             Scheduler_Request;
typedef ABng_Core_System::Scheduler_Caller_var Scheduler_Caller_var;


class Sched_Request;
class Sched_Location;
class Sched_Location_List;
class Sched_Complex_Request;
class Sched_Barrier;
class Sched_Barrier_Id;
class SchedulerImpl;

int operator == ( const Badge_Id &first,
		  const Badge_Id &second);
int operator == ( const ABng_Core_System::Sensor_Id &first,
		  const ABng_Core_System::Sensor_Id &second);

ABng_Core::Time Time_Stamp2Time( const Time_Stamp &);
 

#include "sched_barrier_id.hh"
#include "sched_reply.hh"
#include "sched_barrier.hh"
#include "sched_request.hh"
#include "sched_repos.hh"


 

//-------------------------------------------------------Sched_Location_List
//
class Sched_Location_List : ABng_Guarded {
  RWTPtrHashDictionary< Sched_Barrier_Id, Sched_Barrier> barrierid2barrier;

  Sched_Barrier & _get_barrier( const Sched_Barrier_Id &bar_id);
public:
  Sched_Location_List() : 	barrierid2barrier( Sched_Barrier_Id::hash) {}
  ~Sched_Location_List();
  
  Sched_Barrier & get_barrier( const Sched_Barrier_Id &bar_id);
  void update_location_list( const Sched_Barrier_Id &); 

  void handle_reply( Sched_Reply *reply);       

  static void Sched_Location_List::destroy_element( Sched_Barrier_Id *,
					   Sched_Barrier * &bar,
					   void *);
};

//------------------------------------------------------Sched_Request_Queue
class Sched_Request_Queue : public ABng_Guarded,
                            public RWTPtrDlist< Sched_Complex_Request> {
  static void static_worker( void *arg);

  struct Clean_Params {
    Clean_Params( Sched_Request_Queue &_queue, CORBA::Long _id) :
      queue( _queue), request_id( _id) {}
    Sched_Request_Queue &queue;
    CORBA::Long request_id;
  };

public:
  ~Sched_Request_Queue();    
  Scheduler_Requests* list_user_requests( RWCString user_name);
  Scheduler_Requests* list_all_requests();
  void remove_request( CORBA::Long request_id);
  void clean( Sched_Complex_Request &);
};
 

class Poller_Item {
public:
  RWCString name;
  ABng_Poller::AB_Poller_var poller_ref;
			  
  Poller_Item( RWCString _name) :
    name( _name) {}

  int operator == ( const Poller_Item &pit) const {
    return name == pit.name;
  }
};


//------------------------------------------------------------SchedulerImpl
//
class SchedulerImpl : public BOA_IMPL_BASE(ABng_Core_System,Scheduler),
                      private ABng_Guarded,
                      public Bootstrap_listener,
		      public ServerStatusInterface {
  CORBA::Long my_req_id;

			  
  RWTPtrDlist< Poller_Item> pollers;
  ABng_Poller::AB_Observer_ptr poller_observer_obj;
  ProcessControlImpl *ps_dev;
  
public:
  Sched_SB_Repositories sb_repository; 
			// sensor/badge repository

  Sched_Request_Queue request_queue;
  Sched_Location_List location_rep;

  SchedulerImpl( const char *obj_name);

  static void detach( Poller_Item *pit, void *);
  ~SchedulerImpl();

  ABng_Poller::AB_Poller_var poller( const Sched_Barrier_Id &loc);
  
  ABng_Core_System::Badge_Ids get_all_badge_ids();
  ABng_Core_System::Sensor_Ids get_all_sensor_ids();

  CORBA::Long create_request_id();

  MY_PRINCIPAL_VAR get_principal( ENV_DECL1(env));

  // Bootstrap_listener interface method
  void reference_resolved( const ABng_Server_Manager::Name &server_name, 
  			   CORBA::Object_var server_ref,
			   ABng_Server_Manager::uid callback_id);

  virtual void receive (
	const char * network_group_name, 
	const ABng_Poller::AB_Poller::Message& msg
	ENV_DECL(env));

  virtual void sighting (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	ABng_Poller::AB_Poller::Sighting_Code code, 
	const ABng_Poller::AB_Poller::Field_Id& field, 
	ABng_Poller::AB_Poller::Byte click_num,
	ABng_Poller::AB_Poller::Byte attributes, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DECL(env)); 

  virtual void challenge_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	const ABng_Poller::AB_Poller::Octets& random, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void sound_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	const Tones& sound, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void set_state_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	ABng_Poller::AB_Poller::Byte status, 
	const Tones& sound, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void get_state_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	ABng_Poller::AB_Poller::Byte major_version, 
	ABng_Poller::AB_Poller::Byte minor_version, 
	ABng_Poller::AB_Poller::Byte status, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void set_home_reply (
	const char * network_group_name,
	Network_Id network,
	Station_Id station,
	const Badge_Id& badge,
	const ABng_Poller::AB_Poller::Address& address,
	Seq_Number seq,
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void get_home_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Badge_Id& badge, 
	const ABng_Poller::AB_Poller::Address& address, 
	Seq_Number seq, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void get_station_state_reply (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	ABng_Poller::AB_Poller::Byte major_version, 
	ABng_Poller::AB_Poller::Byte minor_version, 
	ABng_Poller::AB_Poller::Byte status1, 
	ABng_Poller::AB_Poller::Byte status2, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual void null_message (
	const char * network_group_name, 
	Network_Id network, 
	Station_Id station, 
	const Time_Stamp& ts
	ENV_DECL(env));

  virtual Scheduler_Requests*
    list_user_requests (
	const char * user_ids
	ENV_DECL(env));
 
  virtual Scheduler_Requests* 
     list_all_requests ( ENV_DECL1(env));

  virtual void remove_request (
	CORBA::Long request_id
	ENV_DECL(env));

  virtual CORBA::Long issue_sensor_command (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Octets& cmd, 
	CORBA::Boolean has_reply,
	CORBA::Octet reply_code,
	const ABng_Core::Command_Context& ctx, 	
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long issue_badge_command (
	const ABng_Core_System::Sensor_Ids& sensors,
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& cmd, 
	CORBA::Boolean has_reply,
	CORBA::Octet reply_code,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long challenge (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& random, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long sound (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long set_state (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	CORBA::Octet status, 
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long get_state (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long set_home (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core_System::Octets& address, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long get_home (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core_System::Badge_Ids& badges, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long reset_buffer (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long reset_fifos (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long get_station_state (
	const ABng_Core_System::Sensor_Ids& sensors, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long set_output (
	const ABng_Core_System::Sensor_Ids& sensors, 
	CORBA::Octet output_value, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DECL(env));

  virtual CORBA::Long set_persistence (
	const ABng_Core_System::Sensor_Ids& sensors,
	CORBA::Boolean mode, 
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
        ENV_DECL(env));


};



#endif		// __shed_main_hh__

//
