//
//  File:         sensor_i.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 04.10.97
//
// General Description:
//   Implementation file for the Sensor_I class
//

#define EXCEPTIONS
#include "sensor_i.h"
#include "db_utils.hh"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

//
// Implementation of constructor, destructor and operators
//                   _____________________________________

  Sensor_I::Sensor_I (
	const ABng_Core_System::Sensor::Description& descr_init
  ) :
	my_guard(*(Sensor_Rep_I::instance()))
{
	int len = strlen(descr_init.id.group_name) + 1;

      	// init my_descr
	my_descr.item_id = descr_init.item_id;
	my_descr.id.sensor_num = descr_init.id.sensor_num;
	my_descr.id.network_num = descr_init.id.network_num;
	my_descr.id.group_name = ABng_String::copy_string(descr_init.id.group_name);

	// init my_corba_object
	my_corba_object = (ABng_Core_System::Sensor_ptr)NULL;
}

#ifdef FilePersistency
   Sensor_I::Sensor_I (
	istream &iostr
   ) :
	my_guard(*(Sensor_Rep_I::instance()))
{
	// read description from the file
	restore(iostr);

	// init my_corba_object
	my_corba_object = (ABng_Core_System::Sensor_ptr)NULL;
}
#endif

  Sensor_I::~Sensor_I (
  )
{
	ABng_String::free_string(my_descr.id.group_name);
}

  Sensor_I::operator== (
	const Sensor_I& sensor_p
  )
{
	if (my_descr.id.sensor_num != sensor_p.my_descr.id.sensor_num)
		return 0;

	if (my_descr.id.network_num != sensor_p.my_descr.id.network_num)
		return 0;

	if (strcmp(my_descr.id.group_name, sensor_p.my_descr.id.group_name))
		return 0;

	return 1;
}

//
// Implementation of methods enabling access to the state
//

void
  Sensor_I::set_descr (
	const ABng_Core_System::Sensor::Description& descr_update
  )
{
	int len = strlen(descr_update.id.group_name) + 1;
       
	// update my_descr
	/// NO !!!! my_descr.item_id = descr_update.item_id;
	my_descr.id.sensor_num = descr_update.id.sensor_num;
	my_descr.id.network_num = descr_update.id.network_num;
	//	ABng_String::free_string(my_descr.id.group_name);
	// never!!!, IDL structers handles memory deallocation
	my_descr.id.group_name = ABng_String::copy_string(descr_update.id.group_name);
}

const ABng_Core_System::Sensor::Description&
   Sensor_I::get_descr (
   ) const
{
	return my_descr;  
}

void
   Sensor_I::set_corba_object (
	const ABng_Core_System::Sensor_ptr corba_object_p
   )
{
	my_corba_object = corba_object_p;
}

const ABng_Core_System::Sensor_ptr
   Sensor_I::get_corba_object (
   ) 
{
	return my_corba_object;
}

void 
   Sensor_I::set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   )
{
	my_descr.item_id = item_id_p;
}

//
// Implementation of IDL operations of the System_Static_Data_Observed_E interface
//                                         _____________________________

ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   Sensor_I::system_static_data_observer_list (
	ENV_DEF1(env)
   )
		throw (CORBA::SystemException) 
{
	return Sensor_Rep_I::instance()->system_static_data_observer_list(
		my_descr.item_id USE_ENV(env));
}
                
void 
   Sensor_I::attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs  
	ENV_DEF(env)
   )
		throw (CORBA::SystemException)
{
  	// initialize the sequence for this sensor item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this sensor item id into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;
	

	// register observer into the repository list
	Sensor_Rep_I::instance()->attach_system_static_data_observer(obs,
		observed_id_list_p USE_ENV(env));
}

void 
   Sensor_I::detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs 
	ENV_DEF(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer) 
{
	// initialize the sequence for this sensor item id
	Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

	// put this sensor item it into the sequance
	observed_id_list_p.length(1);
	observed_id_list_p[0] = my_descr.item_id;

	// register observer into the repository list
     	try {
       		Sensor_Rep_I::instance()->detach_system_static_data_observer(obs,
			observed_id_list_p USE_ENV(env));
     	}
     	catch ( Domain_Repository::Unknown_Observer &ex) {
       		throw ex;
    	}
     	catch ( ...) {}
}

//
// Implementation of IDL operations of Scheduler_Request_Queue interface
//                                     _______________________

ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests*
   Sensor_I::list_user_requests (
	const char * user_id
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Scheduler_var sch =
		Sensor_Rep_I::instance()->scheduler();

	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch))
		return sch->list_user_requests(user_id);
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}
   
ABng_Core_System::Scheduler_Request_Queue::Scheduler_Requests* 
   Sensor_I::list_all_requests (
	ENV_DEF1(env)
   )
		throw (CORBA::SystemException,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Scheduler_var sch = 
		Sensor_Rep_I::instance()->scheduler();

	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch))
		return sch->list_all_requests();
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

void 
   Sensor_I::remove_request (
	CORBA::Long request_id
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException, 
                       ABng_Core::Unknown_Request_Id,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Scheduler_var sch =
		Sensor_Rep_I::instance()->scheduler();

	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch))
		sch->remove_request(request_id);
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

//
// Implementation of IDL operations of the Sensor_Commander interface
//                                         ________________

void 
   Sensor_I::prepare_scheduler_params ( 
	const ABng_Core_System::Badges& badges,
	ABng_Core_System::Badge_Ids &bids,
	ABng_Core_System::Sensor_Ids &sids
) 
      		throw (ABng_Core::Invalid_Argument) 
{
	sids.length(1);
	sids[0] = my_descr.id;

	bids.length(badges.length());
	for (int i=0; i < badges.length(); i++) 
		try {	
			bids[i] = *(badges[i]->id());
		}
		catch(...) {
			throw ABng_Core::Invalid_Argument();

		}
}

void 
   Sensor_I::map_exceptions(
	 CORBA::Exception &ex
   ) 
      		throw (ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
 
{
  	try {
    		throw ex;
  	}
    	catch (ABng_Core_System::Unknown_Sensor_Id &ex) {
      		throw ABng_Core::Invalid_Argument();
    	}
    	catch (ABng_Core_System::Invalid_Sensor_Id &ex) {
      		throw ABng_Core::Invalid_Argument();
    	}
    	catch (ABng_Core_System::Unknown_Badge_Id &ex) {
      		throw ABng_Core::Invalid_Argument();
    	}
    	catch (ABng_Core_System::Invalid_Badge_Id &ex) {
      		throw ABng_Core::Invalid_Argument();
    	}
    	catch (CORBA::SystemException &ex) {
      		throw Domain_Repository::Destination_Unreachable("scheduler");
    	}  
}

CORBA::Long 
   Sensor_I::issue_sensor_command (
	const ABng_Core_System::Octets& cmd,
	const CORBA::Boolean has_reply,
	const CORBA::Octet reply_code, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable) 
{
	ABng_Core_System::Sensor_Ids sids(1);
	sids.length(1);
	sids[0] = my_descr.id;

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->issue_sensor_command(sids, cmd, has_reply, 
				reply_code, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::issue_badge_command (
	const ABng_Core_System::Badges& badges, 
	const ABng_Core_System::Octets& cmd,
	const CORBA::Boolean has_reply,
	const CORBA::Octet reply_code, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->issue_badge_command(sids, bids, cmd, has_reply,
				 reply_code, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::challenge (
	const ABng_Core_System::Badges& badges,
	const ABng_Core_System::Octets& random, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DEF(env)
  ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable) 
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->challenge(sids, bids, random, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::sound (
	const ABng_Core_System::Badges& badges, 
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable) 
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->sound(sids, bids, sound, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::set_state (
	const ABng_Core_System::Badges& badges,
	CORBA::Octet status,
	const ABng_Core_System::Octets& sound, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->set_state(sids, bids, status, sound, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::get_state (
	const ABng_Core_System::Badges& badges, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->get_state(sids, bids, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::set_home (
	const ABng_Core_System::Badges& badges,
	const ABng_Core_System::Octets& address,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->set_home(sids, bids, address, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::get_home (
	const ABng_Core_System::Badges& badges,
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	ABng_Core_System::Badge_Ids bids(badges.length());
	prepare_scheduler_params(badges, bids, sids);

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->get_home(sids, bids, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::reset_buffer (
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	sids.length(1);
	sids[0] = my_descr.id;

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->reset_buffer(sids, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::reset_fifos (
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to 
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)
{
	ABng_Core_System::Sensor_Ids sids(1);
	sids.length(1);
	sids[0] = my_descr.id;

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->reset_fifos(sids, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::get_station_state (
	const ABng_Core::Command_Context& ctx, 
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable) 
{
	ABng_Core_System::Sensor_Ids sids(1);
	sids.length(1);
	sids[0] = my_descr.id;

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->get_station_state(sids, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::set_output (
	CORBA::Octet output_value, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable)

{
	ABng_Core_System::Sensor_Ids sids(1);
	sids.length(1);
	sids[0] = my_descr.id;

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->set_output(sids, output_value, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

CORBA::Long 
   Sensor_I::set_persistence (
	CORBA::Boolean mode, 
	const ABng_Core::Command_Context& ctx,
	ABng_Core_System::Scheduler_Caller_ptr reply_to
	ENV_DEF(env)
   ) 
		throw (CORBA::SystemException,
		       ABng_Core::Invalid_Request_Time,
		       ABng_Core::Invalid_Argument,
		       Domain_Repository::Destination_Unreachable) 
{
	ABng_Core_System::Sensor_Ids sids(1);
	sids.length(1);
	sids[0] = my_descr.id;

	ABng_Core_System::Scheduler_var sch = Sensor_Rep_I::instance()->scheduler();
	if (!CORBA::is_nil((ABng_Core_System::Scheduler_ptr)sch)) {
		try {
			return sch->set_persistence(sids, mode, ctx, reply_to);
		}
    		catch (CORBA::Exception &ex) {
      			map_exceptions(ex);
    		}
  	} 
	else 
		throw Domain_Repository::Destination_Unreachable("scheduler");
}

//
// Implementation of IDL operations of the Sensor interface
//                                         ______
 
ABng_Core_System::Sensor::Description* 
  Sensor_I::descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException) 
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate my_descr
	ABng_Core_System::Sensor::Description* my_descr_duplicated =
		new ABng_Core_System::Sensor::Description(my_descr);

	return my_descr_duplicated;
}
 
ABng_Core_System::Sensor::Value_Description* 
  Sensor_I::value_descr (
	ENV_DEF1(env)
  ) 
		throw (CORBA::SystemException) 
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate my_descr
	ABng_Core_System::Sensor::Value_Description* my_value_descr_duplicated =
		new ABng_Core_System::Sensor::Description(my_descr);

	return my_value_descr_duplicated;
}

Domain_Repository::Repository_Item_Id 
   Sensor_I::item_id (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException) 
{
	// no lock here: item_id is immutable

	return my_descr.item_id;
}

ABng_Core_System::Sensor_Id* 
   Sensor_I::id (
	ENV_DEF1(env)
   ) 
		throw (CORBA::SystemException) 
{
	ABng_Guarded::auto_lock local_lock(my_guard);

	// duplicate id
        ABng_Core_System::Sensor_Id* id_duplicated =
		new ABng_Core_System::Sensor_Id(my_descr.id);

	return id_duplicated;
}

#ifdef FilePersistency
//
//		Store and restore state methods
//		_________________

void
   Sensor_I::save (
	ostream &iostr
   )
{
	iostr << my_descr.item_id        << ' ';
	iostr << my_descr.id.sensor_num  << ' ';
	iostr << my_descr.id.network_num << ' ';
	save_string( iostr, my_descr.id.group_name);
	iostr << endl;
}

void
   Sensor_I::restore (
	istream &iostr
   )
{
	iostr >> my_descr.item_id;
	iostr >> my_descr.id.sensor_num;
	iostr >> my_descr.id.network_num;
	char *tmp= NULL;
	restore_string( iostr, tmp);
	my_descr.id.group_name = tmp;
}
#endif /* FilePersistency */


