// -*- Mode: C++; -*-
//
//  File:         sensor_rep_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 02.10.97
//
// General Description:
//   Header file for Sensor_Rep implementation class
//

#ifndef sensor_rep_i_h
#define sensor_rep_i_h

#include "sensor_i.h"
#include "basic_rep_i.h"
#include "bootstrap_n.hh"
#include "process_status.hh"

class S_SDO_Types {

public:
  	typedef ABng_Core_System::System_Static_Data_Observed_R Observed;
  	typedef ABng_Core_System::System_Static_Data_Observer Observer;
  	typedef ABng_Core_System::System_Static_Data_Observer_ptr Observer_ptr;
  	typedef ABng_Core_System::System_Static_Data_Observed_E::
    		System_Static_Data_Observers Observers;
};

class Sensor_Types {

public:
	typedef ABng_Core_System::Sensor Item;  
	typedef ABng_Core_System::Sensor_ptr Item_ptr;  
  	typedef ABng_Core_System::Sensors Items;  
  	typedef Sensor_I Item_I;  
  	typedef ABng_Core_System::Sensor::Description Initial_Data;
  	typedef ABng_Core_System::Sensor::Description Description;
  	typedef ABng_Core_System::System_Static_Data_Observer::
		System_Static_Data_Object Data_Object;
};

class Sensor_Rep_I : public Bootstrap_listener,
                     public Basic_Rep_I<Sensor_Types, S_SDO_Types>,
                     public ServerStatusInterface 
{
 public:
  // ctor:
  Sensor_Rep_I (
#ifdef ObjectStore
	os_database* db
#elif defined FilePersistency
    	const char* repository_name_p
#endif
  );

  virtual void 
    init (
	int cold_start_p = 0
    );

  // get instance of this class 
  //  - Singleton pattern (E. Gamm & al.,"Design Paterns", p. 127)
  static Sensor_Rep_I* 
    instance ();

  ABng_Core_System::Scheduler_var 
   scheduler (
   );

  // dctor:
  virtual 
     ~Sensor_Rep_I ();

 private:
  // pointer to an unique istance of the class
  static Sensor_Rep_I* _instance;

  // repository
  ABng_Core_Management::Sensor_Rep* my_rep;

  ProcessControlImpl *ps_dev;

  // references of servers this one is depended on
  ABng_Core_System::Scheduler_var r_scheduler;

  virtual void 
   init_corba_object (
	Sensor_I *item_p
   );

  virtual void 
   create_corba_objects (
   );

  virtual void 
   destroy_corba_objects (
   );

  void 
   reference_resolved (
	const ABng_Server_Manager::Name &name,
	CORBA::Object_var server_ref,
	ABng_Server_Manager::uid callback_id
   );

  virtual DB_Header*
   create_db_header (
   );

  // calculate the hash value of the sensor
  static unsigned 
   hash_sensor (
	const Sensor_I& sensor_i_p
   ); 
     
  // put the current sensor into the container
  static void
   gather_sensor (
	Sensor_I* current_sensor,
	void* apply_data
   );

  // definition of structure containing parameters of match_sensor method
  struct MatchSensorApplyDataStruct {
    ABng_Core_System::Sensors* _found_sensors;
    const ABng_Core_System::Sensor::Pattern& _data_record_pattern;

    // its constructor
    MatchSensorApplyDataStruct(
	ABng_Core_System::Sensors* _found_sensors_p,
   	const ABng_Core_System::Sensor::Pattern& _data_record_pattern_p):
	 	_found_sensors(_found_sensors_p),
		_data_record_pattern(_data_record_pattern_p) {}
  };

  // if the current sensor matches the pattern puts it into the container
  static void
   match_sensor (
	Sensor_I* current_sensor,
	void* apply_data
   );
 
  virtual void 
   construct_data_object (
    	ABng_Core_System::System_Static_Data_Observer::
    	System_Static_Data_Object& data_object_p,
    	ABng_Core_System::Sensor_ptr item_p
   );

  virtual void 
   collect_values (
	Sensor_I *lt, 
    	ABng_Core_System::Sensor::Value_Description& ld
   );

public:
  //
  // IDL operations of the System_Static_Data_Observed_R interface
  //                       _____________________________

  virtual ABng_Core_System::System_Static_Data_Observed_R::Observer_Entries*
   system_static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
             
  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   add_system_static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
           
  virtual void 
   attach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs,  
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids);

  virtual void 
   detach_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs, 
	const Domain_Repository::Repository_Item_Ids& observed_id_list
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Repository_Item_Ids, 
                       Domain_Repository::Unknown_Observer,
		       Domain_Repository::Unmatched_Observer_Item_Ids);
  
  virtual void 
   attach_add_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs  
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_add_system_static_data_observer (
	ABng_Core_System::System_Static_Data_Observer_ptr obs 
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer);

  // Suplementary operation

  virtual ABng_Core_System::System_Static_Data_Observed_E::System_Static_Data_Observers*
   system_static_data_observer_list (
	Domain_Repository::Repository_Item_Id item_id_p
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException);

  //
  // IDL operations of the Sensor_Rep interface
  //                       __________

  virtual ABng_Core_System::Sensors* 
   sensor_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual ABng_Core_System::Sensor_ptr 
   add (
	const ABng_Core_System::Sensor::Description& data_record
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Duplicate_Data_Record);

  virtual void 
   remove (
	ABng_Core_System::Sensor_ptr object_to_remove
	ENV_DECL(env)
   )
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Object_Ref);

  virtual ABng_Core_System::Sensors* 
   find (
	const ABng_Core_System::Sensor::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

  // no locking here 
  ABng_Core_System::Sensors* 
   private_find (
	const ABng_Core_System::Sensor::Pattern& data_record_pattern
	ENV_DECL(env)
   )
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);
  
  virtual ABng_Core_System::Sensor::Value_Descriptions* 
   find_values (
	const ABng_Core_System::Sensor::Pattern& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException,
                       Domain_Repository::Search_Not_Implemented);

  virtual ABng_Core_System::Sensor::Query_Results*
   complex_query(
		 const ABng_Core_System::Sensor::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core_System::System_Static_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return Basic_Rep_I<Sensor_Types, S_SDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual void 
   update (
	ABng_Core_System::Sensor_ptr object_to_update, 
	const ABng_Core_System::Sensor::Description& data_record_pattern
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
		       Domain_Repository::Unknown_Object_Ref,
		       Domain_Repository::Duplicate_Data_Record);
#ifdef ObjectStore
  //
  // for ObjectStore
  //

  static os_typespec* get_os_typespec();
#endif
};

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_ABng_Core_Management_Sensor_Rep(Sensor_Rep_I)
#else
DEF_TIE_ABng_Core_Management_Sensor_Rep(Sensor_Rep_I)
#endif

#elif defined OMNIORB
// DEF_TIE__tie_Sensor_Rep
#endif

#endif /* sensor_rep_i_h */
