//
// $Id: server_mgr.cc,v 1.4 1998/11/30 15:07:02 mencnar Exp $
//
// $Log: server_mgr.cc,v $
// Revision 1.4  1998/11/30 15:07:02  mencnar
// Added support for checking for status of garbage collector
// Removed workarounds for Orbix 2.2 (no longer needed)
//
// Revision 1.3  1998/07/29 14:04:27  mencnar
// "Server_Status" implementation
//
// Revision 1.2  1998/06/18 10:35:02  mencnar
// moved to CVS
//
// Revision 1.1  1998/03/06 13:01:02  mencnar
// Initial revision
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: Implementation of ABng_Server_Manager::Register interface
//              Keeps track of all running servers in ABng system
//
#ifdef ORBIX
#define USE_IIOP
#define EXCEPTIONS
#endif

#ifndef ORBIX
#define CORBA_ORBIX_VER 0
#endif

#include <iostream.h>
#include <fstream.h>
#include "server_mgr.hh"
#include "bootstrap_n.hh"
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <rw/cstring.h>

static CORBA::ORB_ptr the_orb;


struct GCState : public ABng_S_Guarded {
  int active;
  int check_callback;
  CORBA::String_var IOR;
  ABng_Server_Manager::Name name;

  GCState() : active(0) {}
} gcstate;

unsigned hash_server_name( const ABng_Server_Manager::Name &name) {
  if ( name.length() > 0) 
    return RWCString( ( const char *) ( name[ name.length() -1 ])).hash();
  else
    return 0;
}

	       

////////////////////////////////////////////////////////////////////Server_Entry//
//
Server_Entry::Server_Entry( ABng_Server_Manager::uid _item_id,
			    CORBA::Object_var &_server_reference,
			    const ABng_Server_Manager::Name &_name) :
  item_id( _item_id),
  server_reference( _server_reference),
  server_name( _name)
#if defined(ORBIX) && (CORBA_ORBIX_VER < 23)
  , stringified_server( CORBA::is_nil( _server_reference) ? (const char *) "" :
          the_orb->object_to_string( _server_reference,
				   CORBA::default_environment))
#endif
 {}

unsigned Server_Entry::hash( const Server_Entry &server_entry) {
  return hash_server_name( server_entry.server_name);
}

int Server_Entry::operator == ( const Server_Entry &server_entry) const {
  return ( server_name == server_entry.server_name);
}

int Server_Entry::operator >= ( const Server_Entry &server_entry) const {
  return ( server_name >= server_entry.server_name);
}

int Server_Entry::name_is_public( const ABng_Server_Manager::Name &server_name) {
  const char *la= server_name[ server_name.length()-1];
  return la[strlen(la)-1] != '@';  
}

int Server_Entry::is_public() {
  return name_is_public( server_name);
}

/////////////////////////////////////////////////////////////////Notifier_Entry//
//
unsigned Notifier_Entry::hash( const Notifier_Entry &notifier_entry) {
  return hash_server_name( notifier_entry.server_name);
}

int Notifier_Entry::operator == ( const Notifier_Entry &notifier_entry) const {
  return ( server_name == notifier_entry.server_name);
}

/////////////////////////////////////////////////////////////////Callback_Entry//
//
Callback_Entry::Callback_Entry( 
		     ABng_Server_Manager::uid _item_id,
		     ABng_Server_Manager::Callback_var &_callback_object,
		     const ABng_Server_Manager::Name &_server_name,
		     int _context_entry,
		     int _public_entry) :
  item_id( _item_id),
  callback_object( _callback_object),
  server_name( _server_name),
  context_entry( _context_entry),
  public_entry( _public_entry)
#if defined(ORBIX) && (CORBA_ORBIX_VER < 23)
  , stringified_callback( 
	the_orb->object_to_string( _callback_object,
				   CORBA::default_environment))
#endif
{}


unsigned Callback_Entry::hash( const Callback_Entry &callback_entry) {
  return callback_entry.item_id;
}

int Callback_Entry::operator == ( const Callback_Entry &callback_entry) const {
  return ( item_id == callback_entry.item_id &&
	   public_entry == callback_entry.public_entry);
}

ostream & operator << ( ostream &ostr,
			Callback_Entry &callback_entry) {

  ostr << callback_entry.item_id << ' ' << callback_entry.context_entry << ' '
       << callback_entry.server_name << ' ' << callback_entry.public_entry 
       << ' ';

#if defined(ORBIX) && (CORBA_ORBIX_VER < 23)
  return ostr << "\"" << callback_entry.stringified_callback << "\"" << endl;
#else
  char *stringified= the_orb->object_to_string( callback_entry.callback_object);
  ostr << "\"" << stringified << "\"" << endl;
  CORBA::string_free( stringified);
  return ostr;
#endif
}

istream & operator >> ( istream &istr,
			Callback_Entry &callback_entry) {
  istr >> callback_entry.item_id >> callback_entry.context_entry
       >> callback_entry.server_name >> callback_entry.public_entry;
  {
    CORBA::Object_var obj;
    RWCString xbuf;
    istr >> xbuf;
    RWCString buf= xbuf( 1, xbuf.length()-2);
 
    char *str= CORBA::string_dup( ( char *) ((const char *) buf));

    obj= the_orb->string_to_object( str);
    CORBA::string_free( str);
    callback_entry.callback_object= ABng_Server_Manager::Callback::_duplicate( 
			 ABng_Server_Manager::Callback::_narrow( obj));
#if defined(ORBIX) && (CORBA_ORBIX_VER < 23)
    callback_entry.stringified_callback= callback_entry.callback_object->_object_to_string( CORBA::IT_INTEROPERABLE_OR_KIND);
#endif
  }
  return istr;
}

/////////////////////////////////////////////////////////////////////Callback_DB//
//
//-------------------------------------------------------------gather_callbacks--
//
RWTValDlist< Callback_Entry>
  Callback_DB::gather_callbacks( ABng_Server_Manager::Name &name) {

  auto_lock local_lock( *this);
  return private_gather_callbacks( name);
}

RWTValDlist< Callback_Entry>
  Callback_DB::private_gather_callbacks( ABng_Server_Manager::Name &name) {

  RWTValDlist< Callback_Entry> ret;
  int public_name= Server_Entry::name_is_public( name);

  for ( RWTValHashTableIterator< Callback_Entry> iter( db);
	iter(); ) {
    if ( iter.key().context_entry) {
      if ( name >= iter.key().server_name &&
	   iter.key().public_entry <= public_name) ret.insert( iter.key());
    }
    else {
      if ( iter.key().server_name == name &&
	   iter.key().public_entry <= public_name) ret.insert( iter.key());
    }
  }
  return ret;
}

//---------------------------------------------------------gather_all_callbacks--
//
RWTValDlist< Callback_Entry>
  Callback_DB::gather_all_callbacks() {

  auto_lock local_lock( *this);

  RWTValDlist< Callback_Entry> ret;
  for ( RWTValHashTableIterator< Callback_Entry> iter( db);
	iter(); )
    ret.insert( iter.key());
  return ret;
}


//-----------------------------------------------------------gather_all_servers--
//
RWTValDlist< Server_Entry>
  Callback_DB::gather_all_servers() {

  auto_lock local_lock( *this);

  RWTValDlist< Server_Entry> ret;
  for ( RWTValHashTableIterator< Server_Entry> iter( server_db);
	iter(); )
    ret.insert( iter.key());
  return ret;
}

//---------------------------------------------------------------gather_servers--
//
RWTValDlist< Server_Entry> 
  Callback_DB::gather_servers( ABng_Server_Manager::Name &name,
			       int check_NS,
			       int public_callback) {

  auto_lock local_lock( *this);
  return private_gather_servers( name, check_NS, public_callback);
}

RWTValDlist< Server_Entry> 
  Callback_DB::private_gather_servers( ABng_Server_Manager::Name &name,
				       int check_NS,
				       int public_callback) {

  RWTValDlist< Server_Entry> ret;

  if (public_callback > Server_Entry::name_is_public(name)) return ret;

  CORBA::Object_var null_var;

  Server_Entry item( 0,
		     null_var,
		     name);
  Server_Entry found;

  if ( server_db.find( item, found)) {
    ret.insert( found);
    return ret;
  }
  if ( ! check_NS) return ret;

  CORBA::Object_var obj;
  try {
    obj= base.resolve( name);

    found= Server_Entry( unique_id++,
			 obj,
			 name);

    
    server_db.insert( found);
    ret.insert( found);
  }
  catch (...) {
  }
  return ret; 
}

//-----------------------------------------------------gather_servers_in_context--
//
RWTValDlist< Server_Entry>
  Callback_DB::gather_servers_in_context( ABng_Server_Manager::Name &name,
					  int check_NS,
					  int public_callback) {

  auto_lock local_lock( *this);
  return private_gather_servers_in_context( name, check_NS, public_callback);
}

RWTValDlist< Server_Entry>
  Callback_DB::private_gather_servers_in_context( ABng_Server_Manager::Name &name,
						  int check_NS,
						  int public_callback) {

  RWTValDlist< Server_Entry> ret;

  CORBA::Object_var null_var;
  Server_Entry item( 0,
		     null_var,
		     name);

  for ( RWTValHashTableIterator< Server_Entry> iter( server_db);
	iter(); )
    if ( iter.key() >= item &&
	 public_callback <= iter.key().is_public()) {

	ret.insert( iter.key() );
    }

  if ( ! check_NS) return ret;

  try {
    CORBA::Object_var abng_ctx_obj= base.resolve( name);
    CosNaming::NamingContext_var abng_ctx= 
      CosNaming::NamingContext::_narrow( abng_ctx_obj);

    CosNaming::BindingList_var dir_list;
    CosNaming::BindingIterator_var autoiter;

    abng_ctx->list( 100, dir_list, autoiter);

    for ( int i=0; i<dir_list->length(); i++) {
      ABng_Server_Manager::Name new_name= 
	  name+dir_list[ i].binding_name[ 0].id;

      if ( dir_list[ i].binding_type == CosNaming::ncontext) {
	RWTValDlist< Server_Entry> rec_ret= 
	  private_gather_servers_in_context( new_name, check_NS, public_callback);
	for ( RWTValDlistIterator< Server_Entry> rec_iter( rec_ret);
	      rec_iter(); )
	  ret.insert( rec_iter.key());
	continue;
      }

      
      Server_Entry item( 0,
			 null_var,
			 new_name);

      Server_Entry new_item;

      if (server_db.find( item, new_item)) continue;

      CORBA::Object_var server_reference;
      try {
	server_reference= base.resolve( new_name);
      }
      catch (...) {
	continue;
      }
      new_item= Server_Entry( unique_id++,
			      server_reference,
			      new_name);
      if ( public_callback > new_item.is_public()) continue;

      ret.insert( new_item );
      server_db.insert( new_item);
    }
  }
  catch (...) {}
  return ret;
}



//------------------------------------------------------------------------insert--
//
void Callback_DB::insert( Callback_Entry &callback_entry) {

  auto_lock local_lock( *this);

  if ( ! callback_entry.item_id)
    callback_entry.item_id= unique_id++;
  db.insert( callback_entry);

  RWTValDlist< Server_Entry> servers;

  if ( callback_entry.context_entry)
    servers= private_gather_servers_in_context( callback_entry.server_name,
						1,
						callback_entry.public_entry);
  else
    servers= private_gather_servers( callback_entry.server_name,
				     1,
				     callback_entry.public_entry);

  save();
  // inform a callback about all servers it is interested in
  //
  launch_callback_broadcast( callback_entry,
			     servers,
			     send_callback_startup);
}

//-----------------------------------------------------------------------remove--
//
int Callback_DB::remove( Callback_Entry &callback_entry,
			 int unique) {

  auto_lock local_lock( *this);
  return private_remove( callback_entry, unique);
}

int Callback_DB::private_remove( Callback_Entry &callback_entry,
				 int unique) {
  
  Callback_Entry found;
  if ( db.find( callback_entry, found)) 
    if ( found.item_id == callback_entry.item_id || ( ! unique)) {
      db.remove( found);
      save();
      return 1;
    }
  return 0;
}

//----------------------------------------------------------------insert_server--
//
int Callback_DB::insert_server( Server_Entry &entry) {

  auto_lock local_lock( *this);
  
  //  cerr << "Server (1) " << server_db.entries() << endl;

  Server_Entry found;
  RWTValDlist<Server_Entry> slist= private_gather_servers( entry.server_name,
							   1 /* check NS */,
							   0);
  if ( slist.entries() != 0) {
    found= slist[0];
    int dead= 1;
    
    try {
#ifdef ORBIX
      if (!strcmp( found.server_reference->_host(),
		   "149.156.97.51")) dead= 1;
      else dead= DII_non_existent(found.server_reference);
#else
      dead= found.server_reference->_non_existent();
#endif
    }
    catch (CORBA::BAD_OPERATION) {
      dead= 0; /* interoperability problem */
    }
    catch (...) {}
    if ( dead) private_remove_server( entry, 0);
    else throw ABng_Server_Manager::Server_Still_Active();
  }
  entry.item_id= unique_id++;
  //  cerr << "Server (2) " << server_db.entries() << endl;
  server_db.insert( entry);
  // cerr << "Server (3) " << server_db.entries() << endl;
#ifdef DEBUG
  for ( RWTValHashTableIterator< Server_Entry> iter( server_db);
	iter(); )
    cerr << "######" << endl << iter.key().server_name << endl;
#endif

  try {
    base.incremental_bind( entry.server_name, entry.server_reference);
    {
      TransactionLOG::LOGItem logitem( tlog);
      tlog.ostr() << "insert_server::incremental_bind " << entry.server_name;
      logitem.commit();
    }
  }
  catch (...) {}

  RWTValDlist< Callback_Entry> callback_entries=
    private_gather_callbacks( entry.server_name);

  launch_server_broadcast( entry,
			   callback_entries,
			   send_callback_startup);
  return 1;
}

//----------------------------------------------------------------remove_server--
//
int Callback_DB::remove_server( Server_Entry &entry, 
				int unique) {
  auto_lock local_lock( *this);
  return private_remove_server( entry, unique);
}

int Callback_DB::private_remove_server( Server_Entry &entry, 
					int unique) {

  Server_Entry found;
  if ( server_db.find( entry, found)) {
    if ( found.item_id == entry.item_id || ( ! unique)) {
      
      RWTValDlist< Callback_Entry> callback_entries=
	private_gather_callbacks( entry.server_name);
  
      server_db.remove( entry);
#ifdef DEBUG
      cerr << "private_remove_server" << endl;
#endif
    
      try {
	base.unbind( entry.server_name);
	{
	  TransactionLOG::LOGItem logitem( tlog);
	  tlog.ostr() << "private_remove_server::unbind " << entry.server_name;
	  logitem.commit();
	}
      }
      catch (...) {}

      // inform all callbacks registered for updates of the specified server
      //

      launch_server_broadcast( entry,
      			       callback_entries,
      			       send_callback_shutdown);
      return 1;
    }
  }
  else {
    try {
      base.unbind( entry.server_name);
      {
	TransactionLOG::LOGItem logitem( tlog);
	tlog.ostr() << "private_remove_server::unbind " << entry.server_name;
	logitem.commit();
      }
    }
    catch (...) {}
  }
  return 0;
}

//-------------------------------------------------------launch_server_broadcast--
//
void Callback_DB::launch_server_broadcast( 
			 Server_Entry &server_entry,
			 RWTValDlist< Callback_Entry> &callbacks,
			 callback_fun callback) {

  Server_Params *params= new Server_Params( *this, 
					    server_entry,
					    callbacks,
					    callback);

  ( new omni_thread( static_launch_server_broadcast, params))->start();
}

//------------------------------------------------static_launch_server_broadcast--
//
void Callback_DB::static_launch_server_broadcast( void *_params) {
  Server_Params *params= ( Server_Params *) _params;
  params->callback_db.server_broadcast( params->server_entry,
					params->callbacks,
					params->callback);
  delete params;
}

//--------------------------------------------------------------server_broadcast--
//
void Callback_DB::server_broadcast( Server_Entry &server_entry,
				    RWTValDlist< Callback_Entry> &callbacks,
				    callback_fun callback) {

#ifdef DEBUG
  cerr << "server_broadcast" << endl;
  cerr << callbacks.entries() << " entries" << endl;
#endif

  for ( RWTValDlistIterator< Callback_Entry> iter( callbacks);
	iter(); ) {

    try {
      Notifier_Lock notify_lock( 
	  *this, 
	  server_entry, 
	  (callback == send_callback_shutdown) ? Notifier_Lock::SHUTDOWN_INFO : 
	  ((callback == send_callback_startup) ? Notifier_Lock::STARTUP_INFO : 
	  (((callback == check_server_entry) ? Notifier_Lock::CHECK_INFO_SERVER :
	    Notifier_Lock::CHECK_INFO_CALLBACK))),
	  iter.key().callback_object);

      switch ( (this->*callback)( iter.key(), server_entry)) {
      case -1: remove( iter.key(), 1 /* unique */);
	break;
      case +1: 
	{
	  {
	    TransactionLOG::LOGItem logitem( tlog);
	    tlog.ostr() << "server_broadcast::remove_server " 
			<< server_entry.server_name;
	    logitem.commit();
	  }
	  remove_server( server_entry, 1 /* unique */);
	}
	break;
      }
    }
    catch ( const char *info) {
#ifdef DEBUG
      cerr << info << endl;
#endif
    }
  }
}

//-----------------------------------------------------launch_callback_broadcast--
//
void Callback_DB::launch_callback_broadcast( 
			 Callback_Entry &callback_entry,
			 RWTValDlist< Server_Entry> &servers,
			 callback_fun callback) {

  Callback_Params *params= new Callback_Params( *this, 
						callback_entry,
						servers,
						callback);

   ( new omni_thread( static_launch_callback_broadcast, params))->start();
}

//----------------------------------------------static_launch_callback_broadcast--
//
void Callback_DB::static_launch_callback_broadcast( void *_params) {
  Callback_Params *params= ( Callback_Params *) _params;
  params->callback_db.callback_broadcast( params->callback_entry,
					  params->servers,
					  params->callback);
  delete params;
}

//------------------------------------------------------------callback_broadcast--
//
void Callback_DB::callback_broadcast( Callback_Entry &callback_entry,
				      RWTValDlist< Server_Entry> &servers,
				      callback_fun callback) {

#ifdef DEBUG
  cerr << "callback_broadcast" << endl;
  cerr << servers.entries() << " entries" << endl;
#endif

  for ( RWTValDlistIterator< Server_Entry> iter( servers);
	iter(); ) {

    try {
      Notifier_Lock notify_lock( 
	  *this, 
	  iter.key(), 
	  (callback == send_callback_shutdown) ? Notifier_Lock::SHUTDOWN_INFO : 
	  ((callback == send_callback_startup) ? Notifier_Lock::STARTUP_INFO : 
	  (((callback == check_server_entry) ? Notifier_Lock::CHECK_INFO_SERVER :
	    Notifier_Lock::CHECK_INFO_CALLBACK))),
	  iter.key().server_reference);

      switch ( (this->*callback)( callback_entry, iter.key())) {
      case -1: remove( callback_entry, 1 /* unique */ );
	break;
      case +1: 
	{
	  {
	    TransactionLOG::LOGItem logitem( tlog);
	    char *str= the_orb->object_to_string( iter.key().server_reference);

	    tlog.ostr() << "callback_broadcast::remove_server " 
			<< iter.key().server_name
	                << str;
	    CORBA::string_free( str);
	    logitem.commit();
	  }
	  remove_server( iter.key(), 1 /* unique */);
	}
	break;
      }
    }
    catch ( const char *info) {
#ifdef DEBUG
      cerr << info << endl;
#endif
    }
  }
}

//------------------------------------------------------------check_server_entry--
int Callback_DB::check_server_entry( Callback_Entry &,
				     Server_Entry &server_entry) {

  int dead= 1;
  try {
#if defined(ORBIX) && (CORBA_ORBIX_VER < 23)
    char *str= CORBA::string_dup( ( char *) ((const char *) server_entry.stringified_server));

    server_entry.server_reference=
      the_orb->string_to_object( str);
    CORBA::string_free( str);
#endif

#ifdef DEBUG
    cerr << "checking " << server_entry.server_name;
#endif

#ifdef ORBIX
#ifdef DEBUG
    cerr << '#' << server_entry.server_reference->_host() << '#' << endl;
#endif


    // a temporary hack to prevent deadlock while talking to Orbix2.2 server
    if (!strcmp((const char *) server_entry.server_reference->_host(),
		"neta.ics.agh.edu.pl")) return 0;
#endif



#ifdef ORBIX
      dead= DII_non_existent(server_entry.server_reference);
#else
      dead= server_entry.server_reference->_non_existent();
#endif
  }
  catch (CORBA::BAD_OPERATION) {
    dead= 0; /* interoperability problem */
  }
  catch (CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << ex << endl;
#endif
  }
  
  if ( dead) return +1; /* remove server */
  return 0; /* no action required */
}


//---------------------------------------------------------check_callback_entry--
int Callback_DB::check_callback_entry( Callback_Entry &callback_entry,
				       Server_Entry &) {

  int dead= 1;
  try {
#ifdef ORBIX
      dead= DII_non_existent(callback_entry.callback_object);
#else
      dead= callback_entry.callback_object->_non_existent();
#endif

  }
  catch (CORBA::BAD_OPERATION) {
    dead= 0; /* interoperability problem */
  }
  catch (...) {};
  
  if ( dead) return -1; /* remove callback */
  return 0; /* no action required */
}



//---------------------------------------------------------send_callback_startup--
//
int Callback_DB::send_callback_startup( Callback_Entry &callback_entry,
					Server_Entry &server_entry) {

  static omni_mutex mtx;

  mtx.lock();
  try {
#if defined(ORBIX) && (CORBA_ORBIX_VER < 23)
    char *str= CORBA::string_dup( ( char *) ((const char *) server_entry.stringified_server));

    server_entry.server_reference=
      the_orb->string_to_object( str);
    CORBA::string_free( str);
#endif

    callback_entry.callback_object->server_startup( server_entry.server_reference,
    						    server_entry.server_name,
    						    callback_entry.item_id);
#ifdef DEBUG
    cerr << "Callback startup succeeded" << endl;
    cerr << server_entry.server_name << endl;
#ifdef ORBIX
    cerr << server_entry.server_reference->_interfaceMarker() << endl;
#endif
#endif
  }
  catch (...) {
    mtx.unlock();
#ifdef DEBUG
    cerr << "Callback startup failed" << endl;
#endif
    return -1; /* remove callback */
  }
  mtx.unlock();
  return 0;
}

//---------------------------------------------------------send_callback_shutdown--
//
int Callback_DB::send_callback_shutdown( Callback_Entry &callback_entry,
					 Server_Entry &server_entry) {

  try {
    callback_entry.callback_object->server_shutdown( server_entry.server_name,
						     callback_entry.item_id);
#ifdef DEBUG
    cerr << "Callback shutdown succeeded" << endl;
#endif
  }
  catch ( CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "Callback shutdown failed " << ex << endl;
#endif
    return -1; /* remove callback */
  }
  return 0;
}

//-----------------------------------------------------static_garbage_collector--
//
void Callback_DB::static_garbage_collector( void *_params) {
  (( Callback_DB *) _params)->garbage_collector();
}


//---------------------------------------------------------------------sweep_db--
//
void Callback_DB::sweep_db() {
#ifdef DEBUG
    cerr << "Garbage collector running; checking servers" << endl;
#endif
    Callback_Entry void_callback;
    Server_Entry void_server;

    callback_broadcast( void_callback,
			gather_all_servers(),
			check_server_entry);

#ifdef DEBUG
    cerr << "Checking callbacks" << endl;
#endif

    server_broadcast( void_server,
		      gather_all_callbacks(),
		      check_callback_entry);

#ifdef DEBUG
    cerr << "Garbage collector sleeping" << endl;
#endif
}

//------------------------------------------------------------garbage_collector--
//
void Callback_DB::garbage_collector() {

  sleep(60);
#ifdef DEBUG
  cerr << "Garbage collector started" << endl;
#endif
  while ( 1) {
    sweep_db();
    sleep( gc_sleep_time);
  }
#ifdef DEBUG
  cerr << "Garbage collector terminating" << endl;
#endif
}

//-------------------------------------------------------------------------save--
//
void Callback_DB::save() {
  FileTransaction ft( (const char *) file_name,
		      0 /* don't make backup */ );

  ostream &ofs= *(ft.prepare());

  //  cerr << "save<" << endl;
  ofs << unique_id << endl;
  ofs << db.entries() << endl;
  for ( RWTValHashTableIterator< Callback_Entry> iter( db);
	iter(); ) 
    ofs << iter.key();

  ft.commit();
}

//-----------------------------------------------------------------------restore--
//
void Callback_DB::restore() {
  RWTValDlist< Callback_Entry> my_db;
  {
    ifstream ifs( file_name);
    if ( !ifs) return;

    int items;

    ifs >> unique_id >> items;
    while ( ifs.good() && (items--) > 0 ) {
      Callback_Entry new_entry;

      ifs >> new_entry;
      my_db.insert( new_entry);
    }
  }
  for ( RWTValDlistIterator< Callback_Entry> iter( my_db);
	iter(); )
    insert( iter.key());
}

//---------------------------------------------------------------------start_GC--
//
void Callback_DB::start_GC() {
  //  sleep( 5);
  ( new omni_thread( static_garbage_collector, this))->start();
}

//-------------------------------------------------------------get_server_status--
//
void Callback_DB::get_server_status( ostream &ostr) {
  {
    auto_lock local_lock( *this);
    
    ostr << db.entries() << " callback entry(ies)" << endl
	 << server_db.entries() << " server(s) cached" << endl
	 << active_notifiers.entries() << " active notifier(s)" << endl;
  }
  {
    ABng_S_Guarded::auto_lock local_lock( gcstate);
    
    ostr << "GC: " << (gcstate.active ? "running" : "sleeping") << endl;
    if (gcstate.active) {
      ostr << "\tchecking " << (gcstate.check_callback ? "callback for " :
				"server ") << gcstate.name << endl
	   << (const char*) gcstate.IOR << endl;
    }
  }
}

///////////////////////////////////////////////////////////////////Notifier_Lock//
//
Notifier_Lock::Notifier_Lock( Callback_DB &_db,
			      Server_Entry &server_entry,
			      callback_info ci,
			      CORBA::Object_ptr call_obj) :
  db( _db) {

  server_name= server_entry.server_name;


  found= NULL;
  if ( ci == CHECK_INFO_SERVER || ci == CHECK_INFO_CALLBACK) {
    ABng_S_Guarded::auto_lock local_lock( gcstate);
    gcstate.active= 1;
    gcstate.check_callback= ( ci == CHECK_INFO_CALLBACK);
    gcstate.IOR= the_orb->object_to_string( call_obj);
    gcstate.name= server_entry.server_name;
    return;
  }

#ifdef DEBUG
  cerr << "Notifier_Lock" << server_name << endl;
#endif

  {
    ABng_Guarded::auto_lock local_lock( db);

    Notifier_Entry key( server_entry.server_name);

    if ( ( found= db.active_notifiers.find( &key)) == NULL) {
#ifdef DEBUG
      cerr << "NL: notify_entry created" << endl;
#endif
      db.active_notifiers.insert( found= new Notifier_Entry( server_entry.server_name));
    }
    else {
#ifdef DEBUG
      cerr << "NL: notify_entry found" << endl;
#endif
      found->duplicate();
    }
  }
  found->mutex().lock();
  {
    ABng_Guarded::auto_lock local_lock( db);
    
    Server_Entry server_found;
    if ( db.server_db.find( server_entry, server_found)) {
      if ( ci == SHUTDOWN_INFO || server_entry.item_id < server_found.item_id) {
	found->mutex().unlock();
#ifdef DEBUG
	cerr << "NL: unlock " << server_name << endl;
#endif
	if ( found->release() == 0) {

#ifdef DEBUG
	  cerr << "NL: destroy notifier_entry" << endl;
#endif
	  delete db.active_notifiers.remove( found);
	}
	found= NULL;
	throw "Invalid operation";
      }
    }
  }
#ifdef DEBUG
  cerr << "NL: lock acquired " << server_name << endl;
#endif
}

Notifier_Lock::~Notifier_Lock() {

  if ( found) {
#ifdef DEBUG
  cerr << "NL: unlock " << server_name << endl;
#endif
    found->mutex().unlock();
    {
      ABng_Guarded::auto_lock local_lock( db);
    
      if ( found->release() == 0) {

#ifdef DEBUG
        cerr << "NL: destroy notifier_entry" << endl;
#endif
        delete db.active_notifiers.remove( found);
      }
    }
  }
  else {
    ABng_S_Guarded::auto_lock local_lock( gcstate);
    gcstate.active= 0;
  }
}

//////////////////////////////////////////////////////////////////Server_Manager//
Server_Manager::Server_Manager( CORBA::ORB_ptr _orb,
				int gc_sleep_time) :
  Bootstrap_base( _orb),
  tlog( "server_mgr.db"),
  db( "server_mgr.db", 
      *this,
      tlog,
      gc_sleep_time)
{

  // create the TIE (CORBA) object for the C++ object
#ifdef ORBIX
  register_object=
    new TIE_ABng_Server_Manager_Register(Server_Manager) (this);
  register_manager_object=
    new TIE_ABng_Server_Manager_Register_Manager(Server_Manager)(this);

#elif defined OMNIORB
  register_object=
    new TIE_CLASS(ABng_Server_Manager,Register)<Server_Manager,0>(this);  
  CONNECT_IMPL(ABng_Server_Manager::Register_var, register_object);
  register_manager_object=
    new TIE_CLASS(ABng_Server_Manager,Register_Manager)<Server_Manager,0>(this);  
  CONNECT_IMPL(ABng_Server_Manager::Register_Manager_var, 
	       register_manager_object);
#endif

#ifdef DEBUG
    cerr << "Server_Manager ctor" << endl;
#endif
  int dead= 0;
  try {
    CORBA::Object_ptr old_impl= resolve( Bootstrap::SRV_Bootstrap());
#ifdef DEBUG
    cerr << "resolve .." << endl;
#endif
    int is_nil=0;
    try {
      is_nil= CORBA::is_nil( (CORBA::Object_ptr) old_impl);
    }
    catch (...) {}

    if (! is_nil) {
      try {
#ifdef ORBIX
	dead= DII_non_existent(old_impl);
#else
	dead= old_impl->_non_existent();
#endif
	cerr << "_non_existent returned " << dead << endl;
      }
      catch (CORBA::COMM_FAILURE &se) {
	cerr << "COMM_FAILURE, the old instance is dead" << endl;
	dead= 1;
      }
      catch (CORBA::Exception &ex) {}
      cerr << "OK" << endl;
    }
    else {
      cerr << "nilref" << endl;
      dead= 1;
    }
    cerr << "OK" << endl;
  }
  catch (CORBA::SystemException &se) {
    cerr << "Exception during resolve" << endl;
#ifdef OMNIORB
    cerr << se.NP_RepositoryId() << endl;
#elif defined ORBIX
    cerr << se << endl;
#endif
    exit(1);
  }
  catch (...) {
    cerr << "Exception during resolve" << endl;
    dead= 1;
  }

  if (!dead) {
    cerr << "There is another server_manager registered, please kill the other instance" << endl;
    exit( 1);
  }
  try {
    incremental_bind( Bootstrap::SRV_Bootstrap(),
		      ABng_Server_Manager::Register_Manager::_duplicate( register_manager_object));
    incremental_bind( Bootstrap::SRV_Bootstrap_Client(),
		      ABng_Server_Manager::Register::_duplicate( register_object));
  }
  catch (CORBA::SystemException &se) {
    cerr << "incremental_bind system exception" << endl;
#ifdef OMNIORB
    cerr << se.NP_RepositoryId() << endl;
#endif
    throw;
  }
  catch (CORBA::Exception &ex) {
    cerr << "incremental_bind " << ex << endl;
    throw;
  }
  db.restore();
  db.start_GC();
}



//--------------------------------------------------------------register_server--
//
void Server_Manager::register_server( CORBA::Object_ptr server_reference,
				      const ABng_Server_Manager::Name &name
				      ENV_DEF( env) ) {

  TransactionLOG::LOGItem logitem( tlog);
  tlog.ostr() << "register_server " << name;

  if ( CORBA::is_nil( server_reference)) 
    throw ABng_Server_Manager::Invalid_Server_Reference();

  Server_Entry se( 0,
		   CORBA::Object_var( CORBA::Object::_duplicate( server_reference)),
		   name);

  db.insert_server( se);
  logitem.commit();
}

//------------------------------------------------------------unregister_server--
// IDL implementation method
//  
void Server_Manager::unregister_server( const ABng_Server_Manager::Name &name
					ENV_DEF( env) ) {

  CORBA::Object_var null_object;

  Server_Entry se( 0,
		   null_object,
		   name);

  db.remove_server( se, 0 /* non-unique */);
}

//-----------------------------------------------------------register_callback--
// IDL implementation method
//
ABng_Server_Manager::uid 
  Server_Manager::register_callback( 
		       ABng_Server_Manager::Callback_ptr callback_object,
		       const ABng_Server_Manager::Name& server_name
		       ENV_DEF( env) ) {

  TransactionLOG::LOGItem logitem( tlog);
  tlog.ostr() << "register_callback " << server_name;

  Callback_Entry ce( 0,
		     ABng_Server_Manager::Callback_var( 
		       ABng_Server_Manager::Callback::_duplicate( callback_object)),
		     server_name,
		     0 /* normal entry */,
		     0 /* unrestricted - all objects available */);
  db.insert( ce);
  logitem.commit();
  return ce.item_id;
}

ABng_Server_Manager::uid 
  Server_Manager::register_restricted_callback( 
		       ABng_Server_Manager::Callback_ptr callback_object,
		       const ABng_Server_Manager::Name& server_name
		       ENV_DEF( env) ) {

  TransactionLOG::LOGItem logitem( tlog);
  tlog.ostr() << "register_restricted_callback " << server_name;

  Callback_Entry ce( 0,
		     ABng_Server_Manager::Callback_var( 
		       ABng_Server_Manager::Callback::_duplicate( callback_object)),
		     server_name,
		     0 /* normal entry */,
		     1 /* restricted - only public objects */);
  db.insert( ce);
  logitem.commit();
  return ce.item_id;
}

//---------------------------------------------------register_context_callback--
// IDL implementation method
//
ABng_Server_Manager::uid 
  Server_Manager::register_context_callback( 
		       ABng_Server_Manager::Callback_ptr callback_object,
		       const ABng_Server_Manager::Name& server_name
		       ENV_DEF( env) ) 
{

  TransactionLOG::LOGItem logitem( tlog);
  tlog.ostr() << "register_context_callback " << server_name;

  Callback_Entry ce( 0,
		     ABng_Server_Manager::Callback_var( 
		       ABng_Server_Manager::Callback::_duplicate( callback_object)),
		     server_name,
		     1 /* context entry */,
		     0 /* unrestricted - all objects available */);
  db.insert( ce);
  logitem.commit();
  return ce.item_id;
}

ABng_Server_Manager::uid 
  Server_Manager::register_restricted_context_callback( 
		       ABng_Server_Manager::Callback_ptr callback_object,
		       const ABng_Server_Manager::Name& server_name
		       ENV_DEF( env) ) 
{

  TransactionLOG::LOGItem logitem( tlog);
  tlog.ostr() << "register_restricted_context_callback " << server_name;

  Callback_Entry ce( 0,
		     ABng_Server_Manager::Callback_var( 
		       ABng_Server_Manager::Callback::_duplicate( callback_object)),
		     server_name,
		     1 /* context entry */,
		     1 /* restricted - only public objects */);
  db.insert( ce);
  logitem.commit();
  return ce.item_id;
}
//---------------------------------------------------------unregister_callback--
// IDL implementation method
// 
void Server_Manager::unregister_callback( ABng_Server_Manager::uid identifier
					  ENV_DEF( env) ) {

  ABng_Server_Manager::Callback_var null_callback;
  ABng_Server_Manager::Name null_name;

  Callback_Entry ce( identifier,
		     null_callback,
		     null_name,
		     0,
		     0);

  db.remove( ce, 0 /* non-unique */);
}

void Server_Manager::unregister_restricted_callback( 
					  ABng_Server_Manager::uid identifier
					  ENV_DEF( env) ) {

  ABng_Server_Manager::Callback_var null_callback;
  ABng_Server_Manager::Name null_name;

  Callback_Entry ce( identifier,
		     null_callback,
		     null_name,
		     0,
		     1);

  db.remove( ce, 0 /* non-unique */);
}

ABng_Server_Manager::Time Server_Manager::get_current_time( ENV_DEF1( env)) {
  ABng_Server_Manager::Time ret;

  struct timeval mtd;
  gettimeofday( &mtd, NULL);
  ret.seconds= mtd.tv_sec;
  ret.microseconds= mtd.tv_usec;
  return ret;
}

ABng_Server_Manager::ip_address* Server_Manager::get_ip_address( ENV_DEF1( env)){
  struct in_addr myaddr=  get_host_address();
  
  // stands for IP address ver. 4
  //
  ABng_Server_Manager::ip_address *ret= 
    new ABng_Server_Manager::ip_address( 4);
  ret->length( 4);
  
  (*ret)[0]= (CORBA::Octet)((ntohl(myaddr.s_addr) & 0xff000000) >> 24);
  (*ret)[1]= (CORBA::Octet)((ntohl(myaddr.s_addr) & 0x00ff0000) >> 16);
  (*ret)[2]= (CORBA::Octet)((ntohl(myaddr.s_addr) & 0x0000ff00) >> 8);
  (*ret)[3]= (CORBA::Octet)(ntohl(myaddr.s_addr) & 0x000000ff);
  
  return ret;
}

void usage( char *prog_name) {
  cout << "Usage: " << prog_name << endl
       << "\t\t [-g <garbage_collector_sleep_time>]" << endl;
#ifdef ORBIX
  cout << "\t\t [-t <orbix_implementation_timeout>]" << endl;
#endif
  exit( 1);
}


//-------------------------------------------------------------------------main--
int main( int argc, char **argv){
  extern char *optarg;
  extern int optind;

  CORBA::ORB_ptr orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

  the_orb= orb_ptr;  

#ifdef ORBIX
  int implementation_timeout= CORBA::ORB::INFINITE_TIMEOUT;
#endif
  int gc_sleep_time= 120;

  char c;
  while ((c = getopt(argc, argv, "g:t:h")) != EOF)
    switch (c) {
    case 'g':
      gc_sleep_time= atoi( optarg);
      break;
#ifdef ORBIX
    case 't':
      implementation_timeout= atoi( optarg);
      break;
#endif
    default:
      usage( *argv);
      break;
    }

  try {
    Server_Manager srv_mgr( orb_ptr, 
			    gc_sleep_time);
    Bootstrap::init( orb_ptr);
    ProcessControlImpl procimpl( "server_manager", srv_mgr.server_status_interface());


#ifdef DEBUG
    cerr << "Implementation object created, entering event loop" << endl;
#endif

    //
    // Enter the event loop
    //
#ifdef ORBIX
    try {
      boa_ptr->processEvents( implementation_timeout);
    }
    catch ( CORBA::SystemException &ex) {
      cerr << "Caught system exception " << ex
           << " -- impl_is_ready" << endl;
      return 1;
    }
#elif defined OMNIORB
    try {
      boa_ptr->impl_is_ready();
    }
    catch ( CORBA::Exception &ex) {
      cerr << ex << endl;
      return 1;
    }
#else
#error Not implemented
#endif
  }
  catch (CORBA::Exception &ex) {
    cerr << "Caught exception " << ex
         << " -- creating Server_Manager" << endl;
    return 1;
  }
#ifdef DEBUG
  cerr << "Implementation timed out, exiting" << endl;
#endif
  return 0;
}
































