// $Id: locatable_i.cc,v 1.2 1998/07/29 14:57:05 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/locatable_i.cc,v $
//
// $Log: locatable_i.cc,v $
// Revision 1.2  1998/07/29 14:57:05  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:24:29  jasz
// Initial revision
//
//
// File:          locatable_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Definition of the implementation class of the Locatable and 
//  LESP interfaces. To improve performance the LESP interface is not 
//  implemented as a CORBA object but as a usual C++ object.


#include <rw/rwdate.h>
#include <rw/rwdate.h>
#include "locatable_i.h"
#include "locatable_rep_i.h"
#include "location_rep_i.h"
#include "db_utils.hh"
#include "view_i.h"

// The constructor used to create a new locatable
Locatable_I::Locatable_I (
  ABng_Core_System::Badge_Holder_ptr badge_holder_p) :
  badge_holder( ABng_Core_System::Badge_Holder::_duplicate( badge_holder_p)),
  my_location(0),
  my_home_location(0)  {

  ABng_Core_System::Badge_Holder::Description_var bhd;
  ABng_Core_System::Badge_ptr badge;  
  ABng_Core_System::Sensor_ptr sensor;

  // Get Badge_Holder data.
#ifdef DEBUG
  cerr << "Get Badge_Holder data" << endl;
#endif
  try {
    bhd = badge_holder->descr();
  }
  catch (CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "badge_holder_rep unreachable " << ex << endl;
#endif
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");
  }

  //  badge_holder->type_specific_object();
  my_descr.type_specific_object = bhd->type_specific_object;
  my_descr.assigned = bhd->assigned;
  my_descr.visitor = bhd->visitor;

#ifdef ORBIX // nil references buf
  switch (my_descr.type_specific_object._d()) {
  case ABng_Core::LOCATABLE_USER:
    if (CORBA::is_nil( my_descr.type_specific_object.user_obj()))
      my_descr.type_specific_object.user_obj( NULL);
    break;
  case ABng_Core::LOCATABLE_EQUIPMENT:
    if (CORBA::is_nil( my_descr.type_specific_object.equipment_obj()))
      my_descr.type_specific_object.equipment_obj( NULL);
    break;
  }
  switch (bhd->type_specific_object._d()) {
  case ABng_Core::LOCATABLE_USER:
    if (CORBA::is_nil( bhd->type_specific_object.user_obj()))
      bhd->type_specific_object.user_obj( NULL);
    break;
  case ABng_Core::LOCATABLE_EQUIPMENT:
    if (CORBA::is_nil( bhd->type_specific_object.equipment_obj()))
      bhd->type_specific_object.equipment_obj( NULL);
    break;
  }
#endif

  my_badge_holder_item_id = bhd->item_id;
  badge = bhd->current_badge;
  sensor = bhd->home_sensor;

#ifdef DEBUG
  cerr << "get badge description" << endl;
#endif
  if (CORBA::is_nil( badge)) {
#ifdef DEBUG
    cerr << "badge is nil" << endl;
#endif
    throw Domain_Repository::Invalid_Badge_Holder();
  }
  try {
    ABng_Core_System::Badge::Description_var bd = badge->descr();
#ifdef ORBIX
    if (CORBA::is_nil( bd->holder))
      bd->holder= NULL;
#endif
    my_bd = new Comparable_Badge_Descr(bd.operator->()); 
  }
  catch(...) {
#ifdef DEBUG
    cerr << "badge repository unreachable" << endl;
#endif
    throw Domain_Repository::Destination_Unreachable("badge_rep");
  }

#ifdef DEBUG
  cerr << "get sensor description" << endl;
#endif
  if (CORBA::is_nil( sensor)) {
#ifdef DEBUG
    cerr << "sensor is nil" << endl;
#endif
    my_home_sd= new Comparable_Sensor_Descr();
  }
  else {
    try {
      ABng_Core_System::Sensor::Description_var sd =  sensor->descr();
      my_home_sd = new Comparable_Sensor_Descr(sd.operator->());
    }
    catch(...) {
      throw Domain_Repository::Destination_Unreachable("sensor_rep");
    }
  }

  Location_I *loc = Location_Rep_I::instance()->resolve_sensor(
    my_home_sd->id);
  my_home_location = loc;
  if(loc) {
    my_descr.home_location = 
      ABng_Core::Location::_duplicate(loc->get_corba_object());
  }
  else {
    my_descr.home_location = ABng_Core::Location::_nil();
  }

  my_sighting.view_name = 
    CORBA::string_dup(Locatable_Rep_I::instance()->get_view_name());
  my_sighting.my_locatable = my_descr.item_id;
  my_sighting.my_locatable_type = my_descr.type_specific_object._d();
  my_sighting.my_location = (Domain_Repository::Repository_Item_Id) 0;
  my_sighting.message_type = (CORBA::Octet) 0;
  my_sighting.attributes = (CORBA::Octet) 0;
  my_sighting.sequence_number = (ABng_Core::Seq_Number) 0;
  my_sighting.update_time.seconds = (CORBA::ULong) 0;
  my_sighting.update_time.microseconds = (CORBA::ULong) 0;
  my_sighting.q = (ABng_Core::Quality) 0;
}

// The constructor used to make a search pattern.

Locatable_I::Locatable_I(
  Domain_Repository::Repository_Item_Id badge_holder_item_id_p) :
  my_badge_holder_item_id(badge_holder_item_id_p) {
  my_bd= NULL;  
  my_home_sd= NULL;  
  my_location= NULL;
  my_home_location= NULL;
}

// The constructor used to restore a locatable
#ifdef FilePersistency
Locatable_I::Locatable_I(istream &iostr) :
  badge_holder( ABng_Core_System::Badge_Holder::_nil()),
  my_home_location(0),
  my_location(0) {

  my_bd= new Comparable_Badge_Descr();
  my_home_sd= new Comparable_Sensor_Descr();

  my_sighting.view_name = 
    CORBA::string_dup(Locatable_Rep_I::instance()->get_view_name());
  my_sighting.my_locatable_type = my_descr.type_specific_object._d();
  my_sighting.my_location = (Domain_Repository::Repository_Item_Id) 0;
  my_sighting.message_type = (CORBA::Octet) 0;
  my_sighting.attributes = (CORBA::Octet) 0;
  my_sighting.sequence_number = (ABng_Core::Seq_Number) 0;
  my_sighting.update_time.seconds = (CORBA::ULong) 0;
  my_sighting.update_time.microseconds = (CORBA::ULong) 0;
  my_sighting.q = (ABng_Core::Quality) 0;

  // read description from the file
  restore(iostr);
  my_sighting.my_locatable = my_descr.item_id;
  badge_holder = ABng_Core_System::Badge_Holder::_nil();
  my_descr.home_location = ABng_Core::Location::_nil();
  my_descr.type_specific_object.user_obj(Domain_Repository::User::_nil());

}
#endif

Locatable_I::~Locatable_I() {

  if (my_bd) delete my_bd;
  if (my_home_sd) delete my_home_sd;
}

// Operations of the Dynamic_Data_Observed_E interface:

ABng_Core::Dynamic_Data_Observed_E::Dynamic_Data_Observers* 
    Locatable_I::dynamic_data_observer_list (
    ENV_DEF1(env)) {

  return Locatable_Rep_I::instance()->dynamic_data_observer_list(
    my_descr.item_id);
}

void Locatable_I::attach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this location_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);
      
  // put this Locatable_type item id into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
      
  // register observer into the repository list
  Locatable_Rep_I::instance()->attach_dynamic_data_observer(obs,
    observed_id_list_p);
}
              
void Locatable_I::detach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this Locatable_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

  // put this Locatable_type item it into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
	
  // register observer into the repository list
  Locatable_Rep_I::instance()->detach_dynamic_data_observer(obs,
    observed_id_list_p);
}

// Operations of the View_Data_Observed_E interface:

ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    Locatable_I::view_data_observer_list (
    ENV_DEF1(env)) {

  return Locatable_Rep_I::instance()->view_data_observer_list(
    my_descr.item_id);
}
                
void Locatable_I::attach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this Locatable_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);
      
  // put this Locatable_type item id into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
      
  // register observer into the repository list
  Locatable_Rep_I::instance()->attach_view_data_observer(obs,
    observed_id_list_p);
}

void Locatable_I::detach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this Locatable_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

  // put this Locatable_type item it into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
	
  // register observer into the repository list
  Locatable_Rep_I::instance()->detach_view_data_observer(obs,
    observed_id_list_p);
}

  // Operations of the Locatable_Commander interface:

CORBA::Long Locatable_I::issue_command (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->issue_locatable_command(
	 location_list,
	 create_locatable_list(),
	 cmd,
	 has_reply,
	 reply_code,
	 ctx,
	 reply_to);
}
    
CORBA::Long Locatable_I::challenge (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& random, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->challenge(
	 location_list,
 	 create_locatable_list(),
	 random,
	 ctx,
	 reply_to);
}
  
CORBA::Long Locatable_I::sound (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->sound(
	 location_list,
	 create_locatable_list(),
	 sound,
	 ctx,
	 reply_to);
}
  
CORBA::Long Locatable_I::set_state (
    const ABng_Core::Locations& location_list, 
    CORBA::Octet status, 
    const ABng_Core::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->set_state(
         location_list,
	 create_locatable_list(),
	 status,
	 sound,
	 ctx,
	 reply_to);
}
  
CORBA::Long Locatable_I::get_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->get_state(
	 location_list,
	 create_locatable_list(),
	 ctx,
	 reply_to);
}
  
CORBA::Long Locatable_I::set_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& address, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->set_home(
	 location_list,
	 create_locatable_list(),
	 address,
	 ctx,
	 reply_to);
}
  
CORBA::Long Locatable_I::get_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env) ) {

  return View_I::instance()->get_home(
         location_list,
	 create_locatable_list(),
	 ctx,
	 reply_to);
}

  // Operations of Request_Queue:
ABng_Core::Request_Queue::Requests* Locatable_I::list_user_requests (
    const char * user_id
    ENV_DEF( IT_env) ) {

  return View_I::instance()->list_user_requests( user_id);
}

ABng_Core::Request_Queue::Requests* Locatable_I::list_all_requests (
    ENV_DEF1( IT_env) ) {

  return View_I::instance()->list_all_requests();
}

void Locatable_I::remove_request (CORBA::Long request_id
    ENV_DEF( IT_env) ) {

  View_I::instance()->remove_request( request_id);
}

// Operations of the Locatable interface:

ABng_Core::Locatable::Description* Locatable_I::descr (
    ENV_DEF1(env)) {

  if (CORBA::is_nil((ABng_Core_System::Badge_Holder_ptr) badge_holder))
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

  return new ABng_Core::Locatable::Description(my_descr);
}

CORBA::Boolean Locatable_I::assigned( ENV_DEF1(env)) {
  return my_descr.assigned;
}

CORBA::Boolean Locatable_I::visitor( ENV_DEF1(env)) {
  return my_descr.visitor;
}

ABng_Core::Locatable::Value_Description* Locatable_I::value_descr (
    ENV_DEF1( IT_env) ) {

  if (CORBA::is_nil((ABng_Core_System::Badge_Holder_ptr) badge_holder))
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

  ABng_Core::Locatable::Value_Description * ret=
    new ABng_Core::Locatable::Value_Description();

  Domain_Repository::User::Value_Description udef;
  ret->type_specific_object_value.user_description( udef);

  {
    ret->item_id= my_descr.item_id;
    if (!CORBA::is_nil( my_descr.home_location)) {
	ABng_Core::Location::Value_Description_var
	tmp( my_descr.home_location->value_descr());
	ret->home_location= *(tmp.operator->());
    }
    switch (my_descr.type_specific_object._d()) {
    case ABng_Core::LOCATABLE_USER:
      if (!CORBA::is_nil( my_descr.type_specific_object.user_obj())) {
	try {
	  Domain_Repository::User::Value_Description_var 
	    tmp( my_descr.type_specific_object.user_obj()->value_descr());
	  ret->type_specific_object_value.user_description( *(tmp.operator->()));
	}
	catch (...) {
	  delete ret;
	  throw Domain_Repository::Destination_Unreachable("user_rep");
	}
      }
      else {
	Domain_Repository::User::Value_Description tmp;
	ret->type_specific_object_value.user_description( tmp);
      }
      break;
    case ABng_Core::LOCATABLE_EQUIPMENT:
      if (!CORBA::is_nil( my_descr.type_specific_object.equipment_obj())) {
	try {
	  Domain_Repository::Equipment::Value_Description_var
	    tmp( my_descr.type_specific_object.equipment_obj()->value_descr());
	  ret->type_specific_object_value.equipment_description( *(tmp.operator->()));
	}
	catch (Domain_Repository::Destination_Unreachable &) {
	  throw;
	}
	catch (...) {
	  delete ret;
	  throw Domain_Repository::Destination_Unreachable("equipment_rep");
	}
      }
      else {
	Domain_Repository::Equipment::Value_Description tmp;
	ret->type_specific_object_value.equipment_description( tmp);
      }
      break;
    }
  }
  return ret;
}

ABng_Core::Location_ptr Locatable_I::home_location (
    ENV_DEF1(env)) {

  if (CORBA::is_nil((ABng_Core_System::Badge_Holder_ptr) badge_holder))
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");

  return ABng_Core::Location::_duplicate(my_descr.home_location);
}
  
ABng_Core::Locatable_Type_Specific_Object* 
  Locatable_I::type_specific_object (
    ENV_DEF1(env)) {

  if (CORBA::is_nil((ABng_Core_System::Badge_Holder_ptr) badge_holder))
    throw Domain_Repository::Destination_Unreachable("badge_holder_rep");
  return new ABng_Core::Locatable_Type_Specific_Object(
    my_descr.type_specific_object);
}

ABng_Core::Sighting_Description* Locatable_I::get_sighting (
    ENV_DEF1(env)) {

    return new ABng_Core::Sighting_Description(my_sighting);
}

void Locatable_I::process_sighting (
  const ABng_Core_Internals::Internal_Sighting_Description& 
    sighting_descr, 
  Location_I *location
  ENV_DEF(env)) {

  if(my_sighting.my_location == location->item_id()) {
    // The Locatable has not changed a location.      
    // Increase the quality.
    if(my_sighting.q + View_I::QUALITY_STEP <= View_I::MAX_QUALITY) {
      my_sighting.q += View_I::QUALITY_STEP;
    }
  }
  else {
    // The Locatable has changed a location.   
    my_location = location;
    my_sighting.my_location = location->item_id();
    my_sighting.q = View_I::MIN_QUALITY + View_I::QUALITY_STEP;
  }
  // Update other sighting data.
  my_sighting.message_type = sighting_descr.message_type;
  my_sighting.attributes = sighting_descr.attributes;
  my_sighting.sequence_number = sighting_descr.sequence_number;
  my_sighting.update_time = sighting_descr.update_time;
}

void Locatable_I::set_home_sd(Comparable_Sensor_Descr *home_sd_p) {

  my_home_sd = home_sd_p;
  Location_I *loc = Location_Rep_I::instance()->resolve_sensor(
    my_home_sd->id);
  my_home_location = loc;
  if(loc) {
    my_descr.home_location = 
      ABng_Core::Location::_duplicate(loc->get_corba_object());
  }
  else {
    my_descr.home_location = ABng_Core::Location::_nil();
  }
}

void Locatable_I::remove_location(Location_I *location_p) {

  if(my_sighting.my_location == location_p->item_id()) {
    my_sighting.my_location = (Domain_Repository::Repository_Item_Id) 0;
    my_sighting.message_type = (CORBA::Octet) 0;
    my_sighting.attributes = (CORBA::Octet) 0;
    my_sighting.sequence_number = (ABng_Core::Seq_Number) 0;
    my_sighting.update_time.seconds = (CORBA::ULong) 0;
    my_sighting.update_time.microseconds = (CORBA::ULong) 0;
    my_sighting.q = (ABng_Core::Quality) 0;
    my_location = 0;
  }
  if(location_p == my_home_location) {
    my_home_location = 0;
    my_descr.home_location = ABng_Core::Location::_nil();
  }    
}

#ifdef FilePersistency
void Locatable_I::save(ostream &iostr) {

  iostr << dec << my_descr.item_id << ' ';
  iostr << my_badge_holder_item_id << endl;

  iostr << my_sighting.my_location << ' '
        << (int) my_sighting.message_type << ' '
        << (int) my_sighting.attributes << ' '
	<< (int) my_sighting.sequence_number << ' '
        << my_sighting.update_time.seconds << ' '
	<< my_sighting.update_time.microseconds << ' '
	<< my_sighting.q << endl;
}
#endif

#ifdef FilePersistency
void Locatable_I::restore(istream &iostr) {
  int tmp;

  iostr >> dec >> my_descr.item_id >> my_badge_holder_item_id;
  iostr >> my_sighting.my_location;
  iostr >> tmp;
  my_sighting.message_type= tmp;
  iostr >> tmp;
  my_sighting.attributes= tmp;
  iostr >> tmp;
  my_sighting.sequence_number= tmp;
  iostr >> my_sighting.update_time.seconds >> my_sighting.update_time.microseconds
	>> my_sighting.q;
  my_location= Location_Rep_I::instance()->locatable_came_in( my_sighting.my_location,
							      this);

  if(!iostr) {
    cerr << "Locatable_I error: reading from file" << endl;
    exit(1);
  }
}
#endif

// Convert the id of the corresponding Badge_Holder into its reference.
Locatable_I::Resolution_Result Locatable_I::fast_resolve_ids(
  ABng_Core_System::Badge_Holder_ptr badge_holder_p) {

  if (CORBA::is_nil( badge_holder_p)) return INCONSISTENCY;
  badge_holder = 
    ABng_Core_System::Badge_Holder::_duplicate(badge_holder_p);
  return SUCCESS;
}

// Convert the id of the corresponding Badge_Holder into its reference.
Locatable_I::Resolution_Result Locatable_I::resolve_ids(
  ABng_Core_Management::Badge_Holder_Rep_ptr badge_holder_rep_p) {

  // Find the corresponding BH.
  ABng_Core_System::Badge_Holder::Pattern pattern;
  Domain_Repository::User::Pattern user_pattern;
  pattern.is_any = 0;
  pattern.item_id_pattern.is_any = 0;
  pattern.item_id_pattern.value = my_badge_holder_item_id;
  pattern.my_badge_pattern.is_any = 1;
  pattern.current_badge_pattern.is_any = 1;
  pattern.home_sensor_pattern.is_any = 1;
  pattern.type_specific_object_pattern.is_any = 1;
  pattern.type_specific_object_pattern.object_pattern.user_obj_pattern( user_pattern);
  pattern.assigned_pattern.is_any= 1;
  pattern.visitor_pattern.is_any= 1;

  ABng_Core_System::Badge_Holders_var descriptions;
  try {
    descriptions = Locatable_Rep_I::instance()->get_bh_rep()->
      find(pattern);
  }
  catch(Domain_Repository::Destination_Unreachable &) {
    return IGNORE;
  }
  catch(...) {
    return RES_COMM_FAILURE;
  }
  if(descriptions->length() == 0) {
    // There is no corresponding ABng_LD.
    return INCONSISTENCY;
  }
  badge_holder = 
    ABng_Core_System::Badge_Holder::_duplicate(descriptions[0]);
  return SUCCESS;
}

// Refresh locatable's attributes corresponding to information
// from other repositories (Badge_Holder_Rep, Badge_Rep, Sensor_Rep, User_Rep,
// Equipment_Rep) that could have changed. If the id the badge associated 
// with the locatable has been updated return both the old and the new 
// badge description. If the id the home sensor associated 
// with the locatable has been updated return both the old and the new 
// badge description.

Locatable_I::Refreshing_Result Locatable_I::refresh(
  Comparable_Badge_Descr*& new_bd_p, Comparable_Badge_Descr*& old_bd_p,
  Comparable_Sensor_Descr*& new_sd_p, Comparable_Sensor_Descr*& old_sd_p
  ) {

  int changed_badge = 0; 
  int changed_home_location = 0; 
  int changed_obj = 0;
  int changed_visitors = 0;
  int badge_comm_error = 0;
  int sensor_comm_error = 0;
  int obj_comm_error = 0;
  new_bd_p = 0;
  old_bd_p = 0;
  new_sd_p = 0;
  old_sd_p = 0;

  // Get BH data.
  ABng_Core_System::Badge_Holder::Description_var bhd;
  try {
    bhd = badge_holder->descr();
  }
  catch(Domain_Repository::Destination_Unreachable &ex) {
#ifdef DEBUG
    cerr << "badge_holder->descr() " << ex << " " << ex.server_name << endl;
#endif
    return IGNORE_REFRESH;
  }
  catch(CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "badge_holder->descr() " << ex << endl;
#endif
    return COMM_FAILURE;
  }
  if (my_descr.assigned != bhd->assigned) changed_visitors= 1;
  my_descr.assigned= bhd->assigned;
  if (my_descr.visitor != bhd->visitor) changed_visitors= 1;
  my_descr.visitor= bhd->visitor;

#ifdef ORBIX
  if (CORBA::is_nil( bhd->home_sensor))
    bhd->home_sensor= NULL;
  switch (bhd->type_specific_object._d()) {
  case ABng_Core::LOCATABLE_USER:
    if (CORBA::is_nil( bhd->type_specific_object.user_obj()))
      bhd->type_specific_object.user_obj( NULL);
    break;
  case ABng_Core::LOCATABLE_EQUIPMENT:
    if (CORBA::is_nil( bhd->type_specific_object.equipment_obj()))
      bhd->type_specific_object.equipment_obj( NULL);
    break;
  }
#endif

  // Check if you have the current copy of the badge id. 
  // If not, update it. 
  
  ABng_Core_System::Badge::Description_var new_bd;
  try {
    new_bd = bhd->current_badge->descr();
  }
  catch(...) {
    badge_comm_error = 1;
  }

  if(!badge_comm_error) {
    Comparable_Badge_Id new_cbid(new_bd->id);
    Comparable_Badge_Id old_cbid(my_bd->id);
    // Compare your local copy of the badge id to its current value.
    if(!(new_cbid == old_cbid)) {
      changed_badge = 1;
      old_bd_p = my_bd;
#ifdef ORBIX
      if (CORBA::is_nil( new_bd->holder))
	new_bd->holder= NULL;
#endif
      my_bd = new Comparable_Badge_Descr(new_bd.operator->());
      new_bd_p = my_bd;
    }
  }

  // Check if your home location has changed.
  // If so, update it. 
  if (!CORBA::is_nil( bhd->home_sensor)) {
    ABng_Core_System::Sensor::Description_var new_sd;
    // Get the current home Sensor reference and the Sensor id.
    try {
      new_sd = bhd->home_sensor->descr();
    }
    catch(...) {
      // Cannot communicate with Sensor_Rep. Mark the error.   
      sensor_comm_error = 1;
    }

    if(!sensor_comm_error) {
      // Check if the sensor id has changed.
      Comparable_Sensor_Id new_csid(new_sd->id);
      Comparable_Sensor_Id old_csid(my_home_sd->id);
      if(!(new_csid == new_csid)) {
	old_sd_p = new Comparable_Sensor_Descr(my_home_sd); 
	my_home_sd = new Comparable_Sensor_Descr(new_sd.operator->());
	new_sd_p = my_home_sd;
	Location_I *loc = Location_Rep_I::instance()->resolve_sensor(
				      my_home_sd->id);
	if(my_home_location != loc) {
	  changed_home_location = 1;
	  my_home_location = loc;
	  if(loc) {
	    my_descr.home_location = 
	      ABng_Core::Location::_duplicate(loc->get_corba_object());
	  }
	  else {
	    my_descr.home_location = ABng_Core::Location::_nil();
	  }
	}
      }
    }
  }

  // Check if you have the current copy of the User or Equipment reference. 
  // If not, update it. 

  ABng_Core::Locatable_Type_Specific_Object *new_obj = 
    &bhd->type_specific_object;    

  // Check if the associated object has changed its type.
  if(my_descr.type_specific_object._d() != new_obj->_d()) {
    changed_obj = 1;
  }

  if(!changed_obj) {
    if(new_obj->_d() == ABng_Core::LOCATABLE_USER) {
      Domain_Repository::User_ptr new_user = new_obj->user_obj();
      Domain_Repository::Repository_Item_Id new_item_id;
      Domain_Repository::Repository_Item_Id old_item_id;
      // Get the current User reference and the User id.
      if ( CORBA::is_nil( (Domain_Repository::User_ptr) new_user)) {
	//
      }
      else {
	try {
	  new_item_id = new_user->item_id();
	}
	catch(...) {
	  // Cannot communicate with User_Rep. Mark the error.   
	  obj_comm_error = 1;
	}
	if(!obj_comm_error) {
	  // Check if the old User reference is alive. Get User id.
	  try {
	    if (CORBA::is_nil( my_descr.type_specific_object.user_obj()))
	      changed_obj = 1;
	    else 
	      old_item_id = my_descr.type_specific_object.user_obj()->item_id();
	  }
	  catch(...) {
	    // The reference is invalid, so it is out of date for sure.
	    changed_obj = 1;
	  }
	  if(!changed_obj) {
	    // The old User ref is valid. Check both the new and the old User 
	    // references point to the same User.
	    if(new_item_id != old_item_id) {
	      // They point to different User objects.
	      // This is very unlikely to happen.
	      changed_obj = 1;
	    }
	  }
	}
      }
    }
    else if(new_obj->_d() == ABng_Core::LOCATABLE_EQUIPMENT) {
      Domain_Repository::Equipment_ptr new_equipment = 
	new_obj->equipment_obj();
      Domain_Repository::Repository_Item_Id new_item_id;
      Domain_Repository::Repository_Item_Id old_item_id;
      // Get the current Equipment reference and the User id.

      if ( CORBA::is_nil ((Domain_Repository::Equipment_ptr) new_equipment)) {
	//
      }
      else {
	try {
	  new_item_id = new_equipment->item_id();
	}
	catch(...) {
	  // Cannot communicate with Equipment_Rep. Mark the error.   
	  obj_comm_error = 1;
	}
	if(!obj_comm_error) {
	  // Check if the old Equipment reference is alive. Get Equipment id.
	  try {
	    if (CORBA::is_nil( my_descr.type_specific_object.equipment_obj()))
	      changed_obj = 1;
	    else 
	      old_item_id = my_descr.type_specific_object.equipment_obj()->item_id();
	  }
	  catch(...) {
	    // The reference is invalid, so it is out of date for sure.
	    changed_obj = 1;
	  }
	  if(!changed_obj) {
	    // The old Equipment ref is valid. Check both the new and the 
	    // old Equipment references point to the same Equipment.
	    if(new_item_id != old_item_id) {
	      // They point to different Equipment objects.
	      // This is very unlikely to happen.
	      changed_obj = 1;
	    }
	  }
	}
      }
    }
  }
 
  if(changed_obj) {
    // The copy of the TSO refence is out of date. Update it.
    my_descr.type_specific_object = *new_obj;
    my_sighting.my_locatable_type = my_descr.type_specific_object._d();
  }

  // Compute and return the result.
  if((changed_obj || changed_home_location || changed_visitors) && changed_badge) {
    return CHANGED_BOTH;
  }
  if(changed_obj || changed_home_location || changed_visitors) {
    return CHANGED_EXTERNALS;
  }
  if(changed_badge) {
    return CHANGED_INTERNALS;
  }
  if(badge_comm_error || sensor_comm_error || obj_comm_error) {
    return COMM_FAILURE;
  }
  return UNCHANGED;
}

ABng_Core::Locatables Locatable_I::create_locatable_list() {
  ABng_Core::Locatables ret( 1);
  ret.length( 1);
  ret[ 0]= ABng_Core::Locatable::_duplicate( get_corba_object());
  return ret;
}

