// $Id: locatable_i.h,v 1.2 1998/07/29 14:57:06 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/locatable_i.h,v $
//
// $Log: locatable_i.h,v $
// Revision 1.2  1998/07/29 14:57:06  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:46:55  jasz
// Initial revision
//
//
// File:          locatable_i.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Declaration of the implementation class of the Locatable and 
//  LESP interfaces. To improve performance the LESP interface is not 
//  implemented as a CORBA object but as a usual C++ object.

#ifndef __LOCATABLE_I_H__
#define __LOCATABLE_I_H__

#ifdef ObjectStore
#include <oosa.h>
#endif

#ifdef FilePersistency
#include <fstream.h>
#endif

#include "abng_core_management.hh"
#include "abng_guarded.hh"
#include "comparable_badge_descr.h"
#include "comparable_sensor_descr.h"
#include "compatibility.hh"

class Location_I;

class Locatable_I {

public:

  enum Resolution_Result {
    SUCCESS,
    RES_COMM_FAILURE,
    INCONSISTENCY,
    IGNORE
  };

  enum Refreshing_Result {
    COMM_FAILURE,
    CHANGED_EXTERNALS,
    CHANGED_INTERNALS,
    CHANGED_BOTH,
    UNCHANGED,
    IGNORE_REFRESH
  };

  // The cold start constructor.
  Locatable_I(
    ABng_Core_System::Badge_Holder_ptr badge_holder_p);

  // The constructor used to make a search pattern.
  // It should not be used to create real repository items.
  Locatable_I(Domain_Repository::Repository_Item_Id badge_holder_id_p);

  // The hot start constructor.
#ifdef FilePersistency
   Locatable_I(istream &iostr);
#endif

  virtual ~Locatable_I();

  virtual operator==(const Locatable_I& loc_p) {
    return my_badge_holder_item_id == loc_p.my_badge_holder_item_id;
  }

  // Operations of the Dynamic_Data_Observed_E interface:

  virtual ABng_Core::Dynamic_Data_Observed_E::Dynamic_Data_Observers* 
    dynamic_data_observer_list (
    ENV_DECL1(env) );

  virtual void attach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DECL(env) );
                
  virtual void detach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DECL(env) );

  // Operations of the View_Data_Observed_E interface:

  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    view_data_observer_list (
    ENV_DECL1(env) );
                
  virtual void attach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env) );

  virtual void detach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env) );

  // Operations of the Locatable_Commander interface:

  virtual CORBA::Long issue_command (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );
    
  virtual CORBA::Long challenge (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& random, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );
  
  virtual CORBA::Long sound (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );
  
  virtual CORBA::Long set_state (
    const ABng_Core::Locations& location_list, 
    CORBA::Octet status, 
    const ABng_Core::Octets& sound, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );
  
  virtual CORBA::Long get_state (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );
  
  virtual CORBA::Long set_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Octets& address, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );
  
  virtual CORBA::Long get_home (
    const ABng_Core::Locations& location_list, 
    const ABng_Core::Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DECL( IT_env) );

  // Operations of Request_Queue:
  virtual ABng_Core::Request_Queue::Requests* list_user_requests (
    const char * user_id
    ENV_DECL( IT_env) );

  virtual ABng_Core::Request_Queue::Requests* list_all_requests (
    ENV_DECL1( IT_env) );

  virtual void remove_request (CORBA::Long request_id
    ENV_DECL( IT_env) );

  // Operations of the Locatable interface:

  virtual Domain_Repository::Repository_Item_Id item_id (
    ENV_DECL1( IT_env) ) {
    return my_descr.item_id;
  }

  virtual CORBA::Boolean assigned( ENV_DECL1( env));
  virtual CORBA::Boolean visitor( ENV_DECL1(env));

  virtual ABng_Core::Locatable::Description* descr (
    ENV_DECL1( IT_env) );

  virtual ABng_Core::Locatable::Value_Description* value_descr (
    ENV_DECL1( IT_env) );

  virtual ABng_Core::Location_ptr home_location (
    ENV_DECL1(env));
  
  virtual ABng_Core::Locatable_Type_Specific_Object* 
    type_specific_object (
    ENV_DECL1(env) );

  virtual ABng_Core::Sighting_Description* get_sighting (
    ENV_DECL1(env) );

  // Operations of the LESP interface:

  /*
  virtual ABng_Core::Locatable_ptr my_locatable (
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw (CORBA::SystemException);
  
  virtual ABng_Core_System::Badge_Holder_ptr my_badge_holder (
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw (CORBA::SystemException);

  virtual ABng_Core_Internals::Locatable_Sighting_Processor::Sighting* 
    current_sighting (
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw (CORBA::SystemException);
 
  virtual void process_sighting (
    const ABng_Core_Internals::Internal_Sighting_Description& isd, 
    ABng_Core::Location_ptr loc, 
    CORBA::Environment &IT_env=CORBA::default_environment) 
    throw (CORBA::SystemException);
  */

  virtual void process_sighting (
    const ABng_Core_Internals::Internal_Sighting_Description& isd, 
    Location_I *location
    ENV_DECL(env) );

  // Non-IDL public methods.

  // Accessors:

  virtual void set_descr(
    const ABng_Core::Locatable::Description& descr_update) {
    my_descr = descr_update;
    
  }

  const ABng_Core::Locatable::Description& get_descr() const {
    return my_descr;
  }

  virtual void set_corba_object(
    ABng_Core::Locatable_ptr locatable_corba_object_p) {
    my_locatable_corba_object = locatable_corba_object_p;
  }

  ABng_Core::Locatable_ptr get_corba_object() {
    return my_locatable_corba_object;
  }

  /*
  virtual void set_lesp_corba_object(
    ABng_Core_Internals::Locatable_Sighting_Processor_ptr 
      lesp_corba_object_p) {
    my_lesp_corba_object = lesp_corba_object_p;
  }

  ABng_Core_Internals::Locatable_Sighting_Processor_ptr 
    get_lesp_corba_object() {
    return my_lesp_corba_object;
  }
  */

  virtual void set_item_id(
    const Domain_Repository::Repository_Item_Id item_id_p) {
    my_descr.item_id = item_id_p;
    my_sighting.my_locatable = my_descr.item_id;
  }

  virtual ABng_Core_System::Badge_Holder_var get_badge_holder() {
    return badge_holder;
  }

  virtual Domain_Repository::Repository_Item_Id 
    get_badge_holder_item_id() const {
    return my_badge_holder_item_id;
  }

  virtual Comparable_Badge_Descr *get_bd() {
    return my_bd;
  }

  virtual void set_bd(Comparable_Badge_Descr *bd_p) {
    my_bd = bd_p;
  }

  virtual Comparable_Sensor_Descr *get_home_sd() {
    return my_home_sd;
  }

  virtual void set_home_sd(Comparable_Sensor_Descr *home_sd_p);

  virtual Location_I *get_location() const {
    return my_location;
  }

  virtual void remove_location(Location_I *location_p);

  virtual ABng_Core::Sighting_Description &local_get_sighting () {
    return my_sighting;
  }

  ABng_Core::Locatables create_locatable_list();

#ifdef ObjectStore
  // method for ObjectStore
  static os_typespec* get_os_typespec();
#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void save(ostream &iostr);
  virtual void restore(istream &iostr);
#endif

  // Convert the id of the corresponding Badge_Holder into its reference.
  Resolution_Result resolve_ids(
    ABng_Core_Management::Badge_Holder_Rep_ptr badge_holder_rep_p);
  Resolution_Result fast_resolve_ids(
    ABng_Core_System::Badge_Holder_ptr badge_holder_p);

  // Refresh locatable's attributes corresponding to information
  // from other repositories (Badge_Holder_Rep, Badge_Rep) that could have 
  // changed. If the id the badge associated with the locatable
  // has been updated return both the old and the new badge description.
  virtual Refreshing_Result refresh(
    Comparable_Badge_Descr*& new_bd, Comparable_Badge_Descr*& old_bd,
    Comparable_Sensor_Descr*& new_sd, Comparable_Sensor_Descr*& old_sd);


  Location_I *get_home_location() const { return my_home_location; }
protected:

  ABng_Core::Locatable_ptr my_locatable_corba_object;
  /*
  ABng_Core_Internals::Locatable_Sighting_Processor_var my_lesp_corba_object;
  */

  ABng_Core::Locatable::Description my_descr;
  ABng_Core_System::Badge_Holder_var badge_holder;
  Domain_Repository::Repository_Item_Id my_badge_holder_item_id;
  Comparable_Badge_Descr *my_bd;  
  Comparable_Sensor_Descr *my_home_sd;  
  ABng_Core::Sighting_Description my_sighting;

  // The location in which a locatable is currently seen.
  Location_I *my_location;
  Location_I *my_home_location;
};


#endif  /* __LOCATABLE_H__ */



