// -*- Mode: C++; -*-
// $Id: locatable_rep_i.h,v 1.2 1998/07/29 14:57:07 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/locatable_rep_i.h,v $
//
// $Log: locatable_rep_i.h,v $
// Revision 1.2  1998/07/29 14:57:07  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:47:13  jasz
// Initial revision
//
//
// File:          locatable_rep_i.h
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Declaration of the implementation class for the "locatable part" of View,
//  VSP and View_Manager interfaces.

#ifndef __LOCATABLE_REP_I_H__
#define __LOCATABLE_REP_I_H__

#include <rw/rwdate.h>
#include <rw/rwtime.h>
#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <rw/tvhdict.h>
#include <rw/tvdlist.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "abng_core_management.hh"
#include "locatable_i.h"
#include "basic_rep_i.h"
#include "observed_i.h"
#include "view_rep_types.h"
#include "comparable_badge_id.h"
#include "comparable_sensor_id.h"
#include "dyn_dispatcher.hh"

class Locatable_Types {

public:

  typedef ABng_Core::Locatable Item;  
  typedef ABng_Core::Locatable_ptr Item_ptr;  
  typedef ABng_Core::Locatables Items;  
  typedef Locatable_I Item_I;  
  typedef ABng_Core_System::Badge_Holder_ptr Initial_Data;
  typedef ABng_Core::View_Data_Observer::View_Data_Object Data_Object;
};

class Locatable_Rep_I : public No_Update_Basic_Rep_I<Locatable_Types, VDO_Types> { 

public:

  // Hot start constructor
  Locatable_Rep_I (
#ifdef ObjectStore
    os_database* db_p,
#elif defined FilePersistency
    const char *repository_name_p,
#endif
    const char *view_name_p,
    long sightings_dump_period_p,
    unsigned long burst_delay,
    int
  );

  // Cold start constructor
  Locatable_Rep_I (
#ifdef ObjectStore
    os_database* db_p,
#elif defined FilePersistency
    const char *repository_name_p,
#endif
    const char *view_name_p,
    long sightings_dump_period_p,
    unsigned long burst_delay
  );

  virtual ~Locatable_Rep_I ();
  static Locatable_Rep_I *instance ();

  void set_burst_delay( unsigned long new_delay) {
    my_dynamic_observed.set_burst_delay( new_delay);
  }

  void load_dynamic_observers() { my_dynamic_observed.load_observers(); }

  // Operations of Dynamic_Data_Observed_R
  
  virtual ABng_Core::Dynamic_Data_Observed_R::Observer_Entries* 
    dynamic_data_observer_list (
    ENV_DECL1(env)) {

    return my_dynamic_observed.observer_list( USE_ENV1(env));
  }

  virtual void attach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_dynamic_observed.attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    my_dynamic_observed.detach_observer(obs, observed_id_list USE_ENV(env));
  }

  // Suplementary operation

  virtual ABng_Core::Dynamic_Data_Observed_E::Dynamic_Data_Observers*
    dynamic_data_observer_list (
    Domain_Repository::Repository_Item_Id item_id_p
    ENV_DECL(env)) {
       
    return my_dynamic_observed.get_item_observers(item_id_p USE_ENV(env));
  }

  // Operations of Domain_Repository::View_Data_Observed_R:

  virtual ABng_Core::View_Data_Observed_R::Observer_Entries* 
    view_data_observer_list (
    ENV_DECL1(env)) {
   
    return observer_list( USE_ENV1(env));
  }

  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    add_view_data_observer_list (
    ENV_DECL1(env)) {
   
    return add_observer_list( USE_ENV1(env));
  }

  virtual void attach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    attach_observer(obs, observed_id_list USE_ENV(env));
  }

  virtual void detach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs, 
    const Domain_Repository::Repository_Item_Ids& observed_id_list
    ENV_DECL(env)) {

    detach_observer( obs, observed_id_list USE_ENV(env));
  }

  virtual void attach_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    attach_add_observer(obs USE_ENV(env));
  }

  virtual void detach_add_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DECL(env)) {

    detach_add_observer(obs USE_ENV(env));
  }

  // Suplementary operation

  virtual ABng_Core::View_Data_Observed_E::View_Data_Observers*
    view_data_observer_list (
    Domain_Repository::Repository_Item_Id item_id_p
    ENV_DECL(env)) {
       
    return get_item_observers(item_id_p USE_ENV(env));
  }

  // IDL operations of the View and View_Manager interface
  
  virtual ABng_Core::Locatables* locatable_list (
    ENV_DECL1(env)) {

    return item_list( USE_ENV1(env));
  }

  virtual ABng_Core::Locatables* find (
    const ABng_Core::Locatable::Pattern& data_record_pattern
    ENV_DECL(env)) {

    return No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::find(
      data_record_pattern USE_ENV(env));
  }


  virtual ABng_Core::Locatable::Value_Descriptions* find_values (
    const ABng_Core::Locatable::Pattern& data_record_pattern
    ENV_DECL(env)) {
  
    return No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::find_values(
      data_record_pattern USE_ENV(env));
  }

  virtual ABng_Core::Locatable::Query_Results*
   complex_query(
		 const ABng_Core::Locatable::Entity_Set &es,
		 const Domain_Repository::Query_Options &qo,
		 ABng_Core::View_Data_Observer_ptr obs
		 ENV_DECL(env)) {

     return No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::complex_query(
		   es,qo,obs USE_ENV(env));
  }

  virtual ABng_Core::Locatable_ptr add (
    const ABng_Core_System::Badge_Holder_ptr& new_object
    ENV_DECL(env)) {

    return No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::add(new_object USE_ENV(env));
  }

  virtual void remove (
    ABng_Core::Locatable_ptr object_to_remove
    ENV_DECL(env)) {

    No_Update_Basic_Rep_I<Locatable_Types, VDO_Types>::remove(object_to_remove USE_ENV(env));
  }

  virtual ABng_Core_System::Badge_Holder_ptr deref(
    ABng_Core::Locatable_ptr lct_object
    ENV_DECL(env));

  virtual Domain_Repository::Location_Type::Location_Type_Name type (
    ENV_DECL1( IT_env));

  virtual ABng_Core::Sighting_Descriptions* get_sightings (
    const ABng_Core_System::Time& since
    ENV_DECL(env) );
    
  // Operations of the ABng_Core_System::System_Static_Data_Observer interface.
  // Used for intra-core notification mechanism.
  virtual void notify(
    Domain_Repository::Repository_Item_Id id,
    const ABng_Core_System::System_Static_Data_Observer::System_Static_Data_Object& sd, 
    Domain_Repository::Change_Type change
    ENV_DECL(env));

  // Non-IDL public methods used by Locatable_I and View_I classes:

  // Accessors:
  virtual ABng_Core_Management::Badge_Holder_Rep_var get_bh_rep() {
    return my_bh_rep;
  }
  virtual char *get_view_name() {
    return my_view_name;
  }

  // Get a locatable containing the badge with the given id.
  virtual Locatable_I *resolve_badge(const ABng_Core_System::Badge_Id& bid);

  // Notify dynamic data observers registered within the locatable
  virtual void dynamic_notify(Locatable_I *locatable_p);

  // Detach the location for the locatable.
  virtual void remove_location(Location_I *location_p);

  // Refresh all the information that could have changed in the BH_Rep.
  // This method should be called after the BH_Rep is launched.
  virtual void resolve_bh_rep(
    ABng_Core_Management::Badge_Holder_Rep_var bh_rep_p);

  virtual void resolve_badge_rep(
    ABng_Core_Management::Badge_Rep_var badge_rep_p);

  // Send the list of all badges belonging to the view to the MSP.
  // This method should be called after the MSP is launched.
  virtual void resolve_msp(
    ABng_Core_Internals::Main_Sighting_Processor_var msp_p);

  // Generate a list (sequence) of all badges belonging to the view.
  virtual ABng_Core_System::Badge_Ids *get_badge_list();

  ABng_Core_System::Badge_Ids locatables2badge_ids(
    const ABng_Core::Locatables *locs);
  ABng_Core::Locatables badge_ids2locatables(
    const ABng_Core_System::Badge_Ids *bids);
  ABng_Core::Locatable_var badge_id2locatable(
    const ABng_Core_System::Badge_Id &bid)
    throw( ABng_Core::Invalid_Argument);

#ifdef ObjectStore
  static os_typespec* get_os_typespec();
#endif

protected:

  // calculate the hash value of the locatable
  static unsigned hash_locatable(const Locatable_I& locatable_i_p); 

  // definition of structure containing parameters of match_locatable method

  struct MatchLocatableApplyDataStruct {
    RWTValDlist<Locatable_I*> _found_locatables;
    const ABng_Core::Locatable::Pattern& _data_record_pattern;

    MatchLocatableApplyDataStruct(
	   const ABng_Core::Locatable::Pattern& _data_record_pattern_p) :
      _data_record_pattern( _data_record_pattern_p) {}
  };

  // if the current locatable matches the pattern puts it into the container
  static void
    match_locatable( Locatable_I* current_locatable,
		     void *apply_data);
		   
  // Add helper methods
  virtual void init_corba_object(Item_I *item_p);
  virtual void insert_item(Item_I* item_p);

  // Remove helper methods
  virtual void remove_item(Item_I* item_p);

  // Find helper methods
  virtual ABng_Core::Locatables* private_find (
    const ABng_Core::Locatable::Pattern& data_record_pattern
    ENV_DECL(env));
  virtual void collect_values(Locatable_I *lt, 
   ABng_Core::Locatable::Value_Description& ld);

  virtual Description get_null_description( Item_I *item);
  virtual Value_Description get_null_value_description( Item_I *item);

  // Common helper methods
  virtual void construct_data_object(
    ABng_Core::View_Data_Observer::View_Data_Object& data_object_p,
    ABng_Core::Locatable_ptr item_p);
  virtual void save();
  virtual void save(ostream &);
  virtual void restore (istream &);
  virtual DB_Header *create_db_header();
  virtual void create_corba_objects() {}
  virtual void destroy_corba_objects() {}

  // Helper metods used for intra-core notification mechanism:

  // Refresh locatable's attributes corresponding to information
  // from other repositories (BH_Rep, Sensor_Rep, User_Rep, Equipement_Rep) 
  // that could have changed. If notify_msp is non-zero the MSP is 
  // immediately notified if a badge associated with the locatable has changed.
  virtual Locatable_I::Refreshing_Result refresh(Locatable_I *item_p,
    int notify_msp = 1);

  // Replace the local copy of the badge description with the current value.
  virtual void replace_badge(ABng_Core_System::Badge_ptr badge_p);

  // Replace the local copy of the home sensor description with the 
  // current value.
  virtual void replace_sensor(ABng_Core_System::Sensor_ptr sensor_p);

  // Replace the local copy of location's badge description with the 
  // current value. Notify the MSP if it is required.
  virtual void update_badge(Locatable_I *item_p,
    Comparable_Badge_Descr *old_bd, 
    Comparable_Badge_Descr *new_bd, 
    int notify_msp = 1);

  // Replace the local copy of location's home sensor description with the 
  // current value.
  virtual void update_sensor(Locatable_I *item_p,
    Comparable_Sensor_Descr *old_sd, 
    Comparable_Sensor_Descr *new_sd);

  static Locatable_Rep_I* _instance;

  // References of related objects.
  ABng_Core_Management::Badge_Holder_Rep_var my_bh_rep;
  ABng_Core_Management::Badge_Rep_var my_badge_rep;
  ABng_Core_Internals::Main_Sighting_Processor_var my_msp; 

  // The registry of dynamic data observers.
  DynDispatcher my_dynamic_observed;

  // Dictionaries necessary to serve events originating from MSP
  // (sightings) and Badge_Rep.  
  typedef RWTValHashDictionary<Domain_Repository::Repository_Item_Id, 
    Locatable_I*> Badge_Item_Id_Locatable_Dictionary;
  typedef RWTValHashDictionaryIterator<Domain_Repository::Repository_Item_Id, 
    Locatable_I*> Badge_Item_Id_Locatable_Dictionary_Iterator;

  typedef RWTValHashDictionary<Comparable_Badge_Id, Locatable_I*>
    Badge_Id_Locatable_Dictionary;
  typedef RWTValHashDictionaryIterator<Comparable_Badge_Id, Locatable_I*>
    Badge_Id_Locatable_Dictionary_Iterator;

  Badge_Id_Locatable_Dictionary my_badge_ids_to_locatables;
  Badge_Item_Id_Locatable_Dictionary my_badge_item_ids_to_locatables;

  // Dictionaries necessary to serve events originating from MSP
  // (sightings) and Sensor_Rep.
  typedef RWTValHashDictionary<Domain_Repository::Repository_Item_Id, 
    Locatable_I*> Sensor_Item_Id_Locatable_Dictionary;
  typedef RWTValHashDictionaryIterator<Domain_Repository::Repository_Item_Id, 
    Locatable_I*> Sensor_Item_Id_Locatable_Dictionary_Iterator;

  Sensor_Item_Id_Locatable_Dictionary my_sensor_item_ids_to_locatables;

  CORBA::String_var my_view_name;
  RWTime last_save_time;
  long sightings_dump_period;

};

#endif /* __LOCATABLE_REP_I_H__ */

