// $Id: location_i.cc,v 1.2 1998/07/29 14:57:07 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/servers/sighting_processors/location_i.cc,v $
//
// $Log: location_i.cc,v $
// Revision 1.2  1998/07/29 14:57:07  mencnar
// performance tuning,
// receive sightings calls moved to dyn_dispatcher.* for future optimisation
//
// Revision 1.1.1.1  1998/06/18 13:37:37  mencnar
//
// Revision 1.1  1998/03/06 13:40:53  jasz
// Initial revision
//
//
// File:          location_i.cc
// Author:        Jakub Szymaszek (jasz@ics.agh.edu.pl)
// Date Created:  26.01.98
//
// General Description:
//  Definition the implementation class of the Location and
//  LSP interfaces. To improve performance the LSP interface is not 
//  implemented as a CORBA object but as a usual C++ object.

#ifdef ORBIX
#define EXCEPTIONS
#endif

#include <rw/rwdate.h>
#include <rw/rwdate.h>
#include <stdlib.h>
#include "location_rep_i.h"
#include "view_i.h"
#include "db_utils.hh"

#ifdef ObjectStore
static os_typespec *char_type = new os_typespec("char");
#endif

int operator >= ( const ABng_Core::Time &t1,
		  const ABng_Core::Time &t2) {

  ABng_Core::Time x1(t1), x2(t2);
  x1.seconds+= x1.microseconds/1000000; // normalize time
  x2.seconds+= x2.microseconds/1000000;

  if (x1.seconds>x2.seconds) return 1;
  if (x1.seconds<x2.seconds) return 0;
  if (x1.microseconds>=x2.microseconds) return 1;
  return 0;
}

// The cold start constructor.
Location_I::Location_I (
    ABng_Core_System::ABng_Location_Description_ptr abng_ld_p) :
    my_abng_ld( ABng_Core_System::ABng_Location_Description::_duplicate( abng_ld_p)),
    my_sds(Comparable_Sensor_Descr::hash),
    my_locatables(Locatable_Rep_I::hash_locatable) {

  try {
    my_descr.loc_descr = my_abng_ld->loc_descr();
    my_abng_ld_item_id = my_abng_ld->item_id();
  }
  catch( CORBA::Exception &ex) {
    cerr << ex << endl;
    throw( Domain_Repository::Destination_Unreachable(
      "abng_location_description_rep"));
  }
  catch (...) {
    cerr << "Caught unknown exception" << endl;
    throw( Domain_Repository::Destination_Unreachable(
      "abng_location_description_rep"));    
  }

  // Check if the location has a type assumed for the view.
  CORBA::String_var type_name; 
  try {
    type_name = my_descr.loc_descr->type_name();
  }
  catch ( CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << ex << endl;
#endif
    throw Domain_Repository::Destination_Unreachable(
      "location_description_rep");
  }
  if(strcmp( (const char *) type_name, 
	     Location_Rep_I::instance()->get_location_type_name()) != 0) {
#ifdef DEBUG
    cerr << "Invalid type name " << type_name << endl;
#endif
    throw (ABng_Core_Management::View_Manager::Invalid_Location_Type());
  }

  ABng_Core_System::Sensors_var sensors;
  try {
    sensors = abng_ld_p->sensor_list();
  }
  catch(...) {
    throw( Domain_Repository::Destination_Unreachable(
      "abng_location_description_rep"));
  }
  Comparable_Sensor_Descr *sd;
  for(int i = 0; i < sensors->length(); i++) {
    sd = new Comparable_Sensor_Descr(sensors[i]->descr());
    try {
      my_sds.insert(sd);
    }
    catch(...) {
      throw( Domain_Repository::Destination_Unreachable("sensor_rep"));
    }
  }
}

// The constructor used to make a search pattern.

Location_I::Location_I(Domain_Repository::Repository_Item_Id abng_ld_item_id_p) : my_abng_ld_item_id(abng_ld_item_id_p),
 my_sds(Comparable_Sensor_Descr::hash),
 my_locatables(Locatable_Rep_I::hash_locatable)  {
}

// The hot start constructor.
#ifdef FilePersistency
Location_I::Location_I(istream &iostr) :
  my_sds(Comparable_Sensor_Descr::hash),
  my_locatables(Locatable_Rep_I::hash_locatable) {

  // read description from the file
  restore(iostr);
  my_abng_ld = ABng_Core_System::ABng_Location_Description::_nil();
  my_descr.loc_descr = Domain_Repository::Location_Description::_nil();
}
#endif

Location_I::~Location_I() {

  my_sds.clearAndDestroy();
}

// Operations of the Dynamic_Data_Observed_E interface:

ABng_Core::Dynamic_Data_Observed_E::Dynamic_Data_Observers* 
    Location_I::dynamic_data_observer_list (
    ENV_DEF1(env)) {

  return Location_Rep_I::instance()->dynamic_data_observer_list(
    my_descr.item_id USE_ENV(env));
}

void Location_I::attach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this location_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);
      
  // put this location_type item id into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
      
  // register observer into the repository list
  Location_Rep_I::instance()->attach_dynamic_data_observer(obs,
    observed_id_list_p USE_ENV(env));
	
}
              
void Location_I::detach_dynamic_data_observer (
    ABng_Core::Dynamic_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this location_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

  // put this location_type item it into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
	
  // register observer into the repository list
  Location_Rep_I::instance()->detach_dynamic_data_observer(obs,
    observed_id_list_p USE_ENV(env));
}

// Operations of the View_Data_Observed_E interface:

ABng_Core::View_Data_Observed_E::View_Data_Observers* 
    Location_I::view_data_observer_list (
    ENV_DEF1(env)) {

  return Location_Rep_I::instance()->view_data_observer_list(
    my_descr.item_id USE_ENV(env));
}
                
void Location_I::attach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this location_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);
      
  // put this location_type item id into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
      
  // register observer into the repository list
  Location_Rep_I::instance()->attach_view_data_observer(obs,
    observed_id_list_p USE_ENV(env));
}

void Location_I::detach_view_data_observer (
    ABng_Core::View_Data_Observer_ptr obs
    ENV_DEF(env)) {

  // initialize the sequence for this location_type item id
  Domain_Repository::Repository_Item_Ids observed_id_list_p(1);

  // put this location_type item it into the sequance
  observed_id_list_p.length(1);
  observed_id_list_p[0] = my_descr.item_id;
	
  // register observer into the repository list
  Location_Rep_I::instance()->detach_view_data_observer(obs,
    observed_id_list_p USE_ENV(env));
}

// Operations of the Location_Commander interface:

CORBA::Long Location_I::issue_location_command (
    const Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->issue_location_command(
	 create_location_list(),
	 cmd,
	 has_reply,
	 reply_code,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::issue_locatable_command (
    const Locatables& locatable_list, 
    const Octets& cmd, 
    CORBA::Boolean has_reply,
    CORBA::Octet reply_code,
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->issue_locatable_command(
	 create_location_list(),
	 locatable_list,
	 cmd,
	 has_reply,
	 reply_code,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::challenge (const Locatables& locatable_list, 
    const Octets& random, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->challenge(
	 create_location_list(),
         locatable_list,
	 random,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::sound (const Locatables& locatable_list, 
    const Octets& sound, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->sound(
	 create_location_list(),
	 locatable_list,
	 sound,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::set_state (const Locatables& locatable_list, 
    CORBA::Octet status,
    const Octets& sound,
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->set_state(
	 create_location_list(),
         locatable_list,
	 status,
	 sound,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::get_state (const Locatables& locatable_list, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->get_state(
	 create_location_list(),
         locatable_list,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::set_home (const Locatables& locatable_list, 
    const Octets& address, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->set_home(
	 create_location_list(),
         locatable_list,
	 address,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::get_home (const Locatables& locatable_list, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->get_home(
	 create_location_list(),
         locatable_list,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::reset_buffer (const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->reset_buffer(
         create_location_list(),
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::reset_fifos (const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->reset_fifos(
         create_location_list(),
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::get_station_state (const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->get_station_state(
         create_location_list(),
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::set_output (CORBA::Octet output_value, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->set_output(
	 create_location_list(),
	 output_value,
	 ctx,
	 reply_to);
}
           
CORBA::Long Location_I::set_persistence (CORBA::Boolean mode, 
    const Command_Context& ctx, 
    ABng_Core::Caller_ptr reply_to
    ENV_DEF( IT_env)) {

  return View_I::instance()->set_persistence(
         create_location_list(),
	 mode,
	 ctx,
	 reply_to);
}

// Operations of the Request_Queue interface:

ABng_Core::Request_Queue::Requests* Location_I::list_user_requests (
    const char * user_id
    ENV_DEF( IT_env)) {

  return View_I::instance()->list_user_requests( user_id);
}

ABng_Core::Request_Queue::Requests* Location_I::list_all_requests (
    ENV_DEF1( IT_env)) {

  return View_I::instance()->list_all_requests();
}

void Location_I::remove_request (CORBA::Long request_id
    ENV_DEF( IT_env)) {

  View_I::instance()->remove_request( request_id);
}

// Operations of the Location interface:

ABng_Core::Location::Description* Location_I::descr (
    ENV_DEF1(env)) {

  if (CORBA::is_nil((ABng_Core_System::ABng_Location_Description_ptr) my_abng_ld))
    throw Domain_Repository::Destination_Unreachable("abng_location_description_rep");

  return new ABng_Core::Location::Description(my_descr);
}

ABng_Core::Location::Value_Description* Location_I::value_descr (
    ENV_DEF1( IT_env)) {

  if (CORBA::is_nil((ABng_Core_System::ABng_Location_Description_ptr) my_abng_ld))
    throw Domain_Repository::Destination_Unreachable("abng_location_description_rep");

  ABng_Core::Location::Value_Description* ret=
    new ABng_Core::Location::Value_Description();

  try {
    ret->item_id= my_descr.item_id;
    if (!CORBA::is_nil( my_descr.loc_descr)) {
      Domain_Repository::Location_Description::Value_Description_var
	tmp( my_descr.loc_descr->value_descr());
      ret->loc_value_descr= *(tmp.operator->());
    }
  }
  catch (CORBA::Exception &ex) {
    delete ret;
    throw Domain_Repository::Destination_Unreachable("location_description_rep");
  }
  return ret;
}

Domain_Repository::Location_Description_ptr Location_I::loc_descr (
    ENV_DEF1(env)) {

  if (CORBA::is_nil((ABng_Core_System::ABng_Location_Description_ptr) my_abng_ld))
    throw Domain_Repository::Destination_Unreachable("abng_location_description_rep");

  return Domain_Repository::Location_Description::_duplicate(
    my_descr.loc_descr);
}

ABng_Core::Sighting_Descriptions* Location_I::get_sightings (
    const ABng_Core::Time& since
    ENV_DEF(env)) {

  Locatable_List_Iterator iterator(my_locatables);
  int len = my_locatables.entries();
  ABng_Core::Sighting_Descriptions *sightings = 
    new ABng_Core::Sighting_Descriptions(len);

  for(int i = 0; iterator(); ) {
    ABng_Core::Sighting_Description &sd= iterator.key()->local_get_sighting();
    if (sd.update_time >= since) {
      sightings->length( i+1);
      (*sightings)[i++] = sd;
    }
  }
  return sightings; 
}


// A locatable entered or left the location. Attach or detach it 
// to or from the location.

void Location_I::process_sighting (
  Locatable_I *locatable_sp,
  Location_I::Event_Descriptor event
  ENV_DEF(env)) {

  if(event == CAME_OUT) {
#ifdef DEBUG
    cerr << "CAME_OUT " << locatable_sp->item_id() << endl;
#endif
    my_locatables.remove(locatable_sp);
  }
  if(event == CAME_IN) {
#ifdef DEBUG
    cerr << "CAME_IN " << locatable_sp->item_id() << endl;
#endif
    my_locatables.insert(locatable_sp);
  }
}
  
#ifdef FilePersistency
void Location_I::save(ostream &iostr) {

#ifdef DEBUG
  // cerr << "Location_I: saving Location state to file" << endl; 
#endif

  iostr << dec << my_descr.item_id << ' ';
  iostr << my_abng_ld_item_id << endl;

#ifdef DEBUG
  // cerr << "Location_I: state saved" << endl; 
#endif

}
#endif

#ifdef FilePersistency
void Location_I::restore(istream &iostr) {

  iostr >> dec >> my_descr.item_id >> my_abng_ld_item_id;
  if(!iostr) {
    cerr << "Location_I error: reading from file" << endl;
    exit(1);
  }
}
#endif


void Location_I::set_descr(
  const ABng_Core::Location::Description& descr_update) {

  my_descr = descr_update;
}

// Refresh all the information that could have changed in the 
// corresponding ABng_LD.

Location_I::Resolution_Result Location_I::resolve_ids(
  ABng_Core_Management::ABng_Location_Description_Rep_ptr abng_ld_rep_p) {
  
  // Find the corresponding ABng_LD.
  ABng_Core_System::ABng_Location_Description::Pattern pattern;
  pattern.is_any = 0;
  pattern.item_id_pattern.is_any = 0;
  pattern.item_id_pattern.value = my_abng_ld_item_id;
  pattern.loc_descr_pattern.is_any = 1;
  pattern.loc_descr_pattern.property_list_pattern.constraints= Domain_Repository::UNRESTRICTED;
  pattern.sensor_list_pattern.is_any = 1;
  pattern.sensor_list_pattern.constraints= Domain_Repository::UNRESTRICTED;

  ABng_Core_System::ABng_Location_Descriptions_var descriptions;
  try {
    descriptions = abng_ld_rep_p->find(pattern);
  }
  catch (Domain_Repository::Destination_Unreachable &ex_du) {
#ifdef DEBUG
    cerr << "Location_I::resolve_ids destination unreachable " << ex_du.server_name << endl;
#endif
    return IGNORE;
  }
  catch (CORBA::Exception &ex) {
#ifdef DEBUG
    cerr << "Location_I::resolve_ids exception " << ex << endl;
#endif
    return RES_COMM_FAILURE;
  }
  catch(...) {
    return RES_COMM_FAILURE;
  }
  if(descriptions->length() == 0) {
    // There is no corresponding ABng_LD.
    return INCONSISTENCY;
  }
  my_abng_ld = 
    ABng_Core_System::ABng_Location_Description::_duplicate( descriptions[0]);
  return SUCCESS;
}

// Refresh the location attributes corresponding to information
// from other repositories (ABng_LD_Rep, Sensor_Rep) that could have 
// changed. If the sensor list associated with the location has been
// updated return both the old and the new sensor lists.

Location_I::Refreshing_Result Location_I::refresh(
  Sensor_Descr_List& old_sds, Sensor_Descr_List& new_sds) {

  int changed_sensors = 0; 
  int changed_ld = 0;
  int ld_comm_error = 0;
  int sensor_comm_error = 0;
  //  old_sds = new_sds = 0;

  // Check if you have the current copy of the sensor list. 
  // If not, update it. 

  // Get the current sensor list (sequence). 
  ABng_Core_System::Sensors_var sensors;
  try {
    sensors = my_abng_ld->sensor_list();
  }
  catch (Domain_Repository::Destination_Unreachable &) {
    return IGNORE_REFRESH;
  }  
  catch(...) {
    // Cannot communicate with ABng_LD_Rep.
    return COMM_FAILURE;
  }

  // Copy the sensor sequence to a list.
  Comparable_Sensor_Descr *csd;
  for(int i = 0; i < sensors->length(); i++) {
    ABng_Core_System::Sensor::Description_var sd = sensors[i]->descr();
    csd = new Comparable_Sensor_Descr(sd.operator->());
    try {
      new_sds.insert(csd);
    }
    catch(...) {
      // Cannot communicate with Sensor_Rep. 
      // Delete the sensor list. Mark the error.
      new_sds.clearAndDestroy();
      sensor_comm_error = 1;
      break;
    }
  }

  if(!sensor_comm_error) {
    // Compare the new sensor list to the old one
    if(new_sds.entries() != my_sds.entries()) {
      // They are of different length.
      changed_sensors = 1;
    }
    else {
      // They are of the same length.
      Sensor_Descr_List_Iterator iterator(my_sds);
      while (iterator()) {
        if(!new_sds.find(iterator.key())) {
	  // They have at least one pair of different sensors.
	  changed_sensors = 1;
        }
      }
    }
    if(changed_sensors) { 
      // The sensor list has been updated. 
      // Store the old list and save the new one.
      old_sds = my_sds;
      my_sds = new_sds;
    }
    else {
      // The sensor list has not been updated. 
      new_sds.clearAndDestroy();
    }
  }

  // Check if you have the current copy of the LD reference. 
  // If not, update it. 
  Domain_Repository::Location_Description_var ld;
  Domain_Repository::Repository_Item_Id new_item_id;
  Domain_Repository::Repository_Item_Id old_item_id;
  // Get the current LD reference and the LD id.
  try {
    ld = my_abng_ld->loc_descr();    
    new_item_id = ld->item_id();
  }
  catch (Domain_Repository::Destination_Unreachable &) {
    return IGNORE_REFRESH;
  }  
  catch(...) {
    // Cannot communicate with LD_Rep. Mark the error.   
    ld_comm_error = 1;
  }

  if(!ld_comm_error) {
    // Check if the old LD reference is alive. Get LD id.
    try {
      if (CORBA::is_nil( my_descr.loc_descr)) changed_ld = 1;
      else old_item_id = my_descr.loc_descr->item_id();
    }
    catch(...) {
      // The reference is invalid, so it is out of date for sure.
      changed_ld = 1;
    }
    if(!changed_ld) {
      // The old LD ref is valid. Check both the new and the old LD 
      // references point to the same LD.
      if(new_item_id != old_item_id) {
        // They point to different LD objects.
        // This is very unlikely to happen.
        changed_ld = 1;
      }
    }
    if(changed_ld) {
      // The copy of the LD refence is out of date. Update it.
      my_descr.loc_descr = ld;
    }
  }
  
  // Compute and return the result.
  if(changed_ld && changed_sensors) {
    return CHANGED_BOTH;
  }
  if(changed_ld) {
    return CHANGED_EXTERNALS;
  }
  if(changed_sensors) {
    return CHANGED_INTERNALS;
  }
  if(ld_comm_error || sensor_comm_error) {
    return COMM_FAILURE;
  }
  return UNCHANGED;
}

ABng_Core::Locations Location_I::create_location_list() {
  ABng_Core::Locations ret( 1);
  ret.length( 1);
  ret[ 0]= ABng_Core::Location::_duplicate( get_corba_object());
  return ret;
}



