// -*- Mode: C++; -*-
//
//  File:         user_i.h
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 28.01.98
//
// General Description:
//   Header file for the implementation class of the User interface
//

#ifndef user_i_h
#define user_i_h

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <errno.h>

#ifdef ObjectStore
#include <oosa.h>
#endif

#include "domain_rep_management.hh"
#include "abng_guarded.hh"
#include "abng_string.h"
#include "compatibility.hh"

class User_I
{
 private:
  // state of the IDL User object
  Domain_Repository::User::Description my_descr;

  // reference of CORBA object encapsulated this C++ object
  Domain_Repository::User_ptr my_corba_object;

  // state lock
  ABng_Guarded &my_guard;

 public:
  // ctor
   User_I (
	const Domain_Repository::User::Description& descr_init
   );

#ifdef FilePersistency
   User_I (
	istream &iostr
   );
#endif

  virtual 
   ~User_I ();

   operator==(	
	const User_I&
   );

  // methods enabling access to the state

  void
   set_descr (
	const Domain_Repository::User::Description& descr_update
   );
 
  const Domain_Repository::User::Description&
   get_descr () const;

  void
   set_corba_object (
	const Domain_Repository::User_ptr corba_object_p
   );

  const Domain_Repository::User_ptr
   get_corba_object ();

  void 
   set_item_id(
	const Domain_Repository::Repository_Item_Id item_id_p
   );

  //
  // IDL operations of the Static_Data_Observed_E interface
  //                       ______________________

  virtual Domain_Repository::Static_Data_Observed_E::Static_Data_Observers*
   static_data_observer_list (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
                
  virtual void 
   attach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs  
	ENV_DECL(env)
   )
		throw (CORBA::SystemException);

  virtual void 
   detach_static_data_observer (
	Domain_Repository::Static_Data_Observer_ptr obs
	ENV_DECL(env)
   ) 
		throw (CORBA::SystemException, 
                       Domain_Repository::Unknown_Observer);

  //
  // IDL operations of the User interface
  //                       ____
 
  virtual Domain_Repository::User::Description* 
   descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);
 
  virtual Domain_Repository::User::Value_Description* 
   value_descr (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual Domain_Repository::Repository_Item_Id 
   item_id (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   uid (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   forename (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   surname (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   position (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   email (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

  virtual char* 
   phone (
	ENV_DECL1(env)
   ) 
		throw (CORBA::SystemException);

#ifdef ObjectStore
  //
  // method for ObjectStore
  //

  static os_typespec* get_os_typespec();

#elif defined FilePersistency
  // methods to store and restore object of this class
  virtual void
   save (
	ostream &iostr
   );

  virtual void
   restore (
	istream &iostr
   );
#endif
};   

#ifdef ORBIX
#ifdef ObjectStore
DEF_ObjectStoreTIE_Domain_Repository_User(User_I)
#else
DEF_TIE_Domain_Repository_User(User_I)
#endif
#elif defined OMNIORB
// DEF_TIE__tie_User
#endif

// implementation of this class need method "instance" of class User_Rep_I
#include "user_rep_i.h"

#endif /* user_i_h */
