#ifdef ORBIX
#define EXCEPTIONS
#endif

#include <iostream.h>
#include "bootstrap_n.hh"

#include <stdlib.h>
#include <string.h>

Bootstrap_base *bbase= NULL;

CosNaming::Name create_name( char *str) {
  int len=0;
  CosNaming::Name ret(len);
  
  char *beg= str;
  char c= '*';

  while (c) {
    while (*str && *str!='.' && *str!='-')
      str++;
    c= *str;
    *str= '\0';
    if (beg==str && !c) break;
    ret.length(++len);
    ret[len-1].id= (const char *) beg;
    ret[len-1].kind= (const char *) beg;
    beg= ++str;

    if (c=='-') {
      while (*str && *str!='.' && *str!='-')
	str++;
      c= *str;
      *str= '\0';
      ret[len-1].kind= (const char *) beg;
      if (c=='\0') break;
      beg= ++str;
    }
  }
  /*
  cerr << ret.length() << ' ';
  for (int i=0; i<ret.length(); i++)
    cerr << (const char *) ret[i].id << '-' << (const char *) ret[i].kind << '.';
  cerr << endl;
  */
  return ret; 
}

CosNaming::NamingContext_var get_context( CosNaming::Name &name) {
  if (name.length() > 0) {
    try {
      CORBA::Object_var abng_ctx_obj= 
	bbase->root_ns->resolve( name);
      CosNaming::NamingContext_var abng_ctx= 
	CosNaming::NamingContext::_narrow( abng_ctx_obj);
      if (CORBA::is_nil( (CosNaming::NamingContext_ptr) abng_ctx)) {
	cerr << "Name does not resolve to a naming context" << endl;
	exit(1);
      }
      return abng_ctx;
    }
    catch (CosNaming::NamingContext::NotFound &) {
      cerr << "Caught exception CosNaming::NotFound" << endl;
      exit(1);
    }
    catch (CosNaming::NamingContext::CannotProceed &) {
      cerr << "Caught exception CosNaming::CannotProceed" << endl;
      exit(1);
    }
    catch (CosNaming::NamingContext::InvalidName &) {
      cerr << "Caught exception CosNaming::InvalidName" << endl;
      exit(1);
    }
    catch (CORBA::SystemException &ex) {
      cerr << "Caught system exception resolving a naming context " << ex << endl;
      exit(1);
    }
  }
  else return bbase->root_ns;
}

CORBA::Object_var get_object( CosNaming::Name &name) {
  try {
    CORBA::Object_var abng_ctx_obj= 
      bbase->root_ns->resolve( name);
    return abng_ctx_obj;
  }
  catch (CosNaming::NamingContext::NotFound &) {
    cerr << "Caught exception CosNaming::NotFound" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::CannotProceed &) {
    cerr << "Caught exception CosNaming::CannotProceed" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::InvalidName &) {
      cerr << "Caught exception CosNaming::InvalidName" << endl;
      exit(1);
  }
  catch (CORBA::SystemException &ex) {
    cerr << "Caught system exception resolving a naming context " << ex << endl;
    exit(1);
  }
  exit(2);
}

void bind_object( CosNaming::NamingContext_var ctx,
		  CosNaming::Name name,
		  CORBA::Object_var obj) {

  try {
    ctx->bind( name, obj);
    return;
  }
  catch (CosNaming::NamingContext::NotFound &) {
    cerr << "Caught exception CosNaming::NotFound" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::CannotProceed &) {
    cerr << "Caught exception CosNaming::CannotProceed" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::InvalidName &) {
      cerr << "Caught exception CosNaming::InvalidName" << endl;
      exit(1);
  }
  catch (CosNaming::NamingContext::AlreadyBound &) {
      cerr << "Caught exception CosNaming::AlreadyBound" << endl;
      exit(1);
  }
  catch (CORBA::SystemException &ex) {
    cerr << "Caught system exception resolving a naming context " << ex << endl;
    exit(1);
  }
  exit(2);
}

void bind_symlink( CosNaming::NamingContext_var ctx,
		   CosNaming::NamingContext_var old_ctx,
		   CosNaming::Name name) {

  try {
    ctx->bind_context( name, old_ctx);
    return;
  }
  catch (CosNaming::NamingContext::NotFound &) {
    cerr << "Caught exception CosNaming::NotFound" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::CannotProceed &) {
    cerr << "Caught exception CosNaming::CannotProceed" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::InvalidName &) {
      cerr << "Caught exception CosNaming::InvalidName" << endl;
      exit(1);
  }
  catch (CosNaming::NamingContext::AlreadyBound &) {
      cerr << "Caught exception CosNaming::AlreadyBound" << endl;
      exit(1);
  }
  catch (CORBA::SystemException &ex) {
    cerr << "Caught system exception resolving a naming context " << ex << endl;
    exit(1);
  }
  exit(2);
}

void bind_context( CosNaming::NamingContext_var ctx,
		   CosNaming::Name name) {

  try {
    ctx->bind_new_context( name);
    return;
  }
  catch (CosNaming::NamingContext::NotFound &) {
    cerr << "Caught exception CosNaming::NotFound" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::CannotProceed &) {
    cerr << "Caught exception CosNaming::CannotProceed" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::InvalidName &) {
      cerr << "Caught exception CosNaming::InvalidName" << endl;
      exit(1);
  }
  catch (CosNaming::NamingContext::AlreadyBound &) {
      cerr << "Caught exception CosNaming::AlreadyBound" << endl;
      exit(1);
  }
  catch (CORBA::SystemException &ex) {
    cerr << "Caught system exception resolving a naming context " << ex << endl;
    exit(1);
  }
  exit(2);
}

void unbind_object( CosNaming::NamingContext_var ctx,
		    CosNaming::Name name) {

  try {
    ctx->unbind( name);
    return;
  }
  catch (CosNaming::NamingContext::NotFound &) {
    cerr << "Caught exception CosNaming::NotFound" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::CannotProceed &) {
    cerr << "Caught exception CosNaming::CannotProceed" << endl;
    exit(1);
  }
  catch (CosNaming::NamingContext::InvalidName &) {
      cerr << "Caught exception CosNaming::InvalidName" << endl;
      exit(1);
  }
  catch (CORBA::SystemException &ex) {
    cerr << "Caught system exception resolving a naming context " << ex << endl;
    exit(1);
  }
  exit(2);
}

void list_context( CosNaming::NamingContext_var ctx) {

  CosNaming::BindingList_var dir_list;
  CosNaming::BindingIterator_var autoiter;

  ctx->list( 100, dir_list, autoiter);
  
  for ( int i=0; i<dir_list->length(); i++) {
    if ( dir_list[ i].binding_type == CosNaming::ncontext)
      cout << "CTX ";
    else
      cout << "OBJ ";
    for (int j=0; j<dir_list[i].binding_name.length(); j++)
      cout << "(" << j << ") id: " << dir_list[i].binding_name[j].id
	   << " kind: " << dir_list[i].binding_name[j].kind << endl;
  }
}

void usage( char *prog_name) {
  cout << prog_name << "\n"
       << "\t-t \t\t\t-list root context\n"
       << "\t-l context_name\t\t-list context\n"
       << "\t-c context_name\t\t-bind new context\n"
       << "\t-p name\t\t\t-put an object in NamingService\n"
       << "\t\t\t\t\t(IOR read from standard input)\n"
       << "\t-s name\t\t\t-symlink a context\n"
       << "\t\t\t\t\t(a source IOR read from standard input)\n"
       << "\t-g name\t\t\t-get an object from NamingService\n"
       << "\t-i name\t\t\t-ping an object\n"
       << "\t-r name\t\t\t-remove (unbind) an object from NamingService\n"
       << endl;
  exit(0);
}

int main( int argc, char **argv) {
  CORBA::ORB_ptr orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

  if (argc==1) usage( *argv);

#ifdef ORBIX
  orb_ptr->useRemoteIsACalls( 1);
#endif

#ifdef ORBIX
  orb_ptr->setDiagnostics( 0);  
#endif

  bbase= new Bootstrap_base( orb_ptr);

  char c;
  while ((c = getopt(argc, argv, "l:tp:c:g:i:r:s:h")) != EOF)
    switch (c) {
    case 'l':
      /* list */
      list_context( get_context( create_name( optarg)));
      break;

    case 't':
      /* list */
      list_context( get_context( create_name( "")));
      break;

    case 'c':
      /* put new context */
      {
        CosNaming::Name name= create_name( optarg);
	bind_context( bbase->root_ns,
		      name);
      }
      break;

    case 'p':
      /* put */
      {
        CosNaming::Name name= create_name( optarg);
	if (name.length()<1) {
	  cerr << "Cannot bind an object to root context" << endl;
	  exit(1);
	}
	char buf[4096];
	cin >> buf;
	CORBA::Object_var obj;
	try {
	  obj= orb_ptr->string_to_object( buf);
	}
	catch (CORBA::SystemException &ex) {
	  cerr << ex << endl;
	  exit(1);
	}
	bind_object( bbase->root_ns,
		     name,
		     obj);
      }
      break;

    case 's':
      /* symlink */
      {
        CosNaming::Name name= create_name( optarg);
	if (name.length()<1) {
	  cerr << "Cannot bind an object to root context" << endl;
	  exit(1);
	}
	char buf[4096];
	cin >> buf;
	CORBA::Object_var obj;
	try {
	  obj= orb_ptr->string_to_object( buf);
	}
	catch (CORBA::SystemException &ex) {
	  cerr << ex << endl;
	  exit(1);
	}
	CosNaming::NamingContext_var old_nc;
	try {
	  old_nc= CosNaming::NamingContext::_narrow( obj);
	}
	catch (CORBA::SystemException &ex) {
	  cerr << "Source IOR is not a CosNaming::NamingContext object" << endl;
	  cerr << ex << endl;
	  exit(1);
	}
	if (CORBA::is_nil( (CosNaming::NamingContext_ptr) old_nc)) {
	  cerr << "Source IOR is not a CosNaming::NamingContext object" << endl;
	  exit(1);
	}

	bind_symlink( bbase->root_ns,
		      old_nc,
		      name);
      }
      break;

    case 'r':
      /* remove (unbind) */
      {
	unbind_object( bbase->root_ns,
		       create_name( optarg));
      }
      break;

    case 'g':
      /* get */
      {
	CORBA::Object_var obj= get_object( create_name( optarg));
	cout << orb_ptr->object_to_string( obj) << endl;
      }
      break;

    case 'i':
      /* ping */ 
      {
	CORBA::Object_var obj= get_object( create_name( optarg));
	cerr << "Pinging..." << endl;
	if (CORBA::is_nil(obj)) {
	  cerr << "Nil object" << endl;
	  exit(1);
	}
	try {
#ifdef ORBIX
	  int r= DII_non_existent( obj);
#else
	  int r= obj->_non_existent();
#endif
	  cerr << "non_existent " << (r ? "TRUE" : "FALSE") 
	       << " == " << (r ? "DEAD" : "ALIVE") << endl;
	  //      	  return !r;
	}
	catch (CORBA::BAD_OPERATION &) {
	  cerr << "CORBA::BAD_OPERATION" << endl;
	  exit(1);
	}
	catch (CORBA::SystemException &ex) {
	  cerr << ex << endl;
	  exit(1);
	}
      }
      break;

    case 'h':
    default:
      usage( *argv);
      break;
    }

  return 0;
}



