//
//  File:         abng_rep_monitor.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 20.11.97
//
// General Description:
//   Implementation of the programe monitoring changes in the ABng repositories
//

#define USE_IIOP
#define USE_IR

#include "CORBA.h"
#include "abng_core_management.hh"

#include <iostream.h>
#include <stdio.h>
#include <fstream.h>
#include <stdlib.h>

#include "badge_rep_observer_i.hh"

#define Badge_Rep_IOR_File "./Badge_Rep_iiop.ref"

int main(void) {
CORBA::Object_ptr obj_ptr;   
	Badge_Rep_ptr my_badge_rep_ptr;
	char ref_str[500];
	ifstream strm(Badge_Rep_IOR_File);
	unsigned long timer = CORBA::Orbix.INFINITE_TIMEOUT;
	
	//
	// init ORB
	//
	try {
		CORBA::Orbix.impl_is_ready("badge_rep_monitor", 0);
	}
	catch (CORBA::SystemException &sysEx) {
		cerr << "Unexpected system exception" << endl;
		cerr << &sysEx;
		exit(-1);
	} 
	catch(...) {
		cerr << "Unexpected exception" << endl;
		exit(-1);
	}

	// initialise the Badge_Rep from the file
	if (!strm){
		cerr << "Badge_Rep monitor error: Unable to open file - ";
		cerr << Badge_Rep_IOR_File << endl;
		return 0;
	}
	strm >> ref_str;

	// get the obj referance
	try {
		obj_ptr = CORBA::Orbix.string_to_object(ref_str);
	} catch (CORBA::SystemException& se) {
		cerr << "Badge_Rep monitor error: Exception! - " << &se << endl;
		exit(1);
	}
	cerr << "Badge_Rep monitor: got Badge_Rep object reference" << endl;

	// check if its nil
	if (CORBA::is_nil(obj_ptr)){     
		cerr << "Badge_Rep monitor error: ";
		cerr << "Badge_Rep object reference is nill!" << endl;
		exit(-1);
	}

	// attempt to narrow
	try {
		my_badge_rep_ptr = ABng_Core_Management::Badge_Rep::_narrow(obj_ptr);
	} catch (CORBA::SystemException& se){
		cerr << "Badge_Rep monitor error: ";
		cerr << "exception when trying to _narrow! " << &se << endl;
	}

	// check if my_badge_rep_ptr is nil
	if (CORBA::is_nil(my_badge_rep_ptr)){
		cerr << "badge_Rep monitor error: ";
		cerr << "failed to narrow object reference!" << endl;
		exit(-1);
	}
	
	// contact with Badge_Rep established  !!

	// create a observer of badge repository
	Badge_Rep_Observer_I* my_badge_rep_observer_i = new Badge_Rep_Observer_I ();

	// create a TIE for the object
	TIE_ABng_Core_System_System_Static_Data_Observer(Badge_Rep_Observer_I)* my_badge_rep_observer = 
		new TIE_ABng_Core_System_System_Static_Data_Observer(Badge_Rep_Observer_I) 
		(my_badge_rep_observer_i);

	my_badge_rep_observer_i->my_corba_object = 
		ABng_Core_System::System_Static_Data_Observer::_duplicate(my_badge_rep_observer);

  	CORBA::Orbix.setServerName("badge_rep_monitor");

	// attempt to register add observer in badge_rep
	try {
		my_badge_rep_ptr->attach_add_system_static_data_observer(my_badge_rep_observer);
	} catch (CORBA::SystemException& se){
		cerr << "Badge_Rep monitor error: ";
		cerr << "exception when trying to register add observer! " << &se << endl;
	}
	cerr << "Badge_Rep monitor: add observer registered in Badge_Rep" << endl;

	//
	// export the server to the network
	//
	try {
		CORBA::Orbix.impl_is_ready("badge_rep_monitor", timer);
	}
	catch (CORBA::SystemException &sysEx) {
		cerr << "Unexpected system exception" << endl;
		cerr << &sysEx;
		exit(-1);
	} 
	catch(...) {
		cerr << "Unexpected exception" << endl;
		exit(-1);
	}

	return 0;
}
