//
//  File:         location_type_rep_cl.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 20.11.97
//
// General Description:
//   Implementation file for the Location_Type_Rep client
//

#define USE_IIOP
#define USE_IR

#include "CORBA.h"
#include "domain_rep_management.hh"

#include <iostream.h>
#include <stdio.h>
#include <fstream.h>
#include <stdlib.h>

#include <rw/rwfile.h>

extern "C" int usleep(unsigned int useconds);

#define Location_Type_Rep_Manager_IOR_File "./Location_Type_Rep_Manager_iiop.ref"
#define Location_Type_Rep_IOR_File "./Location_Type_Rep_iiop.ref"
#define Location_Type_Data_File    "./location_types.dat"

ifstream& operator >> (
                 ifstream &istr, 
                 Domain_Repository::Location_Type::Description &d
          ) {
  int _tmpl;
  istr >> _tmpl;

  char * _name = new char[_tmpl + 1];
  istr >> _name;
  _name[_tmpl-1] = '\0';

  d.name = CORBA::string_dup(_name);
  delete [] _name;

  istr >> _tmpl;
  d.property_description_list = 
      Domain_Repository::Property_Descriptions(_tmpl);

  d.property_description_list.length(_tmpl);

  for(int i = 0; i < d.property_description_list.length(); i ++
  ) {
     istr >> _tmpl;

     _name = new char[_tmpl + 1];
     istr >> _name;
     _name[_tmpl-1] = '\0';

     d.property_description_list[i].name = CORBA::string_dup(_name);
     delete [] _name;

     istr >> _tmpl;

     _name = new char[_tmpl + 1];
     istr >> _name;
     _name[_tmpl-1] = '\0';

     d.property_description_list[i].type = new CORBA::TypeCode(_name);
     delete [] _name;
  }

  return istr;
} 

ostream& operator << (ostream& str, Domain_Repository::Location_Type::Description& descr) {
  str << "name = " << descr.name << endl;
  str << "property_description_list = { " << endl;
  str << "                              _maximum = " 
      << descr.property_description_list.maximum() << endl;
  str << "                              _length = " 
      << descr.property_description_list.length() << endl;
  str << "                              _buffer = {" << endl;
  for(int i = 0; i < descr.property_description_list.length(); i ++) {
       str << "                                          ";
       str << "{ name = " << descr.property_description_list[i].name << ", " 
           << "type = " 
           << (char*)*(descr.property_description_list[i].type) << " }" 
           << endl;
  }
  str << "                                        }" << endl;
  str << "                             } " << endl;
  return str;
}

int main (void) {
	CORBA::Object_ptr obj_ptr;   
	Domain_Repository_Management::Location_Type_Rep_Manager_ptr 
             my_location_type_rep_manager_ptr;
        Domain_Repository::Location_Type_Rep_ptr 
             my_location_type_rep_ptr;

        
	char ref_str[500];

	ifstream strm(Location_Type_Rep_IOR_File);

	// Initialise the Location_Type_Rep from the file
	
	
	
	if (!strm){
	   cerr << "Location_Type_Rep client error: Unable to open file - ";
	   cerr << Location_Type_Rep_IOR_File << endl;
	   return 0;
	}
	strm >> ref_str;
	strm.close();

	// get the obj referance
	try {
		obj_ptr = CORBA::Orbix.string_to_object(ref_str);
	} catch (CORBA::SystemException& se) {
		cerr << "Location_Type_Rep client error: Exception! - " 
                     << &se << endl;
		exit(1);
	}
	cerr << "Location_Type_Rep client: " 
             << "got Location_Type_Rep object reference" << endl;

	// check if its nil
	if (CORBA::is_nil(obj_ptr)){     
		cerr << "Location_Type_Rep client error: ";
		cerr << "Location_Type_Rep object reference is nill!" << endl;
		exit(-1);
	}

	// attempt to narrow
	try {
		my_location_type_rep_ptr = 
                Domain_Repository::Location_Type_Rep::_narrow(obj_ptr);
	} catch (CORBA::SystemException& se){
		cerr << "Location_Type_Rep client error: ";
		cerr << "exception when trying to _narrow! " << &se << endl;
	}

	// check if my_location_type_rep_manager_ptr is nil
	if (CORBA::is_nil(my_location_type_rep_ptr)){
		cerr << "Location_Type_Rep client error: ";
		cerr << "failed to narrow object reference!" << endl;
		exit(-1);
	}

        ifstream _strm(Location_Type_Rep_Manager_IOR_File);

	// Initialise the Location_Type_Rep from the file
	if (!_strm){
	   cerr << "Location_Type_Rep client error: Unable to open file - ";
	   cerr << Location_Type_Rep_Manager_IOR_File << endl;
	   return 0;
	}
	_strm >> ref_str;
	_strm.close();

	// get the obj referance
	try {
		obj_ptr = CORBA::Orbix.string_to_object(ref_str);
	} catch (CORBA::SystemException& se) {
		cerr << "Location_Type_Rep client error: Exception! - " 
                     << &se << endl;
		exit(1);
	}
	cerr << "Location_Type_Rep client: " 
             << "got Location_Type_Rep object reference" << endl;

	// check if its nil
	if (CORBA::is_nil(obj_ptr)){     
		cerr << "Location_Type_Rep client error: ";
		cerr << "Location_Type_Rep object reference is nill!" << endl;
		exit(-1);
	}

	// attempt to narrow
	try {
		my_location_type_rep_manager_ptr = 
                Domain_Repository_Management::Location_Type_Rep_Manager::_narrow(obj_ptr);
	} catch (CORBA::SystemException& se){
		cerr << "Location_Type_Rep client error: ";
		cerr << "exception when trying to _narrow! " << &se << endl;
	}

	// check if my_location_type_rep_manager_ptr is nil
	if (CORBA::is_nil(my_location_type_rep_manager_ptr)){
		cerr << "Location_Type_Rep client error: ";
		cerr << "failed to narrow object reference!" << endl;
		exit(-1);
	}
	
	// Contact with Location_Type_Rep established
        ifstream strm_location_type_data(Location_Type_Data_File);
	int location_type_number;
	Domain_Repository::Location_Type::Description 
              location_type_data_record;

	if (!strm_location_type_data){
	     cerr << "Location_Type_Rep client error: Unable to open file - ";
	     cerr << Location_Type_Data_File << endl;
	     exit(-1);
	}

        strm_location_type_data >> location_type_number;
        
	Domain_Repository::Location_Type_ptr new_location_type_ptr;

	for(int i = 0; i < location_type_number; i++){
		cerr << "Location_Type_Rep client: adding " << i 
                     << " location_type to the repository" << endl;
		
		strm_location_type_data >> location_type_data_record;
		cout << location_type_data_record;

		//		cout << "CLIENT:" << location_type_data_record;

		try {
			// try to add this location_type
    			new_location_type_ptr = 
				my_location_type_rep_manager_ptr->add(
                                        location_type_data_record
                                );
		} catch (Domain_Repository::Duplicate_Data_Record &ex) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to add failed, duplicated record"
                             << endl << endl;
			continue;
		} catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to add failed, unexpected system exception: " 
                        << &sysEx << endl;
		   exit(1);
		} 

		CORBA::string stringObj; 
		try {
			stringObj = my_location_type_rep_manager_ptr->
				_object_to_string(
                                     CORBA::IT_INTEROPERABLE_OR_KIND
                                );
  		}
 		catch(CORBA::SystemException &sysEx) {
			cerr << "Unexpected system exception: " << &sysEx 
                             << endl; 
		}
			
		cout << "Location_Type IOR: " << endl << stringObj << endl 
                     << endl;
		CORBA::string_free(stringObj);

		try {
		  // try to get item id of this location_type
    		  cerr << "Location_Type_Rep client: location_type Item id = " 
                       << new_location_type_ptr->item_id() << endl;
		} catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to get item_id failed: " << &sysEx << endl;
		   exit(1);
		}
		
                if(location_type_data_record.property_description_list.length())
                    location_type_data_record.property_description_list[0].name
                      = CORBA::string_dupl("gosia");

usleep(1000000);
		try {
			// try to update this location_type
			my_location_type_rep_manager_ptr->update(
                             new_location_type_ptr, location_type_data_record
                        );
		} catch (Domain_Repository::Duplicate_Data_Record &ex) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to update failed, duplicated record"
                             << endl << endl;
			//continue;
		} catch (Domain_Repository::Unknown_Object_Ref &ex) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to update failed, unknown object ref"
                             << endl << endl;
			continue;
		} catch (CORBA::SystemException &sysEx) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to remove failed: " << &sysEx << endl;
			exit(1);
		}

		cerr << "Location_Type_Rep client: " << i 
                     << " location_type updated" << endl;
		     
usleep(1000000);
	try {
			// try to remove this location_type
			my_location_type_rep_manager_ptr->remove(
                                new_location_type_ptr
                        );
		} catch (Domain_Repository::Unknown_Object_Ref &ex) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to remove failed, unknown object ref"
                             << endl << endl;
			continue;
		} catch (CORBA::SystemException &sysEx) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to remove failed, unexpected system exception: " << &sysEx << endl;
			exit(1);
		}
		cerr << "Location_Type_Rep client: " << i 
                     << " location_type removed" << endl << endl;
usleep(1000000);
		    
	}

        strm_location_type_data.close();

        strm_location_type_data.open(Location_Type_Data_File);

	if (!strm_location_type_data){
	     cerr << "Location_Type_Rep client error: Unable to open file - ";
	     cerr << Location_Type_Data_File << endl;
	     exit(-1);
	}

        strm_location_type_data >> location_type_number;

        Domain_Repository::Location_Type_ptr* tab = 
           new Domain_Repository::Location_Type_ptr[location_type_number];

	for(i = 0; i < location_type_number; i++){
		cerr << "Location_Type_Rep client: adding " << i 
                     << " location_type to the repository" << endl;
		
		strm_location_type_data >> location_type_data_record;
                cout << location_type_data_record.name << endl;

		//	cout << "CLIENT:" << location_type_data_record;

		try {
			// try to add this location_type
    			tab[i] = my_location_type_rep_manager_ptr->add(
                                        location_type_data_record
                                 );
		} catch (Domain_Repository::Duplicate_Data_Record &ex) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to add failed, duplicated record"
                             << endl << endl;
			continue;
		} catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to add failed, unexpected system exception: " 
                        << &sysEx << endl;
		   exit(1);
		} 


		try {
		  // try to get item id of this location_type
    		  cerr << "Location_Type_Rep client: location_type Item id = " 
                       << tab[i]->item_id() << endl;
		} catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to get item_id failed: " << &sysEx << endl;
		   exit(1);
		}
	}
        strm_location_type_data.close();

	Domain_Repository::Location_Type_Rep::Location_Types *loc_seq;
	Domain_Repository::Location_Type::Pattern pattern;
        pattern.is_any = 1;
	
    cout << "-------------------------- FIND -----------------------" << endl;
        try {
           loc_seq = my_location_type_rep_ptr->find(pattern);
        } catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to find failed: " << &sysEx << endl;
		   exit(1);
	}

        for(i = 0; i < loc_seq->length(); i ++) {
		try {
		  // try to get item id of this location_type
    		  cerr << "Location_Type_Rep client: location_type Item id = " 
                       << (*loc_seq)[i]->item_id() << endl;
		} catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to get item_id failed: " << &sysEx << endl;
		   exit(1);
		}
        }
	
        Domain_Repository::Location_Type::Value_Descriptions* val_desc;
        try {
           val_desc = my_location_type_rep_ptr->find_values(pattern);
        } catch (CORBA::SystemException &sysEx) {
		   cerr << "Location_Type_Rep client error: ";
		   cerr << "call to find failed: " << &sysEx << endl;
		   exit(1);
	}

        for(i = 0; i < val_desc->length(); i ++) {
	  cout << "DESCRIPTION: " << i << endl;
          cout << (*val_desc)[i] << endl;
        }
	/*
	for(i = 0; i < location_type_number; i++){
		try {
			// try to remove this location_type
			my_location_type_rep_manager_ptr->remove(tab[i]);
		} catch (Domain_Repository::Unknown_Object_Ref &ex) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to remove failed, unknown object ref"
                             << endl << endl;
			continue;
		} catch (CORBA::SystemException &sysEx) {
			cerr << "Location_Type_Rep client error: ";
			cerr << "call to remove failed, unexpected system exception: " << &sysEx << endl;
			exit(1);
		}
		cerr << "Location_Type_Rep client: " << i 
                     << " location_type removed" << endl << endl;
		    
	}
	*/
        
	cout << endl << "exiting.. Everything OK!!" << endl << endl;
  
	return 0;
}






