#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/un.h>            /* Unix stream */
#include <netdb.h>             /* host entries database */
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <time.h>
#include <omnithread.h>

#ifndef max
#define max(a,b) (a>b ? a : b)
#endif

unsigned long buffer_length;
unsigned long buffer_pos= 0;
unsigned char *buffer;
unsigned long listener_pos= 0;
enum { STATE_WAIT_FOR_CONNECTION, STATE_CONNECTED } state= STATE_WAIT_FOR_CONNECTION;
enum { SERVER_CONNECTED, SERVER_FINISHED } server_state= SERVER_CONNECTED;

omni_mutex listener_guard;
omni_condition listener_cond( &listener_guard);
omni_mutex accept_guard;
omni_condition accept_cond( &accept_guard);
int gsockfd, gnewsockfd;


void add_to_buffer( const void *src, unsigned long length) {
  listener_guard.lock();
  length= (length>=buffer_length) ? buffer_length : length;
  unsigned long len;
  if (buffer_pos+length>buffer_length) {
    len= buffer_length/10;
    len= max(length,len);
    memmove( buffer, buffer+len, buffer_length-len);
    listener_pos= listener_pos>len ? listener_pos-len : 0;
    buffer_pos-= len;
  }
  memcpy( buffer+buffer_pos, src, length);
  buffer_pos+=length;
  listener_cond.signal();
  listener_guard.unlock();
}

void check_client( void *args) {
  int fd= (int) args;
  char buf[1];
  while (read( fd, buf, 1) > 0)
    ;
  listener_guard.lock();
  state= STATE_WAIT_FOR_CONNECTION;
  listener_cond.signal();
  listener_guard.unlock();
}

void read_stdin( void *args) {
  char buf[256];
  for (;;) {
    int len= read(0, buf, 256);
    if (len<1) {
      listener_guard.lock();
      server_state= SERVER_FINISHED;
      listener_cond.signal();
      listener_guard.unlock();
      sprintf(buf,"Server finished\n");
      add_to_buffer(buf,strlen(buf));
      return;
    }
    else add_to_buffer(buf,len);
  }
}

unsigned long read_from_buffer( void *dst, unsigned long max_buffer) {
  listener_guard.lock();
  unsigned long len;
  while ((len= max_buffer<buffer_pos-listener_pos ? max_buffer :
	  buffer_pos-listener_pos) == 0) {
    if (state!= STATE_CONNECTED) break;
    if (server_state != SERVER_CONNECTED) break;
    listener_cond.wait();
  }
  memcpy( dst, buffer+listener_pos, len);
  listener_pos+= len;
  listener_guard.unlock();
  return len;
}

int routine_cli_UNIXSTR( char *path, int bufsize, int kill_proc= 0)
{
  int                   sockfd,nwritten;
  struct sockaddr_un    serv_addr;
  char *                buf= (char *) malloc( bufsize);
  int exc= 0;

  if (!buf) return -1;

  bzero( (char *) &serv_addr, sizeof( serv_addr));
  serv_addr.sun_family= AF_UNIX;
  strcpy( serv_addr.sun_path, path);

  if ((sockfd= socket( AF_UNIX, SOCK_STREAM, 0)) <0)
  {
    perror("Can't get socket");
    free(buf);
    return -1;
  }
  if (connect(sockfd, (struct sockaddr *) &serv_addr,
              strlen(path)+sizeof(serv_addr.sun_family))<0)
  {
    perror("Can't connect to server");
    close(sockfd);
    free(buf);
    return -1;
  }
  buf[0]= kill_proc ? 'K' : 'L';
  write( sockfd, buf, 1);
  if (kill_proc) {
    return 0;
  }
  while (1)
  {
    nwritten= read( sockfd, buf, bufsize);
    if (nwritten>0)
      write( 0, buf, nwritten);

    if (nwritten<0) {
      perror("error reading from unix stream");
      exit(-1);
    }
  }
}
             

void accept_thread( void *args) {
  int clilen, newsockfd;
  struct sockaddr_un cli_addr;
  char buf[80];

  for (;;) {
    clilen= sizeof( cli_addr);
    newsockfd= accept( gsockfd, (struct sockaddr *) &cli_addr, &clilen);
    if (newsockfd == 0) {
      perror("Accept error");
      close(gsockfd);
      exit(-1);
    }
    
    if (read(newsockfd, buf, 1)) {
      if (buf[0]=='K') { // kill
	listener_guard.lock();
	if (server_state == SERVER_FINISHED) {
	  unlink( (char*) args);
	  exit(0);
	}
	listener_guard.unlock();
	close(newsockfd);
	continue;
      }
    }
    else {
      close(newsockfd);
      continue;
    }
    if (state==STATE_CONNECTED) {
      close(newsockfd);
      continue;
    }
    accept_guard.lock();
    gnewsockfd= newsockfd;
    accept_cond.signal();
    accept_guard.unlock();
  }
}

int routine_srv_UNIXSTR( char *path, int bufsize)
{
  int                         nread;
  struct sockaddr_un          serv_addr, cli_addr;
  char *buf= (char *) malloc( bufsize);
  int exc= 0, datasize= 0;
             
  if (!buf) return -1;

    if ((gsockfd= socket(AF_UNIX, SOCK_STREAM, 0))<0) {
	perror("Can't get socket");
	free(buf);
	return -1;
    }
    //    unlink(path);
    bzero( (char *) &serv_addr, sizeof( serv_addr));
    serv_addr.sun_family= AF_UNIX;
    strcpy(serv_addr.sun_path, path);

    for (int retries=0;; retries++) {
      if (bind(gsockfd, (struct sockaddr *) &serv_addr,
	       strlen( path)+ sizeof( serv_addr.sun_family)) < 0) {
	if (retries>3) exit(-1);
	perror("Can't bind local address");
	routine_cli_UNIXSTR( path, bufsize, 1 /* kill*/ );
	sleep(1);
      }
      else {
	break;
      }
    }
    listen(gsockfd,5);
    (new omni_thread( accept_thread, path))->start();
  for (;;) {
    accept_guard.lock();
    accept_cond.wait();
    accept_guard.unlock();

    state= STATE_CONNECTED;
    (new omni_thread( check_client, (void*)gnewsockfd))->start();
    listener_pos= 0;
    for (;;) {
      nread= read_from_buffer( buf, bufsize);
      listener_guard.lock();
      if (state!=STATE_CONNECTED) {
	listener_guard.unlock();
	break;
      }
      listener_guard.unlock();
      nread= write(gnewsockfd, buf, nread);
      if (nread<0) {
	perror("error writing unix stream");
	break;
      }
    }
    close(gnewsockfd);
  }
}

void usage( char *prog) {
  exit(-1);
}

void handler( int sig) {
  fprintf(stderr,"signal %d\n", sig);
}

int main( int argc, char **argv) {
  buffer_length= 20*1024;
  buffer= new unsigned char[buffer_length];

  sigset( SIGPIPE, handler);
  //  signal( SIGTTIN, handler);
  //  signal( SIGTTOU, handler);

  (new omni_thread( read_stdin, NULL))->start();

  if (argc<2) usage(argv[0]);
  if (argc == 2) 
    routine_srv_UNIXSTR( argv[1], 1024) ;
  else
    routine_cli_UNIXSTR( argv[1], 1024);
}







