//
// $Id: abngps.cc,v 1.2 1998/07/29 14:13:47 mencnar Exp $
//
// $Log: abngps.cc,v $
// Revision 1.2  1998/07/29 14:13:47  mencnar
// better formatted results
//
// Revision 1.1.1.1  1998/06/18 14:38:47  mencnar
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: A utility to list all running servers in ABng system, check
//              their status, or kill them
//

#ifdef ORBIX
#define EXCEPTIONS
#endif

#include <iostream.h>
#include "abng_server_manager.hh"
#include "compatibility.hh"
#include "bootstrap_n.hh"
#include "abng_guarded.hh"
#include <rw/rwstring.h>
#include <stdio.h>
#include <errno.h>

int timeout = 10; // seconds

Bootstrap_base *bbase;


class ProcessCheck : public ABng_S_Guarded {
public:
  omni_condition cond;

  ProcessCheck() : cond( & mutex()),
                   error_occured( (const char*) ""),
                   kill_server(0) {}

    
  // "in" parameter(s):
  //
  CORBA::Object_var server_reference;
  int check_status;
  int kill_server;

  // "out" parameters:
  //
  RWCString error_occured;

  // the following fields are filled if check_status is zero
  //
  CORBA::Long pid;
  RWCString host_name;
  RWCString process_owner;
  CORBA::Long uptime;

  // this field is filled if check_status != 0
  //
  RWCString server_status;
};

void query_worker( ProcessCheck &pcheck) {
  RWCString err;
  ABng_Server_Manager::ProcessControl_var ps_reference;
  try {	
    ps_reference= 
      ABng_Server_Manager::ProcessControl::_narrow( 
			      (CORBA::Object_ptr) pcheck.server_reference);
    
  }
  catch (CORBA::COMM_FAILURE &) {
    err= (const char *) "not responding";
  }
  catch (...) {
  }

  if ( CORBA::is_nil( (ABng_Server_Manager::ProcessControl_ptr) ps_reference)) {
    ABng_S_Guarded::auto_lock local_lock(pcheck);
    if (err=="") err= "does not resolve to ABng_Server_Manager::ProcessControl object";
    pcheck.error_occured= err;
    pcheck.cond.signal();
    return;
  }

  if (pcheck.kill_server) {
    try {
      ps_reference->kill_server();
    }
    catch (...) {
      ABng_S_Guarded::auto_lock local_lock(pcheck);
      pcheck.error_occured= "not responding";
      pcheck.cond.signal();
      return;
    }
    {
      ABng_S_Guarded::auto_lock local_lock(pcheck);
      pcheck.cond.signal();
      return;
    }
  }

  if (pcheck.check_status) {
    RWCString sstatus;
    try {
      CORBA::String_var status( ps_reference->get_server_status());
      sstatus= (const char *) status;
    }
    catch (...) {
      ABng_S_Guarded::auto_lock local_lock(pcheck);
      pcheck.error_occured= "not responding";
      pcheck.cond.signal();
      return;
    }
    {
      ABng_S_Guarded::auto_lock local_lock(pcheck);
      pcheck.server_status= sstatus;
      pcheck.cond.signal();
      return;
    }    
  }
  else {
    CORBA::Long pid;
    RWCString host_name;
    RWCString process_owner;
    CORBA::Long uptime;
    try {
      pid= ps_reference->get_pid();
      CORBA::String_var p1( ps_reference->get_host_name());
      host_name= (const char *) p1;
      CORBA::String_var p2( ps_reference->get_process_owner());
      process_owner= (const char *) p2;
      uptime= ps_reference->uptime();
    }
    catch (...) {
      ABng_S_Guarded::auto_lock local_lock(pcheck);
      pcheck.error_occured= "not responding";
      pcheck.cond.signal();
      return;
    }
    {
      ABng_S_Guarded::auto_lock local_lock(pcheck);
      pcheck.pid= pid;
      pcheck.host_name= host_name;
      pcheck.process_owner= process_owner;
      pcheck.uptime= uptime;
      pcheck.cond.signal();
      return;
    }
  }
}

void query_worker_wrapper( void *args) {
  ProcessCheck &pcheck= *((ProcessCheck*)args);
  query_worker( pcheck);
}

void list_servers() {
  ABng_Server_Manager::Name name= Bootstrap::CTX_ABng();

  name = name+"ps_dev";
  
  try {
    CORBA::Object_var abng_ctx_obj= bbase->resolve( name);
    CosNaming::NamingContext_var abng_ctx= 
      CosNaming::NamingContext::_narrow( abng_ctx_obj);

    CosNaming::BindingList_var dir_list;
    CosNaming::BindingIterator_var autoiter;

    abng_ctx->list( 400, dir_list, autoiter);
    ProcessCheck ps_tab[ 200];

    printf("%-38s%5s %11s %8s %s\n--------------------------------------------------------------------------\n",
	   "SERVER NAME", "PID", "HOST NAME", "OWNER", "RUNNING FOR");

    for ( int i=0; i<dir_list->length(); i++) {

      if ( dir_list[ i].binding_type == CosNaming::ncontext) continue;

      printf( "%-38.38s", (const char *) dir_list[ i].binding_name[ 0].id);
      
      ABng_Server_Manager::Name new_name= 
	  name+dir_list[ i].binding_name[ 0].id;

      CORBA::Object_var server_reference;
      ABng_Server_Manager::ProcessControl_var ps_reference;
      try {
	server_reference= bbase->resolve( new_name);

	ps_tab[i].server_reference= server_reference;
	ps_tab[i].check_status= 0;
	ABng_S_Guarded::auto_lock local_lock( ps_tab[i]);
	(new omni_thread( query_worker_wrapper, &ps_tab[i]))->start();
	unsigned long sec,nsec;
	int wait_ret;
	omni_thread::get_time(    &sec, &nsec, 
				  timeout, 0);
#ifdef OMNIORB
	wait_ret= ps_tab[i].cond.timedwait( sec, nsec);
#else   
	wait_ret= ps_tab[i].cond.timed_wait( sec, nsec);
#endif
	
#ifdef OMNIORB
	if (wait_ret == 0)
#else
	if (wait_ret == ETIMEDOUT)
#endif
	  {
	    printf("Timed out (process deadlocked ?)\n");
	    continue;
	  }
	
	if (ps_tab[i].error_occured == "")
	  printf("%5d %11.11s %8.8s %03dd %02d:%02d:%02d\n",
		 (int) ps_tab[i].pid,
		 (const char *) ps_tab[i].host_name,
		 (const char *) ps_tab[i].process_owner,
		 (int) ((ps_tab[i].uptime / 24) / 3600),
		 (int) ((ps_tab[i].uptime % (3600*24)) / 3600),
		 (int) ((ps_tab[i].uptime % 3600) / 60),
		 (int) (ps_tab[i].uptime % 60));
	else
	  printf("%s\n", (const char *) ps_tab[i].error_occured);

      }
      catch (...) {
	printf("\n");
	continue;
      }
      
    }
    printf("\n");
  }
  catch (...) {}
}


void check_server( const char *sname) {
  ABng_Server_Manager::Name name= Bootstrap::CTX_ABng();

  name = (name+"ps_dev")+sname;
  
  CORBA::Object_var server_reference;
  ABng_Server_Manager::ProcessControl_var ps_reference;
  try {
    server_reference= bbase->resolve( name);
    ProcessCheck ps_i;
    
    ps_i.server_reference= server_reference;
    ps_i.check_status= 1;
    ABng_S_Guarded::auto_lock local_lock( ps_i);
    (new omni_thread( query_worker_wrapper, &ps_i))->start();
    unsigned long sec,nsec;
    int wait_ret;
    omni_thread::get_time(    &sec, &nsec, 
			      timeout, 0);
#ifdef OMNIORB
    wait_ret= ps_i.cond.timedwait( sec, nsec);
#else     
    wait_ret= ps_i.cond.timed_wait( sec, nsec);
#endif
	
#ifdef OMNIORB
    if (wait_ret == 0)
#else
      if (wait_ret == ETIMEDOUT)
#endif
	{
	  printf("Timed out (process deadlocked ?)\n");
	  return;
	}
    printf("%s", (const char *) ps_i.server_status);
    if (ps_i.error_occured != "")
      printf("%s\n", (const char *) ps_i.error_occured);
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
  }
}

void kill_server( const char *sname) {
  ABng_Server_Manager::Name name= Bootstrap::CTX_ABng();

  name = (name+"ps_dev")+sname;
  
  CORBA::Object_var server_reference;
  ABng_Server_Manager::ProcessControl_var ps_reference;
  try {
    server_reference= bbase->resolve( name);
    ProcessCheck ps_i;
    
    ps_i.server_reference= server_reference;
    ps_i.kill_server= 1;
    ABng_S_Guarded::auto_lock local_lock( ps_i);
    (new omni_thread( query_worker_wrapper, &ps_i))->start();
    unsigned long sec,nsec;
    int wait_ret;
    omni_thread::get_time(    &sec, &nsec, 
			      timeout, 0);
#ifdef OMNIORB
    wait_ret= ps_i.cond.timedwait( sec, nsec);
#else     
    wait_ret= ps_i.cond.timed_wait( sec, nsec);
#endif
	
#ifdef OMNIORB
    if (wait_ret == 0)
#else
    if (wait_ret == ETIMEDOUT)
#endif
      {
	printf("Timed out (process deadlocked ?)\n");
	return;
      }
    if (ps_i.error_occured != "")
      printf("%s\n", (const char *) ps_i.error_occured);
  }
  catch (CORBA::Exception &ex) {
    cerr << ex << endl;
  }
}

void usage( char *prog_name) {
  cout << prog_name << endl
       << "\t -l\t\t\tlist all servers" << endl
       << "\t -c server_name\t\tcheck detailed server status" << endl
       << "\t -k server_name\t\tkill named server" << endl;
}

int main( int argc, char **argv) {
  extern char *optarg;
  extern int optind;

  CORBA::ORB_ptr orb_ptr= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa_ptr= orb_ptr->BOA_init( argc, argv, BOA_IDENTIFIER);

#ifdef ORBIX
  orb_ptr->setDiagnostics( 0);  
#endif

  bbase= new Bootstrap_base( orb_ptr);

  char c;
  while ((c = getopt(argc, argv, "lc:k:")) != EOF)
    switch (c) {
    case 'l':
      list_servers();
      return 0;
    case 'c':
      check_server( optarg);
      return 0;
    case 'k':
      kill_server( optarg);
      return 0;
    }
  usage( *argv);
  return 1;
}


