#include <iostream.h>
#include "mtime.h"
#include "bootstrap_n.hh"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fstream.h>

ABng_Server_Manager::Time diff_time;
long clock_drift=0; // microseconds per second
ABng_Server_Manager::Time last_synchronized;


int operator > ( const ABng_Server_Manager::Time &t1, 
		 const ABng_Server_Manager::Time &t2) {
  ABng_Server_Manager::Time x1= t1;
  ABng_Server_Manager::Time x2= t2;
  normalize_time( x1);
  normalize_time( x2);
  if (x1.seconds > x2.seconds) return 1;
  if (x1.seconds < x2.seconds) return 0;
  if (x1.microseconds > x2.microseconds) return 1;
  return 0;
}

ABng_Server_Manager::Time operator +( const ABng_Server_Manager::Time &t1, 
				      const ABng_Server_Manager::Time &t2) {
  ABng_Server_Manager::Time ret;
  ret.seconds= t1.seconds+t2.seconds;
  ret.microseconds= t1.microseconds+t2.microseconds;
  //  normalize_time(ret);
  return ret;
}

ABng_Server_Manager::Time operator -( const ABng_Server_Manager::Time &t1, 
				      const ABng_Server_Manager::Time &t2) {
  ABng_Server_Manager::Time ret;
  ret.seconds= t1.seconds-t2.seconds;
  ret.microseconds= t1.microseconds-t2.microseconds;
  //  normalize_time(ret);
  return ret;
}

ABng_Server_Manager::Time operator /( const ABng_Server_Manager::Time &t, 
				      int dv) {
  ABng_Server_Manager::Time ret= t;
  ret.microseconds+= t.seconds*1000000;
  ret.microseconds= t.microseconds/dv;
  ret.seconds= 0;
  return ret;
}

inline ABng_Server_Manager::Time get_clock_diff() {
  struct timeval mtd;
  gettimeofday( &mtd, NULL);

  ABng_Server_Manager::Time new_diff= diff_time;
  new_diff.microseconds-= ((long) mtd.tv_sec + diff_time.seconds - last_synchronized.seconds)*clock_drift;
  //  cerr << "Diff correction " << ((long) mtd.tv_sec + diff_time.seconds - last_synchronized.seconds)*clock_drift;
  normalize_time( new_diff);
  return new_diff;
}

ABng_Server_Manager::Time currtime() {
  struct timeval mtd;
  gettimeofday( &mtd, NULL);

  ABng_Server_Manager::Time diff= get_clock_diff();
  ABng_Server_Manager::Time ret;
  ret.seconds= (long) mtd.tv_sec + diff.seconds;
  ret.microseconds= (long) mtd.tv_usec + diff.microseconds;
  normalize_time( ret);
  return ret;
}

extern "C" int getrusage(int who, struct rusage *rusage);

void start_timer( struct rusage *ru, struct timeval *td) {
  getrusage( RUSAGE_SELF, ru);
  gettimeofday( td, NULL);
}

void stop_timer( struct rusage *ru, struct timeval *td) {
  struct rusage mru;
  struct timeval mtd;
  getrusage( RUSAGE_SELF, &mru);
  gettimeofday( &mtd, NULL);

  ru->ru_utime.tv_sec= mru.ru_utime.tv_sec-ru->ru_utime.tv_sec;
  if (ru->ru_utime.tv_usec > mru.ru_utime.tv_usec) {
    ru->ru_utime.tv_usec= (1000000+mru.ru_utime.tv_usec)-ru->ru_utime.tv_usec;
    ru->ru_utime.tv_sec--;
  }
  else {
    ru->ru_utime.tv_usec= mru.ru_utime.tv_usec-ru->ru_utime.tv_usec;
  }
  ru->ru_stime.tv_sec= mru.ru_stime.tv_sec-ru->ru_stime.tv_sec;
  if (ru->ru_stime.tv_usec > mru.ru_stime.tv_usec) {
    ru->ru_stime.tv_usec= (1000000+mru.ru_stime.tv_usec)-ru->ru_stime.tv_usec;
    ru->ru_stime.tv_sec--;
  }
  else {
    ru->ru_stime.tv_usec= mru.ru_stime.tv_usec-ru->ru_stime.tv_usec;
  }

  td->tv_sec= mtd.tv_sec-td->tv_sec;
  if (td->tv_usec > mtd.tv_usec) {
    td->tv_usec= (1000000+mtd.tv_usec)-td->tv_usec;
    td->tv_sec--;
  }
  else {
    td->tv_usec= mtd.tv_usec-td->tv_usec;
  }
}

void show_timer( struct rusage *ru, struct timeval *td) {
  fprintf( stderr, "User time: %d.%6d\n", 
	   ru->ru_utime.tv_sec, ru->ru_utime.tv_usec);
  fprintf( stderr, "System time: %d.%6d\n", 
	   ru->ru_stime.tv_sec, ru->ru_stime.tv_usec);
  fprintf( stderr, "Real time: %d.%6d\n", 
	   td->tv_sec, td->tv_usec);	   
}

void my_synch_time( long eps_microseconds);

void synch_time( long eps_microseconds) {
  diff_time.seconds= diff_time.microseconds= 0;

  cerr << "Synchronizing for the 1st time" << endl;
  my_synch_time( eps_microseconds);
  last_synchronized= currtime();
  ABng_Server_Manager::Time diff1= diff_time;
  cerr << "Sleeping for 30 seconds " << endl;
  sleep(30);
  my_synch_time( eps_microseconds);
  ABng_Server_Manager::Time drift= (diff1-diff_time);
  ABng_Server_Manager::Time last_diff= diff_time;
  normalize_time( drift);
  drift= drift/((currtime()-last_synchronized).seconds);
  last_synchronized= currtime();
  cerr << "Drift " << drift.seconds << " " << drift.microseconds << endl;
  
  clock_drift= drift.microseconds;

  diff1= diff_time;
  cerr << "Sleeping for 30 seconds " << endl;
  sleep(30);
  my_synch_time( eps_microseconds);
  drift= (diff1-diff_time);
  normalize_time( drift);
  drift= drift/((currtime()-last_synchronized).seconds);
  cerr << "Drift " << drift.seconds << " " << drift.microseconds << endl;
}

void my_synch_time( long eps_microseconds) {
  char buf[2048];

  ABng_Server_Manager::TimeServer_var timesrv;

  try {
    CORBA::Object_var tmp_obj= Bootstrap::resolve( Bootstrap::SRV_Bootstrap());
    timesrv= ABng_Server_Manager::TimeServer::_narrow( tmp_obj);
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Could not narrow stringified reference TIME_SRV_IOR_FILE" << endl << ex << endl;
    return;
  }

  try {
    timesrv->_non_existent();

    ABng_Server_Manager::Time org_timestamp;
    ABng_Server_Manager::Time recv_timestamp;
    ABng_Server_Manager::Time time_err;
    ABng_Server_Manager::Time rt;
    ABng_Server_Manager::Time sum_rt;
    ABng_Server_Manager::Time mean_rt;

    mean_rt.seconds= 1000;
    mean_rt.microseconds= 0;

    int first= 0;
    int norm_cnt= 0;
    int loop=0;
    int sum_counter= 0;
    sum_rt.seconds= sum_rt.microseconds= 0;

    do {
      loop++;
      org_timestamp= currtime();
      recv_timestamp= timesrv->get_current_time();
      normalize_time( rt= currtime()-org_timestamp);
      
      if (loop>4) {
	sum_rt= sum_rt+rt;
	mean_rt= sum_rt/(++sum_counter);
      }

      if ( rt > mean_rt) {
	cout << "Roundtrip " << rt.seconds << " " << rt.microseconds
	     << "  -- discarded" << endl;
	continue;
      }
	  
      if (first) time_err= (recv_timestamp-org_timestamp);
      time_err= (recv_timestamp-org_timestamp)+((org_timestamp-currtime())/2);
      normalize_time(time_err);
      cout << "Time corrected by " << time_err.seconds << " "
	   << time_err.microseconds << endl;
      cout << "Roundtrip " << rt.seconds << " " << rt.microseconds << endl;
      first= 0;
      diff_time= diff_time+time_err;
      normalize_time( diff_time);
      cout << "diff " << diff_time.seconds << " " << diff_time.microseconds << endl;


      if (time_err.seconds==0 && time_err.microseconds<eps_microseconds)
	norm_cnt++;
      else norm_cnt= 0;
      sleep(1);
    }
    while ( norm_cnt<5);
  }
  catch (CORBA::Exception &ex) {
    cerr << "synch_time caught " << ex << endl;
  } 
}



