// $Id: view_dyn_cl.cc,v 1.1.1.1 1998/06/18 14:44:37 mencnar Exp $
//
// $Source: /home/iris/grant/abng/CVS/sources/utils/sighting_processors_util/view_dyn_cl.cc,v $
//
// $Log: view_dyn_cl.cc,v $
// Revision 1.1.1.1  1998/06/18 14:44:37  mencnar
//
// Revision 1.1  1998/03/30 08:02:35  mencnar
// Initial revision
//
//
//----------------------------------------------------------------
// File:          view_dyn_cl.cc
// Author:        Daniel Mencnarowski (mencnar@ics.agh.edu.pl)
// Date Created:  23.03.98
//
// General Description:
//   Test client, observers all observable activities within a view
//

#include <rw/rwtime.h>
#include <rw/rwdate.h>
#include "abng_core_management.hh"
#include "bootstrap_n.hh"
#include <stdlib.h>
#include <iostream.h>
#include "event_log_i.hh"

ABng_Core::View_var view;

long call_counter= 0;
//Synch::Time start_time,last_time;

ostream & operator << ( ostream &ostr, 
			const ABng_Core::Time &time) {

  RWTime offset( RWDate( 1, 1,1970), 0,0,0, RWZone::utc());
  RWTime t( offset.seconds()+time.seconds);
  ostr << t << " " << time.microseconds << " us (local time)";
  return ostr;
}

ostream & operator << ( ostream &ostr, const ABng_Core::Sighting_Description &sd) {
  ostr << "\tview_name\t\t" << sd.view_name << endl;
  ostr << "\tmy_locatable\t\t" << sd.my_locatable << endl;
  ostr << "\tmy_locatable_type\t";

  switch (sd.my_locatable_type) {
  case ABng_Core::LOCATABLE_USER:
    ostr << "LOCATABLE_USER" << endl;
    break;
  case ABng_Core::LOCATABLE_EQUIPMENT:
    ostr << "LOCATABLE_EQUIPMENT" << endl;
    break;
  default:
    ostr << "unknown" << endl;
    break;
  }
  ostr << "\tmy_location\t\t" << sd.my_location << endl;
  ostr << "\tmessage_type (hex) " << hex << (unsigned) sd.message_type << endl;
  ostr << "\tattributes (hex) " << hex << (unsigned) sd.attributes << endl;
  ostr << "\tsequence_number (hex) " << hex << (unsigned) sd.sequence_number << endl;
  ostr << dec << "\tupdate_time\t\t" << sd.update_time << endl;
  ostr << "\tquality\t\t" << (unsigned) sd.q << endl;
  return ostr;
}

class View_Data_ObserverImpl : public BOA_IMPL_BASE(ABng_Core,View_Data_Observer) {
  void notify( Domain_Repository::Repository_Item_Id item_id,
	       const ABng_Core::View_Data_Observer::View_Data_Object &vdo,
	       Domain_Repository::Change_Type ct
	       ENV_DECL( IT_env) );

  void abort( ENV_DECL1( IT_env) );
} *vdo_impl;

class Dynamic_Data_ObserverImpl : public BOA_IMPL_BASE(ABng_Core,Dynamic_Data_Observer) {
  const char *name;
public:
  Dynamic_Data_ObserverImpl( const char *_name) : name( _name) {}

  void receive_sighting( const ABng_Core::Sighting_Description &sd
			 ENV_DECL( IT_env) );
  void receive_sightings( const ABng_Core::Sighting_Descriptions &sds
			  ENV_DECL( IT_env) );
  void abort( ENV_DECL1( IT_env) );
} *ddo_impl_locatable, *ddo_impl_location;

void View_Data_ObserverImpl::notify( 
               Domain_Repository::Repository_Item_Id item_id,
	       const ABng_Core::View_Data_Observer::View_Data_Object &vdo,
	       Domain_Repository::Change_Type ct
	       ENV_DEF( IT_env) ) {

  Domain_Repository::Repository_Item_Ids rids( 1);
  rids.length( 1);

  cout << "VDO::notify (item_id= " << item_id
       << "; object= " << ((vdo._d() == ABng_Core::View_Data_Observer::LOCATION_VD) ?
			   "location" : "locatable")
       << "; type= " << (ct == Domain_Repository::OBJECT_ADDED ? "added" :
			 (ct == Domain_Repository::OBJECT_UPDATED ? "updated" :
			  (ct == Domain_Repository::OBJECT_REMOVED ? "removed" : "?")))
       << endl;
  try {
    if ( ct == Domain_Repository::OBJECT_ADDED) {
      switch ( vdo._d()) {
      case ABng_Core::View_Data_Observer::LOCATION_VD:
	rids[ 0]= vdo.location_obj()->item_id();
	view->attach_location_view_data_observer( this, rids);
	view->attach_location_dynamic_data_observer( ddo_impl_location, rids);
	break;
      case ABng_Core::View_Data_Observer::LOCATABLE_VD:
	rids[ 0]= vdo.locatable_obj()->item_id();
	view->attach_locatable_view_data_observer( this, rids);
	view->attach_locatable_dynamic_data_observer( ddo_impl_locatable, rids);
	break;
      }
    }
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception " << ex << endl;
  }
}

void View_Data_ObserverImpl::abort( ENV_DEF1( IT_env)) {
  cout << "VDO::abort" << endl;
}

void Dynamic_Data_ObserverImpl::receive_sighting( 
					const ABng_Core::Sighting_Description &sd
					ENV_DEF( IT_env) ) {
  LOGTIMESTAMP( sd.update_time );

  cout << "DDO::notify " << name << endl;
  cout << sd << endl;
  return;
  /*
  if (call_counter%200 == 10 || call_counter%200==199) {
    Synch::Time ct= currtime();
    cerr << "Sighting received at " << ct.seconds << " " << ct.microseconds << endl;
  }
  call_counter++;
  return;

  if (call_counter==1) start_time= currtime();
  else last_time= currtime();
  if (call_counter%1000 == 0 && call_counter>1) {
    cerr << "Start time " << start_time.seconds << " " << start_time.microseconds << endl;
    cerr << "call counter " << call_counter << endl;
    Synch::Time avt= (last_time-start_time);
    normalize_time(avt);
    cerr << "Total time " << avt.seconds << " " << avt.microseconds << endl;
    call_counter= -1;
  }
  call_counter++;
  return;
  cout << "DDO::notify " << name << endl;
  cout << sd << endl;
  */
}

void Dynamic_Data_ObserverImpl::receive_sightings( 
					const ABng_Core::Sighting_Descriptions &sds
					ENV_DEF( IT_env) ) {
  if (sds.length())
    LOGTIMESTAMP( sds[0].update_time );
  cerr << (int) sds.length()  << " sightings received" << endl;
}

void Dynamic_Data_ObserverImpl::abort( ENV_DEF1( IT_env)) {

  cout << "DDO::abort" << endl;
}

//-----------------------------------------------------------------------------------
//
void usage( char *prog_name) {
  cout << prog_name << endl
       << "\t-v view_name" << endl
       << "\t-o observer_name" << endl
       << "\t[-l multiply_register]" << endl;
  exit(1);
}

int main( int argc, char **argv) {
  CORBA::ORB_ptr orb= CORBA::ORB_init( argc, argv, ORB_IDENTIFIER);
  CORBA::BOA_ptr boa= orb->BOA_init( argc, argv, BOA_IDENTIFIER);

  if (argc<2) usage(argv[0]);

  Bootstrap::init( orb);
  Bootstrap::synch_time( 10000); // 1 millisecond should be enough
  Bootstrap::need_synch_time( 10000);

  View_Data_Observer_var vdo_impl_var=
    ( vdo_impl= new View_Data_ObserverImpl());
  Dynamic_Data_Observer_var ddo_impl_locatable_var=
    ( ddo_impl_locatable= new Dynamic_Data_ObserverImpl("Locatable"));
  Dynamic_Data_Observer_var ddo_impl_location_var=
    ( ddo_impl_location= new Dynamic_Data_ObserverImpl("Location"));

  int c;
  CORBA::Object_var tmp_obj;
  CORBA::String_var observer_name;
  int multiply_register= 1;
  
  while (( c = getopt( argc, argv, "v:o:l:")) != -1)
    switch (c) {
    case 'v':
      try {
	tmp_obj= Bootstrap::resolve( 
				    Bootstrap::CTX_ABng_Core_Public_Views()+optarg);
	view= ABng_Core::View::_narrow( tmp_obj);
      }
      catch ( CORBA::Exception &ex) {
	cerr << "Cannot bind to view" << endl;
	cerr << ex << endl;
	exit( 1);
      }
      break;
    case 'o':
      observer_name= (const char *) optarg;
      break;
    case 'l':
      multiply_register= atoi( optarg);
      break;
    default:
      usage( argv[0]);
    }

  if ( CORBA::is_nil( (ABng_Core::View_ptr) view)) {
    cerr << "View object not bound (use -v option)" << endl;
    usage( argv[0]);
  }
  if ( (const char *) observer_name == NULL) {
    cerr << "Observer name not set (use -o option)" << endl;
    usage( argv[0]);
  }
  init_event_log( (const char *) observer_name);

  Domain_Repository::Repository_Item_Ids rids( 1);
  rids.length( 1);

  try {
    ABng_Core::Locations_var locations= view->location_list();
    ABng_Core::Locatables_var locatables= view->locatable_list();
    view->attach_location_add_view_data_observer( vdo_impl);
    view->attach_locatable_add_view_data_observer( vdo_impl);
    
    for (int j=0; j<multiply_register; j++) {
      Dynamic_Data_ObserverImpl *obs= new Dynamic_Data_ObserverImpl("Location");
      CONNECT_IMPL( ABng_Core::Dynamic_Data_Observer_var, obs);
      for (int i=0; i<locatables->length(); i++) {
	rids[ 0]= locatables[ i]->item_id();
	if (j==0) view->attach_locatable_view_data_observer( vdo_impl,
							     rids);

	view->attach_locatable_dynamic_data_observer( obs, rids);
      }
    }
    /*
    for (int j=0; j<locatables->length(); j++) {
      rids[ 0]= locatables[ j]->item_id();
      view->attach_locatable_view_data_observer( vdo_impl,
						 rids);
      view->attach_locatable_dynamic_data_observer( ddo_impl_locatable,
						    rids);
    }
    */
    //    synch_time( 200);
    cerr << "Implementation ready" << endl;
#ifdef ORBIX
    boa->processEvents( CORBA::ORB::INFINITE_TIMEOUT);
#elif defined OMNIORB
    boa->impl_is_ready();
#endif
  }
  catch ( Domain_Repository::Destination_Unreachable &du) {
    cerr << "Destination unreachable " << du.server_name << endl;
  }
  catch ( CORBA::SystemException &se) {
    cerr << "Caught system exception" << endl;
  }
  catch ( CORBA::Exception &ex) {
    cerr << "Caught exception" << endl;
    cerr << ex << endl;
  }
  return 1;
}




