//
//  File:         user_rep_monitor.cc
//  Author:       Andrzej Uszok (uszok@ics.agh.edu.pl)
//  Date Created: 29.01.98
//
// General Description:
//   Implementation of the programe monitoring changes in the ABng repositories
//

#define USE_IIOP
#define USE_IR

#include "user_rep_observer_i.h"

#define User_Rep_IOR_File "./User_Rep_iiop.ref"

int main(void) {
	CORBA::Object_ptr obj_ptr;   
	User_Rep_ptr my_user_rep_ptr;
	char ref_str[500];
	ifstream strm(User_Rep_IOR_File);
	unsigned long timer = CORBA::Orbix.INFINITE_TIMEOUT;
	
	//
	// init ORB
	//
	try {
		CORBA::Orbix.impl_is_ready("user_rep_monitor", 0);
	}
	catch (CORBA::SystemException &sysEx) {
		cerr << "Unexpected system exception" << endl;
		cerr << &sysEx;
		exit(-1);
	} 
	catch(...) {
		cerr << "Unexpected exception" << endl;
		exit(-1);
	}

	// initialise the User_Rep from the file
	if (!strm){
		cerr << "User_Rep monitor error: Unable to open file - ";
		cerr << User_Rep_IOR_File << endl;
		return 0;
	}
	strm >> ref_str;

	// get the obj referance
	try {
		obj_ptr = CORBA::Orbix.string_to_object(ref_str);
	} catch (CORBA::SystemException& se) {
		cerr << "User_Rep monitor error: Exception! - " << &se << endl;
		exit(1);
	}
	cerr << "User_Rep monitor: got User_Rep object reference" << endl;

	// check if its nil
	if (CORBA::is_nil(obj_ptr)){     
		cerr << "User_Rep monitor error: ";
		cerr << "User_Rep object reference is nill!" << endl;
		exit(-1);
	}

	// attempt to narrow
	try {
		my_user_rep_ptr = Domain_Repository::User_Rep::_narrow(obj_ptr);
	} catch (CORBA::SystemException& se){
		cerr << "User_Rep monitor error: ";
		cerr << "exception when trying to _narrow! " << &se << endl;
	}

	// check if my_user_rep_ptr is nil
	if (CORBA::is_nil(my_user_rep_ptr)){
		cerr << "User_Rep monitor error: ";
		cerr << "failed to narrow object reference!" << endl;
		exit(-1);
	}
	
	// contact with User_Rep established  !!

	// create a observer of user repository
	User_Rep_Observer_I* my_user_rep_observer_i = new User_Rep_Observer_I ();

	// create a TIE for the object
	TIE_Domain_Repository_Static_Data_Observer(User_Rep_Observer_I)* my_user_rep_observer = 
		new TIE_Domain_Repository_Static_Data_Observer(User_Rep_Observer_I) 
		(my_user_rep_observer_i);

	my_user_rep_observer_i->my_corba_object = 
		Domain_Repository::Static_Data_Observer::_duplicate(my_user_rep_observer);

  	CORBA::Orbix.setServerName("user_rep_monitor");

	// attempt to register add observer in user_rep
	try {
		my_user_rep_ptr->attach_add_static_data_observer(my_user_rep_observer);
	} catch (CORBA::SystemException& se){
		cerr << "User_Rep monitor error: ";
		cerr << "exception when trying to register add observer! " << &se << endl;
	}
	cerr << "User_Rep monitor: add observer registered in User_Rep" << endl;

	//
	// export the server to the network
	//
	try {
		CORBA::Orbix.impl_is_ready("user_rep_monitor", timer);
	}
	catch (CORBA::SystemException &sysEx) {
		cerr << "Unexpected system exception" << endl;
		cerr << &sysEx;
		exit(-1);
	} 
	catch(...) {
		cerr << "Unexpected exception" << endl;
		exit(-1);
	}

	return 0;
}
